@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2004-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,6 +53,7 @@ class ColorSwatch : public QDockWidget
     QAction *movableAction;
     QAction *floatableAction;
     QAction *floatingAction;
+    QAction *verticalTitleBarAction;
 
     QActionGroup *allowedAreasActions;
     QAction *allowLeftAction;
@@ -51,15 +67,22 @@ class ColorSwatch : public QDockWidget
     QAction *topAction;
     QAction *bottomAction;
 
+    QAction *changeSizeHintsAction;
+
+    QMenu *tabMenu;
+    QMenu *splitHMenu;
+    QMenu *splitVMenu;
+
+    QAction *windowModifiedAction;
+
 public:
-    ColorSwatch(const QString &colorName, QWidget *parent = 0, Qt::WFlags flags = 0);
+    ColorSwatch(const QString &colorName, QWidget *parent = 0, Qt::WindowFlags flags = 0);
 
     QMenu *menu;
+    void setCustomSizeHint(const QSize &size);
 
 protected:
     virtual void contextMenuEvent(QContextMenuEvent *event);
-    virtual bool event(QEvent *);
-
 
 private:
     void allow(Qt::DockWidgetArea area, bool allow);
@@ -70,6 +93,8 @@ private slots:
     void changeMovable(bool on);
     void changeFloatable(bool on);
     void changeFloating(bool on);
+    void changeVerticalTitleBar(bool on);
+    void updateContextMenu();
 
     void allowLeft(bool a);
     void allowRight(bool a);
@@ -80,6 +105,9 @@ private slots:
     void placeRight(bool p);
     void placeTop(bool p);
     void placeBottom(bool p);
+
+    void splitInto(QAction *action);
+    void tabInto(QAction *action);
 };
 
 #endif
