@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2004-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -37,6 +52,8 @@
 #include <QMessageBox>
 #include <QSignalMapper>
 #include <QApplication>
+#include <QPainter>
+#include <QMouseEvent>
 #include <qdebug.h>
 
 static const char * const message =
@@ -57,7 +74,8 @@ static const char * const message =
 #endif
     ;
 
-MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
+MainWindow::MainWindow(const QMap<QString, QSize> &customSizeHints,
+                        QWidget *parent, Qt::WindowFlags flags)
     : QMainWindow(parent, flags)
 {
     setObjectName("MainWindow");
@@ -65,7 +83,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
 
     setupToolBar();
     setupMenuBar();
-    setupDockWidgets();
+    setupDockWidgets(customSizeHints);
 
     QTextEdit *center = new QTextEdit(this);
     center->setReadOnly(true);
@@ -83,9 +101,11 @@ void MainWindow::actionTriggered(QAction *action)
 
 void MainWindow::setupToolBar()
 {
-    toolbar = new ToolBar(this);
-    toolbar->setAllowedAreas(Qt::TopToolBarArea | Qt::BottomToolBarArea);
-    addToolBar(toolbar);
+    for (int i = 0; i < 3; ++i) {
+        ToolBar *tb = new ToolBar(QString::fromLatin1("Tool Bar %1").arg(i + 1), this);
+        toolBars.append(tb);
+        addToolBar(tb);
+    }
 }
 
 void MainWindow::setupMenuBar()
@@ -105,18 +125,57 @@ void MainWindow::setupMenuBar()
 
     menu->addAction(tr("&Quit"), this, SLOT(close()));
 
-    menuBar()->addMenu(toolbar->menu);
+    mainWindowMenu = menuBar()->addMenu(tr("Main window"));
+
+    action = mainWindowMenu->addAction(tr("Animated docks"));
+    action->setCheckable(true);
+    action->setChecked(dockOptions() & AnimatedDocks);
+    connect(action, SIGNAL(toggled(bool)), this, SLOT(setDockOptions()));
+
+    action = mainWindowMenu->addAction(tr("Allow nested docks"));
+    action->setCheckable(true);
+    action->setChecked(dockOptions() & AllowNestedDocks);
+    connect(action, SIGNAL(toggled(bool)), this, SLOT(setDockOptions()));
+
+    action = mainWindowMenu->addAction(tr("Allow tabbed docks"));
+    action->setCheckable(true);
+    action->setChecked(dockOptions() & AllowTabbedDocks);
+    connect(action, SIGNAL(toggled(bool)), this, SLOT(setDockOptions()));
+
+    action = mainWindowMenu->addAction(tr("Force tabbed docks"));
+    action->setCheckable(true);
+    action->setChecked(dockOptions() & ForceTabbedDocks);
+    connect(action, SIGNAL(toggled(bool)), this, SLOT(setDockOptions()));
+
+    action = mainWindowMenu->addAction(tr("Vertical tabs"));
+    action->setCheckable(true);
+    action->setChecked(dockOptions() & VerticalTabs);
+    connect(action, SIGNAL(toggled(bool)), this, SLOT(setDockOptions()));
+
+    QMenu *toolBarMenu = menuBar()->addMenu(tr("Tool bars"));
+    for (int i = 0; i < toolBars.count(); ++i)
+        toolBarMenu->addMenu(toolBars.at(i)->menu);
+
     dockWidgetMenu = menuBar()->addMenu(tr("&Dock Widgets"));
 }
 
-static void dump(const QByteArray &array)
+void MainWindow::setDockOptions()
 {
-    QString s;
-    for (int i = 0; i < array.count(); ++i) {
-        s += ' ';
-        s += QString::number((uchar)array.at(i));
-    }
-    qDebug() << "dump():" << s;
+    DockOptions opts;
+    QList<QAction*> actions = mainWindowMenu->actions();
+
+    if (actions.at(0)->isChecked())
+        opts |= AnimatedDocks;
+    if (actions.at(1)->isChecked())
+        opts |= AllowNestedDocks;
+    if (actions.at(2)->isChecked())
+        opts |= AllowTabbedDocks;
+    if (actions.at(3)->isChecked())
+        opts |= ForceTabbedDocks;
+    if (actions.at(4)->isChecked())
+        opts |= VerticalTabs;
+
+    QMainWindow::setDockOptions(opts);
 }
 
 void MainWindow::saveLayout()
@@ -207,20 +266,115 @@ QAction *addAction(QMenu *menu, const QString &text, QActionGroup *group, QSigna
     return result;
 }
 
-void MainWindow::setupDockWidgets()
+class BlueTitleBar : public QWidget
 {
-    QAction *action = dockWidgetMenu->addAction(tr("Animation"));
-    action->setCheckable(true);
-    action->setChecked(isAnimated());
-    connect(action, SIGNAL(toggled(bool)), this, SLOT(setAnimated(bool)));
+    Q_OBJECT
+public:
+    BlueTitleBar(QWidget *parent = 0);
 
-    action = dockWidgetMenu->addAction(tr("Nesting"));
-    action->setCheckable(true);
-    action->setChecked(isDockNestingEnabled());
-    connect(action, SIGNAL(toggled(bool)), this, SLOT(setDockNestingEnabled(bool)));
+    QSize sizeHint() const { return minimumSizeHint(); }
+    QSize minimumSizeHint() const;
+protected:
+    void paintEvent(QPaintEvent *event);
+    void mousePressEvent(QMouseEvent *event);
+private:
+    QPixmap leftPm, centerPm, rightPm;
+};
 
-    dockWidgetMenu->addSeparator();
+QSize BlueTitleBar::minimumSizeHint() const
+{
+    QDockWidget *dw = qobject_cast<QDockWidget*>(parentWidget());
+    Q_ASSERT(dw != 0);
+    QSize result(leftPm.width() + rightPm.width(), centerPm.height());
+    if (dw->features() & QDockWidget::DockWidgetVerticalTitleBar)
+        result.transpose();
+    return result;
+}
 
+BlueTitleBar::BlueTitleBar(QWidget *parent)
+    : QWidget(parent)
+{
+    leftPm = QPixmap(":/res/titlebarLeft.png");
+    centerPm = QPixmap(":/res/titlebarCenter.png");
+    rightPm = QPixmap(":/res/titlebarRight.png");
+}
+
+void BlueTitleBar::paintEvent(QPaintEvent*)
+{
+    QPainter painter(this);
+    QRect rect = this->rect();
+
+    QDockWidget *dw = qobject_cast<QDockWidget*>(parentWidget());
+    Q_ASSERT(dw != 0);
+
+    if (dw->features() & QDockWidget::DockWidgetVerticalTitleBar) {
+        QSize s = rect.size();
+        s.transpose();
+        rect.setSize(s);
+
+        painter.translate(rect.left(), rect.top() + rect.width());
+        painter.rotate(-90);
+        painter.translate(-rect.left(), -rect.top());
+    }
+
+    painter.drawPixmap(rect.topLeft(), leftPm);
+    painter.drawPixmap(rect.topRight() - QPoint(rightPm.width() - 1, 0), rightPm);
+    QBrush brush(centerPm);
+    painter.fillRect(rect.left() + leftPm.width(), rect.top(),
+                        rect.width() - leftPm.width() - rightPm.width(),
+                        centerPm.height(), centerPm);
+}
+
+void BlueTitleBar::mousePressEvent(QMouseEvent *event)
+{
+    QPoint pos = event->pos();
+
+    QRect rect = this->rect();
+
+    QDockWidget *dw = qobject_cast<QDockWidget*>(parentWidget());
+    Q_ASSERT(dw != 0);
+
+    if (dw->features() & QDockWidget::DockWidgetVerticalTitleBar) {
+        QPoint p = pos;
+        pos.setX(rect.left() + rect.bottom() - p.y());
+        pos.setY(rect.top() + p.x() - rect.left());
+
+        QSize s = rect.size();
+        s.transpose();
+        rect.setSize(s);
+    }
+
+    const int buttonRight = 7;
+    const int buttonWidth = 20;
+    int right = rect.right() - pos.x();
+    int button = (right - buttonRight)/buttonWidth;
+    switch (button) {
+        case 0:
+            event->accept();
+            dw->close();
+            break;
+        case 1:
+            event->accept();
+            dw->setFloating(!dw->isFloating());
+            break;
+        case 2: {
+            event->accept();
+            QDockWidget::DockWidgetFeatures features = dw->features();
+            if (features & QDockWidget::DockWidgetVerticalTitleBar)
+                features &= ~QDockWidget::DockWidgetVerticalTitleBar;
+            else
+                features |= QDockWidget::DockWidgetVerticalTitleBar;
+            dw->setFeatures(features);
+            break;
+        }
+        default:
+            event->ignore();
+            break;
+    }
+}
+
+void MainWindow::setupDockWidgets(const QMap<QString, QSize> &customSizeHints)
+{
     mapper = new QSignalMapper(this);
     connect(mapper, SIGNAL(mapped(int)), this, SLOT(setCorner(int)));
 
@@ -260,9 +414,7 @@ void MainWindow::setupDockWidgets()
 #else
         { "Black", Qt::Drawer, Qt::LeftDockWidgetArea },
 #endif
-        { "White", 0, Qt::LeftDockWidgetArea },
-        { "LightGrey", 0, Qt::RightDockWidgetArea },
-        { "DarkGrey", 0, Qt::RightDockWidgetArea },
+        { "White", 0, Qt::RightDockWidgetArea },
         { "Red", 0, Qt::TopDockWidgetArea },
         { "Green", 0, Qt::TopDockWidgetArea },
         { "Blue", 0, Qt::BottomDockWidgetArea },
@@ -274,6 +426,19 @@ void MainWindow::setupDockWidgets()
         ColorSwatch *swatch = new ColorSwatch(tr(sets[i].name), this, Qt::WindowFlags(sets[i].flags));
         if (i%2)
             swatch->setWindowIcon(QIcon(QPixmap(":/res/qt.png")));
+        if (qstrcmp(sets[i].name, "Blue") == 0) {
+            swatch->setTitleBarWidget(new BlueTitleBar(swatch));
+#ifdef Q_WS_QWS
+            QPalette pal = palette();
+            pal.setBrush(backgroundRole(), QColor(0,0,0,0));
+            swatch->setPalette(pal);
+#endif
+        }
+
+        QString name = QString::fromLatin1(sets[i].name);
+        if (customSizeHints.contains(name))
+            swatch->setCustomSizeHint(customSizeHints.value(name));
+
         addDockWidget(sets[i].area, swatch);
         dockWidgetMenu->addMenu(swatch->menu);
     }
@@ -316,3 +481,5 @@ void MainWindow::switchLayoutDirection()
     else
         qApp->setLayoutDirection(Qt::LeftToRight);
 }
+
+#include "mainwindow.moc"
