@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2004-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -28,6 +43,8 @@
 #include <QPainter>
 #include <QPainterPath>
 #include <QSpinBox>
+#include <QLabel>
+#include <QToolTip>
 
 #include <stdlib.h>
 
@@ -50,10 +67,12 @@ static QPixmap genIcon(const QSize &iconSize, const QString &, const QColor &col
 static QPixmap genIcon(const QSize &iconSize, int number, const QColor &color)
 { return genIcon(iconSize, QString::number(number), color); }
 
-ToolBar::ToolBar(QWidget *parent)
+ToolBar::ToolBar(const QString &title, QWidget *parent)
     : QToolBar(parent), spinbox(0), spinboxAction(0)
 {
-    setWindowTitle(tr("Main Tool Bar"));
+    tip = 0;
+    setWindowTitle(title);
+    setObjectName(title);
 
     setIconSize(QSize(32, 32));
 
@@ -143,11 +162,14 @@ ToolBar::ToolBar(QWidget *parent)
     areaActions->addAction(topAction);
     areaActions->addAction(bottomAction);
 
+    toolBarBreakAction = new QAction(tr("Insert break"), this);
+    connect(toolBarBreakAction, SIGNAL(triggered(bool)), this, SLOT(insertToolBarBreak()));
+
     connect(movableAction, SIGNAL(triggered(bool)), areaActions, SLOT(setEnabled(bool)));
 
     connect(movableAction, SIGNAL(triggered(bool)), allowedAreasActions, SLOT(setEnabled(bool)));
 
-    menu = new QMenu(tr("&Toolbar"), this);
+    menu = new QMenu(title, this);
     menu->addAction(toggleViewAction());
     menu->addSeparator();
     menu->addAction(orderAction);
@@ -161,12 +183,15 @@ ToolBar::ToolBar(QWidget *parent)
     menu->addActions(allowedAreasActions->actions());
     menu->addSeparator();
     menu->addActions(areaActions->actions());
+    menu->addSeparator();
+    menu->addAction(toolBarBreakAction);
+
+    connect(menu, SIGNAL(aboutToShow()), this, SLOT(updateMenu()));
 
     randomize();
 }
 
-
-void ToolBar::polishEvent(QEvent *)
+void ToolBar::updateMenu()
 {
     QMainWindow *mainWindow = qobject_cast<QMainWindow *>(parentWidget());
     Q_ASSERT(mainWindow != 0);
@@ -313,3 +338,43 @@ void ToolBar::placeTop(bool p)
 
 void ToolBar::placeBottom(bool p)
 { place(Qt::BottomToolBarArea, p); }
+
+void ToolBar::insertToolBarBreak()
+{
+    QMainWindow *mainWindow = qobject_cast<QMainWindow *>(parentWidget());
+    Q_ASSERT(mainWindow != 0);
+
+    mainWindow->insertToolBarBreak(this);
+}
+
+void ToolBar::enterEvent(QEvent*)
+{
+/*
+    These labels on top of toolbars look darn ugly
+
+    if (tip == 0) {
+        tip = new QLabel(windowTitle(), this);
+        QPalette pal = tip->palette();
+        QColor c = Qt::black;
+        c.setAlpha(100);
+        pal.setColor(QPalette::Window, c);
+        pal.setColor(QPalette::Foreground, Qt::white);
+        tip->setPalette(pal);
+        tip->setAutoFillBackground(true);
+        tip->setMargin(3);
+        tip->setText(windowTitle());
+    }
+    QPoint c = rect().center();
+    QSize hint = tip->sizeHint();
+    tip->setGeometry(c.x() - hint.width()/2, c.y() - hint.height()/2,
+                        hint.width(), hint.height());
+
+    tip->show();
+*/
+}
+
+void ToolBar::leaveEvent(QEvent*)
+{
+    if (tip != 0)
+        tip->hide();
+}
