@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the demonstration applications of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -105,6 +120,7 @@ void ArthurStyle::drawPrimitive(PrimitiveElement element, const QStyleOption *op
 
             painter->save();
             const QPushButton *pushButton = qobject_cast<const QPushButton *>(widget);
+            Q_ASSERT(pushButton);
             QWidget *parent = pushButton->parentWidget();
             if (parent && qobject_cast<QGroupBox *>(parent)) {
                 QLinearGradient lg(0, 0, 0, parent->height());
@@ -117,7 +133,7 @@ void ArthurStyle::drawPrimitive(PrimitiveElement element, const QStyleOption *op
                 painter->setBrushOrigin(0,0);
             }
 
-            bool down = pushButton && ((button->state & State_Sunken) || (button->state & State_On));
+            bool down = (button->state & State_Sunken) || (button->state & State_On);
 
             QPixmap left, right, mid;
             if (down) {
@@ -346,6 +362,9 @@ QSize ArthurStyle::sizeFromContents(ContentsType type, const QStyleOption *optio
 
 
     switch (type) {
+    case CT_RadioButton:
+        newSize += QSize(20, 0);
+        break;
 
     case CT_PushButton:
         newSize.setHeight(26);
@@ -372,10 +391,12 @@ int ArthurStyle::pixelMetric(PixelMetric pm, const QStyleOption *opt, const QWid
 void ArthurStyle::polish(QWidget *widget)
 {
     if (widget->layout() && qobject_cast<QGroupBox *>(widget)) {
-        if (qFindChildren<QGroupBox *>(widget).size() == 0)
+        if (qFindChildren<QGroupBox *>(widget).size() == 0) {
             widget->layout()->setSpacing(0);
-        else
-            widget->layout()->setMargin(10);
+            widget->layout()->setMargin(12);
+        } else {
+            widget->layout()->setMargin(13);
+        }
     }
 
     if (qobject_cast<QPushButton *>(widget)
