@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,27 +58,27 @@ MultiPageWidgetPlugin::MultiPageWidgetPlugin(QObject *parent)
 
 QString MultiPageWidgetPlugin::name() const
 {
-    return QString("MultiPageWidget");
+    return QLatin1String("MultiPageWidget");
 }
 
 QString MultiPageWidgetPlugin::group() const
 {
-    return QString("Display Widgets [Examples]");
+    return QLatin1String("Display Widgets [Examples]");
 }
 
 QString MultiPageWidgetPlugin::toolTip() const
 {
-    return "";
+    return QString();
 }
 
 QString MultiPageWidgetPlugin::whatsThis() const
 {
-    return "";
+    return QString();
 }
 
 QString MultiPageWidgetPlugin::includeFile() const
 {
-    return QString("multipagewidget.h");
+    return QLatin1String("multipagewidget.h");
 }
 
 QIcon MultiPageWidgetPlugin::icon() const
@@ -107,7 +122,7 @@ void MultiPageWidgetPlugin::initialize(QDesignerFormEditorInterface *formEditor)
 
 QString MultiPageWidgetPlugin::domXml() const
 {
-    return QString("\
+    return QLatin1String("\
     <widget class=\"MultiPageWidget\" name=\"multipagewidget\">\
         <widget class=\"QWidget\" name=\"page\" />\
     </widget>\
@@ -116,10 +131,10 @@ QString MultiPageWidgetPlugin::domXml() const
 
 void MultiPageWidgetPlugin::currentIndexChanged(int index)
 {
+    Q_UNUSED(index);
     MultiPageWidget *widget = qobject_cast<MultiPageWidget*>(sender());
     if (widget) {
-        QDesignerFormWindowInterface *form;
-        form = QDesignerFormWindowInterface::findFormWindow(widget);
+        QDesignerFormWindowInterface *form = QDesignerFormWindowInterface::findFormWindow(widget);
         if (form)
             form->emitSelectionChanged();
     }
@@ -127,6 +142,7 @@ void MultiPageWidgetPlugin::currentIndexChanged(int index)
 
 void MultiPageWidgetPlugin::pageTitleChanged(const QString &title)
 {
+    Q_UNUSED(title);
     MultiPageWidget *widget = qobject_cast<MultiPageWidget*>(sender());
     if (widget) {
         QWidget *page = widget->widget(widget->currentIndex());
@@ -137,7 +153,7 @@ void MultiPageWidgetPlugin::pageTitleChanged(const QString &title)
             QExtensionManager *manager = editor->extensionManager();
             QDesignerPropertySheetExtension *sheet;
             sheet = qt_extension<QDesignerPropertySheetExtension*>(manager, page);
-            int propertyIndex = sheet->indexOf(QLatin1String("windowTitle"));
+            const int propertyIndex = sheet->indexOf(QLatin1String("windowTitle"));
             sheet->setChanged(propertyIndex, true);
         }
     }
