@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -9,42 +9,55 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
+#include <QtDesigner>
+#include <QtGui>
+#include <QtPlugin>
+
 #include "tictactoe.h"
 #include "tictactoeplugin.h"
 #include "tictactoetaskmenu.h"
 
-#include <QtDesigner/QExtensionManager>
-#include <QtDesigner/QDesignerFormEditorInterface>
-
-#include <QIcon>
-#include <QtPlugin>
-
 TicTacToePlugin::TicTacToePlugin(QObject *parent)
-    :QObject(parent)
+    : QObject(parent)
 {
     initialized = false;
 }
 
 QString TicTacToePlugin::name() const
 {
-    return QString("TicTacToe");
+    return "TicTacToe";
 }
 
 QString TicTacToePlugin::group() const
 {
-    return QString("Display Widgets [Examples]");
+    return "Display Widgets [Examples]";
 }
 
 QString TicTacToePlugin::toolTip() const
@@ -59,7 +72,7 @@ QString TicTacToePlugin::whatsThis() const
 
 QString TicTacToePlugin::includeFile() const
 {
-    return QString("tictactoe.h");
+    return "tictactoe.h";
 }
 
 QIcon TicTacToePlugin::icon() const
