@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2004-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -27,10 +42,10 @@
 #include "mainwindow.h"
 #include "pixeldelegate.h"
 
-MainWindow::MainWindow() : QMainWindow()
+MainWindow::MainWindow()
 {
-    currentPath = QDir::home().absolutePath();
-    model = new ImageModel(QImage(), this);
+    currentPath = QDir::homePath();
+    model = new ImageModel(this);
 
     QWidget *centralWidget = new QWidget;
 
@@ -38,13 +53,16 @@ MainWindow::MainWindow() : QMainWindow()
     view->setShowGrid(false);
     view->horizontalHeader()->hide();
     view->verticalHeader()->hide();
+    view->horizontalHeader()->setMinimumSectionSize(1);
+    view->verticalHeader()->setMinimumSectionSize(1);
+    view->setModel(model);
 
     PixelDelegate *delegate = new PixelDelegate(this);
     view->setItemDelegate(delegate);
 
     QLabel *pixelSizeLabel = new QLabel(tr("Pixel size:"));
     QSpinBox *pixelSizeSpinBox = new QSpinBox;
-    pixelSizeSpinBox->setMinimum(1);
+    pixelSizeSpinBox->setMinimum(4);
     pixelSizeSpinBox->setMaximum(32);
     pixelSizeSpinBox->setValue(12);
 
@@ -105,11 +123,7 @@ void MainWindow::openImage(const QString &fileName)
     QImage image;
 
     if (image.load(fileName)) {
-        ImageModel *newModel = new ImageModel(image, this);
-        view->setModel(newModel);
-        delete model;
-        model = newModel;
-
+        model->setImage(image);
         if (!fileName.startsWith(":/")) {
             currentPath = fileName;
             setWindowTitle(tr("%1 - Pixelator").arg(currentPath));
@@ -124,10 +138,11 @@ void MainWindow::printImage()
 {
     if (model->rowCount(QModelIndex())*model->columnCount(QModelIndex())
         > 90000) {
-        int answer = QMessageBox::question(this, tr("Large Image Size"),
+	    QMessageBox::StandardButton answer;
+	    answer = QMessageBox::question(this, tr("Large Image Size"),
             tr("The printed image may be very large. Are you sure that "
                "you want to print it?"),
-            QMessageBox::Yes, QMessageBox::No);
+            QMessageBox::Yes | QMessageBox::No);
         if (answer == QMessageBox::No)
             return;
     }
@@ -163,6 +178,7 @@ void MainWindow::printImage()
     QModelIndex parent = QModelIndex();
 
     QProgressDialog progress(tr("Printing..."), tr("Cancel"), 0, rows, this);
+    progress.setWindowModality(Qt::ApplicationModal);
     float y = ItemSize/2;
 
     for (int row = 0; row < rows; ++row) {
@@ -202,8 +218,6 @@ void MainWindow::showAboutBox()
 
 void MainWindow::updateView()
 {
-    for (int row = 0; row < model->rowCount(QModelIndex()); ++row)
-        view->resizeRowToContents(row);
-    for (int column = 0; column < model->columnCount(QModelIndex()); ++column)
-        view->resizeColumnToContents(column);
+    view->resizeColumnsToContents();
+    view->resizeRowsToContents();
 }
