@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -37,6 +52,7 @@ class QWheelEvent;
 class SvgRasterView : public QWidget
 {
     Q_OBJECT
+
 public:
     SvgRasterView(const QString &file, QWidget *parent=0);
 
@@ -57,6 +73,7 @@ private:
 class SvgNativeView : public QWidget
 {
     Q_OBJECT
+
 public:
     SvgNativeView(const QString &file, QWidget *parent=0);
 
@@ -73,9 +90,12 @@ private:
 class SvgGLView : public QGLWidget
 {
     Q_OBJECT
+
 public:
     SvgGLView(const QString &file, QWidget *parent=0);
 
+    void setHighQualityAntialiasing(bool highQualityAntialiasing);
+
     virtual QSize sizeHint() const;
 protected:
     virtual void paintEvent(QPaintEvent *event);
@@ -83,6 +103,8 @@ protected:
 
 private:
     QSvgRenderer *doc;
+
+    bool highQualityAntialiasing;
 };
 #endif
 
