@@ -9,30 +9,57 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
-#include <QtGui>
 
 #include "interfaces.h"
 #include "mainwindow.h"
 #include "paintarea.h"
 #include "plugindialog.h"
 
-MainWindow::MainWindow()
-{
-    paintArea = new PaintArea;
+#include <QPluginLoader>
+#include <QTimer>
 
-    scrollArea = new QScrollArea;
+#include <QScrollArea>
+#include <QMessageBox>
+#include <QActionGroup>
+#include <QAction>
+#include <QMenu>
+#include <QMenuBar>
+#include <QFileDialog>
+#include <QColorDialog>
+#include <QInputDialog>
+#include <QApplication>
+
+MainWindow::MainWindow() :
+    paintArea(new PaintArea),
+    scrollArea(new QScrollArea)
+{
     scrollArea->setBackgroundRole(QPalette::Dark);
     scrollArea->setWidget(paintArea);
     setCentralWidget(scrollArea);
@@ -51,8 +78,9 @@ MainWindow::MainWindow()
 
 void MainWindow::open()
 {
-    QString fileName = QFileDialog::getOpenFileName(this,
-                                    tr("Open File"), QDir::currentPath());
+    const QString fileName = QFileDialog::getOpenFileName(this,
+                                                          tr("Open File"), 
+                                                          QDir::currentPath());
     if (!fileName.isEmpty()) {
         if (!paintArea->openImage(fileName)) {
             QMessageBox::information(this, tr("Plug & Paint"),
@@ -65,10 +93,10 @@ void MainWindow::open()
 
 bool MainWindow::saveAs()
 {
-    QString initialPath = QDir::currentPath() + "/untitled.png";
+    const QString initialPath = QDir::currentPath() + "/untitled.png";
 
-    QString fileName = QFileDialog::getSaveFileName(this, tr("Save As"),
-                                                    initialPath);
+    const QString fileName = QFileDialog::getSaveFileName(this, tr("Save As"),
+                                                          initialPath);
     if (fileName.isEmpty()) {
         return false;
     } else {
@@ -78,7 +106,7 @@ bool MainWindow::saveAs()
 
 void MainWindow::brushColor()
 {
-    QColor newColor = QColorDialog::getColor(paintArea->brushColor());
+    const QColor newColor = QColorDialog::getColor(paintArea->brushColor());
     if (newColor.isValid())
         paintArea->setBrushColor(newColor);
 }
@@ -86,10 +114,10 @@ void MainWindow::brushColor()
 void MainWindow::brushWidth()
 {
     bool ok;
-    int newWidth = QInputDialog::getInteger(this, tr("Plug & Paint"),
-                                            tr("Select brush width:"),
-                                            paintArea->brushWidth(),
-                                            1, 50, 1, &ok);
+    const int newWidth = QInputDialog::getInteger(this, tr("Plug & Paint"),
+                                                  tr("Select brush width:"),
+                                                  paintArea->brushWidth(),
+                                                  1, 50, 1, &ok);
     if (ok)
         paintArea->setBrushWidth(newWidth);
 }
@@ -98,7 +126,7 @@ void MainWindow::changeBrush()
 {
     QAction *action = qobject_cast<QAction *>(sender());
     BrushInterface *iBrush = qobject_cast<BrushInterface *>(action->parent());
-    QString brush = action->text();
+    const QString brush = action->text();
 
     paintArea->setBrush(iBrush, brush);
 }
@@ -108,7 +136,7 @@ void MainWindow::insertShape()
     QAction *action = qobject_cast<QAction *>(sender());
     ShapeInterface *iShape = qobject_cast<ShapeInterface *>(action->parent());
 
-    QPainterPath path = iShape->generateShape(action->text(), this);
+    const QPainterPath path = iShape->generateShape(action->text(), this);
     if (!path.isEmpty())
         paintArea->insertShape(path);
 }
@@ -119,8 +147,8 @@ void MainWindow::applyFilter()
     FilterInterface *iFilter =
             qobject_cast<FilterInterface *>(action->parent());
 
-    QImage image = iFilter->filterImage(action->text(), paintArea->image(),
-                                        this);
+    const QImage image = iFilter->filterImage(action->text(), paintArea->image(),
+                                              this);
     paintArea->setImage(image);
 }
 
