@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -40,7 +55,6 @@ Calculator::Calculator(QWidget *parent)
     display->setReadOnly(true);
     display->setAlignment(Qt::AlignRight);
     display->setMaxLength(15);
-    display->installEventFilter(this);
 
     QFont font = display->font();
     font.setPointSize(font.pointSize() + 8);
@@ -56,38 +70,38 @@ Calculator::Calculator(QWidget *parent)
                                        SLOT(digitClicked()));
     }
 
-    pointButton = createButton(tr("."), digitColor, SLOT(pointClicked()));
-    changeSignButton = createButton(tr("\261"), digitColor, SLOT(changeSignClicked()));
+    Button *pointButton = createButton(tr("."), digitColor, SLOT(pointClicked()));
+    Button *changeSignButton = createButton(tr("\261"), digitColor, SLOT(changeSignClicked()));
 
-    backspaceButton = createButton(tr("Backspace"), backspaceColor,
+    Button *backspaceButton = createButton(tr("Backspace"), backspaceColor,
                                    SLOT(backspaceClicked()));
-    clearButton = createButton(tr("Clear"), backspaceColor, SLOT(clear()));
-    clearAllButton = createButton(tr("Clear All"), backspaceColor.light(120),
+    Button *clearButton = createButton(tr("Clear"), backspaceColor, SLOT(clear()));
+    Button *clearAllButton = createButton(tr("Clear All"), backspaceColor.light(120),
                                   SLOT(clearAll()));
 
-    clearMemoryButton = createButton(tr("MC"), memoryColor,
+    Button *clearMemoryButton = createButton(tr("MC"), memoryColor,
                                      SLOT(clearMemory()));
-    readMemoryButton = createButton(tr("MR"), memoryColor, SLOT(readMemory()));
-    setMemoryButton = createButton(tr("MS"), memoryColor, SLOT(setMemory()));
-    addToMemoryButton = createButton(tr("M+"), memoryColor,
+    Button *readMemoryButton = createButton(tr("MR"), memoryColor, SLOT(readMemory()));
+    Button *setMemoryButton = createButton(tr("MS"), memoryColor, SLOT(setMemory()));
+    Button *addToMemoryButton = createButton(tr("M+"), memoryColor,
                                      SLOT(addToMemory()));
 
-    divisionButton = createButton(tr("\367"), operatorColor,
+    Button *divisionButton = createButton(tr("\367"), operatorColor,
                                   SLOT(multiplicativeOperatorClicked()));
-    timesButton = createButton(tr("\327"), operatorColor,
+    Button *timesButton = createButton(tr("\327"), operatorColor,
                                SLOT(multiplicativeOperatorClicked()));
-    minusButton = createButton(tr("-"), operatorColor,
+    Button *minusButton = createButton(tr("-"), operatorColor,
                                SLOT(additiveOperatorClicked()));
-    plusButton = createButton(tr("+"), operatorColor,
+    Button *plusButton = createButton(tr("+"), operatorColor,
                               SLOT(additiveOperatorClicked()));
 
-    squareRootButton = createButton(tr("Sqrt"), operatorColor,
+    Button *squareRootButton = createButton(tr("Sqrt"), operatorColor,
                                     SLOT(unaryOperatorClicked()));
-    powerButton = createButton(tr("x\262"), operatorColor,
+    Button *powerButton = createButton(tr("x\262"), operatorColor,
                                SLOT(unaryOperatorClicked()));
-    reciprocalButton = createButton(tr("1/x"), operatorColor,
+    Button *reciprocalButton = createButton(tr("1/x"), operatorColor,
                                     SLOT(unaryOperatorClicked()));
-    equalButton = createButton(tr("="), operatorColor.light(120),
+    Button *equalButton = createButton(tr("="), operatorColor.light(120),
                                SLOT(equalClicked()));
 
     QGridLayout *mainLayout = new QGridLayout;
@@ -127,30 +141,6 @@ Calculator::Calculator(QWidget *parent)
     setWindowTitle(tr("Calculator"));
 }
 
-bool Calculator::eventFilter(QObject *target, QEvent *event)
-{
-    if (target == display) {
-        if (event->type() == QEvent::MouseButtonPress
-                || event->type() == QEvent::MouseButtonDblClick
-                || event->type() == QEvent::MouseButtonRelease
-                || event->type() == QEvent::ContextMenu) {
-            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
-            if (mouseEvent->buttons() & Qt::LeftButton) {
-                QPalette newPalette = palette();
-                newPalette.setColor(QPalette::Base,
-                                    display->palette().color(QPalette::Text));
-                newPalette.setColor(QPalette::Text,
-                                    display->palette().color(QPalette::Base));
-                display->setPalette(newPalette);
-            } else {
-                display->setPalette(palette());
-            }
-            return true;
-        }
-    }
-    return QDialog::eventFilter(target, event);
-}
-
 void Calculator::digitClicked()
 {
     Button *clickedButton = qobject_cast<Button *>(sender());
