@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2004-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -83,14 +98,14 @@ void Window::createDateTimeEdits()
     editsGroup = new QGroupBox(tr("Date and time spin boxes"));
 
     QLabel *dateLabel = new QLabel;
-    QDateTimeEdit *dateEdit = new QDateTimeEdit(QDate::currentDate());
+    QDateEdit *dateEdit = new QDateEdit(QDate::currentDate());
     dateEdit->setDateRange(QDate(2005, 1, 1), QDate(2010, 12, 31));
     dateLabel->setText(tr("Appointment date (between %0 and %1):")
                        .arg(dateEdit->minimumDate().toString(Qt::ISODate))
                        .arg(dateEdit->maximumDate().toString(Qt::ISODate)));
 
     QLabel *timeLabel = new QLabel;
-    QDateTimeEdit *timeEdit = new QDateTimeEdit(QTime::currentTime());
+    QTimeEdit *timeEdit = new QTimeEdit(QTime::currentTime());
     timeEdit->setTimeRange(QTime(9, 0, 0, 0), QTime(16, 30, 0, 0));
     timeLabel->setText(tr("Appointment time (between %0 and %1):")
                        .arg(timeEdit->minimumTime().toString(Qt::ISODate))
@@ -148,7 +163,7 @@ void Window::createDoubleSpinBoxes()
     QLabel *precisionLabel = new QLabel(tr("Number of decimal places "
                                            "to show:"));
     QSpinBox *precisionSpinBox = new QSpinBox;
-    precisionSpinBox->setRange(0, 13);
+    precisionSpinBox->setRange(0, 100);
     precisionSpinBox->setValue(2);
 
     QLabel *doubleLabel = new QLabel(tr("Enter a value between "
