@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -32,8 +47,8 @@
 QString project_builtin_regx() //calculate the builtin regular expression..
 {
     QString ret;
-    QStringList builtin_exts(".c");
-    builtin_exts << Option::ui_ext << Option::yacc_ext << Option::lex_ext << ".ts" << ".qrc";
+    QStringList builtin_exts;
+    builtin_exts << Option::c_ext << Option::ui_ext << Option::yacc_ext << Option::lex_ext << ".ts" << ".qrc";
     builtin_exts += Option::h_ext + Option::cpp_ext;
     for(int i = 0; i < builtin_exts.size(); ++i) {
         if(!ret.isEmpty())
@@ -67,7 +82,9 @@ ProjectGenerator::init()
 
     //figure out target
     if(Option::output.fileName() == "-")
-        v["TARGET"] = QStringList("unknown");
+        v["TARGET_ASSIGN"] = QStringList("unknown");
+    else
+        v["TARGET_ASSIGN"] = QStringList(QFileInfo(Option::output).baseName());
 
     //the scary stuff
     if(project->first("TEMPLATE_ASSIGN") != "subdirs") {
@@ -133,7 +150,7 @@ ProjectGenerator::init()
                     }
                 }
             }
-            if(add_depend && !dir.isEmpty() && !v["DEPENDPATH"].contains(dir)) {
+            if(add_depend && !dir.isEmpty() && !v["DEPENDPATH"].contains(dir, Qt::CaseInsensitive)) {
                 QFileInfo fi(fileInfo(dir));
                 if(fi.absoluteFilePath() != qmake_getpwd())
                     v["DEPENDPATH"] += fileFixify(dir);
@@ -154,7 +171,7 @@ ProjectGenerator::init()
                     newdir = fileFixify(newdir);
                     QStringList &subdirs = v["SUBDIRS"];
                     if(exists(fi.filePath() + QDir::separator() + fi.fileName() + Option::pro_ext) &&
-                       !subdirs.contains(newdir)) {
+                       !subdirs.contains(newdir, Qt::CaseInsensitive)) {
                         subdirs.append(newdir);
                     } else {
                         QStringList profiles = QDir(newdir).entryList(QStringList("*" + Option::pro_ext), QDir::Files);
@@ -167,7 +184,7 @@ ProjectGenerator::init()
                             nd += profiles[i];
                             fileFixify(nd);
                             if(profiles[i] != "." && profiles[i] != ".." &&
-                               !subdirs.contains(nd) && !out_file.endsWith(nd))
+                               !subdirs.contains(nd, Qt::CaseInsensitive) && !out_file.endsWith(nd))
                                 subdirs.append(nd);
                         }
                     }
@@ -175,7 +192,7 @@ ProjectGenerator::init()
                         QStringList dirs = QDir(newdir).entryList(QDir::Dirs);
                         for(int i = 0; i < (int)dirs.count(); i++) {
                             QString nd = fileFixify(newdir + QDir::separator() + dirs[i]);
-                            if(dirs[i] != "." && dirs[i] != ".." && !knownDirs.contains(nd))
+                            if(dirs[i] != "." && dirs[i] != ".." && !knownDirs.contains(nd, Qt::CaseInsensitive))
                                 knownDirs.append(nd);
                         }
                     }
@@ -202,13 +219,13 @@ ProjectGenerator::init()
                             for(int i = 0; i < (int)profiles.count(); i++) {
                                 QString nd = newdir + QDir::separator() + files[i];
                                 fileFixify(nd);
-                                if(files[i] != "." && files[i] != ".." && !subdirs.contains(nd)) {
+                                if(files[i] != "." && files[i] != ".." && !subdirs.contains(nd, Qt::CaseInsensitive)) {
                                     if(newdir + files[i] != Option::output_dir + Option::output.fileName())
                                         subdirs.append(nd);
                                 }
                             }
                         }
-                        if(Option::recursive && !knownDirs.contains(newdir))
+                        if(Option::recursive && !knownDirs.contains(newdir, Qt::CaseInsensitive))
                             knownDirs.append(newdir);
                     }
                 }
@@ -245,7 +262,7 @@ ProjectGenerator::init()
                     if(!file_dir.isEmpty()) {
                         for(int inc_it = 0; inc_it < deplist.size(); ++inc_it) {
                             QMakeLocalFileName inc = deplist[inc_it];
-                            if(inc.local() == file_dir && !v["INCLUDEPATH"].contains(inc.real()))
+                            if(inc.local() == file_dir && !v["INCLUDEPATH"].contains(inc.real(), Qt::CaseInsensitive))
                                 v["INCLUDEPATH"] += inc.real();
                         }
                     }
@@ -263,15 +280,8 @@ ProjectGenerator::init()
                             QString src(dep.left(dep.length() - h_ext.length()) +
                                         Option::cpp_ext.at(cppit));
                             if(exists(src)) {
-                                bool exists = false;
                                 QStringList &srcl = v["SOURCES"];
-                                for(int src_it = 0; src_it < srcl.size(); ++src_it) {
-                                    if(srcl[src_it].toLower() == src.toLower()) {
-                                        exists = true;
-                                        break;
-                                    }
-                                }
-                                if(!exists)
+                                if(!srcl.contains(src, Qt::CaseInsensitive))
                                     srcl.append(src);
                             }
                         }
@@ -279,7 +289,7 @@ ProjectGenerator::init()
                               file_no_path.startsWith(Option::lex_mod)) {
                         addConfig("lex_included");
                     }
-                    if(!h.contains(dep))
+                    if(!h.contains(dep, Qt::CaseInsensitive))
                         h += dep;
                 }
             }
@@ -344,7 +354,7 @@ ProjectGenerator::writeMakefile(QTextStream &t)
         t << endl << "# Directories" << "\n"
           << getWritableVar("SUBDIRS");
     } else {
-        t << getWritableVar("TARGET")
+        t << getWritableVar("TARGET_ASSIGN")
           << getWritableVar("CONFIG", false)
           << getWritableVar("CONFIG_REMOVE", false)
           << getWritableVar("DEPENDPATH")
@@ -402,11 +412,17 @@ ProjectGenerator::addFile(QString file)
                 break;
             }
     }
+    if(where.isEmpty()) {
+        for(int cit = 0; cit < Option::c_ext.size(); ++cit) {
+            if(file.endsWith(Option::c_ext[cit])) {
+                where = "SOURCES";
+                break;
+            }
+        }
+    }
     if(where.isEmpty()) {
         if(file.endsWith(Option::ui_ext))
             where = "FORMS";
-        else if(file.endsWith(".c"))
-            where = "SOURCES";
         else if(file.endsWith(Option::lex_ext))
             where = "LEXSOURCES";
         else if(file.endsWith(Option::yacc_ext))
@@ -418,8 +434,10 @@ ProjectGenerator::addFile(QString file)
     }
 
     QString newfile = fixPathToQmake(fileFixify(file));
-    if(!where.isEmpty() && !project->variables()[where].contains(file)) {
-        project->variables()[where] += newfile;
+
+    QStringList &endList = project->variables()[where];
+    if(!endList.contains(newfile, Qt::CaseInsensitive)) {
+        endList += newfile;
         return true;
     }
     return false;
