@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the qmake application of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -95,9 +110,34 @@ QString XmlOutput::doConversion(const QString &text)
     else if (conversion == NoConversion)
         return text;
 
-    QString output(text);
+    QString output;
+    if (conversion == XMLConversion) {
+
+        // this is a way to escape characters that shouldn't be converted
+        for (int i=0; i<text.count(); ++i) {
+            if (text.at(i) == QLatin1Char('&')) {
+                if ( (i + 7) < text.count() &&
+                    text.at(i + 1) == QLatin1Char('#') &&
+                    text.at(i + 2) == QLatin1Char('x') &&
+                    text.at(i + 7) == QLatin1Char(';') ) {
+                    output += text.at(i);
+                } else {
+                    output += QLatin1String("&amp;");
+                }
+            } else {
+                QChar c = text.at(i);
+                if (c.unicode() < 0x20) {
+                    output += QString("&#x%1;").arg(c.unicode(), 2, 16, QLatin1Char('0'));
+                } else {
+                    output += text.at(i);
+                }
+            }
+        }
+    } else {
+        output = text;
+    }
+
     if (conversion == XMLConversion) {
-        output.replace('&', "&amp;");
         output.replace('\"', "&quot;");
         output.replace('\'', "&apos;");
     } else if (conversion == EscapeConversion) {
