@@ -35,12 +35,9 @@ struct MD5Context {
 	UWORD32 in[16];
 };
 
-void MD5Init(struct MD5Context *context);
-void MD5Update(struct MD5Context *context, md5byte const *buf, unsigned len);
-void MD5Final(unsigned char digest[16], struct MD5Context *context);
-void MD5Transform(UWORD32 buf[4], UWORD32 const in[16]);
-
-void qtMD5(const QByteArray &src, unsigned char *digest);
-QString qtMD5(const QByteArray &src);
+static void MD5Init(struct MD5Context *context);
+static void MD5Update(struct MD5Context *context, md5byte const *buf, unsigned len);
+static void MD5Final(struct MD5Context *context, unsigned char digest[16]);
+static void MD5Transform(UWORD32 buf[4], UWORD32 const in[16]);
 
 #endif /* !MD5_H */
