@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,19 +9,34 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
-// Data structures
+/* Data structures */
 #ifndef QT_NO_QUUID_STRING
 #  define QT_NO_QUUID_STRING
 #endif
@@ -35,13 +50,16 @@
 #  define QT_NO_DATESTRING
 #endif
 
-// Dialogs
+/* Dialogs */
 #ifndef QT_NO_COLORDIALOG
 #  define QT_NO_COLORDIALOG
 #endif
 #ifndef QT_NO_ERRORMESSAGE
 #  define QT_NO_ERRORMESSAGE
 #endif
+#ifndef QT_NO_FILEDIALOG
+#  define QT_NO_FILEDIALOG
+#endif
 #ifndef QT_NO_FONTDIALOG
 #  define QT_NO_FONTDIALOG
 #endif
@@ -51,9 +69,6 @@
 #ifndef QT_NO_MESSAGEBOX
 #  define QT_NO_MESSAGEBOX
 #endif
-#ifndef QT_NO_FILEDIALOG
-#  define QT_NO_FILEDIALOG
-#endif
 #ifndef QT_NO_PRINTDIALOG
 #  define QT_NO_PRINTDIALOG
 #endif
@@ -64,10 +79,13 @@
 #  define QT_NO_TABDIALOG
 #endif
 
-// File I/O
+/* File I/O */
 #ifndef QT_NO_DOM
 #  define QT_NO_DOM
 #endif
+#ifndef QT_NO_FILESYSTEMWATCHER
+#  define QT_NO_FILESYSTEMWATCHER
+#endif
 #ifndef QT_NO_PROCESS
 #  define QT_NO_PROCESS
 #endif
@@ -81,12 +99,15 @@
 #  define QT_NO_LIBRARY
 #endif
 
-// Fonts
+/* Fonts */
 #ifndef QT_NO_FREETYPE
 #  define QT_NO_FREETYPE
 #endif
 
-// Images
+/* Images */
+#ifndef QT_NO_ICON
+#  define QT_NO_ICON
+#endif
 #ifndef QT_NO_IMAGEFORMATPLUGIN
 #  define QT_NO_IMAGEFORMATPLUGIN
 #endif
@@ -111,9 +132,6 @@
 #ifndef QT_NO_IMAGE_HEURISTIC_MASK
 #  define QT_NO_IMAGE_HEURISTIC_MASK
 #endif
-#ifndef QT_NO_ICON
-#  define QT_NO_ICON
-#endif
 #ifndef QT_NO_IMAGE_TEXT
 #  define QT_NO_IMAGE_TEXT
 #endif
@@ -121,7 +139,7 @@
 #  define QT_NO_MOVIE
 #endif
 
-// Internationalization
+/* Internationalization */
 #ifndef QT_NO_BIG_CODECS
 #  define QT_NO_BIG_CODECS
 #endif
@@ -144,10 +162,13 @@
 #  define QT_NO_TRANSLATION_UTF8
 #endif
 
-// ItemViews
+/* ItemViews */
 #ifndef QT_NO_ITEMVIEWS
 #  define QT_NO_ITEMVIEWS
 #endif
+#ifndef QT_NO_DATAWIDGETMAPPER
+#  define QT_NO_DATAWIDGETMAPPER
+#endif
 #ifndef QT_NO_DIRMODEL
 #  define QT_NO_DIRMODEL
 #endif
@@ -157,6 +178,9 @@
 #ifndef QT_NO_PROXYMODEL
 #  define QT_NO_PROXYMODEL
 #endif
+#ifndef QT_NO_SORTFILTERPROXYMODEL
+#  define QT_NO_SORTFILTERPROXYMODEL
+#endif
 #ifndef QT_NO_STANDARDITEMMODEL
 #  define QT_NO_STANDARDITEMMODEL
 #endif
@@ -170,7 +194,7 @@
 #  define QT_NO_TREEVIEW
 #endif
 
-// Kernel
+/* Kernel */
 #ifndef QT_NO_ACTION
 #  define QT_NO_ACTION
 #endif
@@ -198,6 +222,9 @@
 #ifndef QT_NO_SOUND
 #  define QT_NO_SOUND
 #endif
+#ifndef QT_NO_TABLETEVENT
+#  define QT_NO_TABLETEVENT
+#endif
 #ifndef QT_NO_THREAD
 #  define QT_NO_THREAD
 #endif
@@ -205,7 +232,7 @@
 #  define QT_NO_WHEELEVENT
 #endif
 
-// Networking
+/* Networking */
 #ifndef QT_NO_COP
 #  define QT_NO_COP
 #endif
@@ -231,7 +258,7 @@
 #  define QT_NO_FTP
 #endif
 
-// Painting
+/* Painting */
 #ifndef QT_NO_COLORNAMES
 #  define QT_NO_COLORNAMES
 #endif
@@ -244,8 +271,14 @@
 #ifndef QT_NO_PRINTER
 #  define QT_NO_PRINTER
 #endif
+#ifndef QT_NO_CUPS
+#  define QT_NO_CUPS
+#endif
 
-// Qtopia Core
+/* Qtopia Core */
+#ifndef QT_NO_QWSEMBEDWIDGET
+#  define QT_NO_QWSEMBEDWIDGET
+#endif
 #ifndef QT_NO_QWS_ALPHA_CURSOR
 #  define QT_NO_QWS_ALPHA_CURSOR
 #endif
@@ -282,25 +315,57 @@
 #ifndef QT_NO_QWS_SOUNDSERVER
 #  define QT_NO_QWS_SOUNDSERVER
 #endif
+#ifndef QT_NO_SXE
+#  define QT_NO_SXE
+#endif
 #ifndef QT_NO_QWS_PROPERTIES
 #  define QT_NO_QWS_PROPERTIES
 #endif
 
-// Styles
+/* Styles */
 #ifndef QT_NO_STYLE_MOTIF
 #  define QT_NO_STYLE_MOTIF
 #endif
 #ifndef QT_NO_STYLE_CDE
 #  define QT_NO_STYLE_CDE
 #endif
+#ifndef QT_NO_STYLE_CLEANLOOKS
+#  define QT_NO_STYLE_CLEANLOOKS
+#endif
 #ifndef QT_NO_STYLE_PLASTIQUE
 #  define QT_NO_STYLE_PLASTIQUE
 #endif
+#ifndef QT_NO_STYLE_STYLESHEET
+#  define QT_NO_STYLE_STYLESHEET
+#endif
 #ifndef QT_NO_STYLE_WINDOWSXP
 #  define QT_NO_STYLE_WINDOWSXP
 #endif
 
-// Widgets
+/* Utilities */
+#ifndef QT_NO_COMPLETER
+#  define QT_NO_COMPLETER
+#endif
+#ifndef QT_NO_DESKTOPSERVICES
+#  define QT_NO_DESKTOPSERVICES
+#endif
+#ifndef QT_NO_SYSTEMTRAYICON
+#  define QT_NO_SYSTEMTRAYICON
+#endif
+#ifndef QT_NO_UNDOCOMMAND
+#  define QT_NO_UNDOCOMMAND
+#endif
+#ifndef QT_NO_UNDOGROUP
+#  define QT_NO_UNDOGROUP
+#endif
+#ifndef QT_NO_UNDOSTACK
+#  define QT_NO_UNDOSTACK
+#endif
+#ifndef QT_NO_UNDOVIEW
+#  define QT_NO_UNDOVIEW
+#endif
+
+/* Widgets */
 #ifndef QT_NO_GROUPBOX
 #  define QT_NO_GROUPBOX
 #endif
@@ -316,9 +381,15 @@
 #ifndef QT_NO_COMBOBOX
 #  define QT_NO_COMBOBOX
 #endif
+#ifndef QT_NO_FONTCOMBOBOX
+#  define QT_NO_FONTCOMBOBOX
+#endif
 #ifndef QT_NO_SPINBOX
 #  define QT_NO_SPINBOX
 #endif
+#ifndef QT_NO_CALENDARWIDGET
+#  define QT_NO_CALENDARWIDGET
+#endif
 #ifndef QT_NO_DATETIMEEDIT
 #  define QT_NO_DATETIMEEDIT
 #endif
@@ -328,6 +399,9 @@
 #ifndef QT_NO_MENU
 #  define QT_NO_MENU
 #endif
+#ifndef QT_NO_CONTEXTMENU
+#  define QT_NO_CONTEXTMENU
+#endif
 #ifndef QT_NO_MAINWINDOW
 #  define QT_NO_MAINWINDOW
 #endif
@@ -373,6 +447,9 @@
 #ifndef QT_NO_SCROLLAREA
 #  define QT_NO_SCROLLAREA
 #endif
+#ifndef QT_NO_GRAPHICSVIEW
+#  define QT_NO_GRAPHICSVIEW
+#endif
 #ifndef QT_NO_TEXTEDIT
 #  define QT_NO_TEXTEDIT
 #endif
