@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -32,26 +47,38 @@
 #include <QtCore/qtextstream.h>
 #include <QtCore/qstring.h>
 #include <QtCore/qvector.h>
+#include <QtCore/qset.h>
+
+QT_BEGIN_HEADER
 
 QT_MODULE(Core)
 
-#if !defined(QT_NO_DEBUG_STREAM)
 class Q_CORE_EXPORT QDebug
 {
     struct Stream {
-        Stream(QtMsgType t) : ts(&buffer, QIODevice::WriteOnly), ref(1), type(t), space(true){}
+        Stream(QIODevice *device) : ts(device), ref(1), type(QtDebugMsg), space(true), message_output(false) {}
+        Stream(QString *string) : ts(string, QIODevice::WriteOnly), ref(1), type(QtDebugMsg), space(true), message_output(false) {}
+        Stream(QtMsgType t) : ts(&buffer, QIODevice::WriteOnly), ref(1), type(t), space(true), message_output(true) {}
         QTextStream ts;
         QString buffer;
         int ref;
         QtMsgType type;
         bool space;
+        bool message_output;
     } *stream;
 public:
+    inline QDebug(QIODevice *device) : stream(new Stream(device)) {}
+    inline QDebug(QString *string) : stream(new Stream(string)) {}
     inline QDebug(QtMsgType t) : stream(new Stream(t)) {}
     inline QDebug(const QDebug &o):stream(o.stream) { ++stream->ref; }
     inline QDebug &operator=(const QDebug &other);
-    inline ~QDebug()
-        { if (!--stream->ref) { qt_message_output(stream->type, stream->buffer.toLocal8Bit().data()); delete stream; } }
+    inline ~QDebug() {
+        if (!--stream->ref) {
+            if(stream->message_output)
+                qt_message_output(stream->type, stream->buffer.toLocal8Bit().data());
+            delete stream;
+        }
+    }
     inline QDebug &space() { stream->space = true; stream->ts << " "; return *this; }
     inline QDebug &nospace() { stream->space = false; return *this; }
     inline QDebug &maybeSpace() { if (stream->space) stream->ts << " "; return *this; }
@@ -71,7 +98,7 @@ public:
         { stream->ts << QString::number(t); return maybeSpace(); }
     inline QDebug &operator<<(float t) { stream->ts << t; return maybeSpace(); }
     inline QDebug &operator<<(double t) { stream->ts << t; return maybeSpace(); }
-    inline QDebug &operator<<(const char* t) { stream->ts  << t; return maybeSpace(); }
+    inline QDebug &operator<<(const char* t) { stream->ts << QString::fromAscii(t); return maybeSpace(); }
     inline QDebug &operator<<(const QString & t) { stream->ts << "\"" << t  << "\""; return maybeSpace(); }
     inline QDebug &operator<<(const QLatin1String &t) { stream->ts << "\""  << t.latin1() << "\""; return maybeSpace(); }
     inline QDebug &operator<<(const QByteArray & t) { stream->ts  << "\"" << t << "\""; return maybeSpace(); }
@@ -85,6 +112,9 @@ public:
     { stream->ts << m; return *this; }
 };
 
+Q_CORE_EXPORT_INLINE QDebug qWarning() { return QDebug(QtWarningMsg); }
+Q_CORE_EXPORT_INLINE QDebug qCritical() { return QDebug(QtCriticalMsg); }
+
 inline QDebug &QDebug::operator=(const QDebug &other)
 {
     if (this != &other) {
@@ -94,8 +124,13 @@ inline QDebug &QDebug::operator=(const QDebug &other)
     return *this;
 }
 
+#if defined(FORCE_UREF)
+template <class T>
+inline QDebug &operator<<(QDebug debug, const QList<T> &list)
+#else
 template <class T>
 inline QDebug operator<<(QDebug debug, const QList<T> &list)
+#endif
 {
     debug.nospace() << "(";
     for (Q_TYPENAME QList<T>::size_type i = 0; i < list.count(); ++i) {
@@ -107,14 +142,25 @@ inline QDebug operator<<(QDebug debug, const QList<T> &list)
     return debug.space();
 }
 
+#if defined(FORCE_UREF)
+template <typename T>
+inline QDebug &operator<<(QDebug debug, const QVector<T> &vec)
+#else
 template <typename T>
 inline QDebug operator<<(QDebug debug, const QVector<T> &vec)
+#endif
 {
+    debug.nospace() << "QVector";
     return operator<<(debug, vec.toList());
 }
 
+#if defined(FORCE_UREF)
+template <class aKey, class aT>
+inline QDebug &operator<<(QDebug debug, const QMap<aKey, aT> &map)
+#else
 template <class aKey, class aT>
 inline QDebug operator<<(QDebug debug, const QMap<aKey, aT> &map)
+#endif
 {
     debug.nospace() << "QMap(";
     for (typename QMap<aKey, aT>::const_iterator it = map.constBegin();
@@ -125,8 +171,13 @@ inline QDebug operator<<(QDebug debug, const QMap<aKey, aT> &map)
     return debug.space();
 }
 
+#if defined(FORCE_UREF)
+template <class aKey, class aT>
+inline QDebug &operator<<(QDebug debug, const QHash<aKey, aT> &hash)
+#else
 template <class aKey, class aT>
 inline QDebug operator<<(QDebug debug, const QHash<aKey, aT> &hash)
+#endif
 {
     debug.nospace() << "QHash(";
     for (typename QHash<aKey, aT>::const_iterator it = hash.constBegin();
@@ -136,16 +187,27 @@ inline QDebug operator<<(QDebug debug, const QHash<aKey, aT> &hash)
     return debug.space();
 }
 
+#if defined(FORCE_UREF)
+template <class T1, class T2>
+inline QDebug &operator<<(QDebug debug, const QPair<T1, T2> &pair)
+#else
 template <class T1, class T2>
 inline QDebug operator<<(QDebug debug, const QPair<T1, T2> &pair)
+#endif
 {
     debug.nospace() << "QPair(" << pair.first << "," << pair.second << ")";
     return debug.space();
 }
 
-inline Q_CORE_EXPORT QDebug qDebug() { return QDebug(QtDebugMsg); }
-inline Q_CORE_EXPORT QDebug qWarning() { return QDebug(QtWarningMsg); }
-inline Q_CORE_EXPORT QDebug qCritical() { return QDebug(QtCriticalMsg); }
+template <typename T>
+inline QDebug operator<<(QDebug debug, const QSet<T> &set)
+{
+    debug.nospace() << "QSet";
+    return operator<<(debug, set.toList());
+}
+
+#if !defined(QT_NO_DEBUG_STREAM)
+Q_CORE_EXPORT_INLINE QDebug qDebug() { return QDebug(QtDebugMsg); }
 
 #else // QT_NO_DEBUG_STREAM
 
@@ -170,7 +232,7 @@ public:
 };
 #undef qDebug
 inline QNoDebug qDebug() { return QNoDebug(); }
-#define qDebug if(1) ; else qDebug
+#define qDebug QT_NO_QDEBUG_MACRO
 
 #ifdef QT_NO_MEMBER_TEMPLATES
 template<typename T>
@@ -179,4 +241,6 @@ inline QNoDebug operator<<(QNoDebug debug, const T &) { return debug; }
 
 #endif
 
+QT_END_HEADER
+
 #endif // QDEBUG_H
