@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -147,6 +162,7 @@ public:
     Q_PID pid;
     int sequenceNumber;
 
+    bool dying;
     bool emittedReadyRead;
     bool emittedBytesWritten;
 
@@ -177,7 +193,9 @@ public:
     QWinEventNotifier *processFinishedNotifier;
 
     void startProcess();
-    void execChild(const QByteArray &encodedProgramName);
+#ifdef Q_OS_UNIX
+    void execChild(const char *workingDirectory, char **path, char **argv, char **envp);
+#endif
     bool processStarted();
     void terminateProcess();
     void killProcess();
@@ -187,9 +205,11 @@ public:
 #endif
 #ifdef Q_OS_WIN
     void flushPipeWriter();
+    qint64 pipeWriterBytesToWrite() const;
 #endif
 
-    static bool startDetached(const QString &program, const QStringList &arguments);
+    static bool startDetached(const QString &program, const QStringList &arguments, const QString &workingDirectory = QString(),
+                              qint64 *pid = 0);
 
     int exitCode;
     QProcess::ExitStatus exitStatus;
