@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -108,6 +123,8 @@ public:
     void setPath(const QString &path);
     QString path() const;
 
+    bool hasQuery() const;
+
     void setEncodedQuery(const QByteArray &query);
     QByteArray encodedQuery() const;
 
@@ -126,6 +143,7 @@ public:
 
     void setFragment(const QString &fragment);
     QString fragment() const;
+    bool hasFragment() const;
 
     QUrl resolved(const QUrl &relative) const;
 
@@ -155,6 +173,10 @@ public:
                                         const QByteArray &include = QByteArray());
     static QString fromPunycode(const QByteArray &);
     static QByteArray toPunycode(const QString &);
+    static QString fromAce(const QByteArray &);
+    static QByteArray toAce(const QString &);
+    static QStringList idnWhitelist();
+    static void setIdnWhitelist(const QStringList &);
 
 #if defined QT3_SUPPORT
     inline QT3_SUPPORT QString protocol() const { return scheme(); }
@@ -201,6 +223,8 @@ public:
     }
 #endif
 
+    QString errorString() const;
+
 protected:
 #if defined (QT3_SUPPORT)
     inline QT3_SUPPORT void reset() { clear(); }
@@ -208,6 +232,9 @@ protected:
 
 private:
     QUrlPrivate *d;
+public:
+    typedef QUrlPrivate * DataPtr;
+    inline DataPtr &data_ptr() { return d; }
 };
 
 Q_DECLARE_TYPEINFO(QUrl, Q_MOVABLE_TYPE);
