@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -42,7 +57,8 @@
 class Q_CORE_EXPORT QPersistentModelIndexData
 {
 public:
-    QPersistentModelIndexData() : model (0) {}
+    QPersistentModelIndexData() : model(0) {}
+    QPersistentModelIndexData(const QModelIndex &idx) : index(idx), model(idx.model()) {}
     QModelIndex index;
     QAtomic ref;
     const QAbstractItemModel *model;
@@ -55,9 +71,9 @@ class Q_CORE_EXPORT QAbstractItemModelPrivate : public QObjectPrivate
     Q_DECLARE_PUBLIC(QAbstractItemModel)
 
 public:
-    ~QAbstractItemModelPrivate();
-
+    QAbstractItemModelPrivate() : QObjectPrivate(), supportedDragActions(-1) {}
     void removePersistentIndexData(QPersistentModelIndexData *data);
+    void addPersistentIndexData(QPersistentModelIndexData *data);
     void invalidate(int position);
     void rowsAboutToBeInserted(const QModelIndex &parent, int first, int last);
     void rowsInserted(const QModelIndex &parent, int first, int last);
@@ -68,6 +84,28 @@ public:
     void columnsAboutToBeRemoved(const QModelIndex &parent, int first, int last);
     void columnsRemoved(const QModelIndex &parent, int first, int last);
     void reset();
+    static QAbstractItemModel *staticEmptyModel();
+
+    inline QModelIndex createIndex(int row, int column, void *data = 0) const {
+        return q_func()->createIndex(row, column, data);
+    }
+
+    inline QModelIndex createIndex(int row, int column, int id) const {
+        return q_func()->createIndex(row, column, id);
+    }
+
+    inline bool indexValid(const QModelIndex &index) const {
+         return (index.row() >= 0) && (index.column() >= 0) && (index.model() == q_func());
+    }
+
+    inline void invalidatePersistentIndexes() {
+        QVector<QPersistentModelIndexData*>::iterator it = persistent.indexes.begin();
+        for (; it != persistent.indexes.end(); ++it) {
+            Q_ASSERT((*it));
+            (*it)->index = QModelIndex();
+            (*it)->model = 0;
+        }
+    }
 
     struct Change {
         Change() : first(-1), last(-1) {}
@@ -79,10 +117,14 @@ public:
     QStack<Change> changes;
 
     struct Persistent {
-        QList<QPersistentModelIndexData*> indexes;
+        Persistent() : previous(0) {}
+        QVector<QPersistentModelIndexData*> indexes;
         QStack<QList<int> > moved;
         QStack<QList<int> > invalidated;
+        QPersistentModelIndexData *previous; // optimization
     } persistent;
+
+    Qt::DropActions supportedDragActions;
 };
 
 #endif // QABSTRACTITEMMODEL_P_H
