@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -58,10 +73,15 @@ public:
     QCoreApplication(int &argc, char **argv);
     ~QCoreApplication();
 
-    static int argc();
-    static char **argv();
+#ifdef QT_DEPRECATED
+    QT_DEPRECATED static int argc();
+    QT_DEPRECATED static char **argv();
+#endif
     static QStringList arguments();
 
+    static void setAttribute(Qt::ApplicationAttribute attribute, bool on = true);
+    static bool testAttribute(Qt::ApplicationAttribute attribute);
+
     static void setOrganizationDomain(const QString &orgDomain);
     static QString organizationDomain();
     static void setOrganizationName(const QString &orgName);
@@ -78,9 +98,11 @@ public:
 
     static bool sendEvent(QObject *receiver, QEvent *event);
     static void postEvent(QObject *receiver, QEvent *event);
+    static void postEvent(QObject *receiver, QEvent *event, int priority);
     static void sendPostedEvents(QObject *receiver, int event_type);
     static void sendPostedEvents();
     static void removePostedEvents(QObject *receiver);
+    static void removePostedEvents(QObject *receiver, int eventType);
     static bool hasPendingEvents();
 
     virtual bool notify(QObject *, QEvent *);
@@ -102,11 +124,16 @@ public:
     static void installTranslator(QTranslator * messageFile);
     static void removeTranslator(QTranslator * messageFile);
 #endif
-    enum Encoding { DefaultCodec, UnicodeUTF8 };
+    enum Encoding { CodecForTr, UnicodeUTF8, DefaultCodec = CodecForTr };
+    // ### Qt 5: merge
     static QString translate(const char * context,
                              const char * key,
                              const char * comment = 0,
-                             Encoding encoding = DefaultCodec);
+                             Encoding encoding = CodecForTr);
+    static QString translate(const char * context,
+                             const char * key,
+                             const char * comment,
+                             Encoding encoding, int n);
 
     static void flush();
 
@@ -152,6 +179,7 @@ protected:
 
 private:
     static bool sendSpontaneousEvent(QObject *receiver, QEvent *event);
+    bool notifyInternal(QObject *receiver, QEvent *event);
 
     void init();
 
@@ -167,13 +195,14 @@ private:
     friend class QWidgetPrivate;
     friend bool qt_sendSpontaneousEvent(QObject*, QEvent*);
     friend Q_CORE_EXPORT QString qAppName();
+    friend class QClassFactory;
 };
 
 inline bool QCoreApplication::sendEvent(QObject *receiver, QEvent *event)
-{  if (event) event->spont = false; return self ? self->notify(receiver, event) : false; }
+{  if (event) event->spont = false; return self ? self->notifyInternal(receiver, event) : false; }
 
 inline bool QCoreApplication::sendSpontaneousEvent(QObject *receiver, QEvent *event)
-{ if (event) event->spont = true; return self ? self->notify(receiver, event) : false; }
+{ if (event) event->spont = true; return self ? self->notifyInternal(receiver, event) : false; }
 
 inline void QCoreApplication::sendPostedEvents() { sendPostedEvents(0, 0); }
 
@@ -190,8 +219,22 @@ inline QString QCoreApplication::translate(const char *, const char *sourceText,
 #endif
     return QString::fromLatin1(sourceText);
 }
+
+// Simple versions
+inline QString QCoreApplication::translate(const char *, const char *sourceText,
+                                           const char *, Encoding encoding, int)
+{
+#ifndef QT_NO_TEXTCODEC
+    if (encoding == UnicodeUTF8)
+        return QString::fromUtf8(sourceText);
+#else
+    Q_UNUSED(encoding)
+#endif
+    return QString::fromLatin1(sourceText);
+}
 #endif
 
+// ### merge the four functions into two (using "int n = -1")
 #define Q_DECLARE_TR_FUNCTIONS(context) \
 public: \
     static inline QString tr(const char *sourceText, const char *comment = 0) \
@@ -199,6 +242,12 @@ public: \
     static inline QString trUtf8(const char *sourceText, const char *comment = 0) \
         { return QCoreApplication::translate(#context, sourceText, comment, \
                                              QCoreApplication::UnicodeUTF8); } \
+    static inline QString tr(const char *sourceText, const char *comment, int n) \
+        { return QCoreApplication::translate(#context, sourceText, comment, \
+                                             QCoreApplication::CodecForTr, n); } \
+    static inline QString trUtf8(const char *sourceText, const char *comment, int n) \
+        { return QCoreApplication::translate(#context, sourceText, comment, \
+                                             QCoreApplication::UnicodeUTF8, n); } \
 private:
 
 typedef void (*QtCleanUpFunction)();
