@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,6 +40,9 @@
 #define QCOREEVENT_H
 
 #include <QtCore/qnamespace.h>
+#include <QtCore/qbytearray.h>
+
+QT_BEGIN_HEADER
 
 QT_MODULE(Core)
 
@@ -93,7 +111,8 @@ public:
         ChildAdded = 68,                        // new child widget
         ChildPolished = 69,                     // polished child widget
 #ifdef QT3_SUPPORT
-        ChildInserted = 70,                     // compatibility posted insert
+        ChildInsertedRequest = 67,              // send ChildInserted compatibility events to receiver
+        ChildInserted = 70,                     // compatibility child inserted
         LayoutHint = 72,                        // compatibility relayout request
 #endif
         ChildRemoved = 71,                      // deleted child widget
@@ -161,8 +180,10 @@ public:
 #endif
         ToolBarChange = 120,                    // toolbar visibility toggled
 
-        ApplicationActivated = 121,             // application has been changed to active
-        ApplicationDeactivated = 122,           // application has been changed to inactive
+        ApplicationActivate = 121,              // application has been changed to active
+        ApplicationActivated = ApplicationActivate, // deprecated
+        ApplicationDeactivate = 122,            // application has been changed to inactive
+        ApplicationDeactivated = ApplicationDeactivate, // deprecated
 
         QueryWhatsThis = 123,                   // query what's this widget help
         EnterWhatsThisMode = 124,
@@ -185,11 +206,42 @@ public:
 #endif
         AcceptDropsChange = 152,
 
-        MenubarUpdated = 153,                    // Support event for Q3MainWindow, which needs to 
+        MenubarUpdated = 153,                    // Support event for Q3MainWindow, which needs to
                                                  // knwow when QMenubar is updated.
 
         ZeroTimerEvent = 154,                   // Used for Windows Zero timer events
 
+        GraphicsSceneMouseMove = 155,           // GraphicsView
+        GraphicsSceneMousePress = 156,
+        GraphicsSceneMouseRelease = 157,
+        GraphicsSceneMouseDoubleClick = 158,
+        GraphicsSceneContextMenu = 159,
+        GraphicsSceneHoverEnter = 160,
+        GraphicsSceneHoverMove = 161,
+        GraphicsSceneHoverLeave = 162,
+        GraphicsSceneHelp = 163,
+        GraphicsSceneDragEnter = 164,
+        GraphicsSceneDragMove = 165,
+        GraphicsSceneDragLeave = 166,
+        GraphicsSceneDrop = 167,
+        GraphicsSceneWheel = 168,
+
+        KeyboardLayoutChange = 169,             // keyboard layout changed
+
+        DynamicPropertyChange = 170,            // A dynamic property was changed through setProperty/property
+
+        TabletEnterProximity = 171,
+        TabletLeaveProximity = 172,
+
+        NonClientAreaMouseMove = 173,
+        NonClientAreaMouseButtonPress = 174,
+        NonClientAreaMouseButtonRelease = 175,
+        NonClientAreaMouseButtonDblClick = 176,
+
+        MacSizeChange = 177,                    // when the Qt::WA_Mac{Normal,Small,Mini}Size changes
+
+        ContentsRectChange = 178,               // sent by QWidget::setContentsMargins (internal)
+
         User = 1000,                            // first user event id
         MaxUser = 65535                         // last user event id
     };
@@ -264,4 +316,18 @@ public:
 };
 #endif
 
+class Q_CORE_EXPORT QDynamicPropertyChangeEvent : public QEvent
+{
+public:
+    QDynamicPropertyChangeEvent(const QByteArray &name);
+    ~QDynamicPropertyChangeEvent();
+
+    inline QByteArray propertyName() const { return n; }
+
+private:
+    QByteArray n;
+};
+
+QT_END_HEADER
+
 #endif // QCOREEVENT_H
