@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,7 +39,7 @@
 #include "qmetaobject.h"
 #include "qmetatype.h"
 #include "qobject.h"
-#include "private/qobject_p.h"
+
 #include <qcoreapplication.h>
 #include <qcoreevent.h>
 #include <qdatastream.h>
@@ -34,6 +49,11 @@
 #include <qvariant.h>
 #include <qhash.h>
 #include <qdebug.h>
+#include <qsemaphore.h>
+
+#include "private/qobject_p.h"
+#include "private/qmetaobject_p.h"
+
 #include <ctype.h>
 
 /*!
@@ -111,7 +131,7 @@ enum PropertyFlags  {
     Invalid = 0x00000000,
     Readable = 0x00000001,
     Writable = 0x00000002,
-    Resetable = 0x00000004,
+    Resettable = 0x00000004,
     EnumOrFlag = 0x00000008,
     StdCppSet = 0x00000100,
 //    Override = 0x00000200,
@@ -195,32 +215,37 @@ QObject *QMetaObject::cast(QObject *obj) const
 #ifndef QT_NO_TRANSLATION
 /*!
     \internal
-
-    Forwards a tr() call from the Q_OBJECT macro to the QApplication.
 */
 QString QMetaObject::tr(const char *s, const char *c) const
 {
-    if (QCoreApplication::instance())
-        return QCoreApplication::instance()->translate(d.stringdata, s, c, QCoreApplication::DefaultCodec);
-    else
-        return QString::fromLatin1(s);
+    return QCoreApplication::translate(d.stringdata, s, c, QCoreApplication::CodecForTr);
 }
+
 /*!
     \internal
+*/
+QString QMetaObject::tr(const char *s, const char *c, int n) const
+{
+    return QCoreApplication::translate(d.stringdata, s, c, QCoreApplication::CodecForTr, n);
+}
 
-    Forwards a trUtf8() call from the Q_OBJECT macro to the
-    QApplication.
+/*!
+    \internal
 */
 QString QMetaObject::trUtf8(const char *s, const char *c) const
 {
-    if (QCoreApplication::instance())
-        return QCoreApplication::instance()->translate(d.stringdata, s, c, QCoreApplication::UnicodeUTF8);
-    else
-        return QString::fromUtf8(s);
+    return QCoreApplication::translate(d.stringdata, s, c, QCoreApplication::UnicodeUTF8);
+}
+
+/*!
+    \internal
+*/
+QString QMetaObject::trUtf8(const char *s, const char *c, int n) const
+{
+    return QCoreApplication::translate(d.stringdata, s, c, QCoreApplication::UnicodeUTF8, n);
 }
 #endif // QT_NO_TRANSLATION
 
-
 /*!
     Returns the method offset for this class; i.e. the index position
     of this class's first member function.
@@ -582,6 +607,7 @@ QMetaEnum QMetaObject::enumerator(int index) const
 
 /*!
     Returns the meta-data for the property with the given \a index.
+    If no such property exists, a null QMetaProperty is returned.
 
     \sa propertyCount(), propertyOffset(), indexOfProperty()
 */
@@ -606,7 +632,7 @@ QMetaProperty QMetaObject::property(int index) const
             if (!result.menum.isValid()) {
                 QByteArray enum_name = type;
                 QByteArray scope_name = d.stringdata;
-                int s = enum_name.indexOf("::");
+                int s = enum_name.lastIndexOf("::");
                 if (s > 0) {
                     scope_name = enum_name.left(s);
                     enum_name = enum_name.mid(s + 2);
@@ -624,6 +650,24 @@ QMetaProperty QMetaObject::property(int index) const
     return result;
 }
 
+/*!
+    \since 4.2
+
+    Returns the property that has the \c USER flag set to true.
+
+    \sa QMetaProperty::isUser()
+*/
+QMetaProperty QMetaObject::userProperty() const
+{
+    const int propCount = propertyCount();
+    for (int i = propCount - 1; i >= 0; --i) {
+        const QMetaProperty prop = property(i);
+        if (prop.isUser())
+            return prop;
+    }
+    return QMetaProperty();
+}
+
 /*!
     Returns the meta-data for the item of class information with the
     given \a index.
@@ -682,114 +726,68 @@ bool QMetaObject::checkConnectArgs(const char *signal, const char *method)
     return false;
 }
 
-static inline bool is_ident_char(char s)
+static void qRemoveWhitespace(const char *s, char *d)
 {
-    return ((s >= 'a' && s <= 'z')
-            || (s >= 'A' && s <= 'Z')
-            || (s >= '0' && s <= '9')
-            || s == '_'
-       );
+    char last = 0;
+    while (*s && is_space(*s))
+        s++;
+    while (*s) {
+        while (*s && !is_space(*s))
+            last = *d++ = *s++;
+        while (*s && is_space(*s))
+            s++;
+        if (*s && is_ident_char(*s) && is_ident_char(last))
+            last = *d++ = ' ';
+    }
+    *d = '\0';
 }
 
-static inline bool is_space(char s)
+static char *qNormalizeType(char *d, int &templdepth, QByteArray &result)
 {
-    return (s == ' ' || s == '\t');
+    const char *t = d;
+    while (*d && (templdepth
+                   || (*d != ',' && *d != ')'))) {
+        if (*d == '<')
+            ++templdepth;
+        if (*d == '>')
+            --templdepth;
+        ++d;
+    }
+    if (strncmp("void", t, d - t) != 0)
+        result += normalizeTypeInternal(t, d);
+
+    return d;
 }
 
-// WARNING: a copy of this function is in moc.cpp
-static QByteArray normalizeTypeInternal(const char *t, const char *e, bool fixScope = false, bool adjustConst = true)
+
+/*!
+    \since 4.2
+
+    Normalizes a \a type.
+
+    See QMetaObject::normalizedSignature() for a description on how
+    Qt normalizes.
+
+    Example:
+
+    \code
+    QByteArray normType = QMetaObject::normalizedType(" int    const  *");
+    // normType is now "const int*"
+    \endcode
+
+    \sa normalizedSignature()
+ */
+QByteArray QMetaObject::normalizedType(const char *type)
 {
-    int len = e - t;
-    /*
-      Convert 'char const *' into 'const char *'. Start at index 1,
-      not 0, because 'const char *' is already OK.
-    */
-    QByteArray constbuf;
-    for (int i = 1; i < len; i++) {
-        if ( t[i] == 'c'
-             && strncmp(t + i + 1, "onst", 4) == 0
-             && (i + 5 >= len || !is_ident_char(t[i + 5]))
-             && !is_ident_char(t[i-1])
-            ) {
-            constbuf = QByteArray(t, len);
-            if (is_space(t[i-1]))
-                constbuf.remove(i-1, 6);
-            else
-                constbuf.remove(i, 5);
-            constbuf.prepend("const ");
-            t = constbuf.data();
-            e = constbuf.data() + constbuf.length();
-            break;
-        }
-        /*
-          We musn't convert 'char * const *' into 'const char **'
-          and we must beware of 'Bar<const Bla>'.
-        */
-        if (t[i] == '&' || t[i] == '*' ||t[i] == '<')
-            break;
-    }
-    if (adjustConst && e > t + 6 && strncmp("const ", t, 6) == 0) {
-        if (*(e-1) == '&') { // treat const reference as value
-            t += 6;
-            --e;
-        } else if (is_ident_char(*(e-1))) { // treat const value as value
-            t += 6;
-        }
-    }
     QByteArray result;
-    result.reserve(len);
 
-    // some type substitutions for 'unsigned x'
-    if (strncmp("unsigned ", t, 9) == 0) {
-        if (strncmp("int", t+9, 3) == 0) {
-            t += 9+3;
-            result += "uint";
-        } else if (strncmp("long", t+9, 4) == 0
-                   // preserve '[unsigned] long int'
-                   && (strlen(t + 9 + 4) < 4
-                       || strncmp(t + 9 + 4, " int", 4) != 0
-                      )
-                   // preserve '[unsigned] long long'
-                   && (strlen(t + 9 + 4) < 5
-                       || strncmp(t + 9 + 4, " long", 5) != 0
-                      )
-                  ) {
-            t += 9+4;
-            result += "ulong";
-        }
-    }
+    if (!type || !*type)
+        return result;
 
-    while (t != e) {
-        char c = *t++;
-        if (fixScope && c == ':' && *t == ':' ) {
-            ++t;
-            c = *t++;
-            int i = result.size() - 1;
-            while (i >= 0 && is_ident_char(result.at(i)))
-                   --i;
-            result.resize(i + 1);
-        }
-        result += c;
-        if (c == '<') {
-            //template recursion
-            const char* tt = t;
-            int templdepth = 1;
-            while (t != e) {
-                c = *t++;
-                if (c == '<')
-                    ++templdepth;
-                if (c == '>')
-                    --templdepth;
-                if (templdepth == 0) {
-                    result += normalizeTypeInternal(tt, t-1, fixScope, false);
-                    result += c;
-                    if (*t == '>')
-                        result += ' '; // avoid >>
-                    break;
-                }
-            }
-        }
-    }
+    QVarLengthArray<char> stackbuf((int)strlen(type));
+    qRemoveWhitespace(type, stackbuf.data());
+    int templdepth = 0;
+    qNormalizeType(stackbuf.data(), templdepth, result);
 
     return result;
 }
@@ -803,50 +801,26 @@ static QByteArray normalizeTypeInternal(const char *t, const char *e, bool fixSc
     'const' from value types and replaces const references with
     values.
 
-    \sa checkConnectArgs()
+    \sa checkConnectArgs(), normalizedType()
  */
 QByteArray QMetaObject::normalizedSignature(const char *method)
 {
-    const char *s = method;
-    if (!s || !*s)
-        return "";
-    int len = qstrlen(s);
+    QByteArray result;
+    if (!method || !*method)
+        return result;
+    int len = int(strlen(method));
     char stackbuf[64];
     char *buf = (len >= 64 ? new char[len+1] : stackbuf);
+    qRemoveWhitespace(method, buf);
     char *d = buf;
-    char last = 0;
-    while(*s && is_space(*s))
-        s++;
-    while (*s) {
-        while (*s && !is_space(*s))
-            last = *d++ = *s++;
-        while (*s && is_space(*s))
-            s++;
-        if (*s && is_ident_char(*s) && is_ident_char(last))
-            last = *d++ = ' ';
-    }
-    *d = '\0';
-    d = buf;
 
-    QByteArray result;
     result.reserve(len);
 
     int argdepth = 0;
     int templdepth = 0;
     while (*d) {
-        if (argdepth == 1) {
-            const char *t = d;
-            while (*d&& (templdepth
-                           || (*d != ',' && *d != ')'))) {
-                if (*d == '<')
-                    ++templdepth;
-                if (*d == '>')
-                    --templdepth;
-                d++;
-            }
-            if (strncmp("void", t, d - t) != 0)
-                result += normalizeTypeInternal(t, d);
-        }
+        if (argdepth == 1)
+            d = qNormalizeType(d, templdepth, result);
         if (*d == '(')
             ++argdepth;
         if (*d == ')')
@@ -955,20 +929,20 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
     sig.append(member, len);
     sig.append('(');
 
-    enum { ParamCount = 11 };
+    enum { MaximumParamCount = 11 };
     const char *typeNames[] = {ret.name(), val0.name(), val1.name(), val2.name(), val3.name(),
                                val4.name(), val5.name(), val6.name(), val7.name(), val8.name(),
                                val9.name()};
 
-    int i;
-    for (i = 1; i < ParamCount; ++i) {
-        len = qstrlen(typeNames[i]);
+    int paramCount;
+    for (paramCount = 1; paramCount < MaximumParamCount; ++paramCount) {
+        len = qstrlen(typeNames[paramCount]);
         if (len <= 0)
             break;
-        sig.append(typeNames[i], len);
+        sig.append(typeNames[paramCount], len);
         sig.append(',');
     }
-    if (i == 1)
+    if (paramCount == 1)
         sig.append(')'); // no parameters
     else
         sig[sig.size() - 1] = ')';
@@ -985,8 +959,24 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
     // check return type
     if (ret.data()) {
         const char *retType = obj->metaObject()->method(idx).typeName();
-        if (qstrcmp(ret.name(), retType) != 0)
-            return false;
+        if (qstrcmp(ret.name(), retType) != 0) {
+            // normalize the return value as well
+            // the trick here is to make a function signature out of the return type
+            // so that we can call normalizedSignature() and avoid duplicating code
+            QByteArray unnormalized;
+            int len = qstrlen(ret.name());
+
+            unnormalized.reserve(len + 3);
+            unnormalized = "_(";        // the function is called "_"
+            unnormalized.append(ret.name());
+            unnormalized.append(')');
+
+            QByteArray normalized = QMetaObject::normalizedSignature(unnormalized.constData());
+            normalized.truncate(normalized.length() - 1); // drop the ending ')'
+
+            if (qstrcmp(normalized.constData() + 2, retType) != 0)
+                return false;
+        }
     }
     void *param[] = {ret.data(), val0.data(), val1.data(), val2.data(), val3.data(), val4.data(),
                      val5.data(), val6.data(), val7.data(), val8.data(), val9.data()};
@@ -996,20 +986,20 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
                : Qt::QueuedConnection;
     }
 
-    if (type != Qt::QueuedConnection) {
+    if (type == Qt::DirectConnection) {
         return obj->qt_metacall(QMetaObject::InvokeMetaMethod, idx, param) < 0;
     } else {
         if (ret.data()) {
             qWarning("QMetaObject::invokeMethod: Unable to invoke methods with return values in queued "
-                     "connections.");
+                     "connections");
             return false;
         }
         int nargs = 1; // include return type
-        void **args = (void **) qMalloc(ParamCount * sizeof(void *));
-        int *types = (int *) qMalloc(ParamCount * sizeof(int));
+        void **args = (void **) qMalloc(paramCount * sizeof(void *));
+        int *types = (int *) qMalloc(paramCount * sizeof(int));
         types[0] = 0; // return type
         args[0] = 0;
-        for (i = 1; i < ParamCount; ++i) {
+        for (int i = 1; i < paramCount; ++i) {
             types[i] = QMetaType::type(typeNames[i]);
             if (types[i]) {
                 args[i] = QMetaType::construct(types[i], param[i]);
@@ -1021,7 +1011,25 @@ bool QMetaObject::invokeMethod(QObject *obj, const char *member, Qt::ConnectionT
             }
         }
 
-        QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, nargs, types, args));
+        if (type == Qt::QueuedConnection) {
+            QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, -1, -1, nargs, types, args));
+        } else {
+            if (QThread::currentThread() == obj->thread()) {
+                qWarning("QMetaObject::invokeMethod: Dead lock detected in BlockingQueuedConnection: "
+                         "Receiver is %s(%p)",
+                         obj->metaObject()->className(), obj);
+            }
+
+            // blocking queued connection
+#ifdef QT_NO_THREAD
+            QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, -1, -1, nargs, types, args));
+#else
+            QSemaphore semaphore;
+            QCoreApplication::postEvent(obj, new QMetaCallEvent(idx, 0, -1, -1, nargs, types, args,
+                                                                &semaphore));
+            semaphore.acquire();
+#endif // QT_NO_THREAD
+        }
     }
     return true;
 }
@@ -1389,19 +1397,19 @@ int QMetaEnum::keyToValue(const char *key) const
 {
     if (!mobj || !key)
         return -1;
-    int scope = 0;
+    uint scope = 0;
     const char *qualified_key = key;
-    const char *s = key;
-    while (*s  && *s != ':')
-        ++s;
-    if (*s && *(s+1)==':') {
-        scope = s - key;
+    const char *s = key + qstrlen(key);
+    while (s > key && *s != ':')
+        --s;
+    if (s > key && *(s-1)==':') {
+        scope = s - key - 1;
         key += scope + 2;
     }
     int count = mobj->d.data[handle + 2];
     int data = mobj->d.data[handle + 3];
     for (int i = 0; i < count; ++i)
-        if ((!scope || strncmp(qualified_key, mobj->d.stringdata, scope) == 0)
+        if ((!scope || (qstrlen(mobj->d.stringdata) == scope && strncmp(qualified_key, mobj->d.stringdata, scope) == 0))
              && strcmp(key, mobj->d.stringdata + mobj->d.data[data + 2*i]) == 0)
             return mobj->d.data[data + 2*i + 1];
     return -1;
@@ -1428,9 +1436,9 @@ const char* QMetaEnum::valueToKey(int value) const
 }
 
 /*!
-    Returns the value derived from combining together the values of the
-    \a keys using the OR operator. Note that the strings in \a keys
-    must be '|'-separated.
+    Returns the value derived from combining together the values of
+    the \a keys using the OR operator, or -1 if \a keys is not
+    defined. Note that the strings in \a keys must be '|'-separated.
 
     \sa isFlag(), valueToKey(), valueToKeys()
 */
@@ -1447,17 +1455,17 @@ int QMetaEnum::keysToValue(const char *keys) const
         QString trimmed = l.at(li).trimmed();
         QByteArray qualified_key = trimmed.toLatin1();
         const char *key = qualified_key.constData();
-        int scope = 0;
-        const char *s = key;
-        while (*s  && *s != ':')
-            ++s;
-        if (*s && *(s+1)==':') {
-            scope = s - key;
+        uint scope = 0;
+        const char *s = key + qstrlen(key);
+        while (s > key && *s != ':')
+            --s;
+        if (s > key && *(s-1)==':') {
+            scope = s - key - 1;
             key += scope + 2;
         }
         int i;
         for (i = count-1; i >= 0; --i)
-            if ((!scope || strncmp(qualified_key.constData(), mobj->d.stringdata, scope) == 0)
+            if ((!scope || (qstrlen(mobj->d.stringdata) == scope && strncmp(qualified_key.constData(), mobj->d.stringdata, scope) == 0))
                  && strcmp(key, mobj->d.stringdata + mobj->d.data[data + 2*i]) == 0) {
                 value |= mobj->d.data[data + 2*i + 1];
                 break;
@@ -1482,7 +1490,7 @@ QByteArray QMetaEnum::valueToKeys(int value) const
     int count = mobj->d.data[handle + 2];
     int data = mobj->d.data[handle + 3];
     int v = value;
-    for(int i = count - 1; i >= 0; --i) {
+    for(int i = 0; i < count; i++) {
         int k = mobj->d.data[data + 2*i + 1];
         if ((k != 0 && (v & k) == k ) ||  (k == value))  {
             v = v & ~k;
@@ -1503,7 +1511,7 @@ QByteArray QMetaEnum::valueToKeys(int value) const
 
     A property has a name() and a type(), as well as various
     attributes that specify its behavior: isReadable(), isWritable(),
-    isDesignable(), isScriptable(), isStored(), and isEditable().
+    isDesignable(), isScriptable(), and isStored().
 
     If the property is an enumeration, isEnumType() returns true; if the
     property is an enumeration that is also a flag (i.e. its values
@@ -1571,24 +1579,43 @@ const char *QMetaProperty::typeName() const
     Returns this property's type. The return value is one
     of the values of the QVariant::Type enumeration.
 
-    \sa typeName(), name()
+    \sa userType(), typeName(), name()
 */
 QVariant::Type QMetaProperty::type() const
 {
     if (!mobj)
         return QVariant::Invalid;
     int handle = priv(mobj->d.data)->propertyData + 3*idx;
-    int flags = mobj->d.data[handle + 2];
+    uint flags = mobj->d.data[handle + 2];
 
-    QVariant::Type type = QVariant::Type(flags >> 24);
+    uint type = flags >> 24;
+    if (type == 0xff) // special value for QVariant
+        type = QVariant::LastType;
     if (type)
-        return type;
+        return QVariant::Type(type);
     if (isEnumType())
         return QVariant::Int;
 
     return QVariant::UserType;
 }
 
+/*!
+    \since 4.2
+
+    Returns this property's user type. The return value is one
+    of the values that are registered with QMetaType, or 0 if
+    the type is not registered.
+
+    \sa type(), QMetaType, typeName()
+ */
+int QMetaProperty::userType() const
+{
+    QVariant::Type tp = type();
+    if (tp != QVariant::UserType)
+        return tp;
+    return QMetaType::type(typeName());
+}
+
 /*!
     Returns true if the property's type is an enumeration value that
     is used as a flag; otherwise returns false.
@@ -1659,22 +1686,27 @@ QVariant QMetaProperty::read(const QObject *object) const
     if (!object || !mobj)
         return QVariant();
 
-    int  t = QVariant::Int;
+    uint t = QVariant::Int;
     if (!isEnumType()) {
         int handle = priv(mobj->d.data)->propertyData + 3*idx;
-        int flags = mobj->d.data[handle + 2];
+        uint flags = mobj->d.data[handle + 2];
         const char *typeName = mobj->d.stringdata + mobj->d.data[handle + 1];
         t = (flags >> 24);
+        if (t == 0xff) // special value for QVariant
+            t = QVariant::LastType;
         if (t == QVariant::Invalid)
             t = QMetaType::type(typeName);
         if (t == QVariant::Invalid)
             t = QVariant::nameToType(typeName);
-        if (t == QVariant::Invalid || t == QVariant::UserType)
+        if (t == QVariant::Invalid || t == QVariant::UserType) {
+            if (t == QVariant::Invalid)
+                qWarning("QMetaProperty::read: Unable to handle unregistered datatype '%s' for property '%s::%s'", typeName, mobj->className(), name());
             return QVariant();
+        }
     }
     QVariant value;
     void *argv[1];
-    if (t == int(QVariant::LastType)) {
+    if (t == QVariant::LastType) {
         argv[0] = &value;
     } else {
         value = QVariant(t, (void*)0);
@@ -1683,7 +1715,7 @@ QVariant QMetaProperty::read(const QObject *object) const
     const_cast<QObject*>(object)->qt_metacall(QMetaObject::ReadProperty,
                                               idx + mobj->propertyOffset(),
                                               argv);
-    if (t != int(QVariant::LastType) && argv[0] != value.data())
+    if (t != QVariant::LastType && argv[0] != value.data())
         return QVariant((QVariant::Type)t, argv[0]);
     return value;
 }
@@ -1717,9 +1749,11 @@ bool QMetaProperty::write(QObject *object, const QVariant &value) const
         v.convert(QVariant::Int);
     } else {
         int handle = priv(mobj->d.data)->propertyData + 3*idx;
-        int flags = mobj->d.data[handle + 2];
+        uint flags = mobj->d.data[handle + 2];
         const char *typeName = mobj->d.stringdata + mobj->d.data[handle + 1];
         t = flags >> 24;
+        if (t == 0xff) // special value for QVariant
+            t = QVariant::LastType;
         if (t == QVariant::Invalid) {
             const char *vtypeName = value.typeName();
             if (vtypeName && strcmp(typeName, vtypeName) == 0)
@@ -1729,7 +1763,7 @@ bool QMetaProperty::write(QObject *object, const QVariant &value) const
         }
         if (t == QVariant::Invalid)
             return false;
-        if (t != QVariant::LastType && t != (uint)value.userType() && (t < QVariant::UserType && !v.convert((QVariant::Type)t)))
+        if (t != QVariant::LastType && t != (uint)value.userType() && (t < QMetaType::User && !v.convert((QVariant::Type)t)))
             return false;
     }
 
@@ -1770,7 +1804,7 @@ bool QMetaProperty::isResettable() const
     if (!mobj)
         return false;
     int flags = mobj->d.data[handle + 2];
-    return flags & Resetable;
+    return flags & Resettable;
 }
 
 /*!
@@ -1809,7 +1843,7 @@ bool QMetaProperty::isWritable() const
     \c{Q_PROPERTY()}'s \c DESIGNABLE attribute is false; otherwise
     returns true (if the attribute is true or is a function or expression).
 
-    \sa isScriptable(), isStored(), isEditable()
+    \sa isScriptable(), isStored()
 */
 bool QMetaProperty::isDesignable(const QObject *object) const
 {
@@ -1832,10 +1866,10 @@ bool QMetaProperty::isDesignable(const QObject *object) const
     otherwise returns false.
 
     If no \a object is given, the function returns false if the
-    \c{Q_PROPERTY()}'s \c DESIGNABLE attribute is false; otherwise returns
+    \c{Q_PROPERTY()}'s \c SCRIPTABLE attribute is false; otherwise returns
     true (if the attribute is true or is a function or expression).
 
-    \sa isDesignable(), isStored(), isEditable()
+    \sa isDesignable(), isStored()
 */
 bool QMetaProperty::isScriptable(const QObject *object) const
 {
@@ -1856,10 +1890,10 @@ bool QMetaProperty::isScriptable(const QObject *object) const
     false.
 
     If no \a object is given, the function returns false if the
-    \c{Q_PROPERTY()}'s \c DESIGNABLE attribute is false; otherwise returns
+    \c{Q_PROPERTY()}'s \c STORED attribute is false; otherwise returns
     true (if the attribute is true or is a function or expression).
 
-    \sa isDesignable(), isScriptable(), isEditable()
+    \sa isDesignable(), isScriptable()
 */
 bool QMetaProperty::isStored(const QObject *object) const
 {
@@ -1884,7 +1918,7 @@ bool QMetaProperty::isStored(const QObject *object) const
     \c{Q_PROPERTY()}'s \c USER attribute is false; otherwise returns
     true (if the attribute is true or is a function or expression).
 
-    \sa isDesignable(), isScriptable(), isEditable()
+    \sa QMetaObject::userProperty(), isDesignable(), isScriptable()
 */
 bool QMetaProperty::isUser(const QObject *object) const
 {
@@ -1901,11 +1935,13 @@ bool QMetaProperty::isUser(const QObject *object) const
 }
 
 /*!
+    \obsolete
+
     Returns true if the property is editable for the given \a object;
     otherwise returns false.
 
     If no \a object is given, the function returns false if the
-    \c{Q_PROPERTY()}'s \c DESIGNABLE attribute is false; otherwise returns
+    \c{Q_PROPERTY()}'s \c EDITABLE attribute is false; otherwise returns
     true (if the attribute is true or is a function or expression).
 
     \sa isDesignable(), isScriptable(), isStored()
