@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -32,7 +47,9 @@
 #include "qlocale.h"
 
 #ifdef QT_BOOTSTRAPPED
+# ifndef QT_NO_GEOM_VARIANT
 #  define QT_NO_GEOM_VARIANT
+# endif
 #else
 #  include "qbitarray.h"
 #  include "qurl.h"
@@ -167,6 +184,7 @@
     \value QLocale QLocale
     \value QBitmap QBitmap
     \value QMatrix QMatrix
+    \value QTransform QTransform
 
     \value User  Base value for user types
 
@@ -273,6 +291,7 @@ static const struct { const char * typeName; int type; } types[] = {
     {"QTextLength", QMetaType::QTextLength},
     {"QTextFormat", QMetaType::QTextFormat},
     {"QMatrix", QMetaType::QMatrix},
+    {"QTransform", QMetaType::QTransform},
 
     /* All Metatype builtins */
     {"void*", QMetaType::VoidStar},
@@ -443,17 +462,20 @@ int QMetaType::registerType(const char *typeName, Destructor destructor,
 }
 
 /*!
-    Returns true if the custom datatype with ID \a type is registered;
+    Returns true if the datatype with ID \a type is registered;
     otherwise returns false.
 
     \sa type(), typeName(), Type
 */
 bool QMetaType::isRegistered(int type)
 {
+    if (type >= 0 && type < User) {
+        // predefined type
+        return true;
+    }
     QReadLocker locker(customTypesLock());
-
     const QVector<QCustomTypeInfo> * const ct = customTypes();
-    return (type < User) || ((type >= User) && (ct && ct->count() > type - User));
+    return ((type >= User) && (ct && ct->count() > type - User));
 }
 
 /*!
@@ -633,6 +655,7 @@ bool QMetaType::save(QDataStream &stream, int type, const void *data)
     case QMetaType::QTextLength:
     case QMetaType::QTextFormat:
     case QMetaType::QMatrix:
+    case QMetaType::QTransform:
         if (!qMetaTypeGuiHelper)
             return false;
         qMetaTypeGuiHelper[type - FirstGuiType].saveOp(stream, data);
@@ -821,6 +844,7 @@ bool QMetaType::load(QDataStream &stream, int type, void *data)
     case QMetaType::QTextLength:
     case QMetaType::QTextFormat:
     case QMetaType::QMatrix:
+    case QMetaType::QTransform:
         if (!qMetaTypeGuiHelper)
             return false;
         qMetaTypeGuiHelper[type - FirstGuiType].loadOp(stream, data);
