@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -86,11 +101,6 @@ public:
     uint receiveChildEvents : 1;
     uint unused : 25;
     int postedEvents;
-#ifdef QT3_SUPPORT
-    int postedChildInsertedEvents;
-#else
-    int reserved;
-#endif
 };
 
 
@@ -108,14 +118,19 @@ public:
     virtual bool eventFilter(QObject *, QEvent *);
 
 #ifdef qdoc
-    static QString tr(const char *, const char *);
-    static QString trUtf8(const char *, const char *);
+    static QString tr(const char *sourceText, const char *comment = 0, int n = -1);
+    static QString trUtf8(const char *sourceText, const char *comment = 0, int n = -1);
     virtual const QMetaObject *metaObject() const;
+    static const QMetaObject staticMetaObject;
 #endif
 #ifdef QT_NO_TRANSLATION
+    static QString tr(const char *sourceText, const char *, int)
+        { return QString::fromLatin1(sourceText); }
     static QString tr(const char *sourceText, const char * = 0)
         { return QString::fromLatin1(sourceText); }
 #ifndef QT_NO_TEXTCODEC
+    static QString trUtf8(const char *sourceText, const char *, int)
+        { return QString::fromUtf8(sourceText); }
     static QString trUtf8(const char *sourceText, const char * = 0)
         { return QString::fromUtf8(sourceText); }
 #endif
@@ -197,6 +212,7 @@ public:
 #ifndef QT_NO_PROPERTIES
     bool setProperty(const char *name, const QVariant &value);
     QVariant property(const char *name) const;
+    QList<QByteArray> dynamicPropertyNames() const;
 #endif // QT_NO_PROPERTIES
 
 #ifndef QT_NO_USERDATA
@@ -283,12 +299,10 @@ public:
 };
 #endif
 
-
 Q_CORE_EXPORT void qt_qFindChildren_helper(const QObject *parent, const QString &name, const QRegExp *re,
-                         const QMetaObject &mo, QList<void*> *list);
+                                           const QMetaObject &mo, QList<void *> *list);
 Q_CORE_EXPORT QObject *qt_qFindChild_helper(const QObject *parent, const QString &name, const QMetaObject &mo);
 
-
 #if defined Q_CC_MSVC && _MSC_VER < 1300
 
 template<typename T>
@@ -299,8 +313,12 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QString &name, T)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, name, 0, ((T)0)->staticMetaObject,
-                        reinterpret_cast<QList<void *>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, name, 0, ((T)0)->staticMetaObject, u.voidList);
     return list;
 }
 
@@ -325,8 +343,12 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QRegExp &re, T)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, 0, &re, ((T)0)->staticMetaObject,
-                        reinterpret_cast<QList<void*>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, 0, &re, ((T)0)->staticMetaObject, u.voidList);
     return list;
 }
 
@@ -336,6 +358,11 @@ inline QList<T> qFindChildren(const QObject *o, const QRegExp &re)
 
 #endif
 
+#ifdef Q_MOC_RUN
+# define Q_DECLARE_INTERFACE(IFace, IId) Q_DECLARE_INTERFACE(IFace, IId)
+#endif // Q_MOC_RUN
+
+
 template <class T> inline T qobject_cast_helper(QObject *object, T)
 { return static_cast<T>(((T)0)->staticMetaObject.cast(object)); }
 
@@ -350,11 +377,13 @@ template <class T>
 inline T qobject_cast(const QObject *object)
 { return qobject_cast_helper<T>(object, T(0)); }
 
-#define Q_DECLARE_INTERFACE(IFace, IId) \
-template <> inline IFace *qobject_cast_helper<IFace *>(QObject *object, IFace *) \
-{ return (IFace *)(object ? object->qt_metacast(IId) : 0); } \
-template <> inline IFace *qobject_cast_helper<IFace *>(const QObject *object, IFace *) \
-{ return (IFace *)(object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0); }
+#ifndef Q_MOC_RUN
+#  define Q_DECLARE_INTERFACE(IFace, IId) \
+    template <> inline IFace *qobject_cast_helper<IFace *>(QObject *object, IFace *) \
+    { return (IFace *)(object ? object->qt_metacast(IId) : 0); } \
+    template <> inline IFace *qobject_cast_helper<IFace *>(const QObject *object, IFace *) \
+    { return (IFace *)(object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0); }
+#endif // Q_MOC_RUN
 
 #else
 
@@ -366,8 +395,12 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QString &name)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, name, 0, reinterpret_cast<T>(0)->staticMetaObject,
-                         reinterpret_cast<QList<void *>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, name, 0, reinterpret_cast<T>(0)->staticMetaObject, u.voidList);
     return list;
 }
 
@@ -376,26 +409,42 @@ template<typename T>
 inline QList<T> qFindChildren(const QObject *o, const QRegExp &re)
 {
     QList<T> list;
-    qt_qFindChildren_helper(o, 0, &re, reinterpret_cast<T>(0)->staticMetaObject,
-                        reinterpret_cast<QList<void*>*>(&list));
+    union {
+        QList<T> *typedList;
+        QList<void *> *voidList;
+    } u;
+    u.typedList = &list;
+    qt_qFindChildren_helper(o, QString(), &re, reinterpret_cast<T>(0)->staticMetaObject, u.voidList);
     return list;
 }
 #endif
 
 template <class T>
 inline T qobject_cast(QObject *object)
-{ return static_cast<T>(reinterpret_cast<T>(0)->staticMetaObject.cast(object)); }
+{
+#if !defined(QT_NO_MEMBER_TEMPLATES) && !defined(QT_NO_QOBJECT_CHECK)
+    reinterpret_cast<T>(0)->qt_check_for_QOBJECT_macro(*reinterpret_cast<T>(object));
+#endif
+    return static_cast<T>(reinterpret_cast<T>(0)->staticMetaObject.cast(object));
+}
 
 template <class T>
 inline T qobject_cast(const QObject *object)
-{ return static_cast<T>(const_cast<const QObject *>(reinterpret_cast<T>(0)->staticMetaObject.cast(const_cast<QObject *>(object)))); }
+{
+#if !defined(QT_NO_MEMBER_TEMPLATES) && !defined(QT_NO_QOBJECT_CHECK)
+    reinterpret_cast<T>(0)->qt_check_for_QOBJECT_macro(*reinterpret_cast<T>(object));
+#endif
+    return static_cast<T>(const_cast<const QObject *>(reinterpret_cast<T>(0)->staticMetaObject.cast(const_cast<QObject *>(object))));
+}
 
 
-#define Q_DECLARE_INTERFACE(IFace, IId) \
-template <> inline IFace *qobject_cast<IFace *>(QObject *object) \
-{ return reinterpret_cast<IFace *>((object ? object->qt_metacast(IId) : 0)); } \
-template <> inline IFace *qobject_cast<IFace *>(const QObject *object) \
-{ return reinterpret_cast<IFace *>((object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0)); }
+#ifndef Q_MOC_RUN
+#  define Q_DECLARE_INTERFACE(IFace, IId) \
+    template <> inline IFace *qobject_cast<IFace *>(QObject *object) \
+    { return reinterpret_cast<IFace *>((object ? object->qt_metacast(IId) : 0)); } \
+    template <> inline IFace *qobject_cast<IFace *>(const QObject *object) \
+    { return reinterpret_cast<IFace *>((object ? const_cast<QObject *>(object)->qt_metacast(IId) : 0)); }
+#endif // Q_MOC_RUN
 
 #endif
 
