@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -71,10 +86,11 @@ QFactoryLoader::QFactoryLoader(const char *iid,
             }
             library = QLibraryPrivate::findOrCreate(QFileInfo(fileName).canonicalFilePath());
             if (!library->isPlugin()) {
-                library->release();
                 if (qt_debug_component()) {
+                    qDebug() << library->errorString;
                     qDebug() << "         not a plugin";
                 }
+                library->release();
                 continue;
             }
             QString regkey = QString::fromLatin1("Qt Factory Cache %1.%2/%3:/%4")
@@ -90,6 +106,7 @@ QFactoryLoader::QFactoryLoader(const char *iid,
             } else {
                 if (!library->loadPlugin()) {
                     if (qt_debug_component()) {
+                        qDebug() << library->errorString;
                         qDebug() << "           could not load";
                     }
                     library->release();
@@ -152,7 +169,7 @@ QStringList QFactoryLoader::keys() const
     for (int i = 0; i < instances.count(); ++i)
         if (QFactoryInterface *factory = qobject_cast<QFactoryInterface*>(instances.at(i)))
             if (instances.at(i)->qt_metacast(d->iid))
-                    keys += factory->keys();
+                keys += factory->keys();
     return keys;
 }
 
