@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -174,30 +189,36 @@ QMutex::~QMutex()
 */
 void QMutex::lock()
 {
-    ulong self = d->self();
-
-    int sentinel;
-    forever {
-        sentinel = d->lock;
-        if (d->lock.testAndSetAcquire(sentinel, sentinel + 1))
-            break;
-    }
-    if (sentinel != 0) {
-        if (!d->recursive || d->owner != self) {
-            if (d->owner == self) {
-                qWarning("QMutex::lock(): Deadlock detected in thread %ld", d->owner);
-            }
-
-            // didn't get the lock, wait for it
-            d->wait();
+    ulong self = 0;
+#ifndef QT_NO_DEBUG
+    self = d->self();
+#endif
+    if (d->recursive) {
+        self = d->self();
+        if (d->owner == self) {
+            ++d->count;
+            Q_ASSERT_X(d->count != 0, "QMutex::lock", "Overflow in recursion counter");
+            return;
         }
+    }
+    
+    bool isLocked = d->contenders.fetchAndAddAcquire(1) == 0;
+    if (!isLocked) {
+#ifndef QT_NO_DEBUG
+        if (d->owner == self)
+            qWarning("QMutex::lock: Deadlock detected in thread %ld", d->owner);
+#endif
+        
+        // didn't get the lock, wait for it
+        isLocked = d->wait();
+        Q_ASSERT_X(isLocked, "QMutex::lock", "Internal error, infinite wait has timed out.");
 
         // don't need to wait for the lock anymore
-        d->lock.deref();
+        d->contenders.deref();
     }
     d->owner = self;
     ++d->count;
-    Q_ASSERT_X(d->count != 0, "QMutex::lock()", "Overflow in recursion counter");
+    Q_ASSERT_X(d->count != 0, "QMutex::lock", "Overflow in recursion counter");
 }
 
 /*!
@@ -212,29 +233,80 @@ void QMutex::lock()
 */
 bool QMutex::tryLock()
 {
-    ulong self = d->self();
-
-    int sentinel;
-    forever {
-        sentinel = d->lock;
-        if (d->lock.testAndSetAcquire(sentinel, sentinel + 1))
-            break;
-    }
-    if (sentinel != 0) {
-        // we're not going to wait for lock
-        d->lock.deref();
-
-        if (!d->recursive || d->owner != self) {
-            // some other thread has the mutex locked, or we tried to
-            // recursively lock an non-recursive mutex
-            return false;
+    ulong self = 0;
+#ifndef QT_NO_DEBUG
+    self = d->self();
+#endif
+    if (d->recursive) {
+        self = d->self();
+        if (d->owner == self) {
+            ++d->count;
+            Q_ASSERT_X(d->count != 0, "QMutex::tryLock", "Overflow in recursion counter");
+            return true;
         }
     }
+    
+    bool isLocked = d->contenders.testAndSetAcquire(0, 1);
+    if (!isLocked) {
+        // some other thread has the mutex locked, or we tried to
+        // recursively lock an non-recursive mutex
+        return isLocked;
+    }
     d->owner = self;
     ++d->count;
+    Q_ASSERT_X(d->count != 0, "QMutex::tryLock", "Overflow in recursion counter");
+    return isLocked;
+}
+
+/*! \overload
+
+    Attempts to lock the mutex. This function returns true if the lock
+    was obtained; otherwise it returns false. If another thread has
+    locked the mutex, this function will wait for at most \a timeout
+    milliseconds for the mutex to become available.
+
+    Note: Passing a negative number as the \a timeout is equivalent to
+    calling lock(), i.e. this function will wait forever until mutex
+    can be locked if \a timeout is negative.
+
+    If the lock was obtained, the mutex must be unlocked with unlock()
+    before another thread can successfully lock it.
+
+    \sa lock(), unlock()
+*/
+bool QMutex::tryLock(int timeout)
+{
+    ulong self = 0;
+#ifndef QT_NO_DEBUG
+    self = d->self();
+#endif
+    if (d->recursive) {
+        self = d->self();
+        if (d->owner == self) {
+            ++d->count;
+            Q_ASSERT_X(d->count != 0, "QMutex::tryLock", "Overflow in recursion counter");
+            return true;
+        }
+    }
+    
+    bool isLocked = d->contenders.fetchAndAddAcquire(1) == 0;
+    if (!isLocked) {
+        // didn't get the lock, wait for it
+        isLocked = d->wait(timeout);
+
+        // don't need to wait for the lock anymore
+        d->contenders.deref();
+        if (!isLocked)
+            return false;
+    }
+
+    d->owner = self;
+    ++d->count;
+    Q_ASSERT_X(d->count != 0, "QMutex::tryLock", "Overflow in recursion counter");
     return true;
 }
 
+
 /*!
     Unlocks the mutex. Attempting to unlock a mutex in a different
     thread to the one that locked it results in an error. Unlocking a
@@ -249,7 +321,7 @@ void QMutex::unlock()
 
     if (!--d->count) {
         d->owner = 0;
-        if (!d->lock.testAndSetRelease(1, 0))
+        if (!d->contenders.testAndSetRelease(1, 0))
             d->wakeUp();
     }
 }
@@ -285,15 +357,16 @@ void QMutex::unlock()
     \ingroup thread
     \ingroup environment
 
-    The purpose of QMutexLocker is to simplify QMutex locking and
-    unlocking. Locking and unlocking a QMutex in complex functions and
+    Locking and unlocking a QMutex in complex functions and
     statements or in exception handling code is error-prone and
     difficult to debug. QMutexLocker can be used in such situations
     to ensure that the state of the mutex is always well-defined.
 
-    QMutexLocker should be created within a function where a QMutex
-    needs to be locked. The mutex is locked when QMutexLocker is
-    created, and unlocked when QMutexLocker is destroyed.
+    QMutexLocker should be created within a function where a
+    QMutex needs to be locked. The mutex is locked when QMutexLocker
+    is created. You can unlock and relock the mutex with \c unlock()
+    and \c relock(). If locked, the mutex will be unlocked when the
+    QMutexLocker is destroyed.
 
     For example, this complex function locks a QMutex upon entering
     the function and unlocks the mutex at all the exit points:
@@ -377,7 +450,7 @@ void QMutex::unlock()
     has locked the mutex has no way of unlocking the mutex before the
     exception is passed up the stack to the calling function.
 
-    QMutexLocker also provides a mutex() member function that returns
+    QMutexLocker also provides a \c mutex() member function that returns
     the mutex on which the QMutexLocker is operating. This is useful
     for code that needs access to the mutex, such as
     QWaitCondition::wait(). For example:
@@ -436,7 +509,8 @@ void QMutex::unlock()
 /*!
     \fn void QMutexLocker::unlock()
 
-    Unlocks this mutex locker.
+    Unlocks this mutex locker. You can use \c relock() to lock
+    it again. It does not need to be locked when destroyed.
 
     \sa relock()
 */
