@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -60,7 +75,7 @@ struct QWaitConditionPrivate {
     QWaitCondition allows a thread to tell other threads that some
     sort of condition has been met. One or many threads can block
     waiting for a QWaitCondition to set a condition with wakeOne() or
-    wakeAll(). Use wakeOne() to wake one randomly selected event or
+    wakeAll(). Use wakeOne() to wake one randomly selected condition or
     wakeAll() to wake them all.
 
     For example, let's suppose that we have three tasks that should
@@ -126,8 +141,8 @@ struct QWaitConditionPrivate {
                 sleep(1);
                 mutex.lock();
             }
-            mutex.unlock();
             keyPressed.wakeAll();
+            mutex.unlock();          
         }
     \endcode
 
@@ -141,7 +156,7 @@ struct QWaitConditionPrivate {
     controlling access to a circular buffer shared by a producer
     thread and a consumer thread.
 
-    \sa QMutex, QSemaphore, QThread
+    \sa QMutex, QSemaphore, QThread, {Wait Conditions Example}
 */
 
 /*!
@@ -201,7 +216,7 @@ void QWaitCondition::wakeAll()
 }
 
 /*!
-    Releases the locked \a mutex and wait on the thread event object.
+    Releases the locked \a mutex and wait on the wait condition.
     The \a mutex must be initially locked by the calling thread. If
     \a mutex is not in a locked state, this function returns
     immediately. If \a mutex is a recursive mutex, this function
