@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -47,6 +62,16 @@ Q_CORE_EXPORT char *qstrdup(const char *);
 inline uint qstrlen(const char *str)
 { return str ? uint(strlen(str)) : 0; }
 
+inline uint qstrnlen(const char *str, uint maxlen)
+{
+    uint length = 0;
+    if (str) {
+        while (length < maxlen && *str++)
+            length++;
+    }
+    return length;
+}
+
 Q_CORE_EXPORT char *qstrcpy(char *dst, const char *src);
 Q_CORE_EXPORT char *qstrncpy(char *dst, const char *src, uint len);
 
@@ -194,23 +219,23 @@ public:
     QList<QByteArray> split(char sep) const;
 
 #ifndef QT_NO_CAST_TO_ASCII
-    QByteArray &append(const QString &s);
-    QByteArray &insert(int i, const QString &s);
-    QByteArray &replace(const QString &before, const char *after);
-    QByteArray &replace(char c, const QString &after);
-    QByteArray &replace(const QString &before, const QByteArray &after);
+    QT_ASCII_CAST_WARN QByteArray &append(const QString &s);
+    QT_ASCII_CAST_WARN QByteArray &insert(int i, const QString &s);
+    QT_ASCII_CAST_WARN QByteArray &replace(const QString &before, const char *after);
+    QT_ASCII_CAST_WARN QByteArray &replace(char c, const QString &after);
+    QT_ASCII_CAST_WARN QByteArray &replace(const QString &before, const QByteArray &after);
 
-    QByteArray &operator+=(const QString &s);
-    int indexOf(const QString &s, int from = 0) const;
-    int lastIndexOf(const QString &s, int from = -1) const;
+    QT_ASCII_CAST_WARN QByteArray &operator+=(const QString &s);
+    QT_ASCII_CAST_WARN int indexOf(const QString &s, int from = 0) const;
+    QT_ASCII_CAST_WARN int lastIndexOf(const QString &s, int from = -1) const;
 #endif
 #ifndef QT_NO_CAST_FROM_ASCII
-    inline bool operator==(const QString &s2) const;
-    inline bool operator!=(const QString &s2) const;
-    inline bool operator<(const QString &s2) const;
-    inline bool operator>(const QString &s2) const;
-    inline bool operator<=(const QString &s2) const;
-    inline bool operator>=(const QString &s2) const;
+    inline QT_ASCII_CAST_WARN bool operator==(const QString &s2) const;
+    inline QT_ASCII_CAST_WARN bool operator!=(const QString &s2) const;
+    inline QT_ASCII_CAST_WARN bool operator<(const QString &s2) const;
+    inline QT_ASCII_CAST_WARN bool operator>(const QString &s2) const;
+    inline QT_ASCII_CAST_WARN bool operator<=(const QString &s2) const;
+    inline QT_ASCII_CAST_WARN bool operator>=(const QString &s2) const;
 #endif
 
     short toShort(bool *ok = 0, int base = 10) const;
@@ -224,6 +249,7 @@ public:
     float toFloat(bool *ok = 0) const;
     double toDouble(bool *ok = 0) const;
     QByteArray toBase64() const;
+    QByteArray toHex() const;
 
     QByteArray &setNum(short, int base = 10);
     QByteArray &setNum(ushort, int base = 10);
@@ -241,6 +267,7 @@ public:
     static QByteArray number(double, char f = 'g', int prec = 6);
     static QByteArray fromRawData(const char *, int size);
     static QByteArray fromBase64(const QByteArray &base64);
+    static QByteArray fromHex(const QByteArray &hexEncoded);
 
     typedef char *iterator;
     typedef const char *const_iterator;
@@ -309,6 +336,9 @@ private:
 
     friend class QByteRef;
     friend class QString;
+public:
+    typedef Data * DataPtr;
+    inline DataPtr &data_ptr() { return d; }
 };
 
 inline QByteArray::QByteArray(): d(&shared_null) { d->ref.ref(); }
