@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,21 +41,24 @@
 
 #include <QtCore/qhash.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Core)
 
 template <class Key, class T>
 class QCache
 {
     struct Node {
-        inline Node() {}
-        inline Node(const Key &key, T *data, int cost)
-            : k(key), t(data), c(cost), p(0), n(0) {}
-        Key k; T *t; int c; Node *p,*n;
+        inline Node() : keyPtr(0) {}
+        inline Node(T *data, int cost)
+            : keyPtr(0), t(data), c(cost), p(0), n(0) {}
+        const Key *keyPtr; T *t; int c; Node *p,*n;
     };
     Node *f, *l;
     QHash<Key, Node> hash;
-    QHash<int, Node> serial;
+    void *unused;
     int mx, total;
+
     inline void unlink(Node &n) {
         if (n.p) n.p->n = n.n;
         if (n.n) n.n->p = n.p;
@@ -48,10 +66,14 @@ class QCache
         if (f == &n) f = n.n;
         total -= n.c;
         delete n.t;
-        hash.remove(Key(n.k));
+        hash.remove(*n.keyPtr);
     }
-    inline T *&relink(const Key &key) {
-        Node &n = hash[key];
+    inline T *relink(const Key &key) {
+        typename QHash<Key, Node>::iterator i = hash.find(key);
+        if (typename QHash<Key, Node>::const_iterator(i) == hash.constEnd())
+            return 0;
+
+        Node &n = *i;
         if (f != &n) {
             if (n.p) n.p->n = n.n;
             if (n.n) n.n->p = n.p;
@@ -117,8 +139,7 @@ inline void QCache<Key,T>::setMaxCost(int m)
 
 template <class Key, class T>
 inline T *QCache<Key,T>::object(const Key &key) const
-{ if (!hash.contains(key)) return 0;
- return const_cast<QCache<Key,T>*>(this)->relink(key); }
+{ return const_cast<QCache<Key,T>*>(this)->relink(key); }
 
 template <class Key, class T>
 inline T *QCache<Key,T>::operator[](const Key &key) const
@@ -126,12 +147,29 @@ inline T *QCache<Key,T>::operator[](const Key &key) const
 
 template <class Key, class T>
 inline bool QCache<Key,T>::remove(const Key &key)
-{ if (hash.contains(key)) { unlink(hash[key]); return true; } return false; }
+{
+    typename QHash<Key, Node>::iterator i = hash.find(key);
+    if (typename QHash<Key, Node>::const_iterator(i) == hash.constEnd()) {
+        return false;
+    } else {
+        unlink(*i);
+        return true;
+    }
+}
 
 template <class Key, class T>
 inline T *QCache<Key,T>::take(const Key &key)
-{ if (!hash.contains(key)) return 0;
- Node &n = hash[key]; T *t = n.t; n.t = 0; unlink(n); return t; }
+{
+    typename QHash<Key, Node>::iterator i = hash.find(key);
+    if (i == hash.end())
+        return 0;
+
+    Node &n = *i;
+    T *t = n.t;
+    n.t = 0;
+    unlink(n);
+    return t;
+}
 
 template <class Key, class T>
 bool QCache<Key,T>::insert(const Key &akey, T *aobject, int acost)
@@ -142,10 +180,11 @@ bool QCache<Key,T>::insert(const Key &akey, T *aobject, int acost)
         return false;
     }
     trim(mx - acost);
-    Node sn(akey, aobject, acost);
-    hash.insert(akey, sn);
+    Node sn(aobject, acost);
+    typename QHash<Key, Node>::iterator i = hash.insert(akey, sn);
     total += acost;
-    Node *n = &hash[akey];
+    Node *n = &i.value();
+    n->keyPtr = &i.key();
     if (f) f->p = n;
     n->n = f;
     f = n;
@@ -165,4 +204,6 @@ void QCache<Key,T>::trim(int m)
     }
 }
 
+QT_END_HEADER
+
 #endif // QCACHE_H
