@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -168,11 +183,11 @@ public:
     SectionNode sectionNode(int index) const;
     Section sectionType(int index) const;
     QString sectionText(const QString &text, int sectionIndex, int index) const;
-    int getDigit(const QVariant &dt, Section s) const;
-    void setDigit(QVariant &t, Section s, int newval) const;
-    int parseSection(int sectionIndex, QString &txt, int index,
+    int getDigit(const QVariant &dt, int index) const;
+    void setDigit(QVariant &t, int index, int newval) const;
+    int parseSection(const QVariant &currentValue, int sectionIndex, QString &txt, int index,
                      QDateTimeParser::State &state, int *used = 0) const;
-    int absoluteMax(int index) const;
+    int absoluteMax(int index, const QDateTime &value = QDateTime()) const;
     int absoluteMin(int index) const;
     bool parseFormat(const QString &format);
 #ifndef QT_NO_DATESTRING
@@ -200,17 +215,16 @@ public:
     enum FieldInfoFlag {
         Numeric = 0x01,
         FixedWidth = 0x02,
-        AllowPartial = 0x04
+        AllowPartial = 0x04,
+        Fraction = 0x08
     };
     Q_DECLARE_FLAGS(FieldInfo, FieldInfoFlag)
 
     FieldInfo fieldInfo(int index) const;
 
-#ifndef QT_NO_DATESTRING
     virtual QVariant getMinimum() const;
     virtual QVariant getMaximum() const;
     virtual int cursorPosition() const { return -1; }
-#endif
     virtual QString displayText() const { return text; }
     virtual QString getAmPmText(AmPm ap, Case cs) const;
 
