@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,6 +50,8 @@
 
 #include <new>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Core)
 
 template <typename T> class QVector;
@@ -49,7 +66,8 @@ struct Q_CORE_EXPORT QListData {
     };
     enum { DataHeaderSize = sizeof(Data) - sizeof(void *) };
 
-    Data *detach();
+    Data *detach(); // remove in 5.0
+    Data *detach2();
     void realloc(int alloc);
     static Data shared_null;
     Data *d;
@@ -85,11 +103,11 @@ class QList
 
 public:
     inline QList() : d(&QListData::shared_null) { d->ref.ref(); }
-    inline QList(const QList &l) : d(l.d) { d->ref.ref(); if (!d->sharable) detach_helper(); }
+    inline QList(const QList<T> &l) : d(l.d) { d->ref.ref(); if (!d->sharable) detach_helper(); }
     ~QList();
-    QList &operator=(const QList &l);
-    bool operator==(const QList &l) const;
-    inline bool operator!=(const QList &l) const { return !(*this == l); }
+    QList<T> &operator=(const QList<T> &l);
+    bool operator==(const QList<T> &l) const;
+    inline bool operator!=(const QList<T> &l) const { return !(*this == l); }
 
     inline int size() const { return p.size(); }
 
@@ -144,6 +162,7 @@ public:
         inline bool operator<=(const iterator& other) const { return i <= other.i; }
         inline bool operator>(const iterator& other) const { return i > other.i; }
         inline bool operator>=(const iterator& other) const { return i >= other.i; }
+#ifndef QT_STRICT_ITERATORS
         inline bool operator==(const const_iterator &o) const
             { return i == reinterpret_cast<const iterator &>(o).i; }
         inline bool operator!=(const const_iterator &o) const
@@ -156,6 +175,7 @@ public:
             { return i > reinterpret_cast<const iterator &>(other).i; }
         inline bool operator>=(const const_iterator& other) const
             { return i >= reinterpret_cast<const iterator &>(other).i; }
+#endif
         inline iterator &operator++() { ++i; return *this; }
         inline iterator operator++(int) { Node *n = i; ++i; return n; }
         inline iterator &operator--() { i--; return *this; }
@@ -180,7 +200,11 @@ public:
         inline const_iterator() : i(0) {}
         inline const_iterator(Node *n) : i(n) {}
         inline const_iterator(const const_iterator &o): i(o.i) {}
+#ifdef QT_STRICT_ITERATORS
+        inline explicit const_iterator(const iterator &o): i(o.i) {}
+#else
         inline const_iterator(const iterator &o): i(o.i) {}
+#endif
         inline const T &operator*() const { return i->t(); }
         inline const T *operator->() const { return &i->t(); }
         inline const T &operator[](int j) const { return i[j].t(); }
@@ -195,7 +219,7 @@ public:
         inline const_iterator &operator--() { i--; return *this; }
         inline const_iterator operator--(int) { Node *n = i; i--; return n; }
         inline const_iterator &operator+=(int j) { i+=j; return *this; }
-        inline const_iterator &operator-=(int j) { i+=j; return *this; }
+        inline const_iterator &operator-=(int j) { i-=j; return *this; }
         inline const_iterator operator+(int j) const { return const_iterator(i+j); }
         inline const_iterator operator-(int j) const { return const_iterator(i-j); }
         inline int operator-(const_iterator j) const { return i - j.i; }
@@ -244,6 +268,7 @@ public:
     typedef const value_type *const_pointer;
     typedef value_type &reference;
     typedef const value_type &const_reference;
+    typedef ptrdiff_t difference_type;
 
 #ifdef QT3_SUPPORT
     inline QT3_SUPPORT iterator remove(iterator pos) { return erase(pos); }
@@ -260,14 +285,14 @@ public:
 #endif
 
     // comfort
-    QList &operator+=(const QList &l);
-    inline QList operator+(const QList &l) const
+    QList<T> &operator+=(const QList<T> &l);
+    inline QList<T> operator+(const QList<T> &l) const
     { QList n = *this; n += l; return n; }
-    inline QList &operator+=(const T &t)
+    inline QList<T> &operator+=(const T &t)
     { append(t); return *this; }
-    inline QList &operator<< (const T &t)
+    inline QList<T> &operator<< (const T &t)
     { append(t); return *this; }
-    inline QList &operator<<(const QList &l)
+    inline QList<T> &operator<<(const QList<T> &l)
     { *this += l; return *this; }
 
     QVector<T> toVector() const;
@@ -385,7 +410,7 @@ inline T QList<T>::takeLast()
 { T t = last(); removeLast(); return t; }
 
 template <typename T>
-void QList<T>::append(const T &t)
+Q_OUTOFLINE_TEMPLATE void QList<T>::append(const T &t)
 {
     detach();
     if (QTypeInfo<T>::isLarge || QTypeInfo<T>::isStatic) {
@@ -476,6 +501,7 @@ Q_OUTOFLINE_TEMPLATE T QList<T>::value(int i) const
     }
     return reinterpret_cast<Node *>(p.at(i))->t();
 }
+
 template<typename T>
 Q_OUTOFLINE_TEMPLATE T QList<T>::value(int i, const T& defaultValue) const
 {
@@ -486,10 +512,10 @@ template <typename T>
 Q_OUTOFLINE_TEMPLATE void QList<T>::detach_helper()
 {
     Node *n = reinterpret_cast<Node *>(p.begin());
-    QListData::Data *x = p.detach();
-    if (x)
-        free(x);
+    QListData::Data *x = p.detach2();
     node_copy(reinterpret_cast<Node *>(p.begin()), reinterpret_cast<Node *>(p.end()), n);
+    if (!x->ref.deref())
+        free(x);
 }
 
 template <typename T>
@@ -635,4 +661,6 @@ Q_OUTOFLINE_TEMPLATE int QList<T>::count(const T &t) const
 Q_DECLARE_SEQUENTIAL_ITERATOR(List)
 Q_DECLARE_MUTABLE_SEQUENTIAL_ITERATOR(List)
 
+QT_END_HEADER
+
 #endif // QLIST_H
