@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,6 +40,8 @@
 #include "qdatastream.h"
 #include "qdebug.h"
 
+#include <math.h>
+
 /*!
     \class QRect
     \ingroup multimedia
@@ -164,7 +181,7 @@
     bottom(). Another solution is to use QRectF: The QRectF class
     defines a rectangle in the plane using floating point accuracy for
     coordinates, and the QRectF::right() and QRectF::bottom()
-    functions \e do return the true bottom-right corner.
+    functions \e do return the right and bottom coordinates.
 
     It is also possible to add offsets to this rectangle's coordinates
     using the adjust() function, as well as retrieve a new rectangle
@@ -1310,6 +1327,15 @@ QDebug operator<<(QDebug dbg, const QRect &r) {
     \sa setTopLeft(), setSize()
 */
 
+/*!
+    \fn QRectF::QRectF(const QPointF &topLeft, const QPointF &bottomRight)
+    \since 4.3
+
+    Constructs a rectangle with the given \a topLeft and \a bottomRight corners.
+
+    \sa setTopLeft(), setBottomRight()
+*/
+
 /*!
     \fn QRectF::QRectF(qreal x, qreal y, qreal width, qreal height)
 
@@ -1999,8 +2025,8 @@ QRectF QRectF::operator&(const QRectF &r) const
     \fn bool QRectF::intersects(const QRectF &rectangle) const
 
     Returns true if this rectangle intersects with the given \a
-    rectangle (i.e., there is at least one pixel that is within both
-    rectangles), otherwise returns false.
+    rectangle (i.e. there is a non-empty area of overlap between
+    them), otherwise returns false.
 
     The intersection rectangle can be retrieved using the intersected()
     function.
@@ -2014,8 +2040,8 @@ bool QRectF::intersects(const QRectF &r) const
         return false;
     QRectF r1 = normalized();
     QRectF r2 = r.normalized();
-    return qMax(r1.xp, r2.xp) <= qMin(r1.xp + r1.w, r2.xp + r2.w)
-        && qMax(r1.yp, r2.yp) <= qMin(r1.yp + r1.h, r2.yp + r2.h);
+    return qMax(r1.xp, r2.xp) < qMin(r1.xp + r1.w, r2.xp + r2.w)
+        && qMax(r1.yp, r2.yp) < qMin(r1.yp + r1.h, r2.yp + r2.h);
 }
 
 /*!
@@ -2024,9 +2050,29 @@ bool QRectF::intersects(const QRectF &r) const
     Returns a QRect based on the values of this rectangle.  Note that the
     coordinates in the returned rectangle are rounded to the nearest integer.
 
-    \sa QRectF()
+    \sa QRectF(), toAlignedRect()
 */
 
+/*!
+    \fn QRect QRectF::toAlignedRect() const
+    \since 4.3
+
+    Returns a QRect based on the values of this rectangle that is the
+    smallest possible integer rectangle that completely contains this
+    rectangle.
+
+    \sa toRect()
+*/
+
+QRect QRectF::toAlignedRect() const
+{
+    int xmin = int(floor(x()));
+    int xmax = int(ceil(x() + width()));
+    int ymin = int(floor(y()));
+    int ymax = int(ceil(y() + height()));
+    return QRect(xmin, ymin, xmax - xmin, ymax - ymin);
+}
+
 /*!
     \fn void QRectF::moveCenter(const QPointF &position)
 
