@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -67,9 +82,9 @@
     implicitly shared. The procedure is:
 
     \list
-    \i Define the \c Employee class with a single data member variable of
+    \o Define the \c Employee class with a single data member variable of
        type QSharedDataPointer<\c{EmployeeData}>.
-    \i Define an \c EmployeeData class that derives from \l QSharedData
+    \o Define an \c EmployeeData class that derives from \l QSharedData
        and that contains all the variables that you would normally
        put in \c Employee.
     \endlist
@@ -86,41 +101,60 @@
     that modifications to one \c Employee object don't affect other
     \c Employee objects.
 
-    In this example, the \c EmployeeData type is a simple class with a
-    default constructor and a copy constructor provided by C++. If
-    member-per-member copy isn't sufficient for your own data type,
-    you must implement your own copy constructor.
+    The \c EmployeeData type is a simple class that inherits QSharedData
+    and that provides a default constructor, a copy constructor,
+    and a destructor. Normally, this is all you need in the "data"
+    class.
+
+    Here's the implementation of the \c EmployeeData members:
+
+    \quotefromfile snippets/sharedemployee/employee.cpp
+    \skipto ::EmployeeData()
+    \printuntil /^\}/
+    \printline ::EmployeeData(const
+    \printuntil /^\}/
+    \printline ::~EmployeeData
+    \printuntil /^\}/
 
     Let's now see how to implement the \c Employee constructors:
 
-    \quotefile snippets/sharedemployee/employee.cpp
-
-    \skipto ::Employee()
+    \printline ::Employee()
     \printuntil }
 
     In the default constructor, we create an object of type
     \c EmployeeData and assign it to the \c d pointer using operator=().
 
-    \skipto ::Employee(int
+    Behind the scenes, QSharedDataPointer automatically increments or
+    decrements the reference count of the shared data object pointed
+    to by \c d, and deletes shared objects when the reference count
+    reaches 0.
+
+    \printline ::Employee(int
     \printuntil }
 
     In the constructor that takes an ID and an employee's name, we
     also create an object of type \c EmployeeData and assign it to the
     \c d pointer.
 
-    In this example, we don't need to provide a copy constructor, an
-    assignment operator, or a destructor for \c Employee. The default
-    implementations provided by C++, which invoke QSharedDataPointer's
-    copy constructor, assignment operator, or destructor, are
-    sufficient. And this is true in general, i.e. for any QSharedData
-    subclass which only stores values (or implicitly shared classes),
-    such as int, double, QString, QStringList, QList\<QWidget*\>, and
-    so on.
+    \printline ::setName
+    \printuntil /^\}/
 
-    Behind the scenes, QSharedDataPointer automatically increments or
-    decrements the reference count of the shared data object pointed
-    to by \c d, and deletes shared objects when the reference count
-    reaches 0.
+    When we use the \c d pointer from a non-const function, detach()
+    is automatically called to ensure that we work on our own copy
+    of the data.
+
+    \printline ::name
+    \printuntil /^\}/
+
+    When we use the \c d pointer in a const function, detach() is \e not
+    called.
+
+    Notice that there is no need to implement a copy constructor or
+    assignment operator in the \c Employee class. This is because the
+    C++ compiler provides default implementations that simply perform
+    member-by-member copy. Here, the only member is \c d, and its
+    operator=() simply increments a reference count (which is precisely
+    what we want).
 
     \sa QSharedData
 */
@@ -232,7 +266,7 @@
     count.
 */
 
-/*! \fn QSharedDataPointer::QSharedDataPointer(const QSharedDataPointer &other)
+/*! \fn QSharedDataPointer::QSharedDataPointer(const QSharedDataPointer<T> &other)
 
     Constructs a copy of \a other.
 
@@ -240,7 +274,7 @@
     shared data object pointed to by \a{other}.
 */
 
-/*! \fn QSharedDataPointer &QSharedDataPointer::operator=(const QSharedDataPointer &other)
+/*! \fn QSharedDataPointer<T> &QSharedDataPointer::operator=(const QSharedDataPointer<T> &other)
 
     Assigns \a other to this pointer.
 
