@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -52,10 +67,10 @@ class Q_GUI_EXPORT QProgressDialog : public QDialog
     Q_PROPERTY(QString labelText READ labelText WRITE setLabelText)
 
 public:
-    explicit QProgressDialog(QWidget *parent = 0, Qt::WFlags f = 0);
+    explicit QProgressDialog(QWidget *parent = 0, Qt::WindowFlags f = 0);
     QProgressDialog(const QString &labelText, const QString &cancelButtonText,
                     int minimum, int maximum,
-                    QWidget *parent = 0, Qt::WFlags f = 0);
+                    QWidget *parent = 0, Qt::WindowFlags f = 0);
     ~QProgressDialog();
 
     void setLabel(QLabel *label);
@@ -67,7 +82,6 @@ public:
     int minimum() const;
     int maximum() const;
 
-    void setRange(int minimum, int maximum);
     int value() const;
 
     QSize sizeHint() const;
@@ -85,6 +99,7 @@ public Q_SLOTS:
     void reset();
     void setMaximum(int maximum);
     void setMinimum(int minimum);
+    void setRange(int minimum, int maximum);
     void setValue(int progress);
     void setLabelText(const QString &);
     void setCancelButtonText(const QString &);
