@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,8 +39,6 @@
 #ifndef QGRAPHICSITEM_P_H
 #define QGRAPHICSITEM_P_H
 
-#include <QtGui/qmatrix.h>
-
 //
 //  W A R N I N G
 //  -------------
@@ -41,31 +54,40 @@
 
 #if !defined(QT_NO_GRAPHICSVIEW) || (QT_EDITION & QT_MODULE_GRAPHICSVIEW) != QT_MODULE_GRAPHICSVIEW
 
-class QGraphicsItemPrivate
+class Q_AUTOTEST_EXPORT QGraphicsItemPrivate
 {
     Q_DECLARE_PUBLIC(QGraphicsItem)
 public:
     enum Extra {
-        ExtraMatrix,
+        ExtraTransform,
         ExtraToolTip,
         ExtraCursor
     };
 
+    enum AncestorFlag {
+        NoFlag = 0,
+        AncestorHandlesChildEvents = 0x1,
+        AncestorClipsChildren = 0x2,
+        AncestorIgnoresTransformations = 0x4
+    };
+
     inline QGraphicsItemPrivate()
         : z(0), scene(0), parent(0), index(-1), q_ptr(0)
     {
         acceptedMouseButtons = 0x1f;
         visible = 1;
+        explicitlyHidden = 0;
         enabled = 1;
+        explicitlyDisabled = 0;
         selected = 0;
         acceptsHover = 0;
         acceptDrops = 0;
         isMemberOfGroup = 0;
         handlesChildEvents = 0;
-        ancestorHandlesChildEvents = 0;
         itemDiscovered = 0;
-        hasMatrix = 0;
+        hasTransform = 0;
         hasCursor = 0;
+        ancestorFlags = 0;
         flags = 0;
         pad = 0;
     }
@@ -73,10 +95,16 @@ public:
     inline virtual ~QGraphicsItemPrivate()
     { }
 
-    void setAncestorHandlesChildEvents(bool enabled);
+    void updateAncestorFlag(QGraphicsItem::GraphicsItemFlag childFlag,
+                            AncestorFlag flag = NoFlag, bool enabled = false, bool root = true);
     void setIsMemberOfGroup(bool enabled);
     void remapItemPos(QEvent *event, QGraphicsItem *item);
-    
+    QPointF genericMapFromScene(const QPointF &pos, const QWidget *viewport) const;
+    bool itemIsUntransformable() const;
+
+    void setVisibleHelper(bool newVisible, bool explicitly, bool update = true);
+    void setEnabledHelper(bool newEnabled, bool explicitly, bool update = true);
+
     inline QVariant extra(Extra type) const
     {
         for (int i = 0; i < extras.size(); ++i) {
@@ -135,18 +163,20 @@ public:
     int index;
     quint32 acceptedMouseButtons : 5;
     quint32 visible : 1;
+    quint32 explicitlyHidden : 1;
     quint32 enabled : 1;
+    quint32 explicitlyDisabled : 1;
     quint32 selected : 1;
     quint32 acceptsHover : 1;
     quint32 acceptDrops : 1;
     quint32 isMemberOfGroup : 1;
     quint32 handlesChildEvents : 1;
-    quint32 ancestorHandlesChildEvents : 1;
     quint32 itemDiscovered : 1;
-    quint32 hasMatrix : 1;
+    quint32 hasTransform : 1;
     quint32 hasCursor : 1;
+    quint32 ancestorFlags : 3;
     quint32 flags : 11;
-    quint32 pad : 5;
+    quint32 pad : 1;
 
     QGraphicsItem *q_ptr;
 };
