@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -69,16 +84,19 @@
 
 extern Q_GUI_EXPORT qint64 qt_pixmap_id(const QPixmap &pixmap);
 
-static int serialNumCounter = 0;
+QBasicAtomic serialNumCounter = Q_ATOMIC_INIT(1);
 
 class QIconPrivate
 {
 public:
-    QIconPrivate():ref(1),engine(0),serialNum(++serialNumCounter){}
+    QIconPrivate():ref(1), engine(0), serialNum(serialNumCounter.fetchAndAdd(1)), detach_no(0), engine_version(2) {}
+
     ~QIconPrivate() { delete engine; }
     QAtomic ref;
     QIconEngine *engine;
     int serialNum;
+    int detach_no;
+    int engine_version;
 };
 
 
@@ -97,9 +115,10 @@ struct QPixmapIconEngineEntry
     bool isNull() const {return (fileName.isEmpty() && pixmap.isNull()); }
 };
 
-class QPixmapIconEngine : public QIconEngine {
+class QPixmapIconEngine : public QIconEngineV2 {
 public:
     QPixmapIconEngine();
+    QPixmapIconEngine(const QPixmapIconEngine &);
     ~QPixmapIconEngine();
     void paint(QPainter *painter, const QRect &rect, QIcon::Mode mode, QIcon::State state);
     QPixmap pixmap(const QSize &size, QIcon::Mode mode, QIcon::State state);
@@ -107,6 +126,13 @@ public:
     QSize actualSize(const QSize &size, QIcon::Mode mode, QIcon::State state);
     void addPixmap(const QPixmap &pixmap, QIcon::Mode mode, QIcon::State state);
     void addFile(const QString &fileName, const QSize &size, QIcon::Mode mode, QIcon::State state);
+
+    // v2 functions
+    QString key() const;
+    QIconEngineV2 *clone() const;
+    bool read(QDataStream &in);
+    bool write(QDataStream &out) const;
+
 private:
     QPixmapIconEngineEntry *tryMatch(const QSize &size, QIcon::Mode mode, QIcon::State state);
     QVector<QPixmapIconEngineEntry> pixmaps;
@@ -118,6 +144,11 @@ QPixmapIconEngine::QPixmapIconEngine()
 {
 }
 
+QPixmapIconEngine::QPixmapIconEngine(const QPixmapIconEngine &other)
+    : QIconEngineV2(other), pixmaps(other.pixmaps)
+{
+}
+
 QPixmapIconEngine::~QPixmapIconEngine()
 {
 }
@@ -298,11 +329,66 @@ void QPixmapIconEngine::addFile(const QString &fileName, const QSize &size, QIco
     }
 }
 
+QString QPixmapIconEngine::key() const
+{
+    return QLatin1String("QPixmapIconEngine");
+}
 
+QIconEngineV2 *QPixmapIconEngine::clone() const
+{
+    return new QPixmapIconEngine(*this);
+}
+
+bool QPixmapIconEngine::read(QDataStream &in)
+{
+    int num_entries;
+    QPixmap pm;
+    QString fileName;
+    QSize sz;
+    uint mode;
+    uint state;
+
+    in >> num_entries;
+    for (int i=0; i < num_entries; ++i) {
+        if (in.atEnd()) {
+            pixmaps.clear();
+            return false;
+        }
+        in >> pm;
+        in >> fileName;
+        in >> sz;
+        in >> mode;
+        in >> state;
+        if (pm.isNull())
+            addFile(fileName, sz, QIcon::Mode(mode), QIcon::State(state));
+        else
+            addPixmap(pm, QIcon::Mode(mode), QIcon::State(state));
+    }
+    return true;
+}
+
+bool QPixmapIconEngine::write(QDataStream &out) const
+{
+    int num_entries = pixmaps.size();
+    out << num_entries;
+    for (int i=0; i < num_entries; ++i) {
+        if (pixmaps.at(i).pixmap.isNull())
+            out << QPixmap(pixmaps.at(i).fileName);
+        else
+            out << pixmaps.at(i).pixmap;
+        out << pixmaps.at(i).fileName;
+        out << pixmaps.at(i).size;
+        out << (uint) pixmaps.at(i).mode;
+        out << (uint) pixmaps.at(i).state;
+    }
+    return true;
+}
 
 #ifndef QT_NO_LIBRARY
 Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
     (QIconEngineFactoryInterface_iid, QCoreApplication::libraryPaths(), QLatin1String("/iconengines"), Qt::CaseInsensitive))
+Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loaderV2,
+    (QIconEngineFactoryInterfaceV2_iid, QCoreApplication::libraryPaths(), QLatin1String("/iconengines"), Qt::CaseInsensitive))
 #endif
 
 
@@ -330,6 +416,12 @@ Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
     button->setIcon(QIcon("open.xpm"));
   \endcode
 
+  To undo a QIcon, simply set a null icon in its place:
+  
+  \code
+   button->setIcon(QIcon());
+  \endcode
+  
   Use the QImageReader::supportedImageFormats() and
   QImageWriter::supportedImageFormats() functions to retrieve a
   complete list of the supported file formats.
@@ -337,11 +429,11 @@ Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
   When you retrieve a pixmap using pixmap(QSize, Mode, State), and no
   pixmap for this given size, mode and state has been added with
   addFile() or addPixmap(), then QIcon will generate one on the
-  fly. This pixmap generation happens in a QIconEngine. The default
+  fly. This pixmap generation happens in a QIconEngineV2. The default
   engine scales pixmaps down if required, but never up, and it uses
   the current style to calculate a disabled appearance. By using
   custom icon engines, you can customize every aspect of generated
-  icons. With QIconEnginePlugin it is possible to register different
+  icons. With QIconEnginePluginV2 it is possible to register different
   icon engines for different file suffixes, so you could provide a SVG
   icon engine or any other scalable format.
 
@@ -430,13 +522,25 @@ QIcon::QIcon(const QString &fileName)
     QFileInfo info(fileName);
     QString suffix = info.suffix();
 #ifndef QT_NO_LIBRARY
-    if (!suffix.isEmpty())
-        if (QIconEngineFactoryInterface *factory = qobject_cast<QIconEngineFactoryInterface*>(loader()->instance(suffix)))
+    if (!suffix.isEmpty()) {
+        // first try version 2 engines..
+        if (QIconEngineFactoryInterfaceV2 *factory = qobject_cast<QIconEngineFactoryInterfaceV2*>(loaderV2()->instance(suffix))) {
             if (QIconEngine *engine = factory->create(fileName)) {
                 d = new QIconPrivate;
                 d->engine = engine;
                 return;
             }
+        }
+        // ..then fall back and try to load version 1 engines
+        if (QIconEngineFactoryInterface *factory = qobject_cast<QIconEngineFactoryInterface*>(loader()->instance(suffix))) {
+            if (QIconEngine *engine = factory->create(fileName)) {
+                d = new QIconPrivate;
+                d->engine = engine;
+                d->engine_version = 1;
+                return;
+            }
+        }
+    }
 #endif
     addFile(fileName);
 }
@@ -449,9 +553,22 @@ QIcon::QIcon(const QString &fileName)
 QIcon::QIcon(QIconEngine *engine)
     :d(new QIconPrivate)
 {
+    d->engine_version = 1;
     d->engine = engine;
 }
 
+/*!
+    Creates an icon with a specific icon \a engine. The icon takes
+    ownership of the engine.
+*/
+QIcon::QIcon(QIconEngineV2 *engine)
+    :d(new QIconPrivate)
+{
+    d->engine_version = 2;
+    d->engine = engine;
+}
+
+
 /*!
     Destroys the icon.
 */
@@ -484,13 +601,16 @@ QIcon::operator QVariant() const
     return QVariant(QVariant::Icon, this);
 }
 
-/*!
+/*! \obsolete
+
     Returns a number that identifies the contents of this
     QIcon object. Distinct QIcon objects can have
     the same serial number if they refer to the same contents
     (but they don't have to). Also, the serial number of
     a QIcon object may change during its lifetime.
 
+    Use cacheKey() instead.
+
     A null icon always has a serial number of 0.
 
     Serial numbers are mostly useful in conjunction with caching.
@@ -503,7 +623,27 @@ int QIcon::serialNumber() const
     return d ? d->serialNum : 0;
 }
 
-/*!  
+/*!
+    Returns a number that identifies the contents of this QIcon
+    object. Distinct QIcon objects can have the same key if
+    they refer to the same contents.
+    \since 4.3
+
+    The cacheKey() will change when the icon is altered via
+    addPixmap() or addFile().
+
+    Cache keys are mostly useful in conjunction with caching.
+
+    \sa QPixmap::cacheKey()
+*/
+qint64 QIcon::cacheKey() const
+{
+    if (!d)
+        return 0;
+    return (((qint64) d->serialNum) << 32) | ((qint64) (d->detach_no));
+}
+
+/*!
   Returns a pixmap with the requested \a size, \a mode, and \a
   state, generating one if necessary. The pixmap might be smaller than
   requested, but never larger.
@@ -592,6 +732,27 @@ bool QIcon::isDetached() const
     return !d || d->ref == 1;
 }
 
+/*! \internal
+ */
+void QIcon::detach()
+{
+    if (d) {
+        if (d->ref != 1) {
+            QIconPrivate *x = new QIconPrivate;
+            if (d->engine_version > 1) {
+                QIconEngineV2 *engine = static_cast<QIconEngineV2 *>(d->engine);
+                x->engine = engine->clone();
+            } else {
+                x->engine = d->engine;
+            }
+            x->engine_version = d->engine_version;
+            x = qAtomicSetPtr(&d, x);
+            if (!x->ref.deref())
+                delete x;
+        }
+        ++d->detach_no;
+    }
+}
 
 /*!
     Adds \a pixmap to the icon, as a specialization for \a mode and
@@ -609,6 +770,8 @@ void QIcon::addPixmap(const QPixmap &pixmap, Mode mode, State state)
     if (!d) {
         d = new QIconPrivate;
         d->engine = new QPixmapIconEngine;
+    } else {
+        detach();
     }
     d->engine->addPixmap(pixmap, mode, state);
 }
@@ -642,6 +805,8 @@ void QIcon::addFile(const QString &fileName, const QSize &size, Mode mode, State
     if (!d) {
         d = new QIconPrivate;
         d->engine = new QPixmapIconEngine;
+    } else {
+        detach();
     }
     d->engine->addFile(fileName, size, mode, state);
 }
@@ -663,7 +828,20 @@ void QIcon::addFile(const QString &fileName, const QSize &size, Mode mode, State
 
 QDataStream &operator<<(QDataStream &s, const QIcon &icon)
 {
-    if (s.version() >= QDataStream::Qt_4_2) {
+    if (s.version() >= QDataStream::Qt_4_3) {
+        if (icon.isNull()) {
+            s << QString();
+        } else {
+            if (icon.d->engine_version > 1) {
+                QIconEngineV2 *engine = static_cast<QIconEngineV2 *>(icon.d->engine);
+                s << engine->key();
+                engine->write(s);
+            } else {
+                // not really supported
+                qWarning("QIcon: Cannot stream QIconEngine. Use QIconEngineV2 instead.");
+            }
+        }
+    } else if (s.version() == QDataStream::Qt_4_2) {
         if (icon.isNull()) {
             s << 0;
         } else {
@@ -695,7 +873,25 @@ QDataStream &operator<<(QDataStream &s, const QIcon &icon)
 
 QDataStream &operator>>(QDataStream &s, QIcon &icon)
 {
-    if (s.version() >= QDataStream::Qt_4_2) {
+    if (s.version() >= QDataStream::Qt_4_3) {
+        icon = QIcon();
+        QString key;
+        s >> key;
+        if (key == QLatin1String("QPixmapIconEngine")) {
+            icon.d = new QIconPrivate;
+            QIconEngineV2 *engine = new QPixmapIconEngine;
+            icon.d->engine = engine;
+            engine->read(s);
+#ifndef QT_NO_LIBRARY
+        } else if (QIconEngineFactoryInterfaceV2 *factory = qobject_cast<QIconEngineFactoryInterfaceV2*>(loaderV2()->instance(key))) {
+            if (QIconEngineV2 *engine= factory->create()) {
+                icon.d = new QIconPrivate;
+                icon.d->engine = engine;
+                engine->read(s);
+            }
+#endif
+        }
+    } else if (s.version() == QDataStream::Qt_4_2) {
         icon = QIcon();
         int num_entries;
         QPixmap pm;
@@ -818,3 +1014,13 @@ QSize QIcon::pixmapSize(Size which)
 */
 
 #endif // QT3_SUPPORT
+
+/*!
+    \fn DataPtr &QIcon::data_ptr()
+    \internal
+*/
+
+/*!
+    \typedef QIcon::DataPtr
+    \internal
+*/
