@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,6 +53,8 @@
 #include <QtGui/qpixmap.h>
 #endif
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 class QByteArray;
@@ -105,9 +122,11 @@ public:
     QSize scaledSize();
     void setScaledSize(const QSize &size);
 
-    CacheMode cacheMode();
+    CacheMode cacheMode() const;
     void setCacheMode(CacheMode mode);
 
+    CacheMode cacheMode(); // ### Qt 5: remove me
+
 Q_SIGNALS:
     void started();
     void resized(const QSize &size);
@@ -126,7 +145,7 @@ public Q_SLOTS:
 
 private:
     Q_DISABLE_COPY(QMovie)
-    Q_PRIVATE_SLOT(d_func(), void loadNextFrame())
+    Q_PRIVATE_SLOT(d_func(), void _q_loadNextFrame())
 
 #ifdef QT3_SUPPORT
 public:
@@ -144,5 +163,8 @@ public:
 #endif
 };
 
+QT_END_HEADER
+
 #endif // QT_NO_MOVIE
+
 #endif // QMOVIE_H
