@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,7 +46,6 @@
 #include "qdatastream.h"
 #include "qbuffer.h"
 #include "qapplication.h"
-#include <private/qinternal_p.h>
 #include <private/qwidget_p.h>
 #include "qevent.h"
 #include "qfile.h"
@@ -41,12 +55,30 @@
 #include "qimagereader.h"
 #include "qimagewriter.h"
 #include "qdebug.h"
+#include "qpaintengine.h"
 
+#if !defined(QT_NO_DIRECT3D) && defined(Q_WS_WIN)
+#include <private/qpaintengine_d3d_p.h>
+#include <d3d9.h>
+extern QDirect3DPaintEngine *qt_d3dEngine();
+#endif
+
+extern int qt_defaultDpi();
+extern int qt_defaultDpiX();
+extern int qt_defaultDpiY();
+
+// ### Qt 5: remove
 typedef void (*_qt_pixmap_cleanup_hook)(int);
 Q_GUI_EXPORT _qt_pixmap_cleanup_hook qt_pixmap_cleanup_hook = 0;
+
+// ### Qt 5: rename
+typedef void (*_qt_pixmap_cleanup_hook_64)(qint64);
+Q_GUI_EXPORT _qt_pixmap_cleanup_hook_64 qt_pixmap_cleanup_hook_64 = 0;
+
+// ### Qt 5: remove
 Q_GUI_EXPORT qint64 qt_pixmap_id(const QPixmap &pixmap)
 {
-    return (((qint64) pixmap.data->image.serialNumber()) << 32) | ((qint64) pixmap.data->detach_no);
+    return pixmap.cacheKey();
 }
 
 QPixmap::QPixmap()
@@ -214,6 +246,10 @@ void QPixmap::fill(const QColor &fillColor)
                           fillColor.green() * alpha / 255,
                           fillColor.blue() * alpha / 255,
                           alpha);
+#ifdef Q_WS_QWS
+        } else if (data->image.depth() == 16) {
+            pixel = qt_convRgbTo16(fillColor.rgba());
+#endif
         } else {
             pixel = fillColor.rgba();
         }
@@ -273,6 +309,11 @@ QBitmap QPixmap::mask() const
 
 void QPixmap::setMask(const QBitmap &mask)
 {
+    if (paintingActive()) {
+        qWarning("QPixmap::setMask: Cannot set mask while pixmap is being painted on");
+        return;
+    }
+
     if (mask.size().isEmpty()) {
         if (depth() != 1) {
             detach();
@@ -334,21 +375,29 @@ QBitmap QPixmap::createHeuristicMask(bool clipTight ) const
 }
 #endif
 
+QBitmap QPixmap::createMaskFromColor(const QColor &maskColor, Qt::MaskMode mode) const
+{
+    QImage image = toImage().convertToFormat(QImage::Format_ARGB32);
+    return QBitmap::fromImage(image.createMaskFromColor(maskColor.rgba(), mode));
+}
+
 QBitmap QPixmap::createMaskFromColor(const QColor &maskColor) const
 {
-    QImage maskImage(size(), QImage::Format_MonoLSB);
-    QImage image = toImage();
-    QRgb mColor = maskColor.rgba();
-    for (int w = 0; w < width(); w++) {
-        for (int h = 0; h < height(); h++) {
-            if (image.pixel(w, h) == mColor)
-                maskImage.setPixel(w, h, Qt::color1);
-            else
-                maskImage.setPixel(w, h, Qt::color0);
-        }
+    return createMaskFromColor(maskColor, Qt::MaskInColor);
+}
+
+
+static void sendResizeEvents(QWidget *target)
+{
+    QResizeEvent e(target->size(), QSize());
+    QApplication::sendEvent(target, &e);
+
+    const QObjectList children = target->children();
+    for (int i = 0; i < children.size(); ++i) {
+        QWidget *child = static_cast<QWidget*>(children.at(i));
+        if (child->isWidgetType() && !child->isWindow() && child->testAttribute(Qt::WA_PendingResizeEvent))
+            sendResizeEvents(child);
     }
-    QBitmap m = fromImage(maskImage);
-    return m;
 }
 
 
@@ -357,6 +406,9 @@ QPixmap QPixmap::grabWidget(QWidget *widget, const QRect &rect)
     if (!widget)
         return QPixmap();
 
+    if (widget->testAttribute(Qt::WA_PendingResizeEvent) || !widget->testAttribute(Qt::WA_WState_Created))
+        sendResizeEvents(widget);
+
     QRect r(rect);
     if (r.width() < 0)
         r.setWidth(widget->width() - rect.x());
@@ -366,9 +418,10 @@ QPixmap QPixmap::grabWidget(QWidget *widget, const QRect &rect)
     if (!r.intersects(widget->rect()))
         return QPixmap();
 
-     QPixmap res(r.size());
+    QPixmap res(r.size());
+    widget->render(&res, -r.topLeft(), r,
+                   QWidget::DrawWindowBackground | QWidget::DrawChildren | QWidget::IgnoreMask);
 
-    widget->d_func()->drawWidget(&res, r, -r.topLeft(), QWidgetPrivate::DrawRecursive | QWidgetPrivate::DrawAsRoot | QWidgetPrivate::DrawPaintOnScreen | QWidgetPrivate::DrawInvisible);
     return res;
 }
 
@@ -394,12 +447,21 @@ QPixmap QPixmap::transformed(const QMatrix &matrix, Qt::TransformationMode mode
     return QPixmap::fromImage(data->image.transformed(matrix, mode));
 }
 
+QPixmap QPixmap::transformed(const QTransform &matrix, Qt::TransformationMode mode ) const
+{
+    return QPixmap::fromImage(data->image.transformed(matrix, mode));
+}
 
 QMatrix QPixmap::trueMatrix(const QMatrix &m, int w, int h)
 {
     return QImage::trueMatrix(m, w, h);
 }
 
+QTransform QPixmap::trueMatrix(const QTransform &m, int w, int h)
+{
+    return QImage::trueMatrix(m, w, h);
+}
+
 QImage QPixmap::toImage() const
 {
     return data->image;
@@ -408,26 +470,24 @@ QImage QPixmap::toImage() const
 QPixmap QPixmap::fromImage(const QImage &image, Qt::ImageConversionFlags flags )
 {
     Q_UNUSED(flags);
-    // ### This will create a temporary image.
-    QPixmap pixmap(image.width(), image.height());
+    QPixmap pixmap;
 
     switch (image.format()) {
     case QImage::Format_Mono:
     case QImage::Format_MonoLSB:
-        pixmap.data->image = image.convertToFormat(QImage::Format_RGB32);
+        pixmap.data->image = image.hasAlphaChannel()
+                             ? image.convertToFormat(QImage::Format_ARGB32_Premultiplied)
+                             : image.convertToFormat(QImage::Format_RGB32);
         break;
     case QImage::Format_RGB32:
     case QImage::Format_ARGB32_Premultiplied:
-#ifdef Q_WS_QWS
     case QImage::Format_RGB16:
-#endif
         pixmap.data->image = image;
         break;
     default:
         pixmap.data->image = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
         break;
     }
-
     return pixmap;
 }
 
@@ -442,7 +502,6 @@ bool QPixmap::load(const QString& fileName, const char *format, Qt::ImageConvers
                QLatin1Char('_')).append(QString::number(info.lastModified().toTime_t())).append(
                QLatin1Char('_')).append(QString::number(data->type));
 
-    detach();
     if (QPixmapCache::find(key, *this))
             return true;
     QImage image = QImageReader(fileName, format).read();
@@ -466,7 +525,6 @@ bool QPixmap::loadFromData(const uchar *buf, uint len, const char* format, Qt::I
     QBuffer b(&a);
     b.open(QIODevice::ReadOnly);
 
-    detach();
     QImage image = QImageReader(&b, format).read();
     if (!image.isNull()) {
         if (data->type == BitmapType)
@@ -498,16 +556,41 @@ int QPixmap::serialNumber() const
     return data->image.serialNumber();
 }
 
+qint64 QPixmap::cacheKey() const
+{
+    // avoid exposing the internal QImageData structure..
+#if defined (Q_CC_MINGW) || (defined (_MSC_VER) && _MSC_VER >= 1310)
+    return ((data->image.cacheKey() & 0xffffffff00000000LL) | ((qint64) data->detach_no));
+#else
+    // MSVC 6.0 can't handle 64 bit constants properly..
+    qint64 mask = 0xffffffff;
+    mask <<= 32;
+    return ((data->image.cacheKey() & mask) | ((qint64) data->detach_no));
+#endif
+}
+
 bool QPixmap::isDetached() const
 {
     return data->count == 1;
 }
 
+typedef void (*_qt_pixmap_cleanup_hook_64)(qint64);
+extern _qt_pixmap_cleanup_hook_64 qt_pixmap_cleanup_hook_64;
+
 void QPixmap::detach()
 {
-    ++data->detach_no;
+#if !defined(QT_NO_DIRECT3D) && defined(Q_WS_WIN)
+    if (data->texture)
+        data->texture->Release();
+    data->texture = 0;
+#endif
+
+    if (qt_pixmap_cleanup_hook_64 && data->count == 1)
+        qt_pixmap_cleanup_hook_64(cacheKey());
+
     if (data->count != 1)
         *this = copy();
+    ++data->detach_no;
 }
 
 #ifdef QT3_SUPPORT
@@ -546,7 +629,42 @@ bool QPixmap::convertFromImage(const QImage &image, ColorMode mode)
 
 int QPixmap::metric(PaintDeviceMetric metric) const
 {
-    return data->image.metric(metric);
+    // override the image dpi with the screen dpi when rendering to a pixmap
+    // ### Qt 4.4: remove #ifdef
+#ifdef Q_WS_QWS
+    int dpmX = qRound(qt_defaultDpiX()*100./2.54);
+    int dpmY = qRound(qt_defaultDpiY()*100./2.54);
+#else
+    int dpmX = qRound(qt_defaultDpi()*100./2.54);
+    int dpmY = dpmX;
+#endif
+    switch (metric) {
+    case PdmWidthMM:
+        return qRound(data->image.width() * 1000 / dpmX);
+        break;
+
+    case PdmHeightMM:
+        return qRound(data->image.height() * 1000 / dpmY);
+        break;
+
+    case PdmDpiX:
+        return qRound(dpmX * 0.0254);
+        break;
+
+    case PdmDpiY:
+        return qRound(dpmY * 0.0254);
+        break;
+
+    case PdmPhysicalDpiX:
+        return qRound(dpmX * 0.0254);
+        break;
+
+    case PdmPhysicalDpiY:
+        return qRound(dpmY * 0.0254);
+        break;
+    default:
+        return data->image.metric(metric);
+    }
 }
 
 bool QPixmap::doImageIO(QImageWriter *writer, int quality) const
@@ -567,16 +685,28 @@ void QPixmap::init(int w, int h, Type type)
     data = new QPixmapData;
     data->type = type;
     data->detach_no = 0;
-    if (type == PixmapType) {
-        data->image = QImage(w, h, QImage::Format_RGB32);
-    } else {
-        data->image = data->createBitmapImage(w, h);
+#if !defined(QT_NO_DIRECT3D) && defined(Q_WS_WIN)
+    data->texture = 0;
+#endif
+    if (w > 0 && h > 0) {
+        if (type == PixmapType) {
+            data->image = QImage(w, h, QImage::Format_RGB32);
+        } else {
+            data->image = data->createBitmapImage(w, h);
+        }
     }
 }
 
 void QPixmap::deref()
 {
     if(data && data->deref()) { // Destroy image if last ref
+#if !defined(QT_NO_DIRECT3D) && defined(Q_WS_WIN)
+        if (data->texture)
+            data->texture->Release();
+        data->texture = 0;
+#endif
+        if (qt_pixmap_cleanup_hook_64)
+            qt_pixmap_cleanup_hook_64(cacheKey());
         delete data;
         data = 0;
     }
@@ -599,12 +729,16 @@ QDataStream &operator<<(QDataStream &s, const QPixmap &pixmap)
 
 QDataStream &operator>>(QDataStream &s, QPixmap &pixmap)
 {
-    QImage img;
-    s >> img;
-    if (pixmap.data->type == QPixmap::BitmapType)
-        pixmap = QBitmap::fromImage(img);
-    else
-        pixmap = QPixmap::fromImage(img);
+    QImage image;
+    s >> image;
+
+    if (image.isNull()) {
+        pixmap = QPixmap();
+    } else if (image.depth() == 1) {
+        pixmap = QBitmap::fromImage(image);
+    } else {
+        pixmap = QPixmap::fromImage(image);
+    }
     return s;
 }
 #endif // QT_NO_DATASTREAM
@@ -616,6 +750,11 @@ QPaintEngine *QPixmap::paintEngine() const
 
 void QPixmap::setAlphaChannel(const QPixmap &alphaChannel)
 {
+    if (paintingActive()) {
+        qWarning("QPixmap::setAlphaChannel: Cannot set alpha channel while pixmap is being painted on");
+        return;
+    }
+
     detach();
     data->image.setAlphaChannel(alphaChannel.toImage());
 }
@@ -643,10 +782,23 @@ Q_GUI_EXPORT void copyBlt(QPixmap *dst, int dx, int dy,
     Q_ASSERT_X(dst, "::copyBlt", "Destination pixmap must be non null");
     Q_ASSERT_X(src, "::copyBlt", "Source pixmap must be non null");
 
-    QImage image = dst->toImage();
-    QPainter p(&image);
-    p.setCompositionMode(QPainter::CompositionMode_Source);
-    p.drawImage(dx, dy, src->toImage(), sx, sy, sw, sh);
-    *dst = QPixmap::fromImage(image);
+    if (src->hasAlphaChannel()) {
+        if (dst->paintEngine()->hasFeature(QPaintEngine::PorterDuff)) {
+            QPainter p(dst);
+            p.setCompositionMode(QPainter::CompositionMode_Source);
+            p.drawPixmap(dx, dy, *src, sx, sy, sw, sh);
+        } else {
+            QImage image = dst->toImage().convertToFormat(QImage::Format_ARGB32_Premultiplied);
+            QPainter p(&image);
+            p.setCompositionMode(QPainter::CompositionMode_Source);
+            p.drawPixmap(dx, dy, *src, sx, sy, sw, sh);
+            p.end();
+            *dst = QPixmap::fromImage(image);
+        }
+    } else {
+        QPainter p(dst);
+        p.drawPixmap(dx, dy, *src, sx, sy, sw, sh);
+    }
+
 }
 #endif
