@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -68,10 +83,11 @@ inline static void qSafeXDestroyImage(XImage *x)
     XDestroyImage(x);
 }
 
-QBitmap QPixmapData::mask_to_bitmap() const
+QBitmap QPixmapData::mask_to_bitmap(int screen) const
 {
     if (!x11_mask)
         return QBitmap();
+    QPixmap::x11SetDefaultScreen(screen);
     QBitmap bm(w, h);
     GC gc = XCreateGC(X11->display, bm.data->hd, 0, 0);
     XCopyArea(X11->display, x11_mask, bm.data->hd, gc, 0, 0, bm.data->w, bm.data->h, 0, 0);
@@ -365,6 +381,8 @@ QPixmapData::~QPixmapData()
     delete paintEngine;
 }
 
+typedef void (*_qt_pixmap_cleanup_hook_64)(qint64);
+extern _qt_pixmap_cleanup_hook_64 qt_pixmap_cleanup_hook_64;
 
 /*!
     Detaches the pixmap from shared pixmap data.
@@ -383,13 +401,14 @@ QPixmapData::~QPixmapData()
     The detach() function returns immediately if there is just a
     single reference or if the pixmap has not been initialized yet.
 */
-
 void QPixmap::detach()
 {
-    ++data->detach_no;
+    if (qt_pixmap_cleanup_hook_64 && data->count == 1)
+        qt_pixmap_cleanup_hook_64(cacheKey());
+
     if (data->count != 1)
         *this = copy();
-
+    ++data->detach_no;
     data->uninit = false;
 
     // reset the cache data
@@ -424,6 +443,7 @@ void QPixmap::fill(const QColor &fillColor)
     if (fillColor.alpha() != 255) {
 #ifndef QT_NO_XRENDER
         if (data->picture && data->d == 32) {
+            detach();
             ::Picture src  = X11->getSolidFill(data->xinfo.screen(), fillColor);
             XRenderComposite(X11->display, PictOpSrc, src, 0, data->picture,
                              0, 0, width(), height(),
@@ -453,15 +473,15 @@ void QPixmap::fill(const QColor &fillColor)
 }
 
 /*!
-    Returns the alpha channel of the pixmap as a new grayscale QPixmap in which 
-    each pixel's red, green, and blue values are given the alpha value of the 
-    original pixmap. The color depth of the returned pixmap is the system depth 
+    Returns the alpha channel of the pixmap as a new grayscale QPixmap in which
+    each pixel's red, green, and blue values are given the alpha value of the
+    original pixmap. The color depth of the returned pixmap is the system depth
     on X11 and 8-bit on Windows and Mac OS X.
 
-    You can use this function while debugging 
-    to get a visible image of the alpha channel. If the pixmap doesn't have an 
+    You can use this function while debugging
+    to get a visible image of the alpha channel. If the pixmap doesn't have an
     alpha channel, i.e., the alpha channel's value for all pixels equals
-    0xff), a null pixmap is returned. You can check this with the \c isNull() 
+    0xff), a null pixmap is returned. You can check this with the \c isNull()
     function.
 
     We show an example:
@@ -535,7 +555,7 @@ QBitmap QPixmap::mask() const
     if (depth() == 1) {
         mask = *this;
     } else {
-        mask = data->mask_to_bitmap();
+        mask = data->mask_to_bitmap(data->xinfo.screen());
     }
     return mask;
 }
@@ -719,7 +739,7 @@ QImage QPixmap::toImage() const
     int            h  = height();
     int            d  = depth();
     Visual *visual = (Visual *) data->xinfo.visual();
-    bool    trucol = (visual->c_class >= TrueColor) && d > 8;
+    bool    trucol = (visual->c_class >= TrueColor) && d > 1;
 
     QImage::Format format = QImage::Format_Mono;
     if (d > 1 && d <= 8) {
@@ -1054,7 +1074,7 @@ QPixmap QPixmap::fromImage(const QImage &img, Qt::ImageConversionFlags flags)
         }
     }
 
-    if (d == 1) {
+    if (d == 1 || d == 16) {
         QImage im = image.convertToFormat(QImage::Format_RGB32, flags);
         return fromImage(im);
     }
@@ -1789,23 +1809,10 @@ QPixmap QPixmap::grabWindow(WId window, int x, int y, int w, int h)
     pm.data->uninit = false;
     pm.x11SetScreen(scr);
 
-#ifndef QT_NO_XRENDER
-    if (pm.data->picture) {
-        XRenderPictFormat *format = XRenderFindVisualFormat(dpy, window_attr.visual);
-        XRenderPictureAttributes pattr;
-        pattr.subwindow_mode = IncludeInferiors;
-        Picture src_pict = XRenderCreatePicture(dpy, window, format, CPSubwindowMode, &pattr);
-        Picture dst_pict = pm.x11PictureHandle();
-        XRenderComposite(dpy, PictOpSrc, src_pict, 0, dst_pict, x, y, x, y, 0, 0, w, h);
-        XRenderFreePicture(dpy, src_pict);
-    } else
-#endif
-        {
-            GC gc = XCreateGC(dpy, pm.handle(), 0, 0);
-            XSetSubwindowMode(dpy, gc, IncludeInferiors);
-            XCopyArea(dpy, window, pm.handle(), gc, x, y, w, h, 0, 0);
-            XFreeGC(dpy, gc);
-        }
+    GC gc = XCreateGC(dpy, pm.handle(), 0, 0);
+    XSetSubwindowMode(dpy, gc, IncludeInferiors);
+    XCopyArea(dpy, window, pm.handle(), gc, x, y, w, h, 0, 0);
+    XFreeGC(dpy, gc);
 
     return pm;
 }
@@ -1828,27 +1835,28 @@ QPixmap QPixmap::grabWindow(WId window, int x, int y, int w, int h)
     \sa trueMatrix(), {QPixmap#Pixmap Transformations}{Pixmap
     Transformations}
 */
-
-QPixmap QPixmap::transformed(const QMatrix &matrix, Qt::TransformationMode mode) const
+QPixmap QPixmap::transformed(const QTransform &matrix, Qt::TransformationMode mode) const
 {
-    uint          w = 0;
-    uint          h = 0;                                // size of target pixmap
-    uint          ws, hs;                                // size of source pixmap
+    uint   w = 0;
+    uint   h = 0;                               // size of target pixmap
+    uint   ws, hs;                              // size of source pixmap
     uchar *dptr;                                // data in target pixmap
-    uint          dbpl, dbytes;                        // bytes per line/bytes total
+    uint   dbpl, dbytes;                        // bytes per line/bytes total
     uchar *sptr;                                // data in original pixmap
-    int           sbpl;                                // bytes per line in original
-    int           bpp;                                        // bits per pixel
+    int    sbpl;                                // bytes per line in original
+    int    bpp;                                 // bits per pixel
     bool   depth1 = depth() == 1;
     Display *dpy = X11->display;
 
-    if (isNull())                                // this is a null pixmap
+    if (isNull())
         return copy();
 
     ws = width();
     hs = height();
 
-    QMatrix mat(matrix.m11(), matrix.m12(), matrix.m21(), matrix.m22(), 0., 0.);
+    QTransform mat(matrix.m11(), matrix.m12(), matrix.m13(),
+                   matrix.m21(), matrix.m22(), matrix.m23(),
+                   0., 0., 1);
     bool complex_xform = false;
     qreal scaledWidth;
     qreal scaledHeight;
@@ -1874,10 +1882,11 @@ QPixmap QPixmap::transformed(const QMatrix &matrix, Qt::TransformationMode mode)
 
 
     bool invertible;
-    mat = mat.inverted(&invertible);                // invert matrix
+    mat = mat.inverted(&invertible);  // invert matrix
 
     if (h == 0 || w == 0 || !invertible
-        || qAbs(scaledWidth) >= 32768 || qAbs(scaledHeight) >= 32768 )	// error, return null pixmap
+        || qAbs(scaledWidth) >= 32768 || qAbs(scaledHeight) >= 32768 )
+	// error, return null pixmap
         return QPixmap();
 
     if (mode == Qt::SmoothTransformation) {
@@ -2013,7 +2022,7 @@ QPixmap QPixmap::transformed(const QMatrix &matrix, Qt::TransformationMode mode)
         XFreeGC(X11->display, gc);
 
         if (data->x11_mask) { // xform mask, too
-            pm.setMask(data->mask_to_bitmap().transformed(matrix));
+            pm.setMask(data->mask_to_bitmap(data->xinfo.screen()).transformed(matrix));
         } else if (data->d != 32 && complex_xform) { // need a mask!
             QBitmap mask(data->w, data->h);
             mask.fill(Qt::color1);
@@ -2023,6 +2032,17 @@ QPixmap QPixmap::transformed(const QMatrix &matrix, Qt::TransformationMode mode)
     }
 }
 
+/*!
+  \overload
+
+  This convenience function loads the \a matrix into a
+  QTransform and calls the overloaded function.
+ */
+QPixmap QPixmap::transformed(const QMatrix &matrix, Qt::TransformationMode mode) const
+{
+    return transformed(QTransform(matrix), mode);
+}
+
 
 /*!
   \internal
@@ -2068,7 +2088,10 @@ void QPixmap::x11SetScreen(int screen)
 
     QImage img = toImage();
     x11SetDefaultScreen(screen);
-    (*this) = fromImage(img);
+    if (img.depth() == 1)
+        (*this) = QBitmap::fromImage(img);
+    else
+        (*this) = fromImage(img);
 }
 
 /*!
