@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,6 +44,8 @@
 #include <QtGui/qitemselectionmodel.h>
 #include <QtGui/qabstractitemdelegate.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_ITEMVIEWS
@@ -41,27 +58,32 @@ class QAbstractItemViewPrivate;
 class Q_GUI_EXPORT QAbstractItemView : public QAbstractScrollArea
 {
     Q_OBJECT
-    Q_ENUMS(SelectionMode SelectionBehavior ScrollHint)
+    Q_ENUMS(SelectionMode SelectionBehavior ScrollHint ScrollMode DragDropMode)
     Q_FLAGS(EditTriggers)
     Q_PROPERTY(bool autoScroll READ hasAutoScroll WRITE setAutoScroll)
     Q_PROPERTY(EditTriggers editTriggers READ editTriggers WRITE setEditTriggers)
     Q_PROPERTY(bool tabKeyNavigation READ tabKeyNavigation WRITE setTabKeyNavigation)
-    Q_PROPERTY(bool showDropIndicator READ showDropIndicator WRITE setDropIndicatorShown)
 #ifndef QT_NO_DRAGANDDROP
+    Q_PROPERTY(bool showDropIndicator READ showDropIndicator WRITE setDropIndicatorShown)
     Q_PROPERTY(bool dragEnabled READ dragEnabled WRITE setDragEnabled)
+    Q_PROPERTY(bool dragDropOverwriteMode READ dragDropOverwriteMode WRITE setDragDropOverwriteMode)
+    Q_PROPERTY(DragDropMode dragDropMode READ dragDropMode WRITE setDragDropMode)
 #endif
     Q_PROPERTY(bool alternatingRowColors READ alternatingRowColors WRITE setAlternatingRowColors)
     Q_PROPERTY(SelectionMode selectionMode READ selectionMode WRITE setSelectionMode)
     Q_PROPERTY(SelectionBehavior selectionBehavior READ selectionBehavior WRITE setSelectionBehavior)
     Q_PROPERTY(QSize iconSize READ iconSize WRITE setIconSize)
     Q_PROPERTY(Qt::TextElideMode textElideMode READ textElideMode WRITE setTextElideMode)
+    Q_PROPERTY(ScrollMode verticalScrollMode READ verticalScrollMode WRITE setVerticalScrollMode)
+    Q_PROPERTY(ScrollMode horizontalScrollMode READ horizontalScrollMode WRITE setHorizontalScrollMode)
 
 public:
     enum SelectionMode {
         NoSelection,
         SingleSelection,
         MultiSelection,
-        ExtendedSelection
+        ExtendedSelection,
+        ContiguousSelection
     };
 
     enum SelectionBehavior {
@@ -73,7 +95,8 @@ public:
     enum ScrollHint {
         EnsureVisible,
         PositionAtTop,
-        PositionAtBottom
+        PositionAtBottom,
+        PositionAtCenter
     };
 
     enum EditTrigger {
@@ -88,6 +111,11 @@ public:
 
     Q_DECLARE_FLAGS(EditTriggers, EditTrigger)
 
+    enum ScrollMode {
+        ScrollPerItem,
+        ScrollPerPixel
+    };
+
     explicit QAbstractItemView(QWidget *parent = 0);
     ~QAbstractItemView();
 
@@ -112,22 +140,42 @@ public:
     void setEditTriggers(EditTriggers triggers);
     EditTriggers editTriggers() const;
 
+    void setVerticalScrollMode(ScrollMode mode);
+    ScrollMode verticalScrollMode() const;
+
+    void setHorizontalScrollMode(ScrollMode mode);
+    ScrollMode horizontalScrollMode() const;
+
     void setAutoScroll(bool enable);
     bool hasAutoScroll() const;
 
     void setTabKeyNavigation(bool enable);
     bool tabKeyNavigation() const;
 
+#ifndef QT_NO_DRAGANDDROP
     void setDropIndicatorShown(bool enable);
     bool showDropIndicator() const;
 
-#ifndef QT_NO_DRAGANDDROP
     void setDragEnabled(bool enable);
     bool dragEnabled() const;
+
+    void setDragDropOverwriteMode(bool overwrite);
+    bool dragDropOverwriteMode() const;
+
+    enum DragDropMode {
+        NoDragDrop,
+        DragOnly,
+        DropOnly,
+        DragDrop,
+        InternalMove
+    };
+
+    void setDragDropMode(DragDropMode behavior);
+    DragDropMode dragDropMode() const;
 #endif
     void setAlternatingRowColors(bool enable);
     bool alternatingRowColors() const;
-    
+
     void setIconSize(const QSize &size);
     QSize iconSize() const;
 
@@ -138,7 +186,7 @@ public:
 
     virtual QRect visualRect(const QModelIndex &index) const = 0;
     virtual void scrollTo(const QModelIndex &index, ScrollHint hint = EnsureVisible) = 0;
-    virtual QModelIndex indexAt(const QPoint &p) const = 0;
+    virtual QModelIndex indexAt(const QPoint &point) const = 0;
 
     QSize sizeHintForIndex(const QModelIndex &index) const;
     virtual int sizeHintForRow(int row) const;
@@ -147,7 +195,26 @@ public:
     void openPersistentEditor(const QModelIndex &index);
     void closePersistentEditor(const QModelIndex &index);
 
-public slots:
+    void setIndexWidget(const QModelIndex &index, QWidget *widget);
+    QWidget *indexWidget(const QModelIndex &index) const;
+
+    void setItemDelegateForRow(int row, QAbstractItemDelegate *delegate);
+    QAbstractItemDelegate *itemDelegateForRow(int row) const;
+
+    void setItemDelegateForColumn(int column, QAbstractItemDelegate *delegate);
+    QAbstractItemDelegate *itemDelegateForColumn(int column) const;
+
+    QAbstractItemDelegate *itemDelegate(const QModelIndex &index) const;
+
+    virtual QVariant inputMethodQuery(Qt::InputMethodQuery query) const;
+
+#ifdef Q_NO_USING_KEYWORD
+    inline void update() { QAbstractScrollArea::update(); }
+#else
+    using QAbstractScrollArea::update;
+#endif
+    
+public Q_SLOTS:
     virtual void reset();
     virtual void setRootIndex(const QModelIndex &index);
     virtual void doItemsLayout();
@@ -155,8 +222,11 @@ public slots:
     void edit(const QModelIndex &index);
     void clearSelection();
     void setCurrentIndex(const QModelIndex &index);
+    void scrollToTop();
+    void scrollToBottom();
+    void update(const QModelIndex &index);
 
-protected slots:
+protected Q_SLOTS:
     virtual void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);
     virtual void rowsInserted(const QModelIndex &parent, int start, int end);
     virtual void rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end);
@@ -173,11 +243,11 @@ protected slots:
     virtual void commitData(QWidget *editor);
     virtual void editorDestroyed(QObject *editor);
 
-signals:
+Q_SIGNALS:
     void pressed(const QModelIndex &index);
     void clicked(const QModelIndex &index);
     void doubleClicked(const QModelIndex &index);
-    
+
     void activated(const QModelIndex &index);
     void entered(const QModelIndex &index);
     void viewportEntered();
@@ -213,7 +283,7 @@ protected:
 #ifndef QT_NO_DRAGANDDROP
     virtual void startDrag(Qt::DropActions supportedActions);
 #endif
-    
+
     virtual QStyleOptionViewItem viewOptions() const;
 
     enum State {
@@ -222,7 +292,8 @@ protected:
         DragSelectingState,
         EditingState,
         ExpandingState,
-        CollapsingState
+        CollapsingState,
+        AnimatingState
     };
 
     State state() const;
@@ -231,19 +302,22 @@ protected:
     void scheduleDelayedItemsLayout();
     void executeDelayedItemsLayout();
 
+    void setDirtyRegion(const QRegion &region);
     void scrollDirtyRegion(int dx, int dy);
     QPoint dirtyRegionOffset() const;
-    
+
     void startAutoScroll();
     void stopAutoScroll();
     void doAutoScroll();
 
+    bool focusNextPrevChild(bool next);
+    bool event(QEvent *event);
     bool viewportEvent(QEvent *event);
     void mousePressEvent(QMouseEvent *event);
     void mouseMoveEvent(QMouseEvent *event);
     void mouseReleaseEvent(QMouseEvent *event);
     void mouseDoubleClickEvent(QMouseEvent *event);
-#ifndef QT_NO_DRAGANDDROP  
+#ifndef QT_NO_DRAGANDDROP
     void dragEnterEvent(QDragEnterEvent *event);
     void dragMoveEvent(QDragMoveEvent *event);
     void dragLeaveEvent(QDragLeaveEvent *event);
@@ -254,12 +328,29 @@ protected:
     void keyPressEvent(QKeyEvent *event);
     void resizeEvent(QResizeEvent *event);
     void timerEvent(QTimerEvent *event);
+    void inputMethodEvent(QInputMethodEvent *event);
+
+#ifndef QT_NO_DRAGANDDROP
+    enum DropIndicatorPosition { OnItem, AboveItem, BelowItem, OnViewport };
+    DropIndicatorPosition dropIndicatorPosition() const;
+#endif
 
 private:
     Q_DECLARE_PRIVATE(QAbstractItemView)
     Q_DISABLE_COPY(QAbstractItemView)
+    Q_PRIVATE_SLOT(d_func(), void _q_columnsAboutToBeRemoved(const QModelIndex&, int, int))
+    Q_PRIVATE_SLOT(d_func(), void _q_columnsRemoved(const QModelIndex&, int, int))
+    Q_PRIVATE_SLOT(d_func(), void _q_rowsRemoved(const QModelIndex&, int, int))
+    Q_PRIVATE_SLOT(d_func(), void _q_modelDestroyed())
+    Q_PRIVATE_SLOT(d_func(), void _q_layoutChanged())
+
+    friend class QTreeViewPrivate; // needed to compile with MSVC
+    friend class QAccessibleItemRow;
 };
+
 Q_DECLARE_OPERATORS_FOR_FLAGS(QAbstractItemView::EditTriggers)
 
 #endif // QT_NO_ITEMVIEWS
+QT_END_HEADER
+
 #endif // QABSTRACTITEMVIEW_H
