@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -92,7 +107,7 @@ void QProxyModel::setModel(QAbstractItemModel *model)
 {
     Q_D(QProxyModel);
     if (d->model && d->model != &d->empty)
-        disconnectFromModel(model);
+        disconnectFromModel(d->model);
     if (model) {
         d->model = model;
         connectToModel(model);
@@ -382,7 +397,7 @@ QModelIndex QProxyModel::setProxyModel(const QModelIndex &source_index) const
 {
     QModelIndex proxy_index = source_index;
     if (proxy_index.isValid())
-        reinterpret_cast<QProxyModelPrivate::QProxyModelIndex*>(&proxy_index)->m = this;
+        proxy_index.m = this;
     return proxy_index;
 }
 
@@ -394,7 +409,7 @@ QModelIndex QProxyModel::setSourceModel(const QModelIndex &proxy_index) const
 {
     Q_D(const QProxyModel);
     QModelIndex source_index = proxy_index;
-    reinterpret_cast<QProxyModelPrivate::QProxyModelIndex*>(&source_index)->m = d->model;
+    source_index.m = d->model;
     return source_index;
 }
 
@@ -404,27 +419,28 @@ QModelIndex QProxyModel::setSourceModel(const QModelIndex &proxy_index) const
 */
 void QProxyModel::connectToModel(const QAbstractItemModel *model) const
 {
-    connect(model, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
-            this, SLOT(_q_sourceDataChanged(const QModelIndex&, const QModelIndex&)));
+    connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+            this, SLOT(_q_sourceDataChanged(QModelIndex,QModelIndex)));
     connect(model, SIGNAL(headerDataChanged(Qt::Orientation,int,int)),
             this, SIGNAL(headerDataChanged(Qt::Orientation,int,int))); // signal to signal
-    connect(model, SIGNAL(rowsAboutToBeInserted(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceRowsAboutToBeInserted(const QModelIndex&,int,int)));
-    connect(model, SIGNAL(rowsInserted(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceRowsInserted(const QModelIndex&,int,int)));
-    connect(model, SIGNAL(rowsAboutToBeRemoved(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceRowsAboutToBeRemoved(const QModelIndex&,int,int)));
-    connect(model, SIGNAL(rowsRemoved(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceRowsRemoved(const QModelIndex&,int,int)));
-    connect(model, SIGNAL(columnsAboutToBeInserted(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceColumnsAboutToBeInserted(const QModelIndex&,int,int)));
-    connect(model, SIGNAL(columnsInserted(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceColumnsInserted(const QModelIndex,int,int)));
-    connect(model, SIGNAL(columnsAboutToBeRemoved(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceColumnsAboutToBeRemoved(const QModelIndex&,int,int)));
-    connect(model, SIGNAL(columnsRemoved(const QModelIndex&,int,int)),
-            this, SLOT(_q_sourceColumnsRemoved(const QModelIndex&,int,int)));
+    connect(model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
+            this, SLOT(_q_sourceRowsAboutToBeInserted(QModelIndex,int,int)));
+    connect(model, SIGNAL(rowsInserted(QModelIndex,int,int)),
+            this, SLOT(_q_sourceRowsInserted(QModelIndex,int,int)));
+    connect(model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
+            this, SLOT(_q_sourceRowsAboutToBeRemoved(QModelIndex,int,int)));
+    connect(model, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+            this, SLOT(_q_sourceRowsRemoved(QModelIndex,int,int)));
+    connect(model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
+            this, SLOT(_q_sourceColumnsAboutToBeInserted(QModelIndex,int,int)));
+    connect(model, SIGNAL(columnsInserted(QModelIndex,int,int)),
+            this, SLOT(_q_sourceColumnsInserted(QModelIndex,int,int)));
+    connect(model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
+            this, SLOT(_q_sourceColumnsAboutToBeRemoved(QModelIndex,int,int)));
+    connect(model, SIGNAL(columnsRemoved(QModelIndex,int,int)),
+            this, SLOT(_q_sourceColumnsRemoved(QModelIndex,int,int)));
     connect(model, SIGNAL(modelReset()), this, SIGNAL(modelReset())); // signal to signal
+    connect(model, SIGNAL(layoutAboutToBeChanged()), this, SIGNAL(layoutAboutToBeChanged())); // signal to signal
     connect(model, SIGNAL(layoutChanged()), this, SIGNAL(layoutChanged())); // signal to signal
 }
 
@@ -434,27 +450,28 @@ void QProxyModel::connectToModel(const QAbstractItemModel *model) const
  */
 void QProxyModel::disconnectFromModel(const QAbstractItemModel *model) const
 {
-    disconnect(model, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
-               this, SLOT(_q_sourceDataChanged(const QModelIndex&, const QModelIndex&)));
+    disconnect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+               this, SLOT(_q_sourceDataChanged(QModelIndex,QModelIndex)));
     disconnect(model, SIGNAL(headerDataChanged(Qt::Orientation,int,int)),
                this, SIGNAL(headerDataChanged(Qt::Orientation,int,int))); // signal to signal
-    disconnect(model, SIGNAL(rowsAboutToBeInserted(const QModelIndex&,int,int)),
-               this, SLOT(_q_sourceRowsAboutToBeInserted(const QModelIndex&,int,int)));
-    disconnect(model, SIGNAL(rowsInserted(const QModelIndex&,int,int)),
-               this, SLOT(rowsInserted(const QModelIndex&,int,int)));
-    disconnect(model, SIGNAL(rowsAboutToBeRemoved(const QModelIndex&,int,int)),
-               this, SLOT(_q_sourceRowsAboutToBeRemoved(const QModelIndex&,int,int)));
-    disconnect(model, SIGNAL(rowsRemoved(const QModelIndex&,int,int)),
-               this, SLOT(_q_sourceRowsRemoved(const QModelIndex&,int,int)));
-    disconnect(model, SIGNAL(columnsAboutToBeInserted(const QModelIndex&,int,int)),
-               this, SLOT(_q_sourceColumnsAboutToBeInserted(const QModelIndex&,int,int)));
-    disconnect(model, SIGNAL(columnsInserted(const QModelIndex&,int,int)),
-               this, SLOT(_q_sourceColumnsInserted(const QModelIndex&,int,int)));
-    disconnect(model, SIGNAL(columnsAboutToBeRemoved(const QModelIndex&,int,int)),
-               this, SLOT(_q_sourceColumnsAboutToBeRemoved(const QModelIndex&,int,int)));
-    disconnect(model, SIGNAL(columnsRemoved(const QModelIndex&,int,int)),
-               this, SLOT(_q_sourceColumnsRemoved(const QModelIndex&,int,int)));
+    disconnect(model, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
+               this, SLOT(_q_sourceRowsAboutToBeInserted(QModelIndex,int,int)));
+    disconnect(model, SIGNAL(rowsInserted(QModelIndex,int,int)),
+               this, SLOT(rowsInserted(QModelIndex,int,int)));
+    disconnect(model, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
+               this, SLOT(_q_sourceRowsAboutToBeRemoved(QModelIndex,int,int)));
+    disconnect(model, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+               this, SLOT(_q_sourceRowsRemoved(QModelIndex,int,int)));
+    disconnect(model, SIGNAL(columnsAboutToBeInserted(QModelIndex,int,int)),
+               this, SLOT(_q_sourceColumnsAboutToBeInserted(QModelIndex,int,int)));
+    disconnect(model, SIGNAL(columnsInserted(QModelIndex,int,int)),
+               this, SLOT(_q_sourceColumnsInserted(QModelIndex,int,int)));
+    disconnect(model, SIGNAL(columnsAboutToBeRemoved(QModelIndex,int,int)),
+               this, SLOT(_q_sourceColumnsAboutToBeRemoved(QModelIndex,int,int)));
+    disconnect(model, SIGNAL(columnsRemoved(QModelIndex,int,int)),
+               this, SLOT(_q_sourceColumnsRemoved(QModelIndex,int,int)));
     disconnect(model, SIGNAL(modelReset()), this, SIGNAL(modelReset())); // signal to signal
+    disconnect(model, SIGNAL(layoutAboutToBeChanged()), this, SIGNAL(layoutAboutToBeChanged())); // signal to signal
     disconnect(model, SIGNAL(layoutChanged()), this, SIGNAL(layoutChanged())); // signal to signal
 }
 
