@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -91,6 +106,7 @@ public:
         QVector<int> proxy_rows;
         QVector<int> proxy_columns;
         QVector<QModelIndex> mapped_children;
+        QMap<QModelIndex, Mapping *>::const_iterator map_iter;
     };
 
     mutable QMap<QModelIndex, Mapping*> source_index_mapping;
@@ -99,6 +115,7 @@ public:
     Qt::SortOrder sort_order;
     Qt::CaseSensitivity sort_casesensitivity;
     int sort_role;
+    bool sort_localeaware;
 
     int filter_column;
     QRegExp filter_regexp;
@@ -121,8 +138,8 @@ public:
         Q_ASSERT(proxy_index.isValid());
         const void *p = proxy_index.internalPointer();
         Q_ASSERT(p);
-        QMap<QModelIndex, Mapping *>::const_iterator it =
-            reinterpret_cast<QMap<QModelIndex, Mapping *>::const_iterator & >(p);
+        QMap<QModelIndex, Mapping *>::const_iterator it = 
+            static_cast<const Mapping*>(p)->map_iter;
         Q_ASSERT(it != source_index_mapping.constEnd());
         Q_ASSERT(it.value());
         return it;
@@ -131,8 +148,7 @@ public:
     inline QModelIndex create_index(int row, int column,
                                     QMap<QModelIndex, Mapping*>::const_iterator it) const
     {
-        const void *p = static_cast<const void *>(it);
-        return q_func()->createIndex(row, column, const_cast<void *>(p));
+        return q_func()->createIndex(row, column, *it);
     }
 
     void _q_sourceDataChanged(const QModelIndex &source_top_left,
@@ -202,6 +218,9 @@ public:
     void handle_filter_changed(
         QVector<int> &source_to_proxy, QVector<int> &proxy_to_source,
         const QModelIndex &source_parent, Qt::Orientation orient);
+
+    void updateChildrenMapping(const QModelIndex &source_parent, Mapping *parent_mapping,
+                               Qt::Orientation orient, int start, int end, int delta_item_count, bool remove);
 };
 
 typedef QMap<QModelIndex, QSortFilterProxyModelPrivate::Mapping *> IndexMap;
@@ -256,6 +275,7 @@ IndexMap::const_iterator QSortFilterProxyModelPrivate::create_mapping(
     build_source_to_proxy_mapping(m->source_columns, m->proxy_columns);
 
     it = IndexMap::const_iterator(source_index_mapping.insert(source_parent, m));
+    m->map_iter = it;
 
     if (source_parent.isValid()) {
         QModelIndex source_grand_parent = source_parent.parent();
@@ -530,12 +550,14 @@ void QSortFilterProxyModelPrivate::insert_source_items(
     Qt::Orientation orient, bool emit_signal)
 {
     Q_Q(QSortFilterProxyModel);
+    QModelIndex proxy_parent = QSortFilterProxyModelPrivate::source_to_proxy(source_parent);
+    if (!proxy_parent.isValid() && source_parent.isValid())
+        return; // nothing to do (source_parent is not mapped)
+
     QVector<QPair<int, QVector<int> > > proxy_intervals;
     proxy_intervals = proxy_intervals_for_source_items_to_add(
         proxy_to_source, source_items, source_parent, orient);
 
-    QModelIndex proxy_parent = QSortFilterProxyModelPrivate::source_to_proxy(source_parent);
-
     for (int i = proxy_intervals.size()-1; i >= 0; --i) {
         QPair<int, QVector<int> > interval = proxy_intervals.at(i);
         int proxy_start = interval.first;
@@ -590,6 +612,11 @@ void QSortFilterProxyModelPrivate::source_items_inserted(
                 // Don't care, since we don't have mapping for the grand parent
                 return;
             }
+            Mapping *gm = it.value();
+            if (gm->proxy_rows.at(source_parent.row()) == -1) {
+                // Don't care, since parent is filtered
+                return;
+            }
         }
         it = create_mapping(source_parent);
         Mapping *m = it.value();
@@ -611,6 +638,9 @@ void QSortFilterProxyModelPrivate::source_items_inserted(
 
     int delta_item_count = end - start + 1;
     int old_item_count = source_to_proxy.size();
+
+    updateChildrenMapping(source_parent, m, orient, start, end, delta_item_count, false);
+
     // Expand source-to-proxy mapping to account for new items
     source_to_proxy.insert(start, delta_item_count, -1);
 
@@ -702,40 +732,62 @@ void QSortFilterProxyModelPrivate::source_items_removed(
     int delta_item_count = end - start + 1;
     source_to_proxy.remove(start, delta_item_count);
 
-    // Adjust "stale" indexes in proxy-to-source mapping
     int proxy_count = proxy_to_source.size();
+    if (proxy_count > source_to_proxy.size()) {
+        // mapping is in an inconsistent state -- redo the whole mapping
+        qWarning("QSortFilterProxyModel: inconsistent changes reported by source model");
+        remove_from_mapping(source_parent);
+        return;
+    }
+
+    // Adjust "stale" indexes in proxy-to-source mapping
     for (int proxy_item = 0; proxy_item < proxy_count; ++proxy_item) {
         int source_item = proxy_to_source.at(proxy_item);
-        if (source_item >= start)
+        if (source_item >= start) {
+            Q_ASSERT(source_item - delta_item_count >= 0);
             proxy_to_source.replace(proxy_item, source_item - delta_item_count);
+        }
     }
     build_source_to_proxy_mapping(proxy_to_source, source_to_proxy);
 
+    updateChildrenMapping(source_parent, m, orient, start, end, delta_item_count, true);
+
+}
+
+
+/*!
+  \internal
+  updates the mapping of the children when inserting or removing items
+*/
+void QSortFilterProxyModelPrivate::updateChildrenMapping(const QModelIndex &source_parent, Mapping *parent_mapping,
+                                                         Qt::Orientation orient, int start, int end, int delta_item_count, bool remove)
+{
     // see if any mapped children should be (re)moved
-    QVector<QModelIndex>::iterator it2 = m->mapped_children.begin();
-    for ( ; it2 != m->mapped_children.end(); ) {
+    QVector<QPair<QModelIndex, Mapping*> > moved_source_index_mappings;
+    QVector<QModelIndex>::iterator it2 = parent_mapping->mapped_children.begin();
+    for ( ; it2 != parent_mapping->mapped_children.end();) {
         const QModelIndex source_child_index = *it2;
         const int pos = (orient == Qt::Vertical)
                         ? source_child_index.row()
                         : source_child_index.column();
         if (pos < start) {
-            // not affected by removal
+            // not affected
             ++it2;
-            continue;
-        } else if (pos <= end) {
+        } else if (remove && pos <= end) {
             // in the removed interval
-            it2 = m->mapped_children.erase(it2);
+            it2 = parent_mapping->mapped_children.erase(it2);
             remove_from_mapping(source_child_index);
         } else {
             // below the removed items -- recompute the index
             QModelIndex new_index;
+            const int newpos = remove ? pos - delta_item_count : pos + delta_item_count;
             if (orient == Qt::Vertical) {
-                new_index = model->index(pos - delta_item_count,
+                new_index = model->index(newpos,
                                          source_child_index.column(),
                                          source_parent);
             } else {
                 new_index = model->index(source_child_index.row(),
-                                         pos - delta_item_count,
+                                         newpos,
                                          source_parent);
             }
             *it2 = new_index;
@@ -744,9 +796,15 @@ void QSortFilterProxyModelPrivate::source_items_removed(
             // update mapping
             Mapping *cm = source_index_mapping.take(source_child_index);
             Q_ASSERT(cm);
-            source_index_mapping.insert(new_index, cm);
+	    // we do not reinsert right away, because the new index might be identical with another, old index
+	    moved_source_index_mappings.append(QPair<QModelIndex, Mapping*>(new_index, cm));
         }
     }
+
+    // reinsert moved, mapped indexes
+    QVector<QPair<QModelIndex, Mapping*> >::iterator it = moved_source_index_mappings.begin();
+    for (; it != moved_source_index_mappings.end(); ++it)
+        (*it).second->map_iter = source_index_mapping.insert((*it).first, (*it).second);
 }
 
 /*!
@@ -875,13 +933,13 @@ void QSortFilterProxyModelPrivate::handle_filter_changed(
 }
 
 void QSortFilterProxyModelPrivate::_q_sourceDataChanged(const QModelIndex &source_top_left,
-                                                     const QModelIndex &source_bottom_right)
+							const QModelIndex &source_bottom_right)
 {
     Q_Q(QSortFilterProxyModel);
     if (!source_top_left.isValid() || !source_bottom_right.isValid())
         return;
     QModelIndex source_parent = source_top_left.parent();
-    IndexMap::const_iterator it = source_index_mapping.constFind(source_parent);
+    IndexMap::const_iterator it = create_mapping(source_parent);
     if (it == source_index_mapping.constEnd()) {
         // Don't care, since we don't have mapping for this index
         return;
@@ -934,7 +992,11 @@ void QSortFilterProxyModelPrivate::_q_sourceDataChanged(const QModelIndex &sourc
                             source_parent, Qt::Vertical, false);
         update_persistent_indexes(source_indexes);
         emit q->layoutChanged();
-    } else if (!source_rows_change.isEmpty()) {
+	// Make sure we also emit dataChanged for the rows
+	source_rows_change += source_rows_resort;
+    }
+
+    if (!source_rows_change.isEmpty()) {
         // Find the proxy row range
         int proxy_start_row;
         int proxy_end_row;
@@ -995,7 +1057,7 @@ void QSortFilterProxyModelPrivate::_q_sourceLayoutChanged()
 {
     Q_Q(QSortFilterProxyModel);
     if (saved_persistent_indexes.isEmpty()) {
-        q->clear();
+        q->invalidate();
         return;
     }
 
@@ -1114,11 +1176,10 @@ void QSortFilterProxyModelPrivate::_q_sourceColumnsRemoved(
     mapFromSource(), mapSelectionToSource(), and
     mapSelectionFromSource().
 
-    In Qt 4.1, sorting and filtering isn't dynamically reapplied
-    whenever the data of the original model changes, as an
-    optimization. Qt 4.2 is expected to offer a mechanism to enable
-    dynamic sorting and filtering.
-
+    \note By default, the model does not dynamically re-sort and re-filter
+    data whenever the original model changes. This behavior can be
+    changed by setting the \l{dynamicSortFilter} property. 
+        
     The \l{itemviews/basicsortfiltermodel}{Basic Sort/Filter Model}
     and \l{itemviews/customsortfiltermodel}{Custom Sort/Filter Model}
     examples illustrate how to use QSortFilterProxyModel to perform
@@ -1160,7 +1221,7 @@ void QSortFilterProxyModelPrivate::_q_sourceColumnsRemoved(
 
     \quotefromfile itemviews/customsortfiltermodel/mysortfilterproxymodel.cpp
     \skipto ::lessThan
-    \printuntil /^\}$/
+    \printuntil /^\}/
 
     (This code snippet comes from the
     \l{itemviews/customsortfiltermodel}{Custom Sort/Filter Model}
@@ -1206,12 +1267,18 @@ void QSortFilterProxyModelPrivate::_q_sourceColumnsRemoved(
 
     \quotefromfile itemviews/customsortfiltermodel/mysortfilterproxymodel.cpp
     \skipto ::filterAcceptsRow
-    \printuntil /^\}$/
+    \printuntil /^\}/
 
     (This code snippet comes from the
     \l{itemviews/customsortfiltermodel}{Custom Sort/Filter Model}
     example.)
 
+    If you are working with large amounts of filtering and have to invoke
+    invalidateFilter() repeatedly, using reset() may be more efficient,
+    depending on the implementation of your model. However, note that reset()
+    returns the proxy model to its original state, losing selection
+    information, and will cause the proxy model to be repopulated.
+
     \section1 Subclassing
 
     \bold{Note:} Some general guidelines for subclassing models are
@@ -1243,10 +1310,11 @@ QSortFilterProxyModel::QSortFilterProxyModel(QObject *parent)
     d->sort_order = Qt::AscendingOrder;
     d->sort_casesensitivity = Qt::CaseSensitive;
     d->sort_role = Qt::DisplayRole;
+    d->sort_localeaware = false;
     d->filter_column = 0;
     d->filter_role = Qt::DisplayRole;
     d->dynamic_sortfilter = false;
-    connect(this, SIGNAL(modelReset()), this, SLOT(clear()));
+    connect(this, SIGNAL(modelReset()), this, SLOT(invalidate()));
 }
 
 /*!
@@ -1387,6 +1455,8 @@ int QSortFilterProxyModel::rowCount(const QModelIndex &parent) const
 {
     Q_D(const QSortFilterProxyModel);
     QModelIndex source_parent = d->proxy_to_source(parent);
+    if (parent.isValid() && !source_parent.isValid())
+        return 0;
     IndexMap::const_iterator it = d->create_mapping(source_parent);
     return it.value()->source_rows.count();
 }
@@ -1398,6 +1468,8 @@ int QSortFilterProxyModel::columnCount(const QModelIndex &parent) const
 {
     Q_D(const QSortFilterProxyModel);
     QModelIndex source_parent = d->proxy_to_source(parent);
+    if (parent.isValid() && !source_parent.isValid())
+        return 0;
     IndexMap::const_iterator it = d->create_mapping(source_parent);
     return it.value()->source_columns.count();
 }
@@ -1409,6 +1481,8 @@ bool QSortFilterProxyModel::hasChildren(const QModelIndex &parent) const
 {
     Q_D(const QSortFilterProxyModel);
     QModelIndex source_parent = d->proxy_to_source(parent);
+    if (parent.isValid() && !source_parent.isValid())
+        return false;
     if (!d->model->hasChildren(source_parent))
         return false;
     QSortFilterProxyModelPrivate::Mapping *m = d->create_mapping(source_parent).value();
@@ -1422,6 +1496,8 @@ QVariant QSortFilterProxyModel::data(const QModelIndex &index, int role) const
 {
     Q_D(const QSortFilterProxyModel);
     QModelIndex source_index = d->proxy_to_source(index);
+    if (index.isValid() && !source_index.isValid())
+        return QVariant();
     return d->model->data(source_index, role);
 }
 
@@ -1432,6 +1508,8 @@ bool QSortFilterProxyModel::setData(const QModelIndex &index, const QVariant &va
 {
     Q_D(QSortFilterProxyModel);
     QModelIndex source_index = d->proxy_to_source(index);
+    if (index.isValid() && !source_index.isValid())
+        return false;
     return d->model->setData(source_index, value, role);
 }
 
@@ -1536,6 +1614,8 @@ bool QSortFilterProxyModel::insertRows(int row, int count, const QModelIndex &pa
     if (row < 0 || count <= 0)
         return false;
     QModelIndex source_parent = d->proxy_to_source(parent);
+    if (parent.isValid() && !source_parent.isValid())
+        return false;
     QSortFilterProxyModelPrivate::Mapping *m = d->create_mapping(source_parent).value();
     if (row > m->source_rows.count())
         return false;
@@ -1554,6 +1634,8 @@ bool QSortFilterProxyModel::insertColumns(int column, int count, const QModelInd
     if (column < 0|| count <= 0)
         return false;
     QModelIndex source_parent = d->proxy_to_source(parent);
+    if (parent.isValid() && !source_parent.isValid())
+        return false;
     QSortFilterProxyModelPrivate::Mapping *m = d->create_mapping(source_parent).value();
     if (column > m->source_columns.count())
         return false;
@@ -1572,6 +1654,8 @@ bool QSortFilterProxyModel::removeRows(int row, int count, const QModelIndex &pa
     if (row < 0 || count <= 0)
         return false;
     QModelIndex source_parent = d->proxy_to_source(parent);
+    if (parent.isValid() && !source_parent.isValid())
+        return false;
     QSortFilterProxyModelPrivate::Mapping *m = d->create_mapping(source_parent).value();
     if (row + count > m->source_rows.count())
         return false;
@@ -1611,6 +1695,8 @@ bool QSortFilterProxyModel::removeColumns(int column, int count, const QModelInd
     if (column < 0 || count <= 0)
         return false;
     QModelIndex source_parent = d->proxy_to_source(parent);
+    if (parent.isValid() && !source_parent.isValid())
+        return false;
     QSortFilterProxyModelPrivate::Mapping *m = d->create_mapping(source_parent).value();
     if (column + count > m->source_columns.count())
         return false;
@@ -1706,6 +1792,8 @@ QSize QSortFilterProxyModel::span(const QModelIndex &index) const
 {
     Q_D(const QSortFilterProxyModel);
     QModelIndex source_index = d->proxy_to_source(index);
+    if (index.isValid() && !source_index.isValid())
+        return QSize();
     return d->model->span(source_index);
 }
 
@@ -1772,7 +1860,7 @@ void QSortFilterProxyModel::setFilterKeyColumn(int column)
     \brief the case sensitivity of the QRegExp pattern used to filter the
     contents of the source model
 
-    By default, the filter is case sensistive.
+    By default, the filter is case sensitive.
 
     \sa filterRegExp, sortCaseSensitivity
 */
@@ -1796,7 +1884,7 @@ void QSortFilterProxyModel::setFilterCaseSensitivity(Qt::CaseSensitivity cs)
     \property QSortFilterProxyModel::sortCaseSensitivity
     \brief the case sensitivity setting used for comparing strings when sorting
 
-    By default, sorting is case sensistive.
+    By default, sorting is case sensitive.
 
     \sa filterCaseSensitivity, lessThan()
 */
@@ -1816,6 +1904,31 @@ void QSortFilterProxyModel::setSortCaseSensitivity(Qt::CaseSensitivity cs)
     d->sort();
 }
 
+/*!
+    \since 4.3
+    \property QSortFilterProxyModel::isSortLocaleAware
+    \brief the local aware setting used for comparing strings when sorting
+
+    By default, sorting is not local aware.
+
+    \sa sortCaseSensitivity, lessThan()
+*/
+bool QSortFilterProxyModel::isSortLocaleAware() const
+{
+    Q_D(const QSortFilterProxyModel);
+    return d->sort_localeaware;
+}
+
+void QSortFilterProxyModel::setSortLocaleAware(bool on)
+{
+    Q_D(QSortFilterProxyModel);
+    if (d->sort_localeaware == on)
+        return;
+
+    d->sort_localeaware = on;
+    d->sort();
+}
+
 /*!
     \overload
 
@@ -1929,9 +2042,9 @@ void QSortFilterProxyModel::setFilterRole(int role)
 }
 
 /*!
-    Clears this sorting filter model, removing all mapping.
+    \obsolete
 
-    \sa filterChanged()
+    This function is obsolete. Use invalidate() instead.
 */
 void QSortFilterProxyModel::clear()
 {
@@ -1942,15 +2055,42 @@ void QSortFilterProxyModel::clear()
 }
 
 /*!
-   \since 4.2
-   Updates the mapping to reflect a change in the filter.
+   \since 4.3
+
+    Invalidates the current sorting and filtering.
+
+    \sa invalidateFilter()
+*/
+void QSortFilterProxyModel::invalidate()
+{
+    Q_D(QSortFilterProxyModel);
+    emit layoutAboutToBeChanged();
+    d->clear_mapping();
+    emit layoutChanged();
+}
+
+/*!
+   \obsolete
+
+    This function is obsolete. Use invalidateFilter() instead.
+*/
+void QSortFilterProxyModel::filterChanged()
+{
+    Q_D(QSortFilterProxyModel);
+    d->filter_changed();
+}
+
+/*!
+   \since 4.3
+
+   Invalidates the current filtering.
 
    This function should be called if you are implementing custom filtering
    (e.g. filterAcceptsRow()), and your filter parameters have changed.
 
-   \sa clear()
+   \sa invalidate()
 */
-void QSortFilterProxyModel::filterChanged()
+void QSortFilterProxyModel::invalidateFilter()
 {
     Q_D(QSortFilterProxyModel);
     d->filter_changed();
@@ -2017,7 +2157,10 @@ bool QSortFilterProxyModel::lessThan(const QModelIndex &left, const QModelIndex
         return l.toDateTime() < r.toDateTime();
     case QVariant::String:
     default:
-        return l.toString().compare(r.toString(), d->sort_casesensitivity) < 0;
+        if (d->sort_localeaware)
+            return l.toString().localeAwareCompare(r.toString()) < 0;
+        else
+            return l.toString().compare(r.toString(), d->sort_casesensitivity) < 0;
     }
     return false;
 }
