@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -32,6 +47,13 @@
 #include "qdebug.h"
 #include <private/qshortcutmap_p.h>
 #include <private/qapplication_p.h>
+#include <private/qmenu_p.h>
+
+#define QAPP_CHECK(functionName) \
+    if (!qApp) { \
+        qWarning("QAction: Initialize QApplication before calling '" functionName "'."); \
+        return; \
+    }
 
 /*
   internal: guesses a descriptive text from a text suited for a menu entry
@@ -121,7 +143,7 @@ void QActionPrivate::redoGrabAlternate(QShortcutMap &map)
     }
     if (!autorepeat) {
         foreach (int id, alternateShortcutIds)
-            map.setShortcutEnabled(false, id, q);
+            map.setShortcutAutoRepeat(false, id, q);
     }
 }
 
@@ -172,7 +194,10 @@ void QActionPrivate::setShortcutEnabled(bool enable, QShortcutMap &map)
     setStatusTip(), setWhatsThis(), and setToolTip(). For menu items,
     it is possible to set an individual font with setFont().
 
-    Actions are added to widgets using QWidget::addAction().
+    Actions are added to widgets using QWidget::addAction(). Note
+    that an action must be added to a widget before it can be used;
+    this is also true when the shortcut should be global (i.e.,
+    Qt::ApplicationShortcut as Qt::ShortcutContext).
 
     Once a QAction has been created it should be added to the relevant
     menu and toolbar, then connected to the slot which will perform
@@ -241,9 +266,9 @@ QAction::QAction(QObject* parent)
 
     The action uses a stripped version of \a text (e.g. "\&Menu
     Option..." becomes "Menu Option") as descriptive text for
-    toolbuttons. You can override this by setting a specific
+    tool buttons. You can override this by setting a specific
     description with setText(). The same text will be used for
-    tooltips unless you specify a different test using
+    tooltips unless you specify a different text using
     setToolTip().
 
 */
@@ -264,9 +289,9 @@ QAction::QAction(const QString &text, QObject* parent)
 
     The action uses a stripped version of \a text (e.g. "\&Menu
     Option..." becomes "Menu Option") as descriptive text for
-    toolbuttons. You can override this by setting a specific
+    tool buttons. You can override this by setting a specific
     description with setText(). The same text will be used for
-    tooltips unless you specify a different test using
+    tooltips unless you specify a different text using
     setToolTip().
 */
 QAction::QAction(const QIcon &icon, const QString &text, QObject* parent)
@@ -325,6 +350,8 @@ QList<QWidget *> QAction::associatedWidgets() const
 */
 void QAction::setShortcut(const QKeySequence &shortcut)
 {
+    QAPP_CHECK("setShortcut");
+
     Q_D(QAction);
     if (d->shortcut == shortcut)
         return;
@@ -355,6 +382,8 @@ void QAction::setShortcuts(const QList<QKeySequence> &shortcuts)
     if (d->shortcut == primary && d->alternateShortcuts == listCopy)
         return;
 
+    QAPP_CHECK("setShortcuts");
+
     d->shortcut = primary;
     d->alternateShortcuts = listCopy;
     d->redoGrab(qApp->d_func()->shortcutMap);
@@ -420,6 +449,7 @@ void QAction::setShortcutContext(Qt::ShortcutContext context)
     Q_D(QAction);
     if (d->shortcutContext == context)
         return;
+    QAPP_CHECK("setShortcutContext");
     d->shortcutContext = context;
     d->redoGrab(qApp->d_func()->shortcutMap);
     d->redoGrabAlternate(qApp->d_func()->shortcutMap);
@@ -447,6 +477,7 @@ void QAction::setAutoRepeat(bool on)
     Q_D(QAction);
     if (d->autorepeat == on)
         return;
+    QAPP_CHECK("setAutoRepeat");
     d->autorepeat = on;
     d->redoGrab(qApp->d_func()->shortcutMap);
     d->redoGrabAlternate(qApp->d_func()->shortcutMap);
@@ -551,8 +582,11 @@ QAction::~QAction()
     if (d->group)
         d->group->removeAction(this);
 #ifndef QT_NO_SHORTCUT
-    if (d->shortcutId && qApp)
+    if (d->shortcutId && qApp) {
         qApp->d_func()->shortcutMap.removeShortcut(d->shortcutId, this);
+        foreach (int id, d->alternateShortcutIds)
+            qApp->d_func()->shortcutMap.removeShortcut(id, this);
+    }
 #endif
 }
 
@@ -634,7 +668,11 @@ QMenu *QAction::menu() const
 void QAction::setMenu(QMenu *menu)
 {
     Q_D(QAction);
+    if (d->menu)
+        d->menu->d_func()->setOverrideMenuAction(0); //we reset the default action of any previous menu
     d->menu = menu;
+    if (menu)
+        menu->d_func()->setOverrideMenuAction(this);
     d->sendDataChanged();
 }
 #endif // QT_NO_MENU
@@ -930,8 +968,9 @@ void QAction::setEnabled(bool b)
     if (b == d->enabled && b != d->forceDisabled)
         return;
     d->forceDisabled = !b;
-    if (b && d->group && !d->group->isEnabled())
+    if (b && (!d->visible || (d->group && !d->group->isEnabled())))
         return;
+    QAPP_CHECK("setEnabled");
     d->enabled = b;
 #ifndef QT_NO_SHORTCUT
     d->setShortcutEnabled(b, qApp->d_func()->shortcutMap);
@@ -961,8 +1000,13 @@ void QAction::setVisible(bool b)
     Q_D(QAction);
     if (b == d->visible && b != d->forceInvisible)
         return;
+    QAPP_CHECK("setVisible");
     d->forceInvisible = !b;
     d->visible = b;
+    d->enabled = b && !d->forceDisabled && (!d->group || d->group->isEnabled()) ;
+#ifndef QT_NO_SHORTCUT
+    d->setShortcutEnabled(d->enabled, qApp->d_func()->shortcutMap);
+#endif
     d->sendDataChanged();
 }
 
@@ -1024,11 +1068,13 @@ QAction::setData(const QVariant &data)
 
 
 /*!
-  Updates the status bar for \a widget. If widget is an appropriate
-  QStatusBar found for for this action based on the parent heirarchy will be used.
+  Updates the relevant status bar for the \a widget specified by sending a
+  QStatusTipEvent to its parent widget. Returns true if an event was sent;
+  otherwise returns false.
+
+  If a null widget is specified, the event is sent to the action's parent.
 
   \sa statusTip
-
 */
 bool
 QAction::showStatusText(QWidget *widget)
