@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,8 +58,12 @@
 #include "QtCore/qmutex.h"
 #include "QtCore/qtranslator.h"
 #include "QtCore/qbasictimer.h"
+#include "QtCore/qhash.h"
 #include "private/qcoreapplication_p.h"
 #include "private/qshortcutmap_p.h"
+#ifdef Q_WS_QWS
+#include "QtGui/qscreen_qws.h"
+#endif
 
 class QWidget;
 class QObject;
@@ -70,6 +89,7 @@ extern QSysInfo::MacVersion qt_macver;
 #endif
 #if defined(Q_WS_QWS)
 class QWSManager;
+class QDirectPainter;
 #endif
 
 #ifndef QT_NO_TABLET
@@ -96,6 +116,12 @@ struct QTabletDeviceData
     int xinput_key_release;
     int xinput_button_press;
     int xinput_button_release;
+    int xinput_proximity_in;
+    int xinput_proximity_out;
+#elif defined(Q_WS_WIN)
+    qint64 llId;
+    int currentDevice;
+    int currentPointerType;
 #endif
 };
 
@@ -185,6 +211,17 @@ public:
     static void x11_initialize_style();
 #endif
 
+    enum KeyPlatform {
+        KB_Win = 1,
+        KB_Mac = 2,
+        KB_X11 = 4,
+        KB_KDE = 8,
+        KB_Gnome = 16,
+        KB_CDE = 32,
+        KB_All = 0xffff
+    };
+
+    static uint currentPlatform();
     bool inPopupMode() const;
     void closePopup(QWidget *popup);
     void openPopup(QWidget *popup);
@@ -235,8 +272,11 @@ public:
     static QPalette *sys_pal;
     static QPalette *set_pal;
     static QFont *app_font;
+    static QFont *sys_font;
+    static QFont *set_font;
     static QWidget *main_widget;
     static QWidget *focus_widget;
+    static QWidget *hidden_focus_widget;
     static QWidget *active_window;
     static QIcon *app_icon;
     static bool obey_desktop_settings;
@@ -257,6 +297,7 @@ public:
     static void setSystemPalette(const QPalette &pal);
     static void setPalette_helper(const QPalette &palette, const char* className, bool clearWidgetPaletteHash);
     static void initializeWidgetPaletteHash();
+    static void setSystemFont(const QFont &font);
 
 #if defined(Q_WS_X11)
     static void applyX11SpecificCommandLineArguments(QWidget *main_widget);
@@ -265,7 +306,6 @@ public:
 #endif
 
 #ifdef Q_WS_MAC
-    bool do_mouse_down(const QPoint &, bool *);
     static OSStatus globalEventProcessor(EventHandlerCallRef, EventRef, void *);
     static OSStatus globalAppleEventProcessor(const AppleEvent *, AppleEvent *, long);
     static void qt_context_timer_callbk(EventLoopTimerRef, void *);
@@ -275,24 +315,39 @@ public:
 #ifdef Q_WS_QWS
     QPointer<QWSManager> last_manager;
 # ifndef QT_NO_DIRECTPAINTER
-    int directPainterID;
-    bool seenRegionEvent;
-    QRegion directPainterRegion;
+    QMap<WId, QDirectPainter *> *directPainters;
 # endif
+    QRect maxWindowRect(const QScreen *screen) const { return maxWindowRects[screen]; }
+    void setMaxWindowRect(const QScreen *screen, const QRect &rect);
 #endif
 
     static QApplicationPrivate *instance() { return self; }
 
     static QString *styleOverride;
 
+    static int app_compile_version;
+
 #ifdef QT_KEYPAD_NAVIGATION
     static bool keypadNavigation;
     static QWidget *oldEditFocus;
 #endif
 
+    static bool tryEmitLastWindowClosedPending;
     void _q_tryEmitLastWindowClosed();
 
+#if defined(Q_WS_MAC) || defined(Q_WS_X11)
+    void _q_alertTimeOut();
+    QHash<QWidget *, QTimer *> alertTimerHash;
+#endif
+#ifndef QT_NO_STYLE_STYLESHEET
+    static QString styleSheet;
+#endif
+
 private:
+#ifdef Q_WS_QWS
+    QMap<const QScreen*, QRect> maxWindowRects;
+#endif
+
     static QApplicationPrivate *self;
 };
 
