@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,18 +9,34 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
+#include "qvariant.h"
 #include "qdesktopwidget.h"
 #include "qapplication.h"
 #include "qt_x11_p.h"
@@ -31,9 +47,6 @@
 // defined in qwidget_x11.cpp
 extern int qt_x11_create_desktop_on_screen;
 
-// defined in qapplication_x11.cpp
-extern bool qt_net_supports(Atom atom);
-
 // function to update the workarea of the screen
 static bool qt_desktopwidget_workarea_dirty = true;
 void qt_desktopwidget_update_workarea()
@@ -229,7 +242,7 @@ const QRect QDesktopWidget::availableGeometry(int screen) const
     if (d->workareas[screen].isValid())
         return d->workareas[screen];
 
-    if (! isVirtualDesktop() && qt_net_supports(ATOM(_NET_WORKAREA))) {
+    if (! isVirtualDesktop() && X11->isSupportedByWM(ATOM(_NET_WORKAREA))) {
         Atom ret;
         int format, e;
         unsigned char *data = 0;
@@ -283,7 +296,7 @@ int QDesktopWidget::screenNumber(const QWidget *widget) const
         int maxScreen = -1;
 
         for (int i = 0; i < d->screenCount; ++i) {
-            QRect sect = d->rects[i].intersect(frame);
+            QRect sect = d->rects[i].intersected(frame);
             int size = sect.width() * sect.height();
             if (size > maxSize && sect.width() > 0 && sect.height() > 0) {
                 maxSize = size;
