@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -66,6 +81,10 @@
     \skipto setHotSpot
     \printuntil ;
 
+    \bold{Note:} on X11, the pixmap may not be able to keep up with the mouse
+    movements if the hot spot causes the pixmap to be displayed
+    directly under the cursor.
+
     The source and target widgets can be found with source() and target().
     These functions are often used to determine whether drag and drop operations
     started and finished at the same widget, so that special behavior can be
@@ -79,7 +98,9 @@
     \l{QWidget::mouseMoveEvent()}{mouseMoveEvent()} to check whether a QDrag is
     required.
 
-    \sa \link dnd.html Drag and Drop\endlink QClipboard QMimeData
+    \sa {Drag and Drop}, QClipboard, QMimeData, QWindowsMime, QMacPasteboardMime,
+        {Draggable Icons Example}, {Draggable Text Example}, {Drop Site Example},
+	    {Fridge Magnets Example}
 */
 
 /*!
@@ -95,6 +116,7 @@ QDrag::QDrag(QWidget *dragSource)
     d->hotspot = QPoint(-10, -10);
     d->possible_actions = Qt::CopyAction;
     d->executed_action = Qt::IgnoreAction;
+    d->defaultDropAction = Qt::IgnoreAction;
 }
 
 /*!
@@ -116,6 +138,8 @@ QDrag::~QDrag()
 void QDrag::setMimeData(QMimeData *data)
 {
     Q_D(QDrag);
+    if (d->data == data)
+        return;
     if (d->data != 0)
         delete d->data;
     d->data = data;
@@ -132,7 +156,8 @@ QMimeData *QDrag::mimeData() const
 
 /*!
     Sets \a pixmap as the pixmap used to represent the data in a drag
-    and drop operation.
+    and drop operation. You can only set a pixmap before the drag is
+    started.
 */
 void QDrag::setPixmap(const QPixmap &pixmap)
 {
@@ -152,6 +177,10 @@ QPixmap QDrag::pixmap() const
 /*!
     Sets the position of the hot spot relative to the top-left corner of the
     pixmap used to the point specified by \a hotspot.
+
+    \bold{Note:} on X11, the pixmap may not be able to keep up with the mouse
+    movements if the hot spot causes the pixmap to be displayed
+    directly under the cursor.
 */
 void QDrag::setHotSpot(const QPoint& hotspot)
 {
@@ -190,6 +219,72 @@ QWidget *QDrag::target() const
 }
 
 /*!
+    \since 4.3
+
+    Starts the drag and drop operation and returns a value indicating the requested
+    drop action when it is completed. The drop actions that the user can choose
+    from are specified in \a supportedActions. The default proposed action will be selected
+    among the allowed actions in the following order : Move, Copy and Link.
+
+    \bold{Note:} Although the drag and drop operation can take some time, this function
+    does not block the event loop. Other events are still delivered to the application
+    while the operation is performed.
+*/
+
+Qt::DropAction QDrag::exec(Qt::DropActions supportedActions)
+{
+    return exec(supportedActions, Qt::IgnoreAction);
+}
+
+/*!
+    \since 4.3
+
+    Starts the drag and drop operation and returns a value indicating the requested
+    drop action when it is completed. The drop actions that the user can choose
+    from are specified in \a supportedActions.
+
+    The \a defaultDropAction determines which action will be proposed when the user performs a
+    drag without using modifier keys.
+
+    \bold{Note:} Although the drag and drop operation is blocking, the function
+    does not block the event loop. Other events are still delivered to the application
+    while the operation is performed.
+*/
+
+Qt::DropAction QDrag::exec(Qt::DropActions supportedActions, Qt::DropAction defaultDropAction)
+{
+    Q_D(QDrag);
+    if (!d->data) {
+        qWarning("QDrag: No mimedata set before starting the drag");
+        return d->executed_action;
+    }
+    QDragManager *manager = QDragManager::self();
+    d->defaultDropAction = Qt::IgnoreAction;
+    d->possible_actions = supportedActions;
+
+    if (manager) {
+        if (defaultDropAction == Qt::IgnoreAction) {
+            if (supportedActions & Qt::MoveAction) {
+                d->defaultDropAction = Qt::MoveAction;
+            } else if (supportedActions & Qt::CopyAction) {
+                d->defaultDropAction = Qt::CopyAction;
+            } else if (supportedActions & Qt::LinkAction) {
+                d->defaultDropAction = Qt::LinkAction;
+            }
+        } else {
+            d->defaultDropAction = defaultDropAction;
+        }
+        d->executed_action = manager->drag(this);
+    }
+
+    return d->executed_action;
+}
+
+/*!
+    \obsolete
+
+    \bold{Note:} It is recommended to use exec() instead of this function.
+
     Starts the drag and drop operation and returns a value indicating the requested
     drop action when it is completed. The drop actions that the user can choose
     from are specified in \a request. Qt::CopyAction is always allowed.
@@ -197,12 +292,18 @@ QWidget *QDrag::target() const
     \bold{Note:} Although the drag and drop operation can take some time, this function
     does not block the event loop. Other events are still delivered to the application
     while the operation is performed.
+
+    \sa exec()
 */
 Qt::DropAction QDrag::start(Qt::DropActions request)
 {
     Q_D(QDrag);
-    Q_ASSERT_X(d->data, "QDrag", "No mimedata set before starting the drag");
+    if (!d->data) {
+        qWarning("QDrag: No mimedata set before starting the drag");
+        return d->executed_action;
+    }
     QDragManager *manager = QDragManager::self();
+    d->defaultDropAction = Qt::IgnoreAction;
     d->possible_actions = request | Qt::CopyAction;
     if (manager)
         d->executed_action = manager->drag(this);
