@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -482,13 +497,13 @@ QWheelEvent::QWheelEvent(const QPoint &pos, const QPoint& globalPos, int delta,
 /*!
     \fn int QWheelEvent::delta() const
 
-    Returns the distance that the wheel is rotated, in eights of a
+    Returns the distance that the wheel is rotated, in eighths of a
     degree. A positive value indicates that the wheel was rotated
     forwards away from the user; a negative value indicates that the
     wheel was rotated backwards toward the user.
 
     Most mouse types work in steps of 15 degrees, in which case the
-    delta value is a multiple of 120 (== 15 * 8).
+    delta value is a multiple of 120; i.e., 120 * 1/8 = 15.
 
     Example:
 
@@ -944,14 +959,14 @@ Qt::FocusReason QFocusEvent::reason() const
 /*!
     \fn bool QFocusEvent::gotFocus() const
 
-    Returns true if type() is QEVent::FocusIn; otherwise returns
+    Returns true if type() is QEvent::FocusIn; otherwise returns
     false.
 */
 
 /*!
     \fn bool QFocusEvent::lostFocus() const
 
-    Returns true if type() is QEVent::FocusOut; otherwise returns
+    Returns true if type() is QEvent::FocusOut; otherwise returns
     false.
 */
 
@@ -1739,6 +1754,13 @@ void QInputMethodEvent::setCommitString(const QString &commitString, int replace
     accepted, it will send a mouse event. This allows applications that
     don't utilize tablets to use a tablet like a mouse, while also
     enabling those who want to use both tablets and mouses differently.
+
+    \section1 Notes for X11 Users
+
+    Qt uses the following hard-coded names to identify tablet
+    devices from the xorg.conf file on X11 (apart from IRIX):
+    'stylus', 'pen', and 'eraser'. If the devices have other names,
+    they will not be picked up Qt.
 */
 
 /*!
@@ -2887,7 +2909,7 @@ QFileOpenEvent::~QFileOpenEvent()
 
     The QToolBarChangeEvent is sent when the toolbar button is clicked. On Mac
     OS X, this is the long oblong button on the right side of the window
-    titlebar. The default implementation is to toggle the appearance (hidden or
+    title bar. The default implementation is to toggle the appearance (hidden or
     shown) of the associated toolbars for the window.
 */
 
