@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,8 +50,14 @@
 
 static int menuBarHeightForWidth(QWidget *menubar, int w)
 {
-    if (menubar && !menubar->isHidden() && !menubar->isWindow())
-        return menubar->heightForWidth(qMax(w, menubar->minimumWidth()));
+    if (menubar && !menubar->isHidden() && !menubar->isWindow()) {
+        int result = menubar->heightForWidth(qMax(w, menubar->minimumWidth()));
+        if (result != -1)
+            return result;
+        result = menubar->sizeHint().height();
+        if (result != -1)
+            return result;
+    }
     return 0;
 }
 
@@ -67,7 +88,8 @@ static int menuBarHeightForWidth(QWidget *menubar, int w)
 
     Geometry management stops when the layout manager is deleted.
 
-    \sa QLayoutItem, {Layout Classes}
+    \sa QLayoutItem, {Layout Classes}, {Basic Layouts Example},
+        {Border Layout Example}, {Flow Layout Example}
 */
 
 
@@ -103,13 +125,13 @@ QLayout::QLayout()
 QLayout::QLayout(QLayoutPrivate &dd, QLayout *lay, QWidget *w)
     : QObject(dd, lay ? static_cast<QObject*>(lay) : static_cast<QObject*>(w))
 {
-   Q_D(QLayout);
-     if (lay) {
+    Q_D(QLayout);
+    if (lay) {
         lay->addItem(this);
     } else if (w) {
         if (w->layout()) {
-            qWarning("Attempting to add QLayout \"%s\" to %s \"%s\", which already has a"
-                     " layout.",
+            qWarning("QLayout: Attempting to add QLayout \"%s\" to %s \"%s\", which"
+                     " already has a layout",
                      qPrintable(QObject::objectName()), w->metaObject()->className(),
                      w->objectName().toLocal8Bit().data());
             setParent(0);
@@ -122,14 +144,28 @@ QLayout::QLayout(QLayoutPrivate &dd, QLayout *lay, QWidget *w)
 }
 
 QLayoutPrivate::QLayoutPrivate()
-    : QObjectPrivate(), insideSpacing(-1), outsideBorder(-1), topLevel(false), enabled(true),
-      activated(true), autoNewChild(false), constraint(QLayout::SetDefaultConstraint)
-      , menubar(0)
+    : QObjectPrivate(), insideSpacing(-1), userLeftMargin(-1), userTopMargin(-1), userRightMargin(-1),
+      userBottomMargin(-1), topLevel(false), enabled(true), activated(true), autoNewChild(false),
+      constraint(QLayout::SetDefaultConstraint), menubar(0)
 {
 }
 
+void QLayoutPrivate::getMargin(int *result, int userMargin, QStyle::PixelMetric pm) const
+{
+    if (!result)
+        return;
 
-
+    Q_Q(const QLayout);
+    if (userMargin >= 0) {
+        *result = userMargin;
+    } else if (!topLevel) {
+        *result = 0;
+    } else if (QWidget *pw = q->parentWidget()) {
+        *result = pw->style()->pixelMetric(pm, 0, pw);
+    } else {
+        *result = 0;
+    }
+}
 
 #ifdef QT3_SUPPORT
 /*!
@@ -144,13 +180,15 @@ QLayoutPrivate::QLayoutPrivate()
 
     There can be only one top-level layout for a widget. It is
     returned by QWidget::layout()
+
+    \sa QWidget::setLayout()
 */
 QLayout::QLayout(QWidget *parent, int margin, int spacing, const char *name)
     : QObject(*new QLayoutPrivate,parent)
 {
     Q_D(QLayout);
-     setObjectName(QString::fromAscii(name));
-    d->outsideBorder = margin;
+    setObjectName(QString::fromAscii(name));
+    setMargin(margin);
     if (spacing < 0)
         d->insideSpacing = margin;
     else
@@ -223,8 +261,14 @@ bool QLayout::autoAdd() const { Q_D(const QLayout); return d->autoNewChild; }
     Implemented in subclasses to add an \a item. How it is added is
     specific to each subclass.
 
-    The ownership of \a item is transferred to the layout, and it's
+    This function is not usually called in application code. To add a widget
+    to a layout, use the addWidget() function; to add a child layout, use the
+    addLayout() function provided by the relevant QLayout subclass.
+
+    \bold{Note:} The ownership of \a item is transferred to the layout, and it's
     the layout's responsibility to delete it.
+
+    \sa addWidget(), QBoxLayout::addLayout(), QGridLayout::addLayout()
 */
 
 /*!
@@ -293,57 +337,148 @@ bool QLayout::setAlignment(QLayout *l, Qt::Alignment alignment)
 /*!
     \property QLayout::margin
     \brief the width of the outside border of the layout
+    \obsolete
 
-    \sa spacing
+    Use setContentsMargins() and getContentsMargins() instead.
+
+    \sa contentsRect(), spacing
 */
 
+/*!
+    \obsolete
+*/
+int QLayout::margin() const
+{
+    int left, top, right, bottom;
+    getContentsMargins(&left, &top, &right, &bottom);
+    if (left == top && top == right && right == bottom) {
+        return left;
+    } else {
+        return -1;
+    }
+}
+
 /*!
     \property QLayout::spacing
     \brief the spacing between widgets inside the layout
 
-    The default value is -1, which signifies that the layout's spacing
-    is inherited from the parent layout, or from the style settings for the parent widget.
+    If no value is explicitly set, the layout's spacing is inherited
+    from the parent layout, or from the style settings for the parent
+    widget.
 
-    \sa margin
+    For QGridLayout, it is possible to set different horizontal and
+    vertical spacings using \l{QGridLayout::}{setHorizontalSpacing()}
+    and \l{QGridLayout::}{setVerticalSpacing()}. In that case,
+    spacing() returns -1.
+
+    \sa contentsRect(), getContentsMargins(), QStyle::layoutSpacing(),
+        QStyle::pixelMetric()
 */
 
-
-int QLayout::margin() const
-{
-    Q_D(const QLayout);
-    if ( d->outsideBorder >= 0 )
-        return d->outsideBorder;
-    if (!d->topLevel)
-        return 0;
-    QWidget *pw = parentWidget();
-    if (pw)
-        return pw->style()->pixelMetric(
-            (pw->isWindow() || (pw->windowType() == Qt::SubWindow))
-            ? QStyle::PM_DefaultTopLevelMargin
-            : QStyle::PM_DefaultChildMargin
-            );
-    return 0;
-}
-
-
 int QLayout::spacing() const
 {
-    Q_D(const QLayout);
-    if (d->insideSpacing >=0) {
-        return d->insideSpacing;
-    } else if (d->topLevel) {
-        QWidget *pw = parentWidget();
-        if (pw)
-            return pw->style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
-        else
-            return QApplication::style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
-    } else if (parent()) {
-        return static_cast<QLayout*>(parent())->spacing();
+    if (const QBoxLayout* boxlayout = qobject_cast<const QBoxLayout*>(this)) {
+        return boxlayout->spacing();
+    } else if (const QGridLayout* gridlayout = qobject_cast<const QGridLayout*>(this)) {
+        return gridlayout->spacing();
     } else {
-        return -1; //this is a layout that hasn't been inserted yet
+        Q_D(const QLayout);
+        if (d->insideSpacing >=0) {
+            return d->insideSpacing;
+        } else {
+            // arbitrarily prefer horizontal spacing to vertical spacing
+            return qSmartSpacing(this, QStyle::PM_LayoutHorizontalSpacing);
+        }
     }
 }
 
+/*!
+    \obsolete
+*/
+void QLayout::setMargin(int margin)
+{
+    setContentsMargins(margin, margin, margin, margin);
+}
+
+void QLayout::setSpacing(int spacing)
+{
+    if (QBoxLayout* boxlayout = qobject_cast<QBoxLayout*>(this)) {
+        boxlayout->setSpacing(spacing);
+    } else if (QGridLayout* gridlayout = qobject_cast<QGridLayout*>(this)) {
+        gridlayout->setSpacing(spacing);
+    } else {
+        Q_D(QLayout);
+        d->insideSpacing = spacing;
+        invalidate();
+    }
+}
+
+/*!
+    \since 4.3
+
+    Sets the \a left, \a top, \a right, and \a bottom margins to use
+    around the layout.
+
+    By default, QLayout uses the values provided by the style. On
+    most platforms, the margin is 11 pixels in all directions.
+
+    \sa getContentsMargins(), QStyle::pixelMetric(),
+        {QStyle::}{PM_LayoutLeftMargin},
+        {QStyle::}{PM_LayoutTopMargin},
+        {QStyle::}{PM_LayoutRightMargin},
+        {QStyle::}{PM_LayoutBottomMargin}
+*/
+void QLayout::setContentsMargins(int left, int top, int right, int bottom)
+{
+    Q_D(QLayout);
+    d->userLeftMargin = left;
+    d->userTopMargin = top;
+    d->userRightMargin = right;
+    d->userBottomMargin = bottom;
+    invalidate();
+}
+
+/*!
+    \since 4.3
+
+    Extracts the left, top, right, and bottom margins used around the
+    layout, and assigns them to *\a left, *\a top, *\a right, and *\a
+    bottom (unless they are null pointers).
+
+    By default, QLayout uses the values provided by the style. On
+    most platforms, the margin is 11 pixels in all directions.
+
+    \sa setContentsMargins(), QStyle::pixelMetric(),
+        {QStyle::}{PM_LayoutLeftMargin},
+        {QStyle::}{PM_LayoutTopMargin},
+        {QStyle::}{PM_LayoutRightMargin},
+        {QStyle::}{PM_LayoutBottomMargin}
+*/
+void QLayout::getContentsMargins(int *left, int *top, int *right, int *bottom) const
+{
+    Q_D(const QLayout);
+    d->getMargin(left, d->userLeftMargin, QStyle::PM_LayoutLeftMargin);
+    d->getMargin(top, d->userTopMargin, QStyle::PM_LayoutTopMargin);
+    d->getMargin(right, d->userRightMargin, QStyle::PM_LayoutRightMargin);
+    d->getMargin(bottom, d->userBottomMargin, QStyle::PM_LayoutBottomMargin);
+}
+
+/*!
+    \since 4.3
+
+    Returns the layout's geometry() rectangle, but taking into account the
+    contents margins.
+
+    \sa setContentsMargins(), getContentsMargins()
+*/
+QRect QLayout::contentsRect() const
+{
+    Q_D(const QLayout);
+    int left, top, right, bottom;
+    getContentsMargins(&left, &top, &right, &bottom);
+    return d->rect.adjusted(+left, +top, -right, -bottom);
+}
+
 #ifdef QT3_SUPPORT
 bool QLayout::isTopLevel() const
 {
@@ -352,21 +487,6 @@ bool QLayout::isTopLevel() const
 }
 #endif
 
-void QLayout::setMargin(int margin)
-{
-    Q_D(QLayout);
-    d->outsideBorder = margin;
-    invalidate();
-}
-
-
-void QLayout::setSpacing(int spacing)
-{
-    Q_D(QLayout);
-    d->insideSpacing = spacing;
-    invalidate();
-}
-
 /*!
     Returns the parent widget of this layout, or 0 if this layout is
     not installed on any widget.
@@ -694,8 +814,13 @@ void QLayout::deleteAllItems()
 #endif
 
 /*!
-    This function is called from \c addLayout() functions in
+    This function is called from \c addLayout() or \c insertLayout() functions in
     subclasses to add layout \a l as a sub-layout.
+
+    The only scenario in which you need to call it directly is if you
+    implement a custom layout that supports nested layouts.
+
+    \sa QBoxLayout::addLayout(), QBoxLayout::insertLayout(), QGridLayout::addLayout()
 */
 void QLayout::addChildLayout(QLayout *l)
 {
@@ -712,6 +837,17 @@ void QLayout::addChildLayout(QLayout *l)
 
 }
 
+#ifdef QT_DEBUG
+static bool layoutDebug()
+{
+    static int checked_env = -1;
+    if(checked_env == -1)
+        checked_env = !!qgetenv("QT_LAYOUT_DEBUG").toInt();
+
+    return checked_env;
+}
+#endif
+
 void QLayoutPrivate::reparentChildWidgets(QWidget *mw)
 {
     Q_Q(QLayout);
@@ -722,18 +858,22 @@ void QLayoutPrivate::reparentChildWidgets(QWidget *mw)
         menubar->setParent(mw);
     }
 #endif
+    bool mwVisible = mw && mw->isVisible();
     for (int i = 0; i < n; ++i) {
         QLayoutItem *item = q->itemAt(i);
         if (QWidget *w = item->widget()) {
             QWidget *pw = w->parentWidget();
 #ifdef QT_DEBUG
-            if (pw && pw != mw) {
+            if (pw && pw != mw && layoutDebug()) {
                 qWarning("QLayout::addChildLayout: widget %s \"%s\" in wrong parent; moved to correct parent",
                          w->metaObject()->className(), w->objectName().toLocal8Bit().data());
             }
 #endif
+            bool needShow = mwVisible && !(w->isHidden() && w->testAttribute(Qt::WA_WState_ExplicitShowHide));
             if (pw != mw)
                 w->setParent(mw);
+            if (needShow)
+                QMetaObject::invokeMethod(w, "_q_showIfNotHidden", Qt::QueuedConnection); //show later
         } else if (QLayout *l = item->layout()) {
             l->d_func()->reparentChildWidgets(mw);
         }
@@ -757,13 +897,20 @@ void QLayout::addChildWidget(QWidget *w)
     //been in a layout.
     if (pw && w->testAttribute(Qt::WA_LaidOut)) {
         QLayout *l = pw->layout();
-        if (l && removeWidgetRecursively(l, w))
-            qWarning("QLayout::addChildWidget: %s \"%s\" is already in a layout; moved to new layout",
-                     w->metaObject()->className(), w->objectName().toLocal8Bit().data());
+        if (l && removeWidgetRecursively(l, w)) {
+#ifdef QT_DEBUG
+            if (layoutDebug())
+                qWarning("QLayout::addChildWidget: %s \"%s\" is already in a layout; moved to new layout",
+                         w->metaObject()->className(), w->objectName().toLocal8Bit().data());
+#endif
+        }
     }
     if (pw && mw && pw != mw) {
-        qWarning("QLayout::addChildWidget: %s \"%s\" in wrong parent; moved to correct parent",
-                 w->metaObject()->className(), w->objectName().toLocal8Bit().data());
+#ifdef QT_DEBUG
+            if (layoutDebug())
+                qWarning("QLayout::addChildWidget: %s \"%s\" in wrong parent; moved to correct parent",
+                         w->metaObject()->className(), w->objectName().toLocal8Bit().data());
+#endif
         pw = 0;
     }
     bool needShow = mw && mw->isVisible() && !(w->isHidden() && w->testAttribute(Qt::WA_WState_ExplicitShowHide));
@@ -771,7 +918,7 @@ void QLayout::addChildWidget(QWidget *w)
         w->setParent(mw);
     w->setAttribute(Qt::WA_LaidOut);
     if (needShow)
-        w->show();
+        QMetaObject::invokeMethod(w, "_q_showIfNotHidden", Qt::QueuedConnection); //show later
 }
 
 #ifdef QT3_SUPPORT
@@ -779,7 +926,7 @@ void QLayout::addChildWidget(QWidget *w)
   \compat
 
     Sets this layout's parent widget to a fixed size with width \a w
-    and height \a h, stopping the user form resizing it, and also
+    and height \a h, stopping the user from resizing it, and also
     prevents the layout from resizing it, even if the layout's size
     hint should change. Does nothing if this is not a top-level
     layout (i.e., if parent()->isWidgetType()).
@@ -933,7 +1080,8 @@ void QLayout::update()
     Redoes the layout for parentWidget() if necessary.
 
     You should generally not need to call this because it is
-    automatically called at the most appropriate times.
+    automatically called at the most appropriate times. It returns
+    true if the layout was redone.
 
     \sa update(), QWidget::updateGeometry()
 */
@@ -1032,7 +1180,7 @@ bool QLayout::activate()
                 for (int i = 0; i < layout->count(); ++i)
                     paintLayout(painter, layout->itemAt(i));
             }
-            painter->drawRect(lay->geometry());
+            painter->drawRect(layout->geometry());
         }
 
         void MyWidget::paintEvent(QPaintEvent *)
@@ -1099,7 +1247,6 @@ int QLayout::indexOf(QWidget *widget) const
     return -1;
 }
 
-
 /*!
     \enum QLayout::SizeConstraint
 
@@ -1211,6 +1358,9 @@ QRect QLayout::alignmentRect(const QRect &r) const
     Removes the widget \a widget from the layout. After this call, it
     is the caller's responsibility to give the widget a reasonable
     geometry or to put the widget back into a layout.
+    
+    \bold{Note:} The ownership of \a widget remains the same as
+    when it was added.
 
     \sa removeItem(), QWidget::setGeometry(), addWidget()
 */
@@ -1336,3 +1486,63 @@ QSize QLayout::closestAcceptableSize(const QWidget *widget, const QSize &size)
     Use sizeConstraint() instead.
 */
 
+void QSizePolicy::setControlType(ControlType type)
+{
+    /*
+        The control type is a flag type, with values 0x1, 0x2, 0x4, 0x8, 0x10,
+        etc. In memory, we pack it onto the available bits (CTSize) in
+        setControlType(), and unpack it here.
+
+        Example:
+
+            0x00000001 maps to 0x00000000
+            0x00000002 maps to 0x00000200
+            0x00000004 maps to 0x00000400
+            0x00000008 maps to 0x00000600
+            etc.
+    */
+
+    int i = 0;
+    while (true) {
+        if (type & (0x1 << i)) {
+            data = (data & ~CTMask) | (i << CTShift);
+            return;
+        }
+        ++i;
+    }
+}
+
+QSizePolicy::ControlType QSizePolicy::controlType() const
+{
+    return QSizePolicy::ControlType(0x1 << ((data & CTMask) >> CTShift));
+}
+
+#ifndef QT_NO_DATASTREAM
+/*!
+    \relates QSizePolicy
+    \since 4.2
+
+    Writes the size \a policy to the data stream \a stream.
+
+    \sa \link datastreamformat.html Format of the QDataStream operators \endlink
+*/
+QDataStream &operator<<(QDataStream &stream, const QSizePolicy &policy)
+{
+    return stream << policy.data;
+}
+
+/*!
+    \relates QSizePolicy
+    \since 4.2
+
+    Reads the size \a policy from the data stream \a stream.
+
+    \sa \link datastreamformat.html Format of the QDataStream operators \endlink
+*/
+QDataStream &operator>>(QDataStream &stream, QSizePolicy &policy)
+{
+    return stream >> policy.data;
+}
+
+#endif
+
