@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,10 +44,11 @@
 static int qt_palette_count = 1;
 class QPalettePrivate {
 public:
-    QPalettePrivate() : ref(1), ser_no(qt_palette_count++) { }
+    QPalettePrivate() : ref(1), ser_no(qt_palette_count++), detach_no(0) { }
     QAtomic ref;
     QBrush br[QPalette::NColorGroups][QPalette::NColorRoles];
     int ser_no;
+    int detach_no;
 };
 
 static QColor qt_mix_colors(QColor a, QColor b)
@@ -134,40 +150,43 @@ void QPalette::setColorGroup(ColorGroup cg, const QColorGroup &g)
 #endif
 
 /*!
-   \fn const QColor &QPalette::color(ColorRole r) const
+   \fn const QColor &QPalette::color(ColorRole role) const
 
    \overload
 
-    Returns the color that has been set for color role \a r in the current ColorGroup.
+    Returns the color that has been set for the given color \a role in
+    the current ColorGroup.
 
     \sa brush() ColorRole
  */
 
 /*!
-    \fn const QBrush &QPalette::brush(ColorRole r) const
+    \fn const QBrush &QPalette::brush(ColorRole role) const
 
     \overload
 
-    Returns the brush that has been set for color role \a r in the current ColorGroup.
+    Returns the brush that has been set for the given color \a role in
+    the current ColorGroup.
 
     \sa color() setBrush() ColorRole
 */
 
 /*!
-    \fn void QPalette::setColor(ColorRole r, const QColor &c)
+    \fn void QPalette::setColor(ColorRole role, const QColor &color)
 
     \overload
 
-    Sets the color used for color role \a r in all color groups to a solid color \a c.
+    Sets the color used for the given color \a role, in all color
+    groups, to the specified solid \a color.
 
     \sa brush() setColor() ColorRole
 */
 
 /*!
-    \fn void QPalette::setBrush(ColorRole cr, const QBrush &brush)
+    \fn void QPalette::setBrush(ColorRole role, const QBrush &brush)
 
-    Sets the brush for color role \a cr to brush \a brush for all
-    groups in the palette.
+    Sets the brush for the given color \a role to the specified \a
+    brush for all groups in the palette.
 
     \sa brush() setColor() ColorRole
 */
@@ -375,8 +394,10 @@ void QPalette::setColorGroup(ColorGroup cg, const QColorGroup &g)
     "base" rather than literal colors like "red" or "turquoise". The color
     roles are enumerated and defined in the \l ColorRole documentation.
 
-    We strongly recommend that you use a system-supplied color group
-    and modify that as necessary.
+				We strongly recommend that you use the default palette of the
+				current style (returned by QApplication::palette()) and
+				modify that as necessary. This is done by Qt's widgets when they
+				are drawn.
 
     To modify a color group you call the functions
     setColor() and setBrush(), depending on whether you want a pure
@@ -384,7 +405,7 @@ void QPalette::setColorGroup(ColorGroup cg, const QColorGroup &g)
 
     There are also corresponding color() and brush() getters, and a
     commonly used convenience function to get the ColorRole for the current ColorGroup:
-    background(), foreground(), base(), etc.
+    window(), windowText(), base(), etc.
 
     You can copy a palette using the copy constructor and test to see
     if two palettes are \e identical using isCopyOf().
@@ -529,14 +550,14 @@ static void qt_palette_from_color(QPalette &pal, const QColor & button)
         base = Qt::black;
     }
     //inactive and active are the same..
-    pal.setColorGroup(QPalette::Active, QBrush(fg), QBrush(btn), QBrush(btn.light(150)),
-                      QBrush(btn.dark()), QBrush(btn.dark(150)), QBrush(fg), QBrush(Qt::white),
+    pal.setColorGroup(QPalette::Active, QBrush(fg), QBrush(btn), QBrush(btn.lighter(150)),
+                      QBrush(btn.darker()), QBrush(btn.darker(150)), QBrush(fg), QBrush(Qt::white),
                       QBrush(base), QBrush(bg));
-    pal.setColorGroup(QPalette::Inactive, QBrush(fg), QBrush(btn), QBrush(btn.light(150)),
-                      QBrush(btn.dark()), QBrush(btn.dark(150)), QBrush(fg), QBrush(Qt::white),
+    pal.setColorGroup(QPalette::Inactive, QBrush(fg), QBrush(btn), QBrush(btn.lighter(150)),
+                      QBrush(btn.darker()), QBrush(btn.darker(150)), QBrush(fg), QBrush(Qt::white),
                       QBrush(base), QBrush(bg));
-    pal.setColorGroup(QPalette::Disabled, QBrush(btn.dark()), QBrush(btn), QBrush(btn.light(150)),
-                      QBrush(btn.dark()), QBrush(btn.dark(150)), QBrush(btn.dark()),
+    pal.setColorGroup(QPalette::Disabled, QBrush(btn.darker()), QBrush(btn), QBrush(btn.lighter(150)),
+                      QBrush(btn.darker()), QBrush(btn.darker(150)), QBrush(btn.darker()),
                       QBrush(Qt::white), QBrush(bg), QBrush(bg));
 }
 
@@ -619,14 +640,14 @@ QPalette::QPalette(const QColor &button, const QColor &window)
         disfg = Qt::darkGray;
     }
     //inactive and active are identical
-    setColorGroup(Inactive, QBrush(fg), QBrush(btn), QBrush(btn.light(150)), QBrush(btn.dark()),
-                  QBrush(btn.dark(150)), QBrush(fg), QBrush(Qt::white), QBrush(base),
+    setColorGroup(Inactive, QBrush(fg), QBrush(btn), QBrush(btn.lighter(150)), QBrush(btn.darker()),
+                  QBrush(btn.darker(150)), QBrush(fg), QBrush(Qt::white), QBrush(base),
                   QBrush(bg));
-    setColorGroup(Active, QBrush(fg), QBrush(btn), QBrush(btn.light(150)), QBrush(btn.dark()),
-                  QBrush(btn.dark(150)), QBrush(fg), QBrush(Qt::white), QBrush(base),
+    setColorGroup(Active, QBrush(fg), QBrush(btn), QBrush(btn.lighter(150)), QBrush(btn.darker()),
+                  QBrush(btn.darker(150)), QBrush(fg), QBrush(Qt::white), QBrush(base),
                   QBrush(bg));
-    setColorGroup(Disabled, QBrush(disfg), QBrush(btn), QBrush(btn.light(150)),
-                  QBrush(btn.dark()), QBrush(btn.dark(150)), QBrush(disfg),
+    setColorGroup(Disabled, QBrush(disfg), QBrush(btn), QBrush(btn.lighter(150)),
+                  QBrush(btn.darker()), QBrush(btn.darker(150)), QBrush(disfg),
                   QBrush(Qt::white), QBrush(base), QBrush(bg));
 }
 
@@ -686,14 +707,19 @@ QPalette::operator QVariant() const
 }
 
 /*!
-    \fn const QColor &QPalette::color(ColorGroup gr, ColorRole r) const
-    Returns the color in color group \a gr, used for color role \a r.
+    \fn const QColor &QPalette::color(ColorGroup group, ColorRole role) const
+
+    Returns the color in the specified color \a group, used for the
+    given color \a role.
 
     \sa brush() setColor() ColorRole
 */
 
 /*!
-    Returns the brush in color group \a gr, used for color role \a cr.
+    \fn const QBrush &QPalette::brush(ColorGroup group, ColorRole role) const
+
+    Returns the brush in the specified color \a group, used for the
+    given color \a role.
 
     \sa color() setBrush() ColorRole
 */
@@ -712,18 +738,20 @@ const QBrush &QPalette::brush(ColorGroup gr, ColorRole cr) const
 }
 
 /*!
-    \fn void QPalette::setColor(ColorGroup gr, ColorRole r, const QColor &c)
-    Sets the brush in color group \a gr, used for color role \a r, to
-    the solid color \a c.
+    \fn void QPalette::setColor(ColorGroup group, ColorRole role, const QColor &color)
+
+    Sets the brush in the specified color \a group, used for the given
+    color \a role, to the specified solid \a color.
 
     \sa setBrush() color() ColorRole
 */
 
 /*!
+    \fn void QPalette::setBrush(ColorGroup group, ColorRole role, const QBrush &brush)
     \overload
 
-    Sets the brush in color group \a cg, used for color role \a cr, to
-    \a b.
+    Sets the brush in the specified color \a group, used for the given
+    color \a role, to \a brush.
 
     \sa brush() setColor() ColorRole
 */
@@ -777,6 +805,7 @@ void QPalette::detach()
         if(!x->ref.deref())
             delete x;
     }
+    ++d->detach_no;
 }
 
 /*!
@@ -857,13 +886,16 @@ bool QPalette::isEqual(QPalette::ColorGroup group1, QPalette::ColorGroup group2)
     return true;
 }
 
-/*!
+/*! \obsolete
+
     Returns a number that identifies the contents of this QPalette
     object. Distinct QPalette objects can only have the same serial
     number if they refer to the same contents (but they don't have
     to). Also, the serial number of a QPalette may change during the
     lifetime of the object.
 
+    Use cacheKey() instead.
+
     \warning The serial number doesn't necessarily change when the
     palette is altered. This means that it may be dangerous to use it
     as a cache key.
@@ -875,6 +907,18 @@ int QPalette::serialNumber() const
     return d->ser_no;
 }
 
+/*!
+    Returns a number that identifies the contents of this QPalette
+    object. Distinct QPalette objects can have the same key if
+    they refer to the same contents.
+
+    The cacheKey() will change when the palette is altered.
+*/
+qint64 QPalette::cacheKey() const
+{
+    return (((qint64) d->ser_no) << 32) | ((qint64) (d->detach_no));
+}
+
 /*!
     Returns a new QPalette that has attributes copied from \a other.
 */
@@ -1111,6 +1155,7 @@ QPalette::setColorGroup(ColorGroup cg, const QBrush &foreground, const QBrush &b
 */
 
 /*! \class QColorGroup
+    \brief The QColorGroup class contains color groups for each widget state.
     \compat
 */
 
