@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -33,11 +48,12 @@
 #include "qshortcut.h"
 #include "qapplication_p.h"
 #include <private/qaction_p.h>
+#include <private/qkeymapper_p.h>
 
 #ifndef QT_NO_SHORTCUT
 
 // To enable verbose output uncomment below
-//#define Debug_QShortcutMap
+//#define DEBUG_QSHORTCUTMAP
 
 /* \internal
     Entry data for QShortcutMap
@@ -48,11 +64,19 @@
 struct QShortcutEntry
 {
     QShortcutEntry()
-        : keyseq(0), context(Qt::WindowShortcut), enabled(false), id(0), owner(0)
+        : keyseq(0), context(Qt::WindowShortcut), enabled(false), autorepeat(1), id(0), owner(0)
+    {}
+
+    QShortcutEntry(const QKeySequence &k)
+        : keyseq(k), context(Qt::WindowShortcut), enabled(false), autorepeat(1), id(0), owner(0)
     {}
 
     QShortcutEntry(QObject *o, const QKeySequence &k, Qt::ShortcutContext c, int i)
-        : keyseq(k), context(c), enabled(true), id(i), owner(o)
+        : keyseq(k), context(c), enabled(true), autorepeat(1), id(i), owner(o)
+    {}
+
+    QShortcutEntry(QObject *o, const QKeySequence &k, Qt::ShortcutContext c, int i, bool a)
+        : keyseq(k), context(c), enabled(true), autorepeat(a), id(i), owner(o)
     {}
 
     bool operator<(const QShortcutEntry &f) const
@@ -61,7 +85,8 @@ struct QShortcutEntry
     QKeySequence keyseq;
     Qt::ShortcutContext context;
     bool enabled : 1;
-    signed int id : 31;
+    bool autorepeat : 1;
+    signed int id;
     QObject *owner;
 };
 
@@ -74,7 +99,8 @@ QDebug &operator<<(QDebug &dbg, const QShortcutEntry *se) {
         return dbg << "QShortcutEntry(0x0)";
     dbg.nospace()
         << "QShortcutEntry(" << se->keyseq
-        << "), id(" << se->id << "), enabled(" << se->enabled << ") owner(" << se->owner << ")";
+        << "), id(" << se->id << "), enabled(" << se->enabled << "), autorepeat(" << se->autorepeat
+        << "), owner(" << se->owner << ")";
     return dbg.space();
 }
 #endif // QT_NO_DEBUGSTREAM
@@ -89,7 +115,10 @@ class QShortcutMapPrivate
 public:
     QShortcutMapPrivate(QShortcutMap* parent)
         : q_ptr(parent), currentId(0), ambigCount(0), currentState(QKeySequence::NoMatch)
-    { identicals.reserve(10); }
+    {
+        identicals.reserve(10);
+        currentSequences.reserve(10);
+    }
     QShortcutMap *q_ptr;                        // Private's parent
 
     QList<QShortcutEntry> sequences;            // All sequences!
@@ -97,7 +126,7 @@ public:
     int currentId;                              // Global shortcut ID number
     int ambigCount;                             // Index of last enabled ambiguous dispatch
     QKeySequence::SequenceMatch currentState;
-    QKeySequence currentSequence;               // Sequence for the current state
+    QVector<QKeySequence> currentSequences;     // Sequence for the current state
     QKeySequence prevSequence;                  // Sequence for the previous identical match
     QVector<const QShortcutEntry*> identicals;  // Last identical matches
 };
@@ -131,11 +160,11 @@ int QShortcutMap::addShortcut(QObject *owner, const QKeySequence &key, Qt::Short
     Q_ASSERT_X(owner, "QShortcutMap::addShortcut", "All shortcuts need an owner");
     Q_ASSERT_X(!key.isEmpty(), "QShortcutMap::addShortcut", "Cannot add keyless shortcuts to map");
     Q_D(QShortcutMap);
-    
-    QShortcutEntry newEntry(owner, key, context, --(d->currentId));
+
+    QShortcutEntry newEntry(owner, key, context, --(d->currentId), true);
     QList<QShortcutEntry>::iterator it = qUpperBound(d->sequences.begin(), d->sequences.end(), newEntry);
     d->sequences.insert(it, newEntry); // Insert sorted
-#if defined(Debug_QShortcutMap)
+#if defined(DEBUG_QSHORTCUTMAP)
     qDebug().nospace()
         << "QShortcutMap::addShortcut(" << owner << ", "
         << key << ", " << context << ") = " << d->currentId;
@@ -145,7 +174,7 @@ int QShortcutMap::addShortcut(QObject *owner, const QKeySequence &key, Qt::Short
 
 /*! \internal
     Removes a shortcut from the global map.
-    If \a owner is 0, all entries in the map with the keysequence specified
+    If \a owner is 0, all entries in the map with the key sequence specified
     is removed. If \a key is null, all sequences for \a owner is removed from
     the map. If \a id is 0, any identical \a key sequences owned by \a owner
     are removed.
@@ -182,7 +211,7 @@ int QShortcutMap::removeShortcut(int id, QObject *owner, const QKeySequence &key
             return itemsRemoved;
         --i;
     }
-#if defined(Debug_QShortcutMap)
+#if defined(DEBUG_QSHORTCUTMAP)
     qDebug().nospace()
         << "QShortcutMap::removeShortcut(" << id << ", " << owner << ", "
         << key << ") = " << itemsRemoved;
@@ -192,7 +221,7 @@ int QShortcutMap::removeShortcut(int id, QObject *owner, const QKeySequence &key
 
 /*! \internal
     Changes the enable state of a shortcut to \a enable.
-    If \a owner is 0, all entries in the map with the keysequence specified
+    If \a owner is 0, all entries in the map with the key sequence specified
     is removed. If \a key is null, all sequences for \a owner is removed from
     the map. If \a id is 0, any identical \a key sequences owned by \a owner
     are changed.
@@ -220,7 +249,7 @@ int QShortcutMap::setShortcutEnabled(bool enable, int id, QObject *owner, const
             return itemsChanged;
         --i;
     }
-#if defined(Debug_QShortcutMap)
+#if defined(DEBUG_QSHORTCUTMAP)
     qDebug().nospace()
         << "QShortcutMap::setShortcutEnabled(" << enable << ", " << id << ", "
         << owner << ", " << key << ") = " << itemsChanged;
@@ -228,6 +257,43 @@ int QShortcutMap::setShortcutEnabled(bool enable, int id, QObject *owner, const
     return itemsChanged;
 }
 
+/*! \internal
+    Changes the auto repeat state of a shortcut to \a enable.
+    If \a owner is 0, all entries in the map with the key sequence specified
+    is removed. If \a key is null, all sequences for \a owner is removed from
+    the map. If \a id is 0, any identical \a key sequences owned by \a owner
+    are changed.
+    Returns the number of sequences which are matched in the map.
+*/
+int QShortcutMap::setShortcutAutoRepeat(bool on, int id, QObject *owner, const QKeySequence &key)
+{
+    Q_D(QShortcutMap);
+    int itemsChanged = 0;
+    bool allOwners = (owner == 0);
+    bool allKeys = key.isEmpty();
+    bool allIds = id == 0;
+
+    int i = d->sequences.size()-1;
+    while (i>=0)
+    {
+        QShortcutEntry entry = d->sequences.at(i);
+        if ((allOwners || entry.owner == owner)
+            && (allIds || entry.id == id)
+            && (allKeys || entry.keyseq == key)) {
+                d->sequences[i].autorepeat = on;
+                ++itemsChanged;
+        }
+        if (id == entry.id)
+            return itemsChanged;
+        --i;
+    }
+#if defined(DEBUG_QSHORTCUTMAP)
+    qDebug().nospace()
+        << "QShortcutMap::setShortcutAutoRepeat(" << on << ", " << id << ", "
+        << owner << ", " << key << ") = " << itemsChanged;
+#endif
+    return itemsChanged;
+}
 
 /*! \internal
     Resets the state of the statemachine to NoMatch
@@ -236,7 +302,7 @@ void QShortcutMap::resetState()
 {
     Q_D(QShortcutMap);
     d->currentState = QKeySequence::NoMatch;
-    clearSequence(d->currentSequence);
+    clearSequence(d->currentSequences);
 }
 
 /*! \internal
@@ -257,6 +323,7 @@ QKeySequence::SequenceMatch QShortcutMap::state()
 bool QShortcutMap::tryShortcutEvent(QWidget *w, QKeyEvent *e)
 {
     Q_D(QShortcutMap);
+
     bool wasAccepted = e->isAccepted();
     if (d->currentState == QKeySequence::NoMatch) {
         ushort orgType = e->t;
@@ -285,7 +352,7 @@ bool QShortcutMap::tryShortcutEvent(QWidget *w, QKeyEvent *e)
         return stateWasAccepted;
     case QKeySequence::ExactMatch:
         resetState();
-        dispatchEvent();
+        dispatchEvent(e);
     default:
 	break;
     }
@@ -320,12 +387,6 @@ QKeySequence::SequenceMatch QShortcutMap::nextState(QKeyEvent *e)
             QKeyEvent pe = QKeyEvent(e->type(), Qt::Key_Tab, e->modifiers(), e->text());
             result = find(&pe);
         }
-        // If still no result, try removing the Shift modifier
-        if (result == QKeySequence::NoMatch) {
-            QKeyEvent pe = QKeyEvent(e->type(), e->key(),
-                                     e->modifiers()&~Qt::ShiftModifier, e->text());
-            result = find(&pe);
-        }
     }
 
     // Should we eat this key press?
@@ -334,10 +395,10 @@ QKeySequence::SequenceMatch QShortcutMap::nextState(QKeyEvent *e)
         e->accept();
     // Does the new state require us to clean up?
     if (result == QKeySequence::NoMatch)
-        clearSequence(d->currentSequence);
+        clearSequence(d->currentSequences);
     d->currentState = result;
 
-#if defined(Debug_QShortcutMap)
+#if defined(DEBUG_QSHORTCUTMAP)
     qDebug().nospace() << "QShortcutMap::nextState(" << e << ") = " << result;
 #endif
     return result;
@@ -355,12 +416,15 @@ QKeySequence::SequenceMatch QShortcutMap::find(QKeyEvent *e)
     Q_D(QShortcutMap);
     if (!d->sequences.count())
         return QKeySequence::NoMatch;
-    
-    static QShortcutEntry newEntry;
-    createNewSequence(e, newEntry.keyseq);
+
+    static QVector<QKeySequence> newEntries;
+    createNewSequences(e, newEntries);
+#if defined(DEBUG_QSHORTCUTMAP)
+    qDebug() << "Possible shortcut key sequences:" << newEntries;
+#endif
 
     // Should never happen
-    if (newEntry.keyseq == d->currentSequence) {
+    if (newEntries == d->currentSequences) {
         Q_ASSERT_X(e->key() != Qt::Key_unknown || e->text().length(),
                    "QShortcutMap::find", "New sequence to find identical to previous");
         return QKeySequence::NoMatch;
@@ -369,34 +433,59 @@ QKeySequence::SequenceMatch QShortcutMap::find(QKeyEvent *e)
     // Looking for new identicals, scrap old
     d->identicals.resize(0);
 
-    QList<QShortcutEntry>::ConstIterator itEnd = d->sequences.constEnd();
-    QList<QShortcutEntry>::ConstIterator it =
-        qLowerBound(d->sequences.constBegin(), itEnd, newEntry);
-
     bool partialFound = false;
     bool identicalDisabledFound = false;
-    QKeySequence::SequenceMatch result = QKeySequence::NoMatch;
-    do {
-        if (it == itEnd)
-            break;
-        result = newEntry.keyseq.matches((*it).keyseq);
-        if (result != QKeySequence::NoMatch && correctContext(*it)) {
-            if (result == QKeySequence::ExactMatch) {
-                if ((*it).enabled)
-                    d->identicals.append(&*it);
-                else
-                    identicalDisabledFound = true;
-            } else if (result == QKeySequence::PartialMatch) {
-                // We don't need partials, if we have identicals
-                if (d->identicals.size())
-                    break;
-                // We only care about enabled partials, so we don't consume
-                // key events when all partials are disabled!
-                partialFound |= (*it).enabled;
+    QVector<QKeySequence> okEntries;
+    int result = QKeySequence::NoMatch;
+    for (int i = newEntries.count()-1; i >= 0 ; --i) {
+        QShortcutEntry entry(newEntries.at(i)); // needed for searching
+        QList<QShortcutEntry>::ConstIterator itEnd = d->sequences.constEnd();
+        QList<QShortcutEntry>::ConstIterator it =
+             qLowerBound(d->sequences.constBegin(), itEnd, entry);
+
+        int oneKSResult = QKeySequence::NoMatch;
+        int tempRes = QKeySequence::NoMatch;
+        do {
+            if (it == itEnd)
+                break;
+            tempRes = matches(entry.keyseq, (*it).keyseq);
+            oneKSResult = qMax(oneKSResult, tempRes);
+            if (tempRes != QKeySequence::NoMatch && correctContext(*it)) {
+                if (tempRes == QKeySequence::ExactMatch) {
+                    if ((*it).enabled)
+                        d->identicals.append(&*it);
+                    else
+                        identicalDisabledFound = true;
+                } else if (tempRes == QKeySequence::PartialMatch) {
+                    // We don't need partials, if we have identicals
+                    if (d->identicals.size())
+                        break;
+                    // We only care about enabled partials, so we don't consume
+                    // key events when all partials are disabled!
+                    partialFound |= (*it).enabled;
+                }
             }
+            ++it;
+            // If we got a valid match on this run, there might still be more keys to check against,
+            // so we'll loop once more. If we get NoMatch, there's guaranteed no more possible
+            // matches in the shortcutmap.
+        } while (tempRes != QKeySequence::NoMatch);
+
+        // If the type of match improves (ergo, NoMatch->Partial, or Partial->Exact), clear the
+        // previous list. If this match is equal or better than the last match, append to the list
+        if (oneKSResult > result) {
+            okEntries.clear();
+#if defined(DEBUG_QSHORTCUTMAP)
+            qDebug() << "Found better match (" << newEntries << "), clearing key sequence list";
+#endif
         }
-        ++it;
-    } while (result != QKeySequence::NoMatch);
+        if (oneKSResult && oneKSResult >= result) {
+            okEntries << newEntries.at(i);
+#if defined(DEBUG_QSHORTCUTMAP)
+            qDebug() << "Added ok key sequence" << newEntries;
+#endif
+        }
+    }
 
     if (d->identicals.size()) {
         result = QKeySequence::ExactMatch;
@@ -405,12 +494,15 @@ QKeySequence::SequenceMatch QShortcutMap::find(QKeyEvent *e)
     } else if (identicalDisabledFound) {
         result = QKeySequence::ExactMatch;
     } else {
-        clearSequence(d->currentSequence);
+        clearSequence(d->currentSequences);
         result = QKeySequence::NoMatch;
     }
     if (result != QKeySequence::NoMatch)
-        d->currentSequence = newEntry.keyseq;
-    return result;
+        d->currentSequences = okEntries;
+#if defined(DEBUG_QSHORTCUTMAP)
+    qDebug() << "Returning shortcut match == " << result;
+#endif
+    return QKeySequence::SequenceMatch(result);
 }
 
 /*! \internal
@@ -420,34 +512,82 @@ QKeySequence::SequenceMatch QShortcutMap::find(QKeyEvent *e)
         key = QKeySequence();
     \endcode
 */
-void QShortcutMap::clearSequence(QKeySequence &seq)
+void QShortcutMap::clearSequence(QVector<QKeySequence> &ksl)
 {
-    seq.setKey(0, 0);
-    seq.setKey(0, 1);
-    seq.setKey(0, 2);
-    seq.setKey(0, 3);
+    ksl.clear();
 }
 
 /*! \internal
     Alters \a seq to the new sequence state, based on the
     current sequence state, and the new key event \a e.
 */
-void QShortcutMap::createNewSequence(QKeyEvent *e, QKeySequence &seq)
+void QShortcutMap::createNewSequences(QKeyEvent *e, QVector<QKeySequence> &ksl)
 {
     Q_D(QShortcutMap);
-    seq.setKey(d->currentSequence[0], 0);
-    seq.setKey(d->currentSequence[1], 1);
-    seq.setKey(d->currentSequence[2], 2);
-    seq.setKey(d->currentSequence[3], 3);
-    int index = d->currentSequence.count();
-    int modifier = translateModifiers(e->modifiers());
+    QList<int> possibleKeys = QKeyMapper::possibleKeys(e);
+    int pkTotal = possibleKeys.count();
+    if (!pkTotal)
+        return;
 
-    // Use the key code, if possible to prevent Ctrl+<Key> text problems,
-    // else use unicode of text
-    if (e->key() && e->key() != Qt::Key_unknown)
-        seq.setKey(e->key() | modifier, index);
-    else
-        seq.setKey((int)e->text().unicode()->toUpper().unicode() | modifier, index);
+    int ssActual = d->currentSequences.count();
+    int ssTotal = qMax(1, ssActual);
+    // Resize to possible permutations of the current sequence(s).
+    ksl.resize(pkTotal * ssTotal);
+
+    int index = ssActual ? d->currentSequences.at(0).count() : 0;
+    for (int pkNum = 0; pkNum < pkTotal; ++pkNum) {
+        for (int ssNum = 0; ssNum < ssTotal; ++ssNum) {
+            int i = (pkNum * ssTotal) + ssNum;
+            QKeySequence &curKsl = ksl[i];
+            if (ssActual) {
+                const QKeySequence &curSeq = d->currentSequences.at(ssNum);
+                curKsl.setKey(curSeq[0], 0);
+                curKsl.setKey(curSeq[1], 1);
+                curKsl.setKey(curSeq[2], 2);
+                curKsl.setKey(curSeq[3], 3);
+            } else {
+                curKsl.setKey(0, 0);
+                curKsl.setKey(0, 1);
+                curKsl.setKey(0, 2);
+                curKsl.setKey(0, 3);
+            }
+            // Filtering keycode here with 0xdfffffff to ignore the Keypad modifier
+            curKsl.setKey(possibleKeys.at(pkNum) & 0xdfffffff, index);
+        }
+    }
+}
+
+/*! \internal
+    Basically the same function as QKeySequence::matches(const QKeySequence &seq) const
+    only that is specially handles Key_hyphen as Key_Minus, as people mix these up all the time and
+    they conceptually the same.
+*/
+QKeySequence::SequenceMatch QShortcutMap::matches(const QKeySequence &seq1,
+                                                  const QKeySequence &seq2) const
+{
+    uint userN = seq1.count(),
+        seqN = seq2.count();
+
+    if (userN > seqN)
+        return QKeySequence::NoMatch;
+
+    // If equal in length, we have a potential ExactMatch sequence,
+    // else we already know it can only be partial.
+    QKeySequence::SequenceMatch match = (userN == seqN
+                                            ? QKeySequence::ExactMatch
+                                            : QKeySequence::PartialMatch);
+
+    for (uint i = 0; i < userN; ++i) {
+        int userKey = seq1[i],
+            sequenceKey = seq2[i];
+        if ((userKey & Qt::Key_unknown) == Qt::Key_hyphen)
+            userKey = userKey & Qt::KeyboardModifierMask | Qt::Key_Minus;
+        if ((sequenceKey & Qt::Key_unknown) == Qt::Key_hyphen)
+            sequenceKey = sequenceKey & Qt::KeyboardModifierMask | Qt::Key_Minus;
+        if (userKey != sequenceKey)
+            return QKeySequence::NoMatch;
+    }
+    return match;
 }
 
 /*! \internal
@@ -464,10 +604,10 @@ bool QShortcutMap::correctContext(const QShortcutEntry &item) {
     // for the shortcut system.
     if (qApp->activePopupWidget())
         active_window = qApp->activePopupWidget();
-    
+
     if (!active_window)
         return false;
-#ifndef QT_NO_ACTION 
+#ifndef QT_NO_ACTION
     if (QAction *a = qobject_cast<QAction *>(item.owner))
         return correctContext(item.context, a, active_window);
 #endif
@@ -513,7 +653,7 @@ bool QShortcutMap::correctContext(Qt::ShortcutContext context, QWidget *w, QWidg
         return sw == focus_widget;
     }
 
-#if defined(Debug_QShortcutMap)
+#if defined(DEBUG_QSHORTCUTMAP)
     qDebug().nospace() << "..true [Pass-through]";
 #endif
     return true;
@@ -523,6 +663,10 @@ bool QShortcutMap::correctContext(Qt::ShortcutContext context, QWidget *w, QWidg
 bool QShortcutMap::correctContext(Qt::ShortcutContext context, QAction *a, QWidget *active_window)
 {
     const QList<QWidget *> &widgets = a->d_func()->widgets;
+#if defined(DEBUG_QSHORTCUTMAP)
+    if (widgets.isEmpty())
+        qDebug() << a << "not connected to any widgets; won't trigger";
+#endif
     for (int i = 0; i < widgets.size(); ++i) {
         QWidget *w = widgets.at(i);
 #ifndef QT_NO_MENU
@@ -568,7 +712,7 @@ QVector<const QShortcutEntry*> QShortcutMap::matches() const
 /*! \internal
     Dispatches QShortcutEvents to widgets who grabbed the matched key sequence.
 */
-void QShortcutMap::dispatchEvent()
+void QShortcutMap::dispatchEvent(QKeyEvent *e)
 {
     Q_D(QShortcutMap);
     if (!d->identicals.size())
@@ -593,21 +737,23 @@ void QShortcutMap::dispatchEvent()
         ++i;
     }
     d->ambigCount = (d->identicals.size() == i ? 0 : d->ambigCount + 1);
-    if (!next)
+    // Don't trigger shortcut if we're autorepeating and the shortcut is
+    // grabbed with not accepting autorepeats.
+    if (!next || (e->isAutoRepeat() && !next->autorepeat))
         return;
     // Dispatch next enabled
-#if defined(Debug_QShortcutMap)
+#if defined(DEBUG_QSHORTCUTMAP)
     qDebug().nospace()
         << "QShortcutMap::dispatchEvent(): Sending QShortcutEvent(\""
-        << (QString)curKey << "\", " << next->id << ", "
+        << (QString)next->keyseq << "\", " << next->id << ", "
         << (bool)(enabledShortcuts>1) << ") to object(" << next->owner << ")";
 #endif
-    QShortcutEvent se(curKey, next->id, enabledShortcuts>1);
+    QShortcutEvent se(next->keyseq, next->id, enabledShortcuts>1);
     QApplication::sendEvent(const_cast<QObject *>(next->owner), &se);
 }
 
 /* \internal
-    QShortcutMap dump function, only available when Debug_QShortcutMap is
+    QShortcutMap dump function, only available when DEBUG_QSHORTCUTMAP is
     defined.
 */
 #if defined(Dump_QShortcutMap)
