@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -72,6 +87,8 @@ class QShowEvent;
 class QHideEvent;
 class QInputContext;
 class QIcon;
+class QWindowSurface;
+class QLocale;
 #if defined(Q_WS_X11)
 class QX11Info;
 #endif
@@ -91,7 +108,8 @@ public:
     mutable uint fstrut_dirty : 1;
     uint context_menu_policy : 3;
     uint window_modality : 2;
-    uint unused : 14;
+    uint in_destructor : 1;
+    uint unused : 13;
     QRect crect;
     mutable QPalette pal;
     QFont fnt;
@@ -162,7 +180,7 @@ class Q_GUI_EXPORT QWidget : public QObject, public QPaintDevice
     Q_PROPERTY(QString windowTitle READ windowTitle WRITE setWindowTitle DESIGNABLE isWindow)
     Q_PROPERTY(QIcon windowIcon READ windowIcon WRITE setWindowIcon DESIGNABLE isWindow)
     Q_PROPERTY(QString windowIconText READ windowIconText WRITE setWindowIconText DESIGNABLE isWindow)
-    Q_PROPERTY(double windowOpacity READ windowOpacity WRITE setWindowOpacity DESIGNABLE false)
+    Q_PROPERTY(double windowOpacity READ windowOpacity WRITE setWindowOpacity DESIGNABLE isWindow)
     Q_PROPERTY(bool windowModified READ isWindowModified WRITE setWindowModified DESIGNABLE isWindow)
 #ifndef QT_NO_TOOLTIP
     Q_PROPERTY(QString toolTip READ toolTip WRITE setToolTip)
@@ -180,17 +198,31 @@ class Q_GUI_EXPORT QWidget : public QObject, public QPaintDevice
     Q_PROPERTY(Qt::LayoutDirection layoutDirection READ layoutDirection WRITE setLayoutDirection RESET unsetLayoutDirection)
     QDOC_PROPERTY(Qt::WindowFlags windowFlags READ windowFlags WRITE setWindowFlags)
     Q_PROPERTY(bool autoFillBackground READ autoFillBackground WRITE setAutoFillBackground)
+#ifndef QT_NO_STYLE_STYLESHEET
+    Q_PROPERTY(QString styleSheet READ styleSheet WRITE setStyleSheet)
+#endif
+    Q_PROPERTY(QLocale locale READ locale WRITE setLocale RESET unsetLocale)
 
 public:
-    explicit QWidget(QWidget* parent = 0, Qt::WFlags f = 0);
+    enum RenderFlag {
+        DrawWindowBackground = 0x1,
+        DrawChildren = 0x2,
+        IgnoreMask = 0x4
+    };
+    Q_DECLARE_FLAGS(RenderFlags, RenderFlag)
+
+    explicit QWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);
 #ifdef QT3_SUPPORT
-    QT3_SUPPORT_CONSTRUCTOR QWidget(QWidget* parent, const char *name, Qt::WFlags f = 0);
+    QT3_SUPPORT_CONSTRUCTOR QWidget(QWidget* parent, const char *name, Qt::WindowFlags f = 0);
 #endif
     ~QWidget();
 
     int devType() const;
 
     WId winId() const;
+    void createWinId(); // internal, going away
+    inline WId internalWinId() const { return data->winid; }
+
     // GUI style setting
     QStyle *style() const;
     void setStyle(QStyle *);
@@ -299,7 +331,19 @@ public:
     QRegion mask() const;
     void clearMask();
 
+    void render(QPaintDevice *target, const QPoint &targetOffset = QPoint(),
+                const QRegion &sourceRegion = QRegion(),
+                RenderFlags renderFlags = RenderFlags(DrawWindowBackground | DrawChildren));
+
+public Q_SLOTS:
     void setWindowTitle(const QString &);
+#ifndef QT_NO_STYLE_STYLESHEET
+    void setStyleSheet(const QString& styleSheet);
+#endif
+public:
+#ifndef QT_NO_STYLE_STYLESHEET
+    QString styleSheet() const;
+#endif
     QString windowTitle() const;
     void setWindowIcon(const QIcon &icon);
     QIcon windowIcon() const;
@@ -335,6 +379,10 @@ public:
     Qt::LayoutDirection layoutDirection() const;
     void unsetLayoutDirection();
 
+    void setLocale(const QLocale &locale);
+    QLocale locale() const;
+    void unsetLocale();
+
     inline bool isRightToLeft() const { return layoutDirection() == Qt::RightToLeft; }
     inline bool isLeftToRight() const { return layoutDirection() == Qt::LeftToRight; }
 
@@ -368,6 +416,7 @@ public:
     int grabShortcut(const QKeySequence &key, Qt::ShortcutContext context = Qt::WindowShortcut);
     void releaseShortcut(int id);
     void setShortcutEnabled(int id, bool enable = true);
+    void setShortcutAutoRepeat(int id, bool enable = true);
 #endif
     static QWidget *mouseGrabber();
     static QWidget *keyboardGrabber();
@@ -419,9 +468,12 @@ public:
     void resize(const QSize &);
     inline void setGeometry(int x, int y, int w, int h);
     void setGeometry(const QRect &);
+    QByteArray saveGeometry() const;
+    bool restoreGeometry(const QByteArray &geometry);
     void adjustSize();
     bool isVisible() const;
     bool isVisibleTo(QWidget*) const;
+    // ### Qt 5: bool isVisibleTo(_const_ QWidget *) const
     inline bool isHidden() const;
 
     bool isMinimized() const;
@@ -452,7 +504,7 @@ public:
     void updateGeometry();
 
     void setParent(QWidget *parent);
-    void setParent(QWidget *parent, Qt::WFlags f);
+    void setParent(QWidget *parent, Qt::WindowFlags f);
 
     void scroll(int dx, int dy);
     void scroll(int dx, int dy, const QRect&);
@@ -496,6 +548,11 @@ public:
     Qt::HANDLE x11PictureHandle() const;
 #endif
 
+#if defined(Q_WS_MAC)
+    Qt::HANDLE macQDHandle() const;
+    Qt::HANDLE macCGHandle() const;
+#endif
+
 #if defined(Q_WS_WIN)
     HDC getDC() const;
     void releaseDC(HDC) const;
@@ -523,6 +580,9 @@ public:
     bool autoFillBackground() const;
     void setAutoFillBackground(bool enabled);
 
+    void setWindowSurface(QWindowSurface *surface);
+    QWindowSurface *windowSurface() const;
+
 Q_SIGNALS:
     void customContextMenuRequested(const QPoint &pos);
 
@@ -547,8 +607,12 @@ protected:
     virtual void resizeEvent(QResizeEvent *);
     virtual void closeEvent(QCloseEvent *);
     virtual void contextMenuEvent(QContextMenuEvent *);
+#ifndef QT_NO_TABLETEVENT
     virtual void tabletEvent(QTabletEvent *);
+#endif
+#ifndef QT_NO_ACTION
     virtual void actionEvent(QActionEvent *);
+#endif
 
 #ifndef QT_NO_DRAGANDDROP
     virtual void dragEnterEvent(QDragEnterEvent *);
@@ -583,7 +647,9 @@ public:
     virtual QVariant inputMethodQuery(Qt::InputMethodQuery) const;
 protected:
     void resetInputContext();
+protected Q_SLOTS:
     void updateMicroFocus();
+protected:
 
     void create(WId = 0, bool initializeWindow = true,
                          bool destroyOldWindow = true);
@@ -595,7 +661,7 @@ protected:
     inline bool focusPreviousChild() { return focusNextPrevChild(false); }
 
 protected:
-    QWidget(QWidgetPrivate &d, QWidget* parent, Qt::WFlags f);
+    QWidget(QWidgetPrivate &d, QWidget* parent, Qt::WindowFlags f);
 private:
 
     bool testAttribute_helper(Qt::WidgetAttribute) const;
@@ -603,6 +669,8 @@ private:
     friend void qt_syncBackingStore(QWidget *);
     friend void qt_syncBackingStore(QRegion, QWidget *);
     friend void qt_syncBackingStore(QRegion, QWidget *, bool);
+    friend QWindowSurface *qt_default_window_surface(QWidget*);
+
     friend class QBackingStoreDevice;
     friend class QWidgetBackingStore;
     friend class QApplication;
@@ -616,26 +684,40 @@ private:
     friend class QLayout;
     friend class QWidgetItem;
     friend class QGLContext;
+    friend class QGLWidget;
     friend class QX11PaintEngine;
     friend class QWin32PaintEngine;
     friend class QShortcutPrivate;
+    friend class QWindowSurface;
+    friend class QD3DWindowSurface;
 
 #ifdef Q_WS_MAC
+#ifdef Q_WS_MAC32
     friend class QMacSavedPortInfo;
-    friend class QQuickDrawPaintEngine;
+#endif
     friend class QCoreGraphicsPaintEnginePrivate;
     friend QPoint qt_mac_posInWindow(const QWidget *w);
+    friend WindowPtr qt_mac_window_for(const QWidget *w);
     friend bool qt_mac_is_metal(const QWidget *w);
+    friend HIViewRef qt_mac_hiview_for(const QWidget *w);
 #endif
 #ifdef Q_WS_QWS
     friend class QWSBackingStore;
     friend class QWSManager;
     friend class QWSManagerPrivate;
     friend class QDecoration;
+    friend class QWSWindowSurface;
+    friend class QScreen;
+    friend class QVNCScreen;
+    friend bool isWidgetOpaque(const QWidget *);
+    friend class QGLWidgetPrivate;
 #endif
 
+    friend Q_GUI_EXPORT QWidgetData *qt_qwidget_data(QWidget *widget);
+
 private:
     Q_DISABLE_COPY(QWidget)
+    Q_PRIVATE_SLOT(d_func(), void _q_showIfNotHidden())
 
     QWidgetData *data;
 
@@ -648,11 +730,11 @@ public:
     inline QT3_SUPPORT void iconify() { showMinimized(); }
     inline QT3_SUPPORT void constPolish() const { ensurePolished(); }
     inline QT3_SUPPORT void polish() { ensurePolished(); }
-    inline QT3_SUPPORT void reparent(QWidget *parent, Qt::WFlags f, const QPoint &p, bool showIt=false)
+    inline QT3_SUPPORT void reparent(QWidget *parent, Qt::WindowFlags f, const QPoint &p, bool showIt=false)
     { setParent(parent, f); setGeometry(p.x(),p.y(),width(),height()); if (showIt) show(); }
     inline QT3_SUPPORT void reparent(QWidget *parent, const QPoint &p, bool showIt=false)
     { setParent(parent, windowFlags() & ~Qt::WindowType_Mask); setGeometry(p.x(),p.y(),width(),height()); if (showIt) show(); }
-    inline QT3_SUPPORT void recreate(QWidget *parent, Qt::WFlags f, const QPoint & p, bool showIt=false)
+    inline QT3_SUPPORT void recreate(QWidget *parent, Qt::WindowFlags f, const QPoint & p, bool showIt=false)
     { setParent(parent, f); setGeometry(p.x(),p.y(),width(),height()); if (showIt) show(); }
     inline QT3_SUPPORT void setSizePolicy(QSizePolicy::Policy hor, QSizePolicy::Policy ver, bool hfw)
     { QSizePolicy sp(hor, ver); sp.setHeightForWidth(hfw); setSizePolicy(sp);}
@@ -743,6 +825,8 @@ protected:
     virtual void languageChange();  // compat
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(QWidget::RenderFlags)
+
 #if defined Q_CC_MSVC && _MSC_VER < 1300
 template <> inline QWidget *qobject_cast_helper<QWidget*>(QObject *o, QWidget *)
 {
@@ -770,9 +854,6 @@ inline Qt::WindowType QWidget::windowType() const
 inline Qt::WindowFlags QWidget::windowFlags() const
 { return data->window_flags; }
 
-inline WId QWidget::winId() const
-{ return data->winid; }
-
 inline bool QWidget::isTopLevel() const
 { return (windowType() & Qt::Window); }
 
