@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,12 +46,15 @@
     \brief The QX11Info class provides information about the X display
     configuration.
 
+    \ingroup shared
+
     The class provides two APIs: a set of non-static functions that
     provide information about a specific widget or pixmap, and a set
     of static functions that provide the default information for the
     application.
 
-    \warning This class is only available on X11.
+    \warning This class is only available on X11. For querying
+    per-screen information in a portable way, use QDesktopWidget.
 
     \sa QWidget::x11Info(), QPixmap::x11Info(), QDesktopWidget
 */
@@ -176,6 +194,11 @@ QX11InfoData* QX11Info::getX11Data(bool def) const
     Returns the horizontal resolution of the given \a screen in terms of the
     number of dots per inch.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa setAppDpiX(), appDpiY()
 */
 int QX11Info::appDpiX(int screen)
@@ -193,6 +216,11 @@ int QX11Info::appDpiX(int screen)
     Sets the horizontal resolution of the given \a screen to the number of
     dots per inch specified by \a xdpi.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa appDpiX(), setAppDpiY()
 */
 
@@ -211,6 +239,11 @@ void QX11Info::setAppDpiX(int screen, int xdpi)
     Returns the vertical resolution of the given \a screen in terms of the
     number of dots per inch.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa setAppDpiY(), appDpiX()
 */
 
@@ -229,6 +262,11 @@ int QX11Info::appDpiY(int screen)
     Sets the vertical resolution of the given \a screen to the number of
     dots per inch specified by \a ydpi.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa appDpiY(), setAppDpiX()
 */
 void QX11Info::setAppDpiY(int screen, int ydpi)
@@ -320,6 +358,11 @@ int QX11Info::appScreen()
 /*!
     Returns a handle for the application's color map on the given \a screen.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa colormap(), defaultColormap()
 */
 Qt::HANDLE QX11Info::appColormap(int screen)
@@ -331,6 +374,11 @@ Qt::HANDLE QX11Info::appColormap(int screen)
     Returns the current visual used by the application on the given
     \a screen.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa visual(), defaultVisual()
 */
 
@@ -342,6 +390,11 @@ void *QX11Info::appVisual(int screen)
 /*!
     Returns a handle for the applications root window on the given \a screen.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa QApplication::desktop()
 */
 Qt::HANDLE QX11Info::appRootWindow(int screen)
@@ -353,6 +406,11 @@ Qt::HANDLE QX11Info::appRootWindow(int screen)
     Returns the color depth (bits per pixel) used by the application on the
     given \a screen.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa depth()
 */
 
@@ -364,6 +422,11 @@ int QX11Info::appDepth(int screen)
 /*!
     Returns the number of cells used by the application on the given \a screen.
 
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa cells()
 */
 
@@ -373,6 +436,11 @@ int QX11Info::appCells(int screen)
 /*!
     Returns true if the application has a default color map on the given
     \a screen; otherwise returns false.
+
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
 */
 bool QX11Info::appDefaultColormap(int screen)
 { return X11 ? X11->screens[screen == -1 ? X11->defaultScreen : screen].defaultColormap : true; }
@@ -380,6 +448,11 @@ bool QX11Info::appDefaultColormap(int screen)
 /*!
     Returns true if the application has a default visual on the given \a screen;
     otherwise returns false.
+
+    The \a screen argument is an X screen number. Be aware that if
+    the user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
 */
 bool QX11Info::appDefaultVisual(int screen)
 { return X11 ? X11->screens[screen == -1 ? X11->defaultScreen : screen].defaultVisual : true; }
@@ -387,6 +460,11 @@ bool QX11Info::appDefaultVisual(int screen)
 /*!
     Returns the number of the screen currently in use.
 
+    The return value is an X screen number. Be aware that if the
+    user's system uses Xinerama (as opposed to traditional X11
+    multiscreen), there is only one X screen. Use QDesktopWidget to
+    query for information about Xinerama screens.
+
     \sa appScreen()
 */
 int QX11Info::screen() const
