@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,16 +51,20 @@
 //
 
 #include "private/qpaintengine_raster_p.h"
+#include "private/qwidget_p.h"
 
 class QWindowSurface;
 
-class QWidgetBackingStore
+class Q_AUTOTEST_EXPORT QWidgetBackingStore
 {
 public:
     QWidgetBackingStore(QWidget *t);
     ~QWidgetBackingStore();
-    void bltRect(const QRect &rect, int dx, int dy, QWidget *widget);
+    bool bltRect(const QRect &rect, int dx, int dy, QWidget *widget);
     void dirtyRegion(const QRegion &rgn, QWidget *widget=0);
+#ifdef Q_RATE_LIMIT_PAINTING
+    void updateDirtyRegion(QWidget *widget);
+#endif
     void cleanRegion(const QRegion &rgn, QWidget *widget=0, bool recursiveCopyToScreen = true);
 #if defined (Q_WS_QWS) || defined (Q_WS_WIN)
     void releaseBuffer();
@@ -57,28 +76,52 @@ public:
 #ifdef Q_WS_WIN
     static void blitToScreen(const QRegion &rgn, QWidget *w);
 #endif
-private:
-    QWidget *tlw;
-    QRegion dirty;
-
-    QWindowSurface *windowSurface;
-
-    QPoint tlwOffset;
+#ifdef Q_WIDGET_USE_DIRTYLIST
+    void removeDirtyWidget(QWidget *w);
+#endif
 
     static bool isOpaque(const QWidget *widget);
+#ifdef Q_RATE_LIMIT_PAINTING
+    static int refreshInterval;
+#endif
+
+private:
+    QWidget *tlw;
+#ifdef Q_WS_QWS
+    QRegion dirtyOnScreen;
+#else
+    QRegion dirty;
+#endif
+#ifdef Q_WIDGET_USE_DIRTYLIST
+    QList<QWidget*> dirtyWidgets;
+#endif
+
+    QWindowSurface *windowSurface;
+#ifdef Q_BACKINGSTORE_SUBSURFACES
+    QList<QWindowSurface*> subSurfaces;
+#endif
+    QPoint tlwOffset;
 
     void copyToScreen(const QRegion &rgn, QWidget *widget, const QPoint &offset, bool recursive = true);
 
-    static void paintSiblingsRecursive(QPaintDevice *pdev, const QObjectList& children, int index, const QRegion &rgn, const QPoint &offset, int flags);
+    static void paintSiblingsRecursive(QPaintDevice *pdev, const QObjectList& children, int index, const QRegion &rgn, const QPoint &offset, int flags
+#ifdef Q_BACKINGSTORE_SUBSURFACES
+                                                 , const QWindowSurface *currentSurface
+#endif
+        );
+
+    static void updateWidget(QWidget *that, const QRegion &rgn);
 
     friend void qt_syncBackingStore(QRegion, QWidget *);
-#if defined(Q_WS_X11) || defined(Q_WS_QWS)
+#if defined(Q_WS_X11) || defined(Q_WS_QWS) || defined(Q_WS_WIN)
     friend void qt_syncBackingStore(QWidget *);
 #endif
     friend class QWidgetPrivate;
     friend class QWidget;
     friend class QWSManagerPrivate;
     friend class QETWidget;
+    friend class QWindowSurface;
+    friend class QWSWindowSurface;
 };
 
 #endif // QBACKINGSTORE_P_H
