@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -45,6 +60,7 @@
 
 class QX11PaintEnginePrivate;
 class QFontEngineFT;
+class QXRenderTessellator;
 
 typedef unsigned long Picture;
 
@@ -69,8 +85,7 @@ public:
     void updateBrush(const QBrush &brush, const QPointF &pt);
     void updateRenderHints(QPainter::RenderHints hints);
     void updateFont(const QFont &font);
-    void updateBackground(Qt::BGMode bgmode, const QBrush &bgBrush);
-    void updateMatrix(const QMatrix &matrix);
+    void updateMatrix(const QTransform &matrix);
     void updateClipRegion_dev(const QRegion &region, Qt::ClipOperation op);
 
     void drawLines(const QLine *lines, int lineCount);
@@ -93,6 +108,8 @@ public:
     void drawTiledPixmap(const QRectF &r, const QPixmap &pixmap, const QPointF &s);
     void drawPath(const QPainterPath &path);
     void drawTextItem(const QPointF &p, const QTextItem &textItem);
+    void drawImage(const QRectF &r, const QImage &img, const QRectF &sr,
+                   Qt::ImageConversionFlags flags = Qt::AutoColor);
 
     virtual Qt::HANDLE handle() const;
     inline Type type() const { return QPaintEngine::X11; }
@@ -106,14 +123,12 @@ protected:
     void drawXLFD(const QPointF &p, const QTextItemInt &si);
 #ifndef QT_NO_FONTCONFIG
     void drawFreetype(const QPointF &p, const QTextItemInt &si);
-    void core_render_glyph(QFontEngineFT *fe, int xp, int yp, uint glyph);
 #endif
 
-    friend void qt_cleanup();
-    friend void qt_draw_transformed_rect(QPaintEngine *pp, int x, int y, int w, int h, bool fill);
-    friend void qt_draw_background(QPaintEngine *pp, int x, int y, int w, int h);
     friend class QPixmap;
     friend class QFontEngineBox;
+    friend Q_GUI_EXPORT GC qt_x11_get_pen_gc(QPainter *);
+    friend Q_GUI_EXPORT GC qt_x11_get_brush_gc(QPainter *);
 
 private:
     Q_DISABLE_COPY(QX11PaintEngine)
@@ -125,18 +140,18 @@ class QX11PaintEnginePrivate : public QPaintEnginePrivate
 public:
     QX11PaintEnginePrivate()
     {
-        dpy = 0;
         scrn = -1;
         hd = 0;
         picture = 0;
-        bg_col = Qt::white;                             // default background color
-        bg_mode = Qt::TransparentMode;                  // default background mode
         gc = gc_brush = 0;
         dpy  = 0;
         xinfo = 0;
-        txop = QPainterPrivate::TxNone;
+        txop = QTransform::TxNone;
         has_clipping = false;
         render_hints = 0;
+#ifndef QT_NO_XRENDER
+        tessellator = 0;
+#endif
     }
     enum GCMode {
         PenGC,
@@ -167,6 +182,7 @@ public:
     int scrn;
     int pdev_depth;
     Qt::HANDLE hd;
+    QPixmap brush_pm;
 #if !defined (QT_NO_XRENDER)
     Qt::HANDLE picture;
     Qt::HANDLE current_brush;
@@ -178,13 +194,11 @@ public:
     GC gc;
     GC gc_brush;
 
-    QColor bg_col;
-    uchar bg_mode;
     QPen cpen;
     QBrush cbrush;
-    QBrush bg_brush;
     QRegion crgn;
-    QMatrix matrix;
+    QTransform matrix;
+    qreal opacity;
 
     uint has_complex_xform : 1;
     uint has_custom_pen : 1;
@@ -202,10 +216,13 @@ public:
 
     const QX11Info *xinfo;
     QPointF bg_origin;
-    QPainterPrivate::TransformationCodes txop;
+    QTransform::TransformationType txop;
     QPolygonClipper<qt_float_point, qt_float_point, float> polygonClipper;
 
     int xlibMaxLinePoints;
+#ifndef QT_NO_XRENDER
+    QXRenderTessellator *tessellator;
+#endif
 };
 
 #endif // QPAINTENGINE_X11_P_H
