@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -172,7 +187,7 @@ QPSPrintEnginePrivate::~QPSPrintEnginePrivate()
 {
 }
 
-#include <QDebug>
+#include <qdebug.h>
 
 static void ps_r7(QPdf::ByteStream& stream, const char * s, int l)
 {
@@ -522,16 +537,7 @@ void QPSPrintEnginePrivate::emitHeader(bool finished)
           << " mm (landscape)\n 90 rotate " << scale << "-" << scale << "scale } def\n";
     }
     s << "%%EndProlog\n";
-
-
-    s << "%%BeginSetup\n";
-    if (copies > 1) {
-        s << "/#copies " << copies << " def\n";
-        s << "/NumCopies " << copies << " SPD\n";
-        s << "/Collate " << (collate ? "true" : "false") << " SPD\n";
-    }
-    s << "%%EndSetup\n";
-
+    
     outDevice->write(header);
     headerDone = true;
 }
@@ -699,9 +705,9 @@ bool QPSPrintEngine::end()
         wrapDSC("%%DocumentFonts: " + d->fontsUsed);
     s << "%%EOF\n";
     d->outDevice->write(trailer);
-    ignoreSigPipe(false);
 
     QPdfBaseEngine::end();
+    ignoreSigPipe(false);
 
     d->firstPage = true;
     d->headerDone = false;
