@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,8 +39,10 @@
 #ifndef QPRINTER_H
 #define QPRINTER_H
 
-#include "QtGui/qpaintdevice.h"
-#include "QtCore/qstring.h"
+#include <QtGui/qpaintdevice.h>
+#include <QtCore/qstring.h>
+
+QT_BEGIN_HEADER
 
 QT_MODULE(Gui)
 
@@ -37,6 +54,7 @@ QT_MODULE(Gui)
 
 class QPrinterPrivate;
 class QPaintEngine;
+class QPrintEngine;
 
 class Q_GUI_EXPORT QPrinter : public QPaintDevice
 {
@@ -45,7 +63,7 @@ public:
     enum PrinterMode { ScreenResolution, PrinterResolution, HighResolution };
 
     explicit QPrinter(PrinterMode mode = ScreenResolution);
-   ~QPrinter();
+    ~QPrinter();
 
     int devType() const;
 
@@ -74,17 +92,27 @@ public:
                        LargeFormat,
 		       LargeCapacity,
 		       Cassette,
-		       FormSource };
+		       FormSource,
+                       MaxPageSource
+    };
 
     enum PrinterState { Idle,
                         Active,
                         Aborted,
                         Error };
-#ifdef QT3_SUPPORT
+
+    enum OutputFormat { NativeFormat, PdfFormat, PostScriptFormat };
+
+    // ### Qt 5: Merge with QAbstractPrintDialog::PrintRange
     enum PrintRange { AllPages, Selection, PageRange };
+
+#ifdef QT3_SUPPORT
     enum PrinterOption { PrintToFile, PrintSelection, PrintPageRange };
 #endif // QT3_SUPPORT
 
+    void setOutputFormat(OutputFormat format);
+    OutputFormat outputFormat() const;
+
     void setPrinterName(const QString &);
     QString printerName() const;
 
@@ -129,6 +157,16 @@ public:
 
     QList<int> supportedResolutions() const;
 
+#ifdef Q_WS_WIN
+    QList<PaperSource> supportedPaperSources() const;
+#endif
+
+    void setFontEmbeddingEnabled(bool enable);
+    bool fontEmbeddingEnabled() const;
+
+    void setDoubleSidedPrinting(bool enable);
+    bool doubleSidedPrinting() const;
+
 #ifdef Q_WS_WIN
     void setWinPageSize(int winPageSize);
     int winPageSize() const;
@@ -148,13 +186,21 @@ public:
     PrinterState printerState() const;
 
     QPaintEngine *paintEngine() const;
+    QPrintEngine *printEngine() const;
 
 #ifdef Q_WS_WIN
     HDC getDC() const;
     void releaseDC(HDC hdc) const;
 #endif
 
-#if defined (QT3_SUPPORT)
+    void setFromTo(int fromPage, int toPage);
+    int fromPage() const;
+    int toPage() const;
+
+    void setPrintRange(PrintRange range);
+    PrintRange printRange() const;
+
+#ifdef QT3_SUPPORT
 #ifdef Q_WS_MAC
     QT3_SUPPORT bool pageSetup(QWidget *parent = 0);
     QT3_SUPPORT bool printSetup(QWidget *parent = 0);
@@ -162,10 +208,6 @@ public:
 
     QT3_SUPPORT bool setup(QWidget *parent = 0);
 
-    QT3_SUPPORT void setFromTo(int fromPage, int toPage);
-    QT3_SUPPORT int fromPage() const;
-    QT3_SUPPORT int toPage() const;
-
     QT3_SUPPORT void setMinMax(int minPage, int maxPage);
     QT3_SUPPORT int minPage() const;
     QT3_SUPPORT int maxPage() const;
@@ -173,9 +215,6 @@ public:
     QT3_SUPPORT void setCollateCopiesEnabled(bool);
     QT3_SUPPORT bool collateCopiesEnabled() const;
 
-    QT3_SUPPORT void setPrintRange(PrintRange range);
-    QT3_SUPPORT PrintRange printRange() const;
-
     QT3_SUPPORT void setOptionEnabled(PrinterOption, bool enable);
     QT3_SUPPORT bool isOptionEnabled(PrinterOption) const;
 
@@ -189,7 +228,8 @@ public:
 #endif
 
 protected:
-    int         metric(PaintDeviceMetric) const;
+    int metric(PaintDeviceMetric) const;
+    void setEngines(QPrintEngine *printEngine, QPaintEngine *paintEngine);
 
 private:
     Q_DISABLE_COPY(QPrinter)
@@ -197,6 +237,7 @@ private:
     QPrinterPrivate *d_ptr;
 
     friend class QPrintDialogWin;
+    friend class QAbstractPrintDialog;
 };
 
 #ifdef QT3_SUPPORT
@@ -220,9 +261,10 @@ inline void QPrinter::margins(uint *top, uint *left, uint *bottom, uint *right)
     if (right)
         *right = paper.right() - page.right();
 }
-
-#endif // QT3_SUPPORT
+#endif
 
 #endif // QT_NO_PRINTER
 
+QT_END_HEADER
+
 #endif // QPRINTER_H
