@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,6 +51,7 @@ class QPlastiqueStylePrivate;
 class Q_GUI_EXPORT QPlastiqueStyle : public QWindowsStyle
 {
     Q_OBJECT
+    Q_DECLARE_PRIVATE(QPlastiqueStyle)
 public:
     QPlastiqueStyle();
     ~QPlastiqueStyle();
@@ -54,12 +70,15 @@ public:
                          SubControl sc, const QWidget *widget) const;
 
     int styleHint(StyleHint hint, const QStyleOption *option = 0, const QWidget *widget = 0,
-		  QStyleHintReturn *returnData = 0) const;
+                  QStyleHintReturn *returnData = 0) const;
     SubControl hitTestComplexControl(ComplexControl control, const QStyleOptionComplex *option,
-				     const QPoint &pos, const QWidget *widget = 0) const;
+                                     const QPoint &pos, const QWidget *widget = 0) const;
 
     int pixelMetric(PixelMetric metric, const QStyleOption *option = 0, const QWidget *widget = 0) const;
 
+    QPixmap standardPixmap(StandardPixmap standardPixmap, const QStyleOption *opt,
+                           const QWidget *widget = 0) const;
+    
     void polish(QWidget *widget);
     void polish(QApplication *app);
     void polish(QPalette &pal);
@@ -68,12 +87,22 @@ public:
 
     QPalette standardPalette() const;
 
+protected Q_SLOTS:
+    QIcon standardIconImplementation(StandardPixmap standardIcon, const QStyleOption *opt = 0,
+                                     const QWidget *widget = 0) const;
+    int layoutSpacingImplementation(QSizePolicy::ControlType control1,
+                                    QSizePolicy::ControlType control2, 
+                                    Qt::Orientation orientation,
+                                    const QStyleOption *option = 0, 
+                                    const QWidget *widget = 0) const;
+
 protected:
     bool eventFilter(QObject *watched, QEvent *event);
     void timerEvent(QTimerEvent *event);
-    
+
 private:
-    QPlastiqueStylePrivate *d;
+    Q_DISABLE_COPY(QPlastiqueStyle)
+    void *reserved;
 };
 
 #endif // QT_NO_STYLE_PLASTIQUE
