@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -48,6 +63,8 @@
 //
 
 class QRenderRule;
+class QAbstractScrollArea;
+class QStyleSheetStylePrivate;
 
 class Q_AUTOTEST_EXPORT QStyleSheetStyle : public QWindowsStyle
 {
@@ -56,6 +73,7 @@ class Q_AUTOTEST_EXPORT QStyleSheetStyle : public QWindowsStyle
     Q_OBJECT
 public:
     QStyleSheetStyle(QStyle *baseStyle);
+    ~QStyleSheetStyle();
 
     void drawComplexControl(ComplexControl cc, const QStyleOptionComplex *opt, QPainter *p,
                             const QWidget *w = 0) const;
@@ -83,6 +101,9 @@ public:
     QPalette standardPalette() const;
     QPixmap standardPixmap(StandardPixmap standardPixmap, const QStyleOption *option = 0,
                            const QWidget *w = 0 ) const;
+    int layoutSpacing(QSizePolicy::ControlType control1, QSizePolicy::ControlType control2,
+                          Qt::Orientation orientation, const QStyleOption *option = 0,
+                          const QWidget *widget = 0) const;
     int styleHint(StyleHint sh, const QStyleOption *opt = 0, const QWidget *w = 0,
                   QStyleHintReturn *shret = 0) const;
     QRect subElementRect(SubElement r, const QStyleOption *opt, const QWidget *widget = 0) const;
@@ -105,17 +126,40 @@ public:
 protected Q_SLOTS:
     QIcon standardIconImplementation(StandardPixmap standardIcon, const QStyleOption *opt = 0,
                                      const QWidget *widget = 0) const;
-
+    int layoutSpacingImplementation(QSizePolicy::ControlType control1,
+                                    QSizePolicy::ControlType control2,
+                                    Qt::Orientation orientation,
+                                    const QStyleOption *option = 0,
+                                    const QWidget *widget = 0) const;
 private Q_SLOTS:
     void widgetDestroyed(QObject *);
 
 private:
-    void setPalette(QWidget *w);
-    QRenderRule renderRule(const QWidget *, const QString &, QStyle::State = QStyle::State_None) const;
-    QRenderRule renderRule(const QWidget *, int, QStyle::State = QStyle::State_None) const;
-    QRenderRule renderRule(const QWidget *, const QStyleOption *, int = 0 /* PseudoElement_None */) const;
-    bool hasStyleRule(const QWidget *, int = 0 /* PseudoElement_None */) const;
     int refcount;
+
+    friend class QRenderRule;
+    int nativeFrameWidth(const QWidget *);
+    QRenderRule renderRule(const QWidget *, int, int = 0) const;
+    QRenderRule renderRule(const QWidget *, const QStyleOption *, int = 0) const;
+    QSize defaultSize(const QWidget *, QSize, const QRect&, int) const;
+    QRect positionRect(const QWidget *, const QRenderRule&, const QRenderRule&, int,
+                       const QRect&, Qt::LayoutDirection) const;
+    QRect positionRect(const QWidget *w, const QRenderRule &rule2, int pe,
+                       const QRect &originRect, Qt::LayoutDirection dir) const;
+
+    void setPalette(QWidget *);
+    void unsetPalette(QWidget *);
+    void setProperties(QWidget *);
+    void setGeometry(QWidget *);
+    QVector<QCss::StyleRule> styleRules(const QWidget *w) const;
+    bool hasStyleRule(const QWidget *w, int part) const;
+
+public:
+    static int numinstances;
+
+private:
+    Q_DISABLE_COPY(QStyleSheetStyle)
+    Q_DECLARE_PRIVATE(QStyleSheetStyle)
 };
 
 #endif // QT_NO_STYLE_STYLESHEET
