@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,11 +50,11 @@
 // We mean it.
 //
 
-#include "qfont.h"
-#include "qmap.h"
-#include "qobject.h"
+#include "QtGui/qfont.h"
+#include "QtCore/qmap.h"
+#include "QtCore/qobject.h"
 #include <private/qunicodetables_p.h>
-#include <qfontdatabase.h>
+#include <QtGui/qfontdatabase.h>
 
 // forwards
 class QFontEngine;
@@ -118,16 +133,11 @@ public:
     ~QFontEngineData();
 
     QAtomic ref;
-    uint lineWidth;
 
-#if defined(Q_WS_X11) || defined(Q_WS_WIN)
+#if !defined(Q_WS_MAC)
     QFontEngine *engines[QUnicodeTables::ScriptCount];
 #else
     QFontEngine *engine;
-#endif // Q_WS_X11 || Q_WS_WIN
-#ifndef Q_WS_MAC
-    enum { widthCacheSize = 0x500 };
-    uchar widthCache[widthCacheSize];
 #endif
 };
 
@@ -143,10 +153,7 @@ public:
     QFontPrivate(const QFontPrivate &other);
     ~QFontPrivate();
 
-#ifndef Q_WS_X11
-    void load(int script);
-#endif
-#if defined(Q_WS_X11) || defined(Q_WS_WIN)
+#if !defined(Q_WS_MAC)
     inline QFontEngine *engineForScript(int script) const
     {
         if (script >= QUnicodeTables::Inherited)
@@ -161,10 +168,10 @@ public:
         if (script >= QUnicodeTables::Inherited)
             script = QUnicodeTables::Common;
         if (!engineData || !engineData->engine)
-            const_cast<QFontPrivate *>(this)->load(script);
+            QFontDatabase::load(this, script);
         return engineData->engine;
     }
-#endif // Q_WS_X11 || Q_WS_WIN
+#endif
 
     QAtomic ref;
     QFontDef request;
@@ -172,6 +179,10 @@ public:
     int dpi;
     int screen;
 
+#ifdef Q_WS_WIN
+    HDC hdc;
+#endif
+
     uint rawMode    :  1;
     uint underline  :  1;
     uint overline   :  1;
@@ -207,8 +218,9 @@ public:
     QFontCache();
     ~QFontCache();
 
-#ifdef Q_WS_QWS
     void clear();
+#if defined(Q_WS_QWS) && !defined(QT_NO_QWS_QPF2)
+    void removeEngineForFont(const QByteArray &fontName);
 #endif
     // universal key structure.  QFontEngineDatas and QFontEngines are cached using
     // the same keys
