@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,6 +41,18 @@
 #include <qpainterpath.h>
 #include "private/qpdf_p.h"
 
+#ifdef Q_WS_X11
+#include "private/qfontengine_x11_p.h"
+#endif
+
+#ifndef QT_NO_FREETYPE
+#if defined(Q_WS_X11) || defined(Q_WS_QWS)
+#    include "private/qfontengine_ft_p.h"
+#endif
+#include <ft2build.h>
+#include FT_FREETYPE_H
+#endif
+
 #ifndef QT_NO_PRINTER
 
 static const char * const agl =
@@ -269,7 +296,7 @@ static FT_Face ft_face(const QFontEngine *engine)
 #ifdef Q_WS_QWS
     if (engine->type() == QFontEngine::Freetype) {
         const QFontEngineFT *ft = static_cast<const QFontEngineFT *>(engine);
-        return ft->face;
+        return ft->non_locked_face();
     }
 #endif
     return 0;
@@ -296,6 +323,8 @@ QByteArray QFontSubset::glyphName(unsigned int glyph, const QVector<int> reverse
             glyphIndex = static_cast<QFontEngineXLFD *>(fontEngine)->glyphIndexToFreetypeGlyphIndex(glyphIndex);
 #endif
         FT_Get_Glyph_Name(face, glyphIndex, &name, 32);
+        if (name[0] == '.') // fix broken PS fonts returning .notdef for many glyphs
+            name[0] = 0;
     }
     if (name[0]) {
         s << "/" << name;
@@ -307,9 +336,8 @@ QByteArray QFontSubset::glyphName(unsigned int glyph, const QVector<int> reverse
         s << "/" << glyphName(uc, false /* ### */);
     } else
 #endif
-    if (reverseMap[glyphIndex]) {
-        char tmp[8];
-        s << "/uni" << QPdf::toHex((ushort)reverseMap[glyphIndex], tmp);
+    if (reverseMap[glyphIndex] && reverseMap[glyphIndex] < 0x10000) {
+        s << "/" << glyphName(reverseMap[glyphIndex], false);
     } else {
         s << "/gl" << (int)glyphIndex;
     }
@@ -532,12 +560,6 @@ private:
     uchar *start;
 };
 
-#define MAKE_TAG( str )                      \
-    ( ( (quint32)str[0] << 24 ) |           \
-      ( (quint32)str[1] << 16 ) |           \
-      ( (quint32)str[2] <<  8 ) |           \
-      (quint32)str[3]         )
-
 struct QTtfTable {
     Tag tag;
     QByteArray data;
@@ -648,7 +670,7 @@ static QTtfTable generateHead(const qttf_head_table &head)
 {
     const int head_size = 54;
     QTtfTable t;
-    t.tag = MAKE_TAG("head");
+    t.tag = MAKE_TAG('h', 'e', 'a', 'd');
     t.data.resize(head_size);
 
     QTtfStream s(t.data);
@@ -719,7 +741,7 @@ static QTtfTable generateHhea(const qttf_hhea_table &hhea)
 {
     const int hhea_size = 36;
     QTtfTable t;
-    t.tag = MAKE_TAG("hhea");
+    t.tag = MAKE_TAG('h', 'h', 'e', 'a');
     t.data.resize(hhea_size);
 
     QTtfStream s(t.data);
@@ -770,7 +792,7 @@ static QTtfTable generateMaxp(const qttf_maxp_table &maxp)
 {
     const int maxp_size = 32;
     QTtfTable t;
-    t.tag = MAKE_TAG("maxp");
+    t.tag = MAKE_TAG('m', 'a', 'x', 'p');
     t.data.resize(maxp_size);
 
     QTtfStream s(t.data);
@@ -848,7 +870,7 @@ static QTtfTable generateName(const QList<QTtfNameRecord> &name)
     const int char_size = 2;
 
     QTtfTable t;
-    t.tag = MAKE_TAG("name");
+    t.tag = MAKE_TAG('n', 'a', 'm', 'e');
 
     const int name_size = 6 + 12*name.size();
     int string_size = 0;
@@ -1206,15 +1228,15 @@ static QList<QTtfTable> generateGlyphTables(qttf_font_tables &tables, const QLis
     tables.hhea.numberOfHMetrics = nGlyphs;
 
     QTtfTable glyf;
-    glyf.tag = MAKE_TAG("glyf");
+    glyf.tag = MAKE_TAG('g', 'l', 'y', 'f');
 
     QTtfTable loca;
-    loca.tag = MAKE_TAG("loca");
+    loca.tag = MAKE_TAG('l', 'o', 'c', 'a');
     loca.data.resize(glyf_size < max_size_small ? (nGlyphs+1)*sizeof(quint16) : (nGlyphs+1)*sizeof(quint32));
     QTtfStream ls(loca.data);
 
     QTtfTable hmtx;
-    hmtx.tag = MAKE_TAG("hmtx");
+    hmtx.tag = MAKE_TAG('h', 'm', 't', 'x');
     hmtx.data.resize(nGlyphs*4);
     QTtfStream hs(hmtx.data);
 
@@ -1313,7 +1335,7 @@ static QByteArray bindFont(const QList<QTtfTable>& _tables)
         for (int i = 0; i < tables.size(); ++i) {
             const QTtfTable &t = tables.at(i);
             const quint32 size = (t.data.size() + 3) & ~3;
-            if (t.tag == MAKE_TAG("head"))
+            if (t.tag == MAKE_TAG('h', 'e', 'a', 'd'))
                 head_offset = table_offset;
             f << t.tag
               << checksum(t.data)
@@ -1437,7 +1459,7 @@ QByteArray QFontSubset::toTruetype() const
     tables.append(generateMaxp(font.maxp));
     // name
     QTtfTable name_table;
-    name_table.tag = MAKE_TAG("name");
+    name_table.tag = MAKE_TAG('n', 'a', 'm', 'e');
     if (!noEmbed)
         name_table.data = fontEngine->getSfntTable(name_table.tag);
     if (name_table.data.isEmpty()) {
@@ -1455,7 +1477,7 @@ QByteArray QFontSubset::toTruetype() const
 
     if (!noEmbed) {
         QTtfTable os2;
-        os2.tag = MAKE_TAG("OS/2");
+        os2.tag = MAKE_TAG('O', 'S', '/', '2');
         os2.data = fontEngine->getSfntTable(os2.tag);
         if (!os2.data.isEmpty())
             tables.append(os2);
