@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -40,7 +55,8 @@
 #ifndef QT_NO_OPENTYPE
 #include <ft2build.h>
 #include FT_FREETYPE_H
-#include "ftxopen.h"
+#include "harfbuzz.h"
+#include "ftglue.h"
 
 enum { PositioningProperties = 0x80000000 };
 
@@ -61,27 +77,31 @@ public:
         Q_ASSERT(script < QUnicodeTables::ScriptCount);
         return supported_scripts[script];
     }
-    void selectScript(unsigned int script, const Features *features = 0);
+    void selectScript(QShaperItem *item, unsigned int script, const Features *features = 0);
 
     bool shape(QShaperItem *item, const unsigned int *properties = 0);
     bool positionAndAdd(QShaperItem *item, int availableGlyphs, bool doLogClusters = true);
 
-    OTL_GlyphItem glyphs() const { return otl_buffer->in_string; }
-    int len() const { return otl_buffer->in_length; }
-    void setProperty(int index, uint property) { otl_buffer->in_string[index].properties = property; }
+    HB_GlyphItem glyphs() const { return hb_buffer->in_string; }
+    int len() const { return hb_buffer->in_length; }
+    void setProperty(int index, uint property) { hb_buffer->in_string[index].properties = property; }
 
 
 private:
     bool checkScript(unsigned int script);
     QFontEngine *fontEngine;
     FT_Face face;
-    TTO_GDEF gdef;
-    TTO_GSUB gsub;
-    TTO_GPOS gpos;
+    HB_GDEF gdef;
+    HB_GSUB gsub;
+    HB_GPOS gpos;
     bool supported_scripts[QUnicodeTables::ScriptCount];
     FT_ULong current_script;
-    bool positioned : 1;
-    OTL_Buffer otl_buffer;
+    bool positioned;
+    bool kerning_feature_selected;
+    bool glyphs_substituted;
+    bool has_features;
+    HB_Buffer hb_buffer;
+    int allocated;
     QGlyphLayout::Attributes *tmpAttributes;
     unsigned int *tmpLogClusters;
     int length;
