@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -481,7 +496,8 @@ bool QTextCursorPrivate::movePosition(QTextCursor::MoveOperation op, QTextCursor
                 return false;
         }
     }
-    setPosition(newPosition);
+
+    const bool moved = setPosition(newPosition);
 
     if (mode == QTextCursor::MoveAnchor) {
         anchor = position;
@@ -493,7 +509,7 @@ bool QTextCursorPrivate::movePosition(QTextCursor::MoveOperation op, QTextCursor
     if (adjustX)
         setX();
 
-    return true;
+    return moved;
 }
 
 QTextTable *QTextCursorPrivate::complexSelectionTable() const
@@ -1109,6 +1125,8 @@ void QTextCursor::insertText(const QString &text, const QTextCharFormat &_format
             d->priv->insert(d->position, QString(text.unicode() + textStart, text.length() - textStart), formatIdx);
     }
     d->priv->endEditBlock();
+    if (!d->priv->isInEditBlock())
+        d->setX();
 }
 
 /*!
@@ -1325,6 +1343,11 @@ static void getText(QString &text, QTextDocumentPrivate *priv, const QString &do
     only returns the text, with no rich text formatting information.
     If you want a document fragment (i.e. formatted rich text) use
     selection() instead.
+
+    \note If the selection obtained from an editor spans a line break,
+    the text will contain a Unicode U+2029 paragraph separator character
+    instead of a newline \c{\n} character. Use QString::replace() to
+    replace these characters with newlines.
 */
 QString QTextCursor::selectedText() const
 {
@@ -1370,6 +1393,12 @@ QString QTextCursor::selectedText() const
     Returns the current selection (which may be empty) with all its
     formatting information. If you just want the selected text (i.e.
     plain text) use selectedText() instead.
+
+    \note Unlike QTextDocumentFragment::toPlainText(),
+    selectedText() may include special unicode characters such as
+    QChar::ParagraphSeparator.
+
+    \sa QTextDocumentFragment::toPlainText()
 */
 QTextDocumentFragment QTextCursor::selection() const
 {
@@ -1432,7 +1461,7 @@ void QTextCursor::mergeBlockFormat(const QTextBlockFormat &modifier)
     Returns the block character format of the block the cursor is in.
 
     The block char format is the format used when inserting text at the
-    beginning of a block.
+    beginning of an empty block.
 
     \sa setBlockCharFormat()
  */
@@ -1472,9 +1501,11 @@ void QTextCursor::mergeBlockCharFormat(const QTextCharFormat &modifier)
 
     d->setBlockCharFormat(modifier, QTextDocumentPrivate::MergeFormat);
 }
+
 /*!
-    Returns the format of the character immediately before the
-    cursor position().
+    Returns the format of the character immediately before the cursor position(). If the cursor is
+    positioned at the beginning of a text block that is not empty then the format of the character
+    immediately after the cursor is returned.
 
     \sa insertText(), blockFormat()
  */
@@ -1485,13 +1516,20 @@ QTextCharFormat QTextCursor::charFormat() const
 
     int idx = d->currentCharFormat;
     if (idx == -1) {
-        int pos = d->position - 1;
+        QTextBlock block = d->block();
+
+        int pos;
+        if (d->position == block.position()
+            && block.length() > 1)
+            pos = d->position;
+        else
+            pos = d->position - 1;
+
         if (pos == -1) {
             idx = d->priv->blockCharFormatIndex(d->priv->blockMap().firstNode());
         } else {
             Q_ASSERT(pos >= 0 && pos < d->priv->length());
 
-
             QTextDocumentPrivate::FragmentIterator it = d->priv->find(pos);
             Q_ASSERT(!it.atEnd());
             idx = it.value()->format;
@@ -1843,6 +1881,11 @@ void QTextCursor::insertFragment(const QTextDocumentFragment &fragment)
     \since 4.2
     Inserts the text \a html at the current position(). The text is interpreted as
     HTML.
+    
+    \note When using this function with a style sheet, the style sheet will
+    only apply to the current block in the document. In order to apply a style
+    sheet throughout a document, use QTextDocument::setDefaultStyleSheet()
+    instead.
 */
 void QTextCursor::insertHtml(const QString &html)
 {
