@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -179,6 +194,14 @@ QTextDocumentPrivate *QTextObject::docHandle() const
     \sa QTextBlock QTextDocument
 */
 
+void QTextBlockGroupPrivate::markBlocksDirty()
+{
+    for (int i = 0; i < blocks.count(); ++i) {
+        const QTextBlock &block = blocks.at(i);
+        pieceTable->documentChange(block.position(), block.length());
+    }
+}
+
 /*!
     \fn QTextBlockGroup::QTextBlockGroup(QTextDocument *document)
 
@@ -220,6 +243,7 @@ void QTextBlockGroup::blockInserted(const QTextBlock &block)
     Q_D(QTextBlockGroup);
     QTextBlockGroupPrivate::BlockList::Iterator it = qLowerBound(d->blocks.begin(), d->blocks.end(), block);
     d->blocks.insert(it, block);
+    d->markBlocksDirty();
 }
 
 // ### DOC: Shouldn't this be removeBlock()?
@@ -231,6 +255,7 @@ void QTextBlockGroup::blockRemoved(const QTextBlock &block)
 {
     Q_D(QTextBlockGroup);
     d->blocks.removeAll(block);
+    d->markBlocksDirty();
     if (d->blocks.isEmpty()) {
         document()->docHandle()->deleteObject(this);
         return;
@@ -567,7 +592,7 @@ void QTextFramePrivate::remove_me()
 
 /*!
     \class QTextFrame::iterator
-    \brief The QTextFrame::iterator class provides an iterator for reading
+    \brief The iterator class provides an iterator for reading
     the contents of a QTextFrame.
 
     \ingroup text
@@ -822,8 +847,8 @@ QTextBlockUserData::~QTextBlockUserData()
     by calling isValid().
 
     QTextBlock provides comparison operators to make it easier to work with
-    blocks: operator==() compares two block for equality, operator!=()
-    compares two blocks for inequality, and operator<() determines whether
+    blocks: \l operator==() compares two block for equality, \l operator!=()
+    compares two blocks for inequality, and \l operator<() determines whether
     a block precedes another in the same document.
 
     \img qtextblock-sequence.png
@@ -1050,8 +1075,8 @@ int QTextBlock::blockFormatIndex() const
 
 /*!
     Returns the QTextCharFormat that describes the block's character
-    format. This is mainly used to draw block-specific additions such
-    as e.g. list markers.
+    format. The block's character format is used when inserting text into
+    an empty block.
 
     \sa blockFormat()
  */
