@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -88,6 +103,9 @@
     format.setBackground(Qt::blue);
     cell.setFormat(format);
 
+    Note that the cell's row or column span cannot be changed through this function. You have
+    to use QTextTable::mergeCells and QTextTable::splitCell instead.
+
     \sa format()
 */
 void QTextTableCell::setFormat(const QTextCharFormat &format)
@@ -96,6 +114,12 @@ void QTextTableCell::setFormat(const QTextCharFormat &format)
     fmt.clearProperty(QTextFormat::ObjectIndex);
     QTextDocumentPrivate *p = table->docHandle();
     QTextDocumentPrivate::FragmentIterator frag(&p->fragmentMap(), fragment);
+
+    QTextFormatCollection *c = p->formatCollection();
+    QTextCharFormat oldFormat = c->charFormat(frag->format);
+    fmt.setTableCellRowSpan(oldFormat.tableCellRowSpan());
+    fmt.setTableCellColumnSpan(oldFormat.tableCellColumnSpan());
+
     p->setCharFormat(frag.position(), 1, fmt, QTextDocumentPrivate::SetFormatAndPreserveObjectIndices);
 }
 
@@ -691,7 +715,7 @@ void QTextTable::insertColumns(int pos, int num)
 
     QTextTableFormat tfmt = format();
     tfmt.setColumns(tfmt.columns()+num);
-    setFormat(tfmt);
+    QTextObject::setFormat(tfmt);
 
 //     qDebug() << "-------- end insertCols" << pos << num;
     p->endEditBlock();
@@ -806,7 +830,7 @@ void QTextTable::removeColumns(int pos, int num)
 
     QTextTableFormat tfmt = format();
     tfmt.setColumns(tfmt.columns()-num);
-    setFormat(tfmt);
+    QTextObject::setFormat(tfmt);
 
     p->endEditBlock();
 //     qDebug() << "-------- end removeCols" << pos << num;
@@ -828,12 +852,11 @@ void QTextTable::mergeCells(int row, int column, int numRows, int numCols)
         d->update();
 
     QTextDocumentPrivate *p = d->pieceTable;
+    QTextFormatCollection *fc = p->formatCollection();
 
     const QTextTableCell cell = cellAt(row, column);
-    if (!cell.isValid())
+    if (!cell.isValid() || row != cell.row() || column != cell.column())
         return;
-    row = cell.row();
-    column = cell.column();
 
     QTextCharFormat fmt = cell.format();
     const int rowSpan = fmt.tableCellRowSpan();
@@ -846,60 +869,117 @@ void QTextTable::mergeCells(int row, int column, int numRows, int numCols)
     if (numRows < rowSpan || numCols < colSpan)
         return;
 
+    // check the edges of the merge rect to make sure no cell spans the edge
+    for (int r = row; r < row + numRows; ++r) {
+        if (cellAt(r, column) == cellAt(r, column - 1))
+            return;
+        if (cellAt(r, column + numCols) == cellAt(r, column + numCols - 1))
+            return;
+    }
+
+    for (int c = column; c < column + numCols; ++c) {
+        if (cellAt(row, c) == cellAt(row - 1, c))
+            return;
+        if (cellAt(row + numRows, c) == cellAt(row + numRows - 1, c))
+            return;
+    }
+
     p->beginEditBlock();
 
     const int origCellPosition = cell.firstPosition() - 1;
 
     const int cellFragment = d->grid[row * d->nCols + column];
 
-    QVarLengthArray<int> cellMarkersToDelete((numCols - colSpan) * rowSpan
-                                             + (numRows - rowSpan) * numCols);
-
-    int idx = 0;
+    // find the position at which to insert the contents of the merged cells
+    QFragmentFindHelper helper(origCellPosition, p->fragmentMap());
+    QList<int>::Iterator it = qBinaryFind(d->cells.begin(), d->cells.end(), helper);
+    Q_ASSERT(it != d->cells.end());
+    Q_ASSERT(*it == cellFragment);
+    const int insertCellIndex = it - d->cells.begin();
+    int insertFragment = d->cells.value(insertCellIndex + 1, d->fragment_end);
+    uint insertPos = p->fragmentMap().position(insertFragment);
 
     d->blockFragmentUpdates = true;
 
-    for (int r = row; r < row + rowSpan; ++r) {
+    bool rowHasText = cell.firstCursorPosition().block().length();
+    bool needsParagraph = rowHasText && colSpan == numCols;
 
-        const int fragment = d->grid[r * d->nCols + column + colSpan];
-        const uint pos = p->fragmentMap().position(fragment);
-        QFragmentFindHelper helper(pos, p->fragmentMap());
-        QList<int>::Iterator it = qBinaryFind(d->cells.begin(), d->cells.end(), helper);
-        Q_ASSERT(it != d->cells.end());
-        Q_ASSERT(*it == fragment);
-        d->cellIndices.remove(it - d->cells.begin(), numCols - colSpan);
-        d->cells.erase(it, it + numCols - colSpan);
+    // find all cells that will be erased by the merge
+    for (int r = row; r < row + numRows; ++r) {
+        int firstColumn = r < row + rowSpan ? column + colSpan : column;
 
-        for (int c = column + colSpan; c < column + numCols; ++c) {
-            const int cell = d->grid[r * d->nCols + c];
-            QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
-            cellMarkersToDelete[idx++] = it.position();
-            d->grid[r * d->nCols + c] = cellFragment;
+        // don't recompute the cell index for the first row
+        int firstCellIndex = r == row ? insertCellIndex + 1 : -1;
+        int cellIndex = firstCellIndex;
+
+        for (int c = firstColumn; c < column + numCols; ++c) {
+            const int fragment = d->grid[r * d->nCols + c];
+
+            // already handled?
+            if (fragment == cellFragment)
+                continue;
+
+            QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), fragment);
+            uint pos = it.position();
+
+            if (firstCellIndex == -1) {
+                QFragmentFindHelper helper(pos, p->fragmentMap());
+                QList<int>::Iterator it = qBinaryFind(d->cells.begin(), d->cells.end(), helper);
+                Q_ASSERT(it != d->cells.end());
+                Q_ASSERT(*it == fragment);
+                firstCellIndex = cellIndex = it - d->cells.begin();
+            }
+
+            ++cellIndex;
+
+            QTextCharFormat fmt = fc->charFormat(it->format);
+
+            const int cellRowSpan = fmt.tableCellRowSpan();
+            const int cellColSpan = fmt.tableCellColumnSpan();
+
+            // update the grid for this cell
+            for (int i = r; i < r + cellRowSpan; ++i)
+                for (int j = c; j < c + cellColSpan; ++j)
+                    d->grid[i * d->nCols + j] = cellFragment;
+
+            // erase the cell marker
+            p->remove(pos, 1);
+
+            const int nextFragment = d->cells.value(cellIndex, d->fragment_end);
+            const uint nextPos = p->fragmentMap().position(nextFragment);
+
+            Q_ASSERT(nextPos >= pos);
+
+            // merge the contents of the cell (if not empty)
+            if (nextPos > pos) {
+                if (needsParagraph) {
+                    needsParagraph = false;
+                    QTextCursor(p, insertPos++).insertBlock();
+                    p->move(pos + 1, insertPos, nextPos - pos);
+                } else if (rowHasText) {
+                    QTextCursor(p, insertPos++).insertText(QLatin1String(" "));
+                    p->move(pos + 1, insertPos, nextPos - pos);
+                } else {
+                    p->move(pos, insertPos, nextPos - pos);
+                }
+
+                insertPos += nextPos - pos;
+                rowHasText = true;
+            }
+        }
+
+        if (rowHasText) {
+            needsParagraph = true;
+            rowHasText = false;
+        }
+
+        // erase cells from last row
+        if (firstCellIndex >= 0) {
+            d->cellIndices.remove(firstCellIndex, cellIndex - firstCellIndex);
+            d->cells.erase(d->cells.begin() + firstCellIndex, d->cells.begin() + cellIndex);
         }
     }
 
-    for (int r = row + rowSpan; r < row + numRows; ++r) {
-
-        const int fragment = d->grid[r * d->nCols + column];
-        const uint pos = p->fragmentMap().position(fragment);
-        QFragmentFindHelper helper(pos, p->fragmentMap());
-        QList<int>::Iterator it = qBinaryFind(d->cells.begin(), d->cells.end(), helper);
-        Q_ASSERT(it != d->cells.end());
-        Q_ASSERT(*it == fragment);
-        d->cellIndices.remove(it - d->cells.begin(), numCols);
-        d->cells.erase(it, it + numCols);
-
-        for (int c = column; c < column + numCols; ++c) {
-            const int cell = d->grid[r * d->nCols + c];
-            QTextDocumentPrivate::FragmentIterator it(&p->fragmentMap(), cell);
-            cellMarkersToDelete[idx++] = it.position();
-            d->grid[r * d->nCols + c] = cellFragment;
-        }
-    }
-
-    for (int i = 0; i < cellMarkersToDelete.size(); ++i)
-        p->remove(cellMarkersToDelete[i] - i, 1);
-
     d->fragment_start = d->cells.first();
 
     fmt.setTableCellRowSpan(numRows);
@@ -964,14 +1044,15 @@ void QTextTable::splitCell(int row, int column, int numRows, int numCols)
 
     QVarLengthArray<int> rowPositions(rowSpan);
 
-    for (int r = row; r < row + numRows; ++r) {
-        const QTextTableCell cell = cellAt(r, column + numCols);
-        rowPositions[r - row] = cell.lastPosition();
-    }
+    rowPositions[0] = cell.lastPosition();
 
-    for (int r = row + numRows; r < row + rowSpan; ++r) {
-        const QTextTableCell cell = cellAt(r, column);
-        rowPositions[r - row] = cell.lastPosition();
+    for (int r = row + 1; r < row + rowSpan; ++r) {
+        // find the cell before which to insert the new cell markers
+        int gridIndex = r * d->nCols + column;
+        QVector<int>::iterator it = qUpperBound(d->cellIndices.begin(), d->cellIndices.end(), gridIndex);
+        int cellIndex = it - d->cellIndices.begin();
+        int fragment = d->cells.value(cellIndex, d->fragment_end);
+        rowPositions[r - row] = p->fragmentMap().position(fragment);
     }
 
     fmt.setTableCellColumnSpan(1);
@@ -1083,6 +1164,13 @@ QTextCursor QTextTable::rowEnd(const QTextCursor &c) const
 
     \sa format()
 */
+void QTextTable::setFormat(const QTextTableFormat &format)
+{
+    QTextTableFormat fmt = format;
+    // don't try to change the number of table columns from here
+    fmt.setColumns(columns());
+    QTextObject::setFormat(fmt);
+}
 
 /*!
     \fn QTextTableFormat QTextTable::format() const
