@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,6 +40,7 @@
 #include "qabstractslider.h"
 #include "qevent.h"
 #include "qabstractslider_p.h"
+#include "qdebug.h"
 #ifndef QT_NO_ACCESSIBILITY
 #include "qaccessible.h"
 #endif
@@ -97,6 +113,7 @@
     QStyle::sliderValueFromPosition() help subclasses and styles to map
     screen coordinates to logical range values.
 
+    \sa QAbstractSpinBox, QSlider, QDial, QScrollBar, {Sliders Example}
 */
 
 /*!
@@ -190,7 +207,7 @@
 
 QAbstractSliderPrivate::QAbstractSliderPrivate()
     : minimum(0), maximum(99), singleStep(1), pageStep(10),
-      value(0), position(0), tracking(true), blocktracking(false), pressed(false),
+      value(0), position(0), pressValue(-1), tracking(true), blocktracking(false), pressed(false),
       invertedAppearance(false), invertedControls(false),
       orientation(Qt::Horizontal), repeatAction(QAbstractSlider::SliderNoAction)
 {
@@ -211,11 +228,15 @@ QAbstractSliderPrivate::~QAbstractSliderPrivate()
 void QAbstractSlider::setRange(int min, int max)
 {
     Q_D(QAbstractSlider);
+    int oldMin = d->minimum;
+    int oldMax = d->maximum;
     d->minimum = min;
     d->maximum = qMax(min, max);
-    sliderChange(SliderRangeChange);
-    emit rangeChanged(d->minimum, d->maximum);
-    setValue(d->value); // re-bound
+    if (oldMin != d->minimum || oldMax != d->maximum) {
+        sliderChange(SliderRangeChange);
+        emit rangeChanged(d->minimum, d->maximum);
+        setValue(d->value); // re-bound
+    }
 }
 
 
@@ -227,7 +248,6 @@ void QAbstractSliderPrivate::setSteps(int single, int page)
     q->sliderChange(QAbstractSlider::SliderStepsChange);
 }
 
-
 /*!
     Constructs an abstract slider.
 
@@ -255,7 +275,6 @@ QAbstractSlider::~QAbstractSlider()
 {
 }
 
-
 /*!
     \property QAbstractSlider::orientation
     \brief the orientation of the slider
@@ -263,7 +282,6 @@ QAbstractSlider::~QAbstractSlider()
     The orientation must be \l Qt::Vertical (the default) or \l
     Qt::Horizontal.
 */
-
 void QAbstractSlider::setOrientation(Qt::Orientation orientation)
 {
     Q_D(QAbstractSlider);
@@ -491,7 +509,7 @@ void QAbstractSlider::setValue(int value)
 {
     Q_D(QAbstractSlider);
     value = d->bound(value);
-    if (d->value == value)
+    if (d->value == value && d->position == value)
         return;
     d->value = value;
     if (d->position != value) {
@@ -513,6 +531,11 @@ void QAbstractSlider::setValue(int value)
     If this property is false (the default), the minimum and maximum will
     be shown in its classic position for the inherited widget. If the
     value is true, the minimum and maximum appear at their opposite location.
+
+    Note: This property makes most sense for sliders and dials. For
+    scroll bars, the visual effect of the scroll bar subcontrols depends on
+    whether or not the styles understand inverted appearance; most styles
+    ignore this property for scroll bars.
 */
 
 bool QAbstractSlider::invertedAppearance() const
@@ -626,7 +649,12 @@ void QAbstractSlider::timerEvent(QTimerEvent *e)
             d->repeatActionTimer.start(d->repeatActionTime, this);
             d->repeatActionTime = 0;
         }
-        triggerAction(d->repeatAction);
+        if (d->repeatAction == SliderPageStepAdd)
+            d->setAdjustedSliderPosition(d->value + d->pageStep);
+        else if (d->repeatAction == SliderPageStepSub)
+            d->setAdjustedSliderPosition(d->value - d->pageStep);
+        else
+            triggerAction(d->repeatAction);
     }
 }
 
@@ -650,6 +678,7 @@ void QAbstractSlider::sliderChange(SliderChange)
 void QAbstractSlider::wheelEvent(QWheelEvent * e)
 {
     Q_D(QAbstractSlider);
+    e->ignore();
     if (e->orientation() != d->orientation && !rect().contains(e->pos()))
         return;
 
@@ -666,11 +695,19 @@ void QAbstractSlider::wheelEvent(QWheelEvent * e)
     offset += e->delta() * step / 120;
     if (d->invertedControls)
         offset = -offset;
+
     if (qAbs(offset) < 1)
         return;
-    setValue(d->value + int(offset));
-    offset -= int(offset);
-    e->accept();
+
+    int prevValue = d->value;
+    d->position = d->value + int(offset); // value will be updated by triggerAction()
+    triggerAction(SliderMove);
+    if (prevValue == d->value) {
+        offset = 0;
+    } else {
+        offset -= int(offset);
+        e->accept();
+    }
 }
 #endif
 /*!
@@ -681,10 +718,29 @@ void QAbstractSlider::keyPressEvent(QKeyEvent *ev)
     Q_D(QAbstractSlider);
     SliderAction action = SliderNoAction;
     switch (ev->key()) {
+#ifdef QT_KEYPAD_NAVIGATION
+        case Qt::Key_Select:
+            if (QApplication::keypadNavigationEnabled())
+                setEditFocus(!hasEditFocus());
+            else
+                ev->ignore();
+            break;
+        case Qt::Key_Back:
+            if (QApplication::keypadNavigationEnabled() && hasEditFocus()) {
+                setValue(d->origValue);
+                setEditFocus(false);
+            } else
+                ev->ignore();
+            break;
+#endif
 
         // It seems we need to use invertedAppearance for Left and right, otherwise, things look weird.
         case Qt::Key_Left:
 #ifdef QT_KEYPAD_NAVIGATION
+            if (QApplication::keypadNavigationEnabled() && !hasEditFocus()) {
+                ev->ignore();
+                return;
+            }
             if (QApplication::keypadNavigationEnabled() && d->orientation == Qt::Vertical)
                 action = d->invertedControls ? SliderSingleStepSub : SliderSingleStepAdd;
             else
@@ -693,6 +749,10 @@ void QAbstractSlider::keyPressEvent(QKeyEvent *ev)
             break;
         case Qt::Key_Right:
 #ifdef QT_KEYPAD_NAVIGATION
+            if (QApplication::keypadNavigationEnabled() && !hasEditFocus()) {
+                ev->ignore();
+                return;
+            }
             if (QApplication::keypadNavigationEnabled() && d->orientation == Qt::Vertical)
                 action = d->invertedControls ? SliderSingleStepAdd : SliderSingleStepSub;
             else
@@ -760,6 +820,17 @@ void QAbstractSlider::changeEvent(QEvent *ev)
 */
 bool QAbstractSlider::event(QEvent *e)
 {
+#ifdef QT_KEYPAD_NAVIGATION
+    Q_D(QAbstractSlider);
+    switch (e->type()) {
+    case QEvent::FocusIn:
+        d->origValue = d->value;
+        break;
+    default:
+        break;
+    }
+#endif
+    
     return QWidget::event(e);
 }
 
