@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -37,6 +52,7 @@
 
 #include "QtCore/qbasictimer.h"
 #include "private/qwidget_p.h"
+#include "qstyle.h"
 
 class QAbstractSliderPrivate : public QWidgetPrivate
 {
@@ -47,7 +63,7 @@ public:
 
     void setSteps(int single, int page);
 
-    int minimum, maximum, singleStep, pageStep, value, position;
+    int minimum, maximum, singleStep, pageStep, value, position, pressValue;
     uint tracking : 1;
     uint blocktracking :1;
     uint pressed : 1;
@@ -58,8 +74,23 @@ public:
     QBasicTimer repeatActionTimer;
     int repeatActionTime;
     QAbstractSlider::SliderAction repeatAction;
+    
+#ifdef QT_KEYPAD_NAVIGATION
+    int origValue;
+#endif
 
     inline int bound(int val) const { return qMax(minimum, qMin(maximum, val)); }
+    inline void setAdjustedSliderPosition(int position)
+    {
+        Q_Q(QAbstractSlider);
+        if (q->style()->styleHint(QStyle::SH_Slider_StopMouseOverSlider, 0, q)) {
+            if ((position > pressValue - 2 * pageStep) && (position < pressValue + 2 * pageStep)) {
+                repeatAction = QAbstractSlider::SliderNoAction;
+                position = pressValue;
+            }
+        }
+        q->setSliderPosition(position);
+    }
 };
 
 #endif // QABSTRACTSLIDER_P_H
