@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -86,11 +101,8 @@ public:
     void setValue(const QVariant &val, EmitPolicy ep, bool updateEdit = true);
     virtual QVariant bound(const QVariant &val, const QVariant &old = QVariant(), int steps = 0) const;
     QLineEdit *lineEdit();
-    void updateButtons();
     virtual void updateEdit();
 
-    virtual QStyleOptionSpinBox getStyleOption() const;
-
     virtual void emitSignals(EmitPolicy ep, const QVariant &old);
     virtual void interpret(EmitPolicy ep);
     virtual QString textFromValue(const QVariant &n) const;
@@ -99,12 +111,14 @@ public:
     void _q_editorTextChanged(const QString &);
     virtual void _q_editorCursorPositionChanged(int oldpos, int newpos);
 
-    QStyle::SubControl newHoverControl(const QPoint &pos);
+    virtual QStyle::SubControl newHoverControl(const QPoint &pos);
     bool updateHoverControl(const QPoint &pos);
 
     void clearCache() const;
+    virtual void updateEditFieldGeometry();
 
     static int variantCompare(const QVariant &arg1, const QVariant &arg2);
+    static QVariant variantBound(const QVariant &min, const QVariant &value, const QVariant &max);
 
     QLineEdit *edit;
     QString prefix, suffix, specialValueText;
@@ -122,6 +136,10 @@ public:
     uint wrapping : 1;
     uint ignoreCursorPositionChanged : 1;
     uint frame : 1;
+    uint accelerate : 1;
+    uint keyboardTracking : 1;
+    QAbstractSpinBox::CorrectionMode correctionMode;
+    int acceleration;
     QStyle::SubControl hoverControl;
     QRect hoverRect;
     QAbstractSpinBox::ButtonSymbols buttonSymbols;
