@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -42,7 +57,10 @@
     buttons except the one that was clicked. By default, a button
     group is exclusive. The buttons in a button group are usually
     checkable QPushButton's, \l{QCheckBox}es (normally for
-    non-exclusive button groups), or \l{QRadioButton}s.
+    non-exclusive button groups), or \l{QRadioButton}s. If you create
+    an exclusive button group, you should ensure that one of the
+    buttons in the group is initially checked; otherwise, the group
+    will initially be in a state where no buttons are checked.
 
     A button is added to the group with addButton(). It can be removed
     from the group with removeButton(). If the group is exclusive, the
@@ -81,15 +99,26 @@
     \property QButtonGroup::exclusive
     \brief whether the button group is exclusive
 
-    If this property is true, then the buttons in the group are
-    checked (toggled), and to untoggle a button you must click on
-    another button in the group. The default value is true.
+    If this property is true then only one button in the group can be checked
+    at any given time. The user can click on any button to check it, and that
+    button will replace the existing one as the checked button in the group.
+
+    In an exclusive group, the user cannot uncheck the currently checked button
+    by clicking on it; instead, another button in the group must be clicked
+    to set the new checked button for that group.
+
+    By default, this property is true.
 */
 
 /*!
     \fn void QButtonGroup::buttonClicked(QAbstractButton *button)
 
-    This signal is emitted when the given \a button is clicked.
+    This signal is emitted when the given \a button is clicked. A
+    button is clicked when it is first pressed and then released, when
+    its shortcut key is typed, or programmatically when
+    QAbstractButton::click() or QAbstractButton::animateClick() is
+    called.
+
 
     \sa checkedButton(), QAbstractButton::clicked()
 */
@@ -103,6 +132,43 @@
     \sa checkedButton(), QAbstractButton::clicked()
 */
 
+/*!
+    \fn void QButtonGroup::buttonPressed(QAbstractButton *button)
+    \since 4.2
+
+    This signal is emitted when the given \a button is pressed down. 
+
+    \sa QAbstractButton::pressed()
+*/
+
+/*!
+    \fn void QButtonGroup::buttonPressed(int id)
+    \since 4.2
+
+    This signal is emitted when a button with the given \a id is
+    pressed down.
+
+    \sa QAbstractButton::pressed()
+*/
+
+/*!
+    \fn void QButtonGroup::buttonReleased(QAbstractButton *button)
+    \since 4.2
+
+    This signal is emitted when the given \a button is released. 
+
+    \sa QAbstractButton::released()
+*/
+
+/*!
+    \fn void QButtonGroup::buttonReleased(int id)
+    \since 4.2
+
+    This signal is emitted when a button with the given \a id is
+    released.
+
+    \sa QAbstractButton::released()
+*/
 
 /*!
     \fn void QButtonGroup::addButton(QAbstractButton *button, int id = -1);
