@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -84,7 +99,8 @@ protected:
         if (combo) {
             QStyleOptionComboBox opt;
             opt.initFrom(combo);
-            if (style()->styleHint(QStyle::SH_ComboBox_Popup, &opt, combo)) {
+            opt.editable = combo->isEditable();
+            if (combo->style()->styleHint(QStyle::SH_ComboBox_Popup, &opt, combo)) {
                 //we paint the empty menu area to avoid having blank space that can happen when scrolling
                 QStyleOptionMenuItem menuOpt;
                 menuOpt.initFrom(this);
@@ -95,7 +111,7 @@ protected:
                 menuOpt.maxIconWidth = 0;
                 menuOpt.tabWidth = 0;
                 QPainter p(viewport());
-                style()->drawControl(QStyle::CE_MenuEmptyArea, &menuOpt, &p, this);
+                combo->style()->drawControl(QStyle::CE_MenuEmptyArea, &menuOpt, &p, this);
             }
         }
         QListView::paintEvent(e);
@@ -198,6 +214,7 @@ public:
     QAbstractItemView *itemView() const;
     void setItemView(QAbstractItemView *itemView);
     int spacing() const;
+    void updateTopBottomMargin();
 
     QTimer blockMouseReleaseTimer;
     QBasicTimer adjustSizeTimer;
