@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,7 +58,7 @@ class QAccessWidget : public QWidget
     friend class QAlphaWidget;
     friend class QRollEffect;
 public:
-    QAccessWidget(QWidget* parent=0, Qt::WFlags f = 0)
+    QAccessWidget(QWidget* parent=0, Qt::WindowFlags f = 0)
         : QWidget(parent, f) {}
 };
 
@@ -58,7 +73,7 @@ class QAlphaWidget: public QWidget, private QEffects
 {
     Q_OBJECT
 public:
-    QAlphaWidget(QWidget* w, Qt::WFlags f = 0);
+    QAlphaWidget(QWidget* w, Qt::WindowFlags f = 0);
 
     void run(int time);
 
@@ -66,6 +81,7 @@ protected:
     void paintEvent(QPaintEvent* e);
     void closeEvent(QCloseEvent*);
     void alphaBlend();
+    bool eventFilter(QObject *, QEvent *);
 
 protected slots:
     void render();
@@ -89,7 +105,7 @@ static QAlphaWidget* q_blend = 0;
 /*
   Constructs a QAlphaWidget.
 */
-QAlphaWidget::QAlphaWidget(QWidget* w, Qt::WFlags f)
+QAlphaWidget::QAlphaWidget(QWidget* w, Qt::WindowFlags f)
     : QWidget(QApplication::desktop()->screen(QApplication::desktop()->screenNumber(w)), f)
 {
 #ifndef Q_WS_WIN
@@ -128,6 +144,11 @@ void QAlphaWidget::run(int time)
     checkTime.start();
 
     showWidget = true;
+
+    //This is roughly equivalent to calling setVisible(true) without actually showing the widget
+    widget->setAttribute(Qt::WA_WState_ExplicitShowHide, true);
+    widget->setAttribute(Qt::WA_WState_Hidden, false);
+
     qApp->installEventFilter(this);
 
     move(widget->geometry().x(),widget->geometry().y());
@@ -152,6 +173,42 @@ void QAlphaWidget::run(int time)
     }
 }
 
+/*
+  \reimp
+*/
+bool QAlphaWidget::eventFilter(QObject *o, QEvent *e)
+{
+    switch (e->type()) {
+    case QEvent::Move:
+	    if (o != widget)
+	        break;
+	    move(widget->geometry().x(),widget->geometry().y());
+	    update();
+	    break;
+    case QEvent::Hide:
+    case QEvent::Close:
+	    if (o != widget)
+	        break;
+    case QEvent::MouseButtonPress:
+	case QEvent::MouseButtonDblClick:
+	    showWidget = false;
+	    render();
+	    break;
+    case QEvent::KeyPress: {
+	        QKeyEvent *ke = (QKeyEvent*)e;
+            if (ke->key() == Qt::Key_Escape) {
+		        showWidget = false;
+            } else {
+		        duration = 0;
+            }
+	        render();
+	        break;
+	}
+    default:
+	    break;
+    }
+    return QWidget::eventFilter(o, e);
+}
 
 /*
   \reimp
@@ -198,6 +255,9 @@ void QAlphaWidget::render()
 #endif
                 widget->hide();
             } else {
+                //Since we are faking the visibility of the widget 
+                //we need to unset the hidden state on it before calling show
+                widget->setAttribute(Qt::WA_WState_Hidden, true);
                 widget->show();
                 lower();
             }
@@ -262,7 +322,7 @@ class QRollEffect : public QWidget, private QEffects
 {
     Q_OBJECT
 public:
-    QRollEffect(QWidget* w, Qt::WFlags f, DirFlags orient);
+    QRollEffect(QWidget* w, Qt::WindowFlags f, DirFlags orient);
 
     void run(int time);
 
@@ -298,7 +358,7 @@ static QRollEffect* q_roll = 0;
 /*
   Construct a QRollEffect widget.
 */
-QRollEffect::QRollEffect(QWidget* w, Qt::WFlags f, DirFlags orient)
+QRollEffect::QRollEffect(QWidget* w, Qt::WindowFlags f, DirFlags orient)
     : QWidget(0, f), orientation(orient)
 {
 #ifndef Q_WS_WIN
@@ -385,6 +445,10 @@ void QRollEffect::run(int time)
     move(widget->geometry().x(),widget->geometry().y());
     resize(qMin(currentWidth, totalWidth), qMin(currentHeight, totalHeight));
 
+    //This is roughly equivalent to calling setVisible(true) without actually showing the widget
+    widget->setAttribute(Qt::WA_WState_ExplicitShowHide, true);
+    widget->setAttribute(Qt::WA_WState_Hidden, false);
+
     show();
     setEnabled(false);
 
@@ -458,6 +522,9 @@ void QRollEffect::scroll()
 #endif
                 widget->hide();
             } else {
+                //Since we are faking the visibility of the widget 
+                //we need to unset the hidden state on it before calling show
+                widget->setAttribute(Qt::WA_WState_Hidden, true);
                 widget->show();
                 lower();
             }
@@ -489,7 +556,7 @@ void qScrollEffect(QWidget* w, QEffects::DirFlags orient, int time)
 
     qApp->sendPostedEvents(w, QEvent::Move);
     qApp->sendPostedEvents(w, QEvent::Resize);
-    Qt::WFlags flags = Qt::ToolTip;
+    Qt::WindowFlags flags = Qt::ToolTip;
 
     // those can be popups - they would steal the focus, but are disabled
     q_roll = new QRollEffect(w, flags, orient);
@@ -512,7 +579,7 @@ void qFadeEffect(QWidget* w, int time)
     qApp->sendPostedEvents(w, QEvent::Move);
     qApp->sendPostedEvents(w, QEvent::Resize);
 
-    Qt::WFlags flags = Qt::ToolTip;
+    Qt::WindowFlags flags = Qt::ToolTip;
 
     // those can be popups - they would steal the focus, but are disabled
     q_blend = new QAlphaWidget(w, flags);
