@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,6 +41,7 @@
 #include "qbitmap.h"
 #include "qstylepainter.h"
 #include "qstyleoption.h"
+#include "qdebug.h"
 #include <private/qwidget_p.h>
 
 class QFocusFramePrivate : public QWidgetPrivate
@@ -40,7 +56,6 @@ public:
     }
     void updateSize();
     void update();
-    QStyleOption getStyleOption() const;
 };
 
 void QFocusFramePrivate::update()
@@ -48,8 +63,11 @@ void QFocusFramePrivate::update()
     Q_Q(QFocusFrame);
     q->setParent(widget->parentWidget());
     updateSize();
-    if (q->parentWidget()->rect().contains(q->geometry())) {
-        q->stackUnder(widget);
+    if (q->parentWidget()->rect().intersects(q->geometry())) {
+        if (q->style()->styleHint(QStyle::SH_FocusFrame_AboveWidget, 0, q))
+            q->raise();
+        else
+            q->stackUnder(widget);
         q->show();
     } else {
         q->hide();
@@ -65,20 +83,28 @@ void QFocusFramePrivate::updateSize()
                widget->width()+(hmargin*2), widget->height()+(vmargin*2));
     if(q->geometry() == geom)
         return;
+
     q->setGeometry(geom);
     QStyleHintReturnMask mask;
-    QStyleOption opt = getStyleOption();
+    QStyleOption opt;
+    q->initStyleOption(&opt);
     if (q->style()->styleHint(QStyle::SH_FocusFrame_Mask, &opt, q, &mask))
         q->setMask(mask.region);
 }
 
-QStyleOption QFocusFramePrivate::getStyleOption() const
+/*!
+    Initialize \a option with the values from this QFocusFrame. This method is useful
+    for subclasses when they need a QStyleOption, but don't want to fill
+    in all the information themselves.
+
+    \sa QStyleOption::initFrom()
+*/
+void QFocusFrame::initStyleOption(QStyleOption *option) const
 {
-    Q_Q(const QFocusFrame);
-    QStyleOption opt;
-    opt.init(q);
-    opt.rect = q->rect();
-    return opt;
+    if (!option)
+        return;
+
+    option->initFrom(this);
 }
 
 /*!
@@ -86,7 +112,7 @@ QStyleOption QFocusFramePrivate::getStyleOption() const
     \brief The QFocusFrame widget provides a focus frame which can be
     outside of a widget's normal paintable area.
 
-    \ingroup basic
+    \ingroup basicwidgets
     \mainclass
 
     Normally an application will not need to create its own
@@ -121,6 +147,7 @@ QFocusFrame::QFocusFrame(QWidget *parent)
     setAttribute(Qt::WA_TransparentForMouseEvents);
     setFocusPolicy(Qt::NoFocus);
     setAttribute(Qt::WA_NoChildEventsForParent, true);
+    setAttribute(Qt::WA_AcceptDrops, style()->styleHint(QStyle::SH_FocusFrame_AboveWidget, 0, this));
 }
 
 /*!
@@ -178,9 +205,10 @@ QFocusFrame::widget() const
 void
 QFocusFrame::paintEvent(QPaintEvent *)
 {
-    Q_D(QFocusFrame);
     QStylePainter p(this);
-    p.drawControl(QStyle::CE_FocusFrame, d->getStyleOption());
+    QStyleOption option;
+    initStyleOption(&option);
+    p.drawControl(QStyle::CE_FocusFrame, option);
 }
 
 
@@ -210,7 +238,10 @@ QFocusFrame::eventFilter(QObject *o, QEvent *e)
             setPalette(d->widget->palette());
             break;
         case QEvent::ZOrderChange:
-            stackUnder(d->widget);
+            if (style()->styleHint(QStyle::SH_FocusFrame_AboveWidget, 0, this))
+                raise();
+            else
+                stackUnder(d->widget);
             break;
         case QEvent::Destroy:
             setWidget(0);
