@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -110,7 +125,7 @@ void QFontFamilyDelegate::paint(QPainter *painter,
 {
     QString text = index.data(Qt::DisplayRole).toString();
     QFont font(option.font);
-    font.setPointSize(font.pointSize() * 3 / 2);
+    font.setPointSize(QFontInfo(font).pointSize() * 3 / 2);
     QFont font2 = font;
     font2.setFamily(text);
 
@@ -171,7 +186,7 @@ QSize QFontFamilyDelegate::sizeHint(const QStyleOptionViewItem &option,
     QString text = index.data(Qt::DisplayRole).toString();
     QFont font(option.font);
 //     font.setFamily(text);
-    font.setPointSize(font.pointSize() * 3/2);
+    font.setPointSize(QFontInfo(font).pointSize() * 3/2);
     QFontMetrics fontMetrics(font);
     return QSize(fontMetrics.width(text), fontMetrics.lineSpacing());
 }
@@ -242,7 +257,7 @@ void QFontComboBoxPrivate::_q_currentChanged(const QString &text)
     select a font family.
 
     \since 4.2
-    \ingroup basic
+    \ingroup basicwidgets
     \ingroup text
 
     The combobox is populated with an alphabetized list of font
