@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -49,11 +64,11 @@ public:
 };
 
 /*!
-    \class QLCDNumber qlcdnumber.h
+    \class QLCDNumber
 
     \brief The QLCDNumber widget displays a number with LCD-like digits.
 
-    \ingroup basic
+    \ingroup basicwidgets
     \mainclass
 
     It can display a number in just about any size. It can display
@@ -87,10 +102,20 @@ public:
     http://www.nvg.ntnu.no/sinclair/computers/zxspectrum/zxspectrum.htm
     Sinclair Spectrum\endlink.
 
-    \inlineimage qlcdnum-m.png Screenshot in Motif style
-    \inlineimage qlcdnum-w.png Screenshot in Windows style
+    \table
+    \row \o \inlineimage motif-lcdnumber.png Screenshot of a Motif style LCD number widget
+    \inlineimage cde-lcdnumber.png Screenshot of a CDE style LCD number widget
+    \inlineimage windows-lcdnumber.png Screenshot of a Windows style LCD number widget
+    \inlineimage windowsxp-lcdnumber.png Screenshot of a Windows XP style LCD number widget
+    \inlineimage macintosh-lcdnumber.png Screenshot of a Macintosh style LCD number widget
+    \inlineimage plastique-lcdnumber.png Screenshot of a Plastique style LCD number widget
+    \row \o LCD number widgets shown in various widget styles (from left to right):
+    \l{Motif Style Widget Gallery}{Motif}, \l{CDE Style Widget Gallery}{CDE},
+    \l{Windows Style Widget Gallery}{Windows}, \l{Windows XP Style Widget Gallery}{Windows XP},
+    \l{Macintosh Style Widget Gallery}{Macintosh}, \l{Plastique Style Widget Gallery}{Plastique}.
+    \endtable
 
-    \sa QLabel, QFrame
+    \sa QLabel, QFrame, {Digital Clock Example}, {Tetrix Example}
 */
 
 /*!
@@ -117,9 +142,9 @@ public:
     This type determines the visual appearance of the QLCDNumber
     widget.
 
-    \value Outline gives raised segments filled with the background brush.
-    \value Filled gives raised segments filled with the foreground brush.
-    \value Flat gives flat segments filled with the foreground brush.
+    \value Outline gives raised segments filled with the background color.
+    \value Filled gives raised segments filled with the windowText color.
+    \value Flat gives flat segments filled with the windowText color.
 */
 
 
@@ -168,18 +193,18 @@ static QString int2string(int num, int base, int ndigits, bool *oflow)
                 } while (n != 0);
                 len = ndigits - len;
                 if (len > 0)
-                s.fill(' ', len);
+                s.fill(QLatin1Char(' '), len);
                 s += QString::fromLatin1(p);
             }
             break;
     }
     if (negative) {
         for (int i=0; i<(int)s.length(); i++) {
-            if (s[i] != ' ') {
+            if (s[i] != QLatin1Char(' ')) {
                 if (i != 0) {
-                    s[i-1] = '-';
+                    s[i-1] = QLatin1Char('-');
                 } else {
-                    s.insert(0, '-');
+                    s.insert(0, QLatin1Char('-'));
                 }
                 break;
             }
@@ -206,10 +231,10 @@ static QString double2string(double num, int base, int ndigits, bool *oflow)
         int nd = ndigits;
         do {
             s.sprintf("%*.*g", ndigits, nd, num);
-            int i = s.indexOf('e');
-            if (i > 0 && s[i+1]=='+') {
-                s[i] = ' ';
-                s[i+1] = 'e';
+            int i = s.indexOf(QLatin1Char('e'));
+            if (i > 0 && s[i+1]==QLatin1Char('+')) {
+                s[i] = QLatin1Char(' ');
+                s[i+1] = QLatin1Char('e');
             }
         } while (nd-- && (int)s.length() > ndigits);
     }
@@ -436,9 +461,9 @@ void QLCDNumber::setNumDigits(int numDigits)
     }
     if (d->digitStr.isNull()) {                  // from constructor
         d->ndigits = numDigits;
-        d->digitStr.fill(' ', d->ndigits);
+        d->digitStr.fill(QLatin1Char(' '), d->ndigits);
         d->points.fill(0, d->ndigits);
-        d->digitStr[d->ndigits - 1] = '0';            // "0" is the default number
+        d->digitStr[d->ndigits - 1] = QLatin1Char('0');            // "0" is the default number
     } else {
         bool doDisplay = d->ndigits == 0;
         if (numDigits == d->ndigits)             // no change
@@ -448,7 +473,7 @@ void QLCDNumber::setNumDigits(int numDigits)
         if (numDigits > d->ndigits) {            // expand
             dif = numDigits - d->ndigits;
             QString buf;
-            buf.fill(' ', dif);
+            buf.fill(QLatin1Char(' '), dif);
             d->digitStr.insert(0, buf);
             d->points.resize(numDigits);
             for (i=numDigits-1; i>=dif; i--)
@@ -739,18 +764,18 @@ void QLCDNumberPrivate::internalSetString(const QString& s)
         if (len == ndigits)
             buffer = s;
         else
-            buffer = s.right(ndigits).rightJustified(ndigits, ' ');
+            buffer = s.right(ndigits).rightJustified(ndigits, QLatin1Char(' '));
     } else {
         int  index = -1;
         bool lastWasPoint = true;
         newPoints.clearBit(0);
         for (i=0; i<len; i++) {
-            if (s[i] == '.') {
+            if (s[i] == QLatin1Char('.')) {
                 if (lastWasPoint) {           // point already set for digit?
                     if (index == ndigits - 1) // no more digits
                         break;
                     index++;
-                    buffer[index] = ' ';        // 2 points in a row, add space
+                    buffer[index] = QLatin1Char(' ');        // 2 points in a row, add space
                 }
                 newPoints.setBit(index);        // set decimal point
                 lastWasPoint = true;
@@ -770,7 +795,7 @@ void QLCDNumberPrivate::internalSetString(const QString& s)
                                    newPoints.testBit(i));
             }
             for(i=0; i<ndigits-index-1; i++) {
-                buffer[i] = ' ';
+                buffer[i] = QLatin1Char(' ');
                 newPoints.clearBit(i);
             }
         }
@@ -1030,8 +1055,7 @@ void QLCDNumberPrivate::drawSegment(const QPoint &pos, char segmentNo, QPainter
             LINETO(0,0);
             break;
         default :
-            qWarning("QLCDNumber::drawSegment: (%s) Internal error."
-                     "  Illegal segment id: %d\n",
+            qWarning("QLCDNumber::drawSegment: (%s) Illegal segment id: %d\n",
                      q->objectName().toLocal8Bit().constData(), segmentNo);
         }
         // End exact copy
@@ -1164,8 +1188,7 @@ void QLCDNumberPrivate::drawSegment(const QPoint &pos, char segmentNo, QPainter
             LINETO(0,0);
             break;
         default :
-            qWarning("QLCDNumber::drawSegment: (%s) Internal error."
-                     "  Illegal segment id: %d\n",
+            qWarning("QLCDNumber::drawSegment: (%s) Illegal segment id: %d\n",
                      q->objectName().toLocal8Bit().constData(), segmentNo);
         }
 
@@ -1227,4 +1250,20 @@ bool QLCDNumber::event(QEvent *e)
     return QFrame::event(e);
 }
 
+/*!
+    \fn void QLCDNumber::setMargin(int margin)
+    Sets the width of the margin around the contents of the widget to \a margin.
+    
+    Use QWidget::setContentsMargins() instead.
+    \sa margin(), QWidget::setContentsMargins()
+*/
+
+/*!
+    \fn int QLCDNumber::margin() const
+    Returns the with of the the margin around the contents of the widget.
+    
+    Use QWidget::getContentsMargins() instead.
+    \sa setMargin(), QWidget::getContentsMargins()
+*/
+
 #endif // QT_NO_LCDNUMBER
