@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -37,6 +52,9 @@ class QValidator;
 class QMenu;
 class QLineEditPrivate;
 class QCompleter;
+class QStyleOptionFrame;
+class QAbstractSpinBox;
+class QDateTimeEdit;
 
 class Q_GUI_EXPORT QLineEdit : public QWidget
 {
@@ -185,6 +203,7 @@ protected:
 #endif
 
     void inputMethodEvent(QInputMethodEvent *);
+    void initStyleOption(QStyleOptionFrame *option) const;
 public:
     QVariant inputMethodQuery(Qt::InputMethodQuery) const;
     bool event(QEvent *);
@@ -209,8 +228,8 @@ public:
 
     QT3_SUPPORT void setFrameRect(QRect) {}
     QT3_SUPPORT QRect frameRect() const { return QRect(); }
-    enum DummyFrame { Box, Sunken, Plain, Raised, MShadow, NoFrame, Panel, StyledPanel, 
-                      HLine, VLine, GroupBoxPanel, WinPanel, ToolBarPanel, MenuBarPanel, 
+    enum DummyFrame { Box, Sunken, Plain, Raised, MShadow, NoFrame, Panel, StyledPanel,
+                      HLine, VLine, GroupBoxPanel, WinPanel, ToolBarPanel, MenuBarPanel,
                       PopupPanel, LineEditPanel, TabWidgetPanel, MShape };
     QT3_SUPPORT void setFrameShadow(DummyFrame) {}
     QT3_SUPPORT DummyFrame frameShadow() const { return Plain; }
@@ -220,10 +239,10 @@ public:
     QT3_SUPPORT int frameStyle() const  { return 0; }
     QT3_SUPPORT int frameWidth() const { return 0; }
     QT3_SUPPORT void setLineWidth(int) {}
-    QT3_SUPPORT int lineWidth() const { return 0; }    
+    QT3_SUPPORT int lineWidth() const { return 0; }
     QT3_SUPPORT void setMargin(int margin) { setContentsMargins(margin, margin, margin, margin); }
-    QT3_SUPPORT int margin() const 
-    { int margin; int dummy; getContentsMargins(&margin, &dummy, &dummy, &dummy);  return margin; }    
+    QT3_SUPPORT int margin() const
+    { int margin; int dummy; getContentsMargins(&margin, &dummy, &dummy, &dummy);  return margin; }
     QT3_SUPPORT void setMidLineWidth(int) {}
     QT3_SUPPORT int midLineWidth() const { return 0; }
 
@@ -232,6 +251,10 @@ Q_SIGNALS:
 #endif
 
 private:
+    friend class QAbstractSpinBox;
+#ifdef QT_KEYPAD_NAVIGATION
+    friend class QDateTimeEdit;
+#endif
     Q_DISABLE_COPY(QLineEdit)
     Q_DECLARE_PRIVATE(QLineEdit)
     Q_PRIVATE_SLOT(d_func(), void _q_clipboardChanged())
