@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -49,22 +64,27 @@ public:
     QRect rect;
     QRubberBand::Shape shape;
     QRegion clipping;
-    QStyleOptionRubberBand getStyleOption() const;
     void updateMask();
 };
 
-QStyleOptionRubberBand QRubberBandPrivate::getStyleOption() const
+/*!
+    Initialize \a option with the values from this QRubberBand. This method
+    is useful for subclasses when they need a QStyleOptionRubberBand, but don't want
+    to fill in all the information themselves.
+
+    \sa QStyleOption::initFrom()
+*/
+void QRubberBand::initStyleOption(QStyleOptionRubberBand *option) const
 {
-    Q_Q(const QRubberBand);
-    QStyleOptionRubberBand opt;
-    opt.init(q);
-    opt.shape = shape;
+    if (!option)
+        return;
+    option->initFrom(this);
+    option->shape = d_func()->shape;
 #ifndef Q_WS_MAC
-    opt.opaque = true;
+    option->opaque = true;
 #else
-    opt.opaque = q->windowFlags() & RUBBERBAND_WINDOW_TYPE;
+    option->opaque = windowFlags() & RUBBERBAND_WINDOW_TYPE;
 #endif
-    return opt;
 }
 
 /*!
@@ -195,7 +215,8 @@ void QRubberBandPrivate::updateMask()
 {
     Q_Q(QRubberBand);
     QStyleHintReturnMask mask;
-    QStyleOptionRubberBand opt = getStyleOption();
+    QStyleOptionRubberBand opt;
+    q->initStyleOption(&opt);
     if (q->style()->styleHint(QStyle::SH_RubberBand_Mask, &opt, q, &mask)) {
         q->setMask(mask.region);
     } else {
@@ -208,10 +229,10 @@ void QRubberBandPrivate::updateMask()
 */
 void QRubberBand::paintEvent(QPaintEvent *)
 {
-    Q_D(QRubberBand);
-
     QStylePainter painter(this);
-    painter.drawControl(QStyle::CE_RubberBand, d->getStyleOption());
+    QStyleOptionRubberBand option;
+    initStyleOption(&option);
+    painter.drawControl(QStyle::CE_RubberBand, option);
 }
 
 /*!
@@ -232,15 +253,8 @@ void QRubberBand::changeEvent(QEvent *e)
         break;
     }
 
-    // calling raise() on LayoutDirectionChange breaks
-    // QWidgetPrivate::setLayoutDirection_helper()
-    // calling raise() on PaletteChange breaks
-    // QWidgetPrivate::propagatePaletteChange()
-    if (e->type() != QEvent::LayoutDirectionChange
-            && e->type() != QEvent::PaletteChange)
+    if (e->type() == QEvent::ZOrderChange)
         raise();
-
-    // ### The raise should probably only be called on the ZOrderChange event
 }
 
 /*!
