@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -26,10 +41,11 @@
 
 #include <QtGui/qwidget.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_SIZEGRIP
-
 class QSizeGripPrivate;
 class Q_GUI_EXPORT QSizeGrip : public QWidget
 {
@@ -39,25 +55,33 @@ public:
     ~QSizeGrip();
 
     QSize sizeHint() const;
-
     void setVisible(bool);
+
 protected:
     void paintEvent(QPaintEvent *);
     void mousePressEvent(QMouseEvent *);
     void mouseMoveEvent(QMouseEvent *);
-
+    void mouseReleaseEvent(QMouseEvent *mouseEvent);
+    void moveEvent(QMoveEvent *moveEvent);
+    void showEvent(QShowEvent *showEvent);
+    void hideEvent(QHideEvent *hideEvent);
     bool eventFilter(QObject *, QEvent *);
     bool event(QEvent *);
+#ifdef Q_WS_WIN
+    bool winEvent(MSG *m, long *result);
+#endif
 
 public:
 #ifdef QT3_SUPPORT
     QT3_SUPPORT_CONSTRUCTOR QSizeGrip(QWidget *parent, const char *name);
 #endif
+
 private:
     Q_DECLARE_PRIVATE(QSizeGrip)
     Q_DISABLE_COPY(QSizeGrip)
 };
-
 #endif // QT_NO_SIZEGRIP
 
+QT_END_HEADER
+
 #endif // QSIZEGRIP_H
