@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -55,6 +70,10 @@ public:
                                   QValidator::State &state) const;
     bool isIntermediateValue(const QString &str) const;
     QChar thousand;
+
+    inline void init() {
+        setLayoutItemMargins(QStyle::SE_SpinBoxLayoutItem);
+    }
 };
 
 class QDoubleSpinBoxPrivate : public QAbstractSpinBoxPrivate
@@ -80,7 +99,7 @@ public:
     \class QSpinBox
     \brief The QSpinBox class provides a spin box widget.
 
-    \ingroup basic
+    \ingroup basicwidgets
     \mainclass
 
     QSpinBox is designed to handle integers and discrete sets of
@@ -177,6 +196,8 @@ public:
 QSpinBox::QSpinBox(QWidget *parent)
     : QAbstractSpinBox(*new QSpinBoxPrivate, parent)
 {
+    Q_D(QSpinBox);
+    d->init();
 }
 
 #ifdef QT3_SUPPORT
@@ -187,7 +208,9 @@ QSpinBox::QSpinBox(QWidget *parent)
 QSpinBox::QSpinBox(QWidget *parent, const char *name)
     : QAbstractSpinBox(*new QSpinBoxPrivate, parent)
 {
+    Q_D(QSpinBox);
     setObjectName(QString::fromAscii(name));
+    d->init();
 }
 
 /*!
@@ -202,6 +225,7 @@ QSpinBox::QSpinBox(int minimum, int maximum, int step, QWidget *parent, const ch
     d->maximum = QVariant(qMax<int>(minimum, maximum));
     d->singleStep = QVariant(step);
     setObjectName(QString::fromAscii(name));
+    d->init();
 }
 
 #endif
@@ -501,19 +525,15 @@ void QSpinBox::fixup(QString &input) const
     \brief The QDoubleSpinBox class provides a spin box widget that
     takes doubles.
 
-    \ingroup basic
+    \ingroup basicwidgets
     \mainclass
 
     QDoubleSpinBox allows the user to choose a value by clicking the
     up and down buttons or by pressing Up or Down on the keyboard to
     increase or decrease the value currently displayed. The user can
-    also type the value in manually. If the value is entered directly
-    into the spin box, the value will be changed and valueChanged()
-    will be emitted with the new value when Enter or Return is
-    pressed, when the spin box loses focus or when the spin box is
-    deactivated (see QWidget::windowActivationChanged()). The spin box
-    supports double values but can be extended to use different
-    strings with validate(), textFromValue() and valueFromText().
+    also type the value in manually. The spin box supports double
+    values but can be extended to use different strings with
+    validate(), textFromValue() and valueFromText().
 
     Every time the value changes QDoubleSpinBox emits the
     valueChanged() signal. The current value can be fetched with
@@ -810,9 +830,10 @@ void QDoubleSpinBox::setRange(double minimum, double maximum)
      \brief the precision of the spin box, in decimals
 
      Sets how many decimals the spinbox will use for displaying and
-     interpreting doubles. The valid decimal range is 0-13. The
-     default is 2. \a decimals will be bounded to a value that is
-     within the valid range.
+     interpreting doubles.
+
+     \warning The results might not be reliable with very high values
+     for \a decimals.
 
      Note: The maximum, minimum and value might change as a result of
      changing this property.
@@ -828,7 +849,7 @@ int QDoubleSpinBox::decimals() const
 void QDoubleSpinBox::setDecimals(int decimals)
 {
     Q_D(QDoubleSpinBox);
-    d->decimals = qBound(0, decimals, 13);
+    d->decimals = qMax(0, decimals);
 
     setRange(minimum(), maximum()); // make sure values are rounded
     setValue(value());
@@ -1042,6 +1063,8 @@ QVariant QSpinBoxPrivate::validateAndInterpret(QString &input, int &pos,
                        || (min >= 0 && copy == QLatin1String("+")))) {
         state = QValidator::Intermediate;
         QSBDEBUG() << __FILE__ << __LINE__<< "num is set to" << num;
+    } else if (copy.startsWith(QLatin1String("-")) && min >= 0) {
+        state = QValidator::Invalid; // special-case -0 will be interpreted as 0 and thus not be invalid with a range from 0-100
     } else {
         bool ok = false;
         bool removedThousand = false;
@@ -1507,6 +1530,13 @@ static bool isIntermediateValueHelper(qint64 num, qint64 min, qint64 max, qint64
 /*! \reimp */
 bool QSpinBox::event(QEvent *event)
 {
+    Q_D(QSpinBox);
+    if (event->type() == QEvent::StyleChange
+#ifdef Q_WS_MAC
+            || event->type() == QEvent::MacSizeChange
+#endif
+            )
+        d->setLayoutItemMargins(QStyle::SE_SpinBoxLayoutItem);
     return QAbstractSpinBox::event(event);
 }
 
