@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -1501,16 +1516,24 @@ QSize QSplitter::minimumSizeHint() const
     ensurePolished();
     int l = 0;
     int t = 0;
-    QObjectList childList = children();
-    for (int i = 0; i < childList.size(); ++i) {
-        if (QWidget *w = qobject_cast<QWidget *>(childList.at(i))) {
-            if (w->isHidden())
-                continue;
-            QSize s = qSmartMinSize(w);
-            if (s.isValid()) {
-                l += d->pick(s);
-                t = qMax(t, d->trans(s));
-            }
+
+    for (int i = 0; i < d->list.size(); ++i) {
+        QSplitterLayoutStruct *s = d->list.at(i);
+        if (!s || !s->widget)
+            continue;
+        if (s->widget->isHidden())
+            continue;
+        QSize widgetSize = qSmartMinSize(s->widget);
+        if (widgetSize.isValid()) {
+            l += d->pick(widgetSize);
+            t = qMax(t, d->trans(widgetSize));
+        }
+        if (!s->handle || s->handle->isHidden())
+            continue;
+        QSize splitterSize = s->handle->sizeHint();
+        if (splitterSize.isValid()) {
+            l += d->pick(splitterSize);
+            t = qMax(t, d->trans(splitterSize));
         }
     }
     return orientation() == Qt::Horizontal ? QSize(l, t) : QSize(t, l);
