@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -84,7 +99,7 @@ public:
     The widget() function returns the widget at a given index
     position. The index of the widget that is shown on screen is given
     by currentIndex() and can be changed using setCurrentIndex(). In a
-    similar manor, the currently shown widget can be retrieved using
+    similar manner, the currently shown widget can be retrieved using
     the currentWidget() function, and altered using the
     setCurrentWidget() function.
 
@@ -92,7 +107,7 @@ public:
     widget is removed from the stacked widget, the currentChanged()
     and widgetRemoved() signals are emitted respectively.
 
-    \sa QStackedLayout, QTabWidget
+    \sa QStackedLayout, QTabWidget, {Config Dialog Example}
 */
 
 /*!
@@ -111,7 +126,7 @@ public:
     \fn void QStackedWidget::widgetRemoved(int index)
 
     This signal is emitted whenever a widget is removed. The widget's
-    index is passed as parameter.
+    \a index is passed as parameter.
 
     \sa removeWidget()
 */
@@ -219,9 +234,14 @@ QWidget *QStackedWidget::currentWidget() const
 
     \sa currentWidget(), setCurrentIndex()
  */
-void QStackedWidget::setCurrentWidget(QWidget *w)
+void QStackedWidget::setCurrentWidget(QWidget *widget)
 {
-    d_func()->layout->setCurrentWidget(w);
+    Q_D(QStackedWidget);
+    if (d->layout->indexOf(widget) == -1) {
+        qWarning("QStackedWidget::setCurrentWidget: widget %p not contained in stack", widget);
+        return;
+    }
+    d->layout->setCurrentWidget(widget);
 }
 
 /*!
