@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,18 +9,35 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
+#include "private/qlayoutengine_p.h"
+#include "qabstractitemdelegate.h"
 #include "qapplication.h"
 #include "qbitmap.h"
 #include "qcursor.h"
@@ -28,11 +45,11 @@
 #include "qpainter.h"
 #include "qstyle.h"
 #include "qstyleoption.h"
+#include "qstylepainter.h"
 #include "qtabwidget.h"
 #include "qtooltip.h"
 #include "qwhatsthis.h"
-#include "qstylepainter.h"
-#include <private/qinternal_p.h>
+#include "private/qtextengine_p.h"
 #ifndef QT_NO_ACCESSIBILITY
 #include "qaccessible.h"
 #endif
@@ -42,65 +59,90 @@
 
 #ifndef QT_NO_TABBAR
 
-QStyleOptionTabV2 QTabBarPrivate::getStyleOption(int tab) const
+inline static bool verticalTabs(QTabBar::Shape shape)
 {
-    Q_Q(const QTabBar);
-    QStyleOptionTabV2 opt;
-    const QTabBarPrivate::Tab *ptab = &tabList.at(tab);
-    opt.init(q);
-    opt.state &= ~(QStyle::State_HasFocus | QStyle::State_MouseOver);
-    opt.rect = q->tabRect(tab);
-    bool isCurrent = tab == currentIndex;
-    opt.row = 0;
-    if (tab == pressedIndex)
-        opt.state |= QStyle::State_Sunken;
+    return shape == QTabBar::RoundedWest
+           || shape == QTabBar::RoundedEast
+           || shape == QTabBar::TriangularWest
+           || shape == QTabBar::TriangularEast;
+}
+
+/*!
+    Initialize \a option with the values from the tab at \a tabIndex. This method
+    is useful for subclasses when they need a QStyleOptionTab or QStyleOptionTabV2, but don't want
+    to fill in all the information themselves. This function will check the version
+    of the QStyleOptionTab and fill in the additional values for a
+    QStyleOptionTabV2.
+
+    \sa QStyleOption::initFrom() QTabWidget::initStyleOption()
+*/
+void QTabBar::initStyleOption(QStyleOptionTab *option, int tabIndex) const
+{
+    Q_D(const QTabBar);
+    int totalTabs = d->tabList.size();
+
+    if (!option || (tabIndex < 0 || tabIndex >= totalTabs))
+        return;
+
+    const QTabBarPrivate::Tab &tab = d->tabList.at(tabIndex);
+    option->initFrom(this);
+    option->state &= ~(QStyle::State_HasFocus | QStyle::State_MouseOver);
+    option->rect = tabRect(tabIndex);
+    bool isCurrent = tabIndex == d->currentIndex;
+    option->row = 0;
+    if (tabIndex == d->pressedIndex)
+        option->state |= QStyle::State_Sunken;
     if (isCurrent)
-        opt.state |= QStyle::State_Selected;
-    if (isCurrent && q->hasFocus())
-        opt.state |= QStyle::State_HasFocus;
-    if (!ptab->enabled)
-        opt.state &= ~QStyle::State_Enabled;
-    if (q->isActiveWindow())
-        opt.state |= QStyle::State_Active;
-    if (opt.rect == hoverRect)
-        opt.state |= QStyle::State_MouseOver;
-    opt.shape = shape;
-    opt.text = ptab->text;
+        option->state |= QStyle::State_Selected;
+    if (isCurrent && hasFocus())
+        option->state |= QStyle::State_HasFocus;
+    if (!tab.enabled)
+        option->state &= ~QStyle::State_Enabled;
+    if (isActiveWindow())
+        option->state |= QStyle::State_Active;
+    if (option->rect == d->hoverRect)
+        option->state |= QStyle::State_MouseOver;
+    option->shape = d->shape;
+    option->text = tab.text;
 
-    if (ptab->textColor.isValid())
-        opt.palette.setColor(q->foregroundRole(), ptab->textColor);
+    if (tab.textColor.isValid())
+        option->palette.setColor(foregroundRole(), tab.textColor);
 
-    opt.icon = ptab->icon;
-    opt.iconSize = q->iconSize();  // Will get the default value then.
+    option->icon = tab.icon;
+    if (QStyleOptionTabV2 *optionV2 = qstyleoption_cast<QStyleOptionTabV2 *>(option))
+        optionV2->iconSize = iconSize();  // Will get the default value then.
 
-    int totalTabs = tabList.size();
 
-    if (tab > 0 && tab - 1 == currentIndex)
-        opt.selectedPosition = QStyleOptionTab::PreviousIsSelected;
-    else if (tab < totalTabs - 1 && tab + 1 == currentIndex)
-        opt.selectedPosition = QStyleOptionTab::NextIsSelected;
+    if (tabIndex > 0 && tabIndex - 1 == d->currentIndex)
+        option->selectedPosition = QStyleOptionTab::PreviousIsSelected;
+    else if (tabIndex < totalTabs - 1 && tabIndex + 1 == d->currentIndex)
+        option->selectedPosition = QStyleOptionTab::NextIsSelected;
     else
-        opt.selectedPosition = QStyleOptionTab::NotAdjacent;
+        option->selectedPosition = QStyleOptionTab::NotAdjacent;
 
-    if (tab == 0) {
+    if (tabIndex == 0) {
         if (totalTabs > 1)
-            opt.position = QStyleOptionTab::Beginning;
+            option->position = QStyleOptionTab::Beginning;
         else
-            opt.position = QStyleOptionTab::OnlyOneTab;
-    } else if (tab == totalTabs - 1) {
-        opt.position = QStyleOptionTab::End;
+            option->position = QStyleOptionTab::OnlyOneTab;
+    } else if (tabIndex == totalTabs - 1) {
+        option->position = QStyleOptionTab::End;
     } else {
-        opt.position = QStyleOptionTab::Middle;
+        option->position = QStyleOptionTab::Middle;
     }
 #ifndef QT_NO_TABWIDGET
-    if (const QTabWidget *tw = qobject_cast<const QTabWidget *>(q->parentWidget())) {
+    if (const QTabWidget *tw = qobject_cast<const QTabWidget *>(parentWidget())) {
         if (tw->cornerWidget(Qt::TopLeftCorner) || tw->cornerWidget(Qt::BottomLeftCorner))
-            opt.cornerWidgets |= QStyleOptionTab::LeftCornerWidget;
+            option->cornerWidgets |= QStyleOptionTab::LeftCornerWidget;
         if (tw->cornerWidget(Qt::TopRightCorner) || tw->cornerWidget(Qt::BottomRightCorner))
-            opt.cornerWidgets |= QStyleOptionTab::RightCornerWidget;
+            option->cornerWidgets |= QStyleOptionTab::RightCornerWidget;
     }
+    int hframe  = style()->pixelMetric(QStyle::PM_TabBarTabHSpace, option, this);
+
+    option->text = fontMetrics().elidedText(option->text, d->elideMode,
+                        1 + (verticalTabs(d->shape) ? tab.rect.height() : tab.rect.width()) - hframe,
+                        Qt::TextShowMnemonic);
 #endif
-    return opt;
 }
 
 /*!
@@ -225,19 +267,24 @@ void QTabBarPrivate::init()
 {
     Q_Q(QTabBar);
     leftB = new QToolButton(q);
+    leftB->setAutoRepeat(true);
     QObject::connect(leftB, SIGNAL(clicked()), q, SLOT(_q_scrollTabs()));
     leftB->hide();
     rightB = new QToolButton(q);
+    rightB->setAutoRepeat(true);
     QObject::connect(rightB, SIGNAL(clicked()), q, SLOT(_q_scrollTabs()));
     rightB->hide();
 #ifdef QT_KEYPAD_NAVIGATION
     if (QApplication::keypadNavigationEnabled()) {
         leftB->setFocusPolicy(Qt::NoFocus);
         rightB->setFocusPolicy(Qt::NoFocus);
-    }
+        q->setFocusPolicy(Qt::NoFocus);
+    } else
 #endif
     q->setFocusPolicy(Qt::TabFocus);
     q->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
+    elideMode = Qt::TextElideMode(q->style()->styleHint(QStyle::SH_TabBar_ElideMode, 0, q));
+    useScrollButtons = !q->style()->styleHint(QStyle::SH_TabBar_PreferNoArrows, 0, q);
 }
 
 QTabBarPrivate::Tab *QTabBarPrivate::at(int index)
@@ -261,15 +308,6 @@ int QTabBarPrivate::indexAtPos(const QPoint &p) const
     return -1;
 }
 
-inline static bool verticalTabs(QTabBar::Shape shape)
-{
-    return shape == QTabBar::RoundedWest
-           || shape == QTabBar::RoundedEast
-           || shape == QTabBar::TriangularWest
-           || shape == QTabBar::TriangularEast;
-}
-
-
 void QTabBarPrivate::layoutTabs()
 {
     Q_Q(QTabBar);
@@ -277,44 +315,99 @@ void QTabBarPrivate::layoutTabs()
     layoutDirty = false;
     QSize size = q->size();
     int last, available;
+    int maxExtent;
+    int i;
     bool vertTabs = verticalTabs(shape);
+    int tabChainIndex = 0;
+    Qt::Alignment tabAlignment = Qt::Alignment(q->style()->styleHint(QStyle::SH_TabBar_Alignment, 0, q));
+    QVector<QLayoutStruct> tabChain(tabList.count() + 2);
+
+    // We put an empty item at the front and back and set its expansive attribute
+    // depending on tabAlignment.
+    tabChain[tabChainIndex].init();
+    tabChain[tabChainIndex].expansive = tabAlignment != Qt::AlignLeft;
+    tabChain[tabChainIndex].empty = true;
+    ++tabChainIndex;
+
+    // We now go through our list of tabs and set the minimum size and the size hint
+    // This will allow us to elide text if necessary. Since we don't set
+    // a maximum size, tabs will EXPAND to fill up the empty space.
+    // Since tab widget is rather *ahem* strict about keeping the geometry of the
+    // tab bar to its absolute minimum, this won't bleed through, but will show up
+    // if you use tab bar on its own (a.k.a. not a bug, but a feature).
+    // Update: if squeezeTabs is true, we DO set a maximum size to prevent the tabs
+    // being wider than necessary.
     if (!vertTabs) {
+        int minx = 0;
         int x = 0;
         int maxHeight = 0;
-        int i;
-        for (i = 0; i < tabList.count(); ++i) {
+        for (i = 0; i < tabList.count(); ++i, ++tabChainIndex) {
             QSize sz = q->tabSizeHint(i);
-            tabList[i].rect = QRect(x, 0, sz.width(), sz.height());
-            maxHeight = qMax(maxHeight, sz.height());
+            tabList[i].maxRect = QRect(x, 0, sz.width(), sz.height());
             x += sz.width();
+            maxHeight = qMax(maxHeight, sz.height());
+            sz = minimumTabSizeHint(i);
+            tabList[i].minRect = QRect(minx, 0, sz.width(), sz.height());
+            minx += sz.width();
+            tabChain[tabChainIndex].init();
+            tabChain[tabChainIndex].sizeHint = tabList.at(i).maxRect.width();
+            tabChain[tabChainIndex].minimumSize = sz.width();
+            tabChain[tabChainIndex].empty = false;
+            tabChain[tabChainIndex].expansive = true;
+
+            if (squeezeTabs)
+                tabChain[tabChainIndex].maximumSize = tabChain[tabChainIndex].sizeHint;
         }
 
-        // Go through the list again and make sure we have a consistent height
-        for (i = 0; i < tabList.count(); ++i)
-            tabList[i].rect.setHeight(maxHeight);
-
-        last = x;
+        last = minx;
         available = size.width();
+        maxExtent = maxHeight;
     } else {
+        int miny = 0;
         int y = 0;
-        int i;
         int maxWidth = 0;
-        for (i = 0; i < tabList.count(); ++i) {
+        for (i = 0; i < tabList.count(); ++i, ++tabChainIndex) {
             QSize sz = q->tabSizeHint(i);
-            tabList[i].rect = QRect(0, y, sz.width(), sz.height());
-            maxWidth = qMax(0, sz.width());
+            tabList[i].maxRect = QRect(0, y, sz.width(), sz.height());
             y += sz.height();
+            maxWidth = qMax(0, sz.width());
+            sz = minimumTabSizeHint(i);
+            tabList[i].minRect = QRect(0, miny, sz.width(), sz.height());
+            miny += sz.height();
+            tabChain[tabChainIndex].init();
+            tabChain[tabChainIndex].sizeHint = tabList.at(i).maxRect.height();
+            tabChain[tabChainIndex].minimumSize = sz.height();
+            tabChain[tabChainIndex].empty = false;
+            tabChain[tabChainIndex].expansive = true;
+
+            if (squeezeTabs)
+                tabChain[tabChainIndex].maximumSize = tabChain[tabChainIndex].sizeHint;
         }
 
-        // Consistent width
-        for (i = 0; i < tabList.count(); ++i)
-            tabList[i].rect.setWidth(maxWidth);
-
-        last = y;
+        last = miny;
         available = size.height();
+        maxExtent = maxWidth;
     }
 
-    if (tabList.count() && last > available) {
+    Q_ASSERT(tabChainIndex == tabChain.count() - 1); // add an assert just to make sure.
+    // Mirror our front item.
+    tabChain[tabChainIndex].init();
+    tabChain[tabChainIndex].expansive = tabAlignment != Qt::AlignRight;
+    tabChain[tabChainIndex].empty = true;
+
+    // Do the calculation
+    qGeomCalc(tabChain, 0, tabChain.count(), 0, qMax(available, last), 0);
+
+    // Use the results
+    for (i = 0; i < tabList.count(); ++i) {
+        const QLayoutStruct &lstruct = tabChain.at(i + 1);
+        if (!vertTabs)
+            tabList[i].rect.setRect(lstruct.pos, 0, lstruct.size, maxExtent);
+        else
+            tabList[i].rect.setRect(0, lstruct.pos, maxExtent, lstruct.size);
+    }
+
+    if (useScrollButtons && tabList.count() && last > available) {
         int extra = extraWidth();
         if (!vertTabs) {
             Qt::LayoutDirection ld = q->layoutDirection();
@@ -370,15 +463,11 @@ void QTabBarPrivate::makeVisible(int index)
     else if (end > scrollOffset + available) // too far right
         scrollOffset = end - available + 1;
 
-    if (scrollOffset && end < available)  // need scrolling at all?
-        scrollOffset = 0;
-
     leftB->setEnabled(scrollOffset > 0);
     const int last = horiz ? tabList.last().rect.right() : tabList.last().rect.bottom();
     rightB->setEnabled(last - scrollOffset >= available);
     if (oldScrollOffset != scrollOffset)
         q->update();
-
 }
 
 void QTabBarPrivate::_q_scrollTabs()
@@ -480,7 +569,7 @@ void QTabBar::setShape(Shape shape)
 
 /*!
     \property QTabBar::drawBase
-    \brief defines whether or not tabbar should draw its base.
+    \brief defines whether or not tab bar should draw its base.
 
     If true then QTabBar draws a base in relation to the styles overlab.
     Otherwise only the tabs are drawn.
@@ -578,13 +667,17 @@ void QTabBar::removeTab(int index)
 #endif
         d->tabList.removeAt(index);
         if (index == d->currentIndex) {
+            // The current tab is going away, in order to make sure
+            // we emit that "current has changed", we need to reset this
+            // around.
+            d->currentIndex = -1;
             if (index == d->tabList.size()) {
                 setCurrentIndex(d->validIndex(index - 1) ? index - 1 : 0);
             } else {
                 setCurrentIndex(d->validIndex(index) ? index : 0);
             }
         } else if (index < d->currentIndex) {
-            --d->currentIndex;
+            setCurrentIndex(d->currentIndex - 1);
         }
         d->refresh();
         tabRemoved(index);
@@ -808,6 +901,28 @@ QRect QTabBar::tabRect(int index) const
     return QRect();
 }
 
+/*!
+    \since 4.3
+    Returns the index of the tab that covers \a position or -1 if no
+    tab covers \a position;
+*/
+
+int QTabBar::tabAt(const QPoint &position) const
+{
+    Q_D(const QTabBar);
+    if (d->validIndex(d->currentIndex)
+        && tabRect(d->currentIndex).contains(position)) {
+        return d->currentIndex;
+    }
+    const int max = d->tabList.size();
+    for (int i = 0; i < max; ++i) {
+        if (tabRect(i).contains(position)) {
+            return i;
+        }
+    }
+    return -1;
+}
+
 /*!
     \property QTabBar::currentIndex
     \brief the index of the tab bar's visible tab
@@ -841,7 +956,10 @@ void QTabBar::setCurrentIndex(int index)
     \brief The size for icons in the tab bar
     \since 4.1
 
-    The default value is style-dependent.
+    The default value is style-dependent. \c iconSize is a maximum
+    size; icons that are smaller are not scaled up.
+
+    \sa QTabWidget::iconSize
 */
 QSize QTabBar::iconSize() const
 {
@@ -883,7 +1001,7 @@ QSize QTabBar::sizeHint() const
         const_cast<QTabBarPrivate*>(d)->layoutTabs();
     QRect r;
     for (int i = 0; i < d->tabList.count(); ++i)
-        r = r.unite(d->tabList.at(i).rect);
+        r = r.united(d->tabList.at(i).maxRect);
     QSize sz = QApplication::globalStrut();
     return r.size().expandedTo(sz);
 }
@@ -893,14 +1011,54 @@ QSize QTabBar::sizeHint() const
 QSize QTabBar::minimumSizeHint() const
 {
     Q_D(const QTabBar);
-    if (style()->styleHint(QStyle::SH_TabBar_PreferNoArrows, 0, this))
-        return sizeHint();
+    if (!d->useScrollButtons) {
+        QRect r;
+        for (int i = 0; i < d->tabList.count(); ++i)
+            r = r.united(d->tabList.at(i).minRect);
+        return r.size().expandedTo(QApplication::globalStrut());
+    }
     if (verticalTabs(d->shape))
         return QSize(sizeHint().width(), d->rightB->sizeHint().height() * 2 + 75);
     else
         return QSize(d->rightB->sizeHint().width() * 2 + 75, sizeHint().height());
 }
 
+static QString computeElidedText(Qt::TextElideMode mode, const QString &text)
+{
+    if (text.length() <= 7)
+        return text;
+
+    static const QLatin1String Ellipses("...");
+    QString ret;
+    switch (mode) {
+    case Qt::ElideRight:
+        ret = text.left(4) + Ellipses;
+        break;
+    case Qt::ElideMiddle:
+        ret = text.left(2) + Ellipses + text.right(2);
+        break;
+    case Qt::ElideLeft:
+        ret = Ellipses + text.right(4);
+        break;
+    case Qt::ElideNone:
+        ret = text;
+        break;
+    }
+    return ret;
+}
+
+QSize QTabBarPrivate::minimumTabSizeHint(int index)
+{
+    Q_Q(QTabBar);
+    // ### Qt 5: make this a protected virtual function in QTabBar
+    Tab &tab = tabList[index];
+    QString oldText = tab.text;
+    tab.text = computeElidedText(elideMode, oldText);
+    QSize size = q->tabSizeHint(index);
+    tab.text = oldText;
+    return size;
+}
+
 /*!
     Returns the size hint for the tab at position \a index.
 */
@@ -908,7 +1066,9 @@ QSize QTabBar::tabSizeHint(int index) const
 {
     Q_D(const QTabBar);
     if (const QTabBarPrivate::Tab *tab = d->at(index)) {
-        QStyleOptionTabV2 opt = d->getStyleOption(index);
+        QStyleOptionTabV2 opt;
+        initStyleOption(&opt, index);
+        opt.text = d->tabList.at(index).text;
         QSize iconSize = tab->icon.isNull() ? QSize() : opt.iconSize;
         int hframe  = style()->pixelMetric(QStyle::PM_TabBarTabHSpace, &opt, this);
         int vframe  = style()->pixelMetric(QStyle::PM_TabBarTabVSpace, &opt, this);
@@ -918,7 +1078,8 @@ QSize QTabBar::tabSizeHint(int index) const
         if (verticalTabs(d->shape))
             csz.transpose();
 
-        return style()->sizeFromContents(QStyle::CT_TabBarTab, &opt, csz, this);
+        QSize retSize = style()->sizeFromContents(QStyle::CT_TabBarTab, &opt, csz, this);
+        return retSize;
     }
     return QSize();
 }
@@ -997,7 +1158,7 @@ bool QTabBar::event(QEvent *e)
         return true;
 #ifndef QT_NO_TOOLTIP
     } else if (e->type() == QEvent::ToolTip) {
-        if (const QTabBarPrivate::Tab *tab = d->at(d->indexAtPos(static_cast<QHelpEvent*>(e)->pos()))) {
+        if (const QTabBarPrivate::Tab *tab = d->at(tabAt(static_cast<QHelpEvent*>(e)->pos()))) {
             if (!tab->toolTip.isEmpty()) {
                 QToolTip::showText(static_cast<QHelpEvent*>(e)->globalPos(), tab->toolTip, this);
                 return true;
@@ -1056,7 +1217,6 @@ void QTabBar::paintEvent(QPaintEvent *)
     optTabBase.init(this);
     optTabBase.shape = d->shape;
     if (theParent && overlap > 0) {
-        QPainter::setRedirected(theParent, this, pos());
         QRect rect;
         switch (tabOverlap.shape) {
         case QTabBar::RoundedNorth:
@@ -1077,9 +1237,6 @@ void QTabBar::paintEvent(QPaintEvent *)
             break;
         }
         optTabBase.rect = rect;
-        QPaintEvent e(rect);
-        QApplication::sendEvent(theParent, &e);
-        QPainter::restoreRedirected(theParent);
     }
     QStylePainter p(this);
     int selected = -1;
@@ -1091,7 +1248,8 @@ void QTabBar::paintEvent(QPaintEvent *)
     QStyleOptionTab cutTab;
     QStyleOptionTab selectedTab;
     for (int i = 0; i < d->tabList.count(); ++i) {
-        QStyleOptionTabV2 tab = d->getStyleOption(i);
+        QStyleOptionTabV2 tab;
+        initStyleOption(&tab, i);
         if (!(tab.state & QStyle::State_Enabled)) {
             tab.palette.setCurrentColorGroup(QPalette::Disabled);
         }
@@ -1119,7 +1277,8 @@ void QTabBar::paintEvent(QPaintEvent *)
 
     // Draw the selected tab last to get it "on top"
     if (selected >= 0) {
-        QStyleOptionTabV2 tab = d->getStyleOption(selected);
+        QStyleOptionTabV2 tab;
+        initStyleOption(&tab, selected);
         p.drawControl(QStyle::CE_TabBarTab, tab);
     }
     if (d->drawBase)
@@ -1211,10 +1370,65 @@ void QTabBar::keyPressEvent(QKeyEvent *e)
 void QTabBar::changeEvent(QEvent *e)
 {
     Q_D(QTabBar);
+    if (e->type() == QEvent::StyleChange) {
+        d->elideMode = Qt::TextElideMode(style()->styleHint(QStyle::SH_TabBar_ElideMode, 0, this));
+        d->useScrollButtons = !style()->styleHint(QStyle::SH_TabBar_PreferNoArrows, 0, this);
+    }
     d->refresh();
     QWidget::changeEvent(e);
 }
 
+/*!
+    \property QTabBar::elideMode
+    \brief how to elide text in the tab bar
+    \since 4.2
+
+    This property controls how items are elided when there is not
+    enough space to show them for a given tab bar size.
+
+    By default the value is style dependent.
+
+    \sa QTabWidget::elideMode usesScrollButtons QStyle::SH_TabBar_ElideMode
+*/
+
+Qt::TextElideMode QTabBar::elideMode() const
+{
+    Q_D(const QTabBar);
+    return d->elideMode;
+}
+
+void QTabBar::setElideMode(Qt::TextElideMode mode)
+{
+    Q_D(QTabBar);
+    d->elideMode = mode;
+}
+
+/*!
+    \property QTabBar::usesScrollButtons
+    \brief Whether or not a tab bar should use buttons to scroll tabs when it
+    has many tabs.
+    \since 4.2
+
+    When there are too many tabs in a tab bar for its size, the tab bar can either choose
+    to expand it's size or to add buttons that allow you to scroll through the tabs.
+
+    By default the value is style dependant.
+
+    \sa elideMode QTabWidget::usesScrollButtons QStyle::SH_TabBar_PreferNoArrows
+*/
+bool QTabBar::usesScrollButtons() const
+{
+    return d_func()->useScrollButtons;
+}
+
+void QTabBar::setUsesScrollButtons(bool useButtons)
+{
+    Q_D(QTabBar);
+    if (d->useScrollButtons == useButtons)
+        return;
+    d->useScrollButtons = useButtons;
+    d->refresh();
+}
 
 /*!
     \fn void QTabBar::setCurrentTab(int index)
