@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -27,6 +42,8 @@
 #include <QtGui/qtextedit.h>
 #include <QtCore/qurl.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_TEXTBROWSER
@@ -42,6 +59,8 @@ class Q_GUI_EXPORT QTextBrowser : public QTextEdit
     Q_OVERRIDE(bool readOnly DESIGNABLE false SCRIPTABLE false)
     Q_OVERRIDE(bool undoRedoEnabled DESIGNABLE false SCRIPTABLE false)
     Q_PROPERTY(QStringList searchPaths READ searchPaths WRITE setSearchPaths)
+    Q_PROPERTY(bool openExternalLinks READ openExternalLinks WRITE setOpenExternalLinks)
+    Q_PROPERTY(bool openLinks READ openLinks WRITE setOpenLinks)
 
 public:
     explicit QTextBrowser(QWidget* parent = 0);
@@ -54,6 +73,16 @@ public:
 
     virtual QVariant loadResource(int type, const QUrl &name);
 
+    bool isBackwardAvailable() const;
+    bool isForwardAvailable() const;
+    void clearHistory();
+
+    bool openExternalLinks() const;
+    void setOpenExternalLinks(bool open);
+
+    bool openLinks() const;
+    void setOpenLinks(bool open);
+
 public Q_SLOTS:
     virtual void setSource(const QUrl &name);
     virtual void backward();
@@ -87,9 +116,13 @@ public:
 private:
     Q_DISABLE_COPY(QTextBrowser)
     Q_DECLARE_PRIVATE(QTextBrowser)
-    Q_PRIVATE_SLOT(d_func(), void documentModified())
+    Q_PRIVATE_SLOT(d_func(), void _q_documentModified())
+    Q_PRIVATE_SLOT(d_func(), void _q_activateAnchor(const QString &))
+    Q_PRIVATE_SLOT(d_func(), void _q_highlightLink(const QString &))
 };
 
 #endif // QT_NO_TEXTBROWSER
 
+QT_END_HEADER
+
 #endif // QTEXTBROWSER_H
