@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -9,20 +9,35 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
-#ifndef QTOOLBAR_P_H
-#define QTOOLBAR_P_H
+#ifndef QDYNAMICTOOLBAR_P_H
+#define QDYNAMICTOOLBAR_P_H
 
 //
 //  W A R N I N G
@@ -35,43 +50,14 @@
 // We mean it.
 //
 
-#include "QtGui/qtoolbar.h"
+#include "qtoolbar.h"
 #include "QtGui/qaction.h"
 #include "private/qwidget_p.h"
 
 #ifndef QT_NO_TOOLBAR
 
-/*
-    internal class to associate a widget with an action
-*/
-class QToolBarWidgetAction : public QAction
-{
-    Q_OBJECT
-
-    QWidget *_widget;
-
-public:
-    inline QToolBarWidgetAction(QWidget *widget, QWidget *parent)
-        : QAction(parent), _widget(widget)
-    {
-        setText(_widget->objectName().isEmpty()
-            ? QString::fromLatin1(_widget->metaObject()->className())
-                : _widget->objectName());
-        setVisible(!(widget->isHidden() && widget->testAttribute(Qt::WA_WState_ExplicitShowHide)));
-    }
-
-    inline QWidget *widget() const
-    { return _widget; }
-};
-
-struct QToolBarItem {
-    QAction *action;
-    QWidget *widget;
-    uint hidden : 1; // toolbar too small to show this item
-};
-
-class QToolBarExtension;
-class QToolBarHandle;
+class QToolBarLayout;
+class QTimer;
 
 class QToolBarPrivate : public QWidgetPrivate
 {
@@ -82,8 +68,7 @@ public:
         : explicitIconSize(false), explicitToolButtonStyle(false), movable(false),
           allowedAreas(Qt::AllToolBarAreas), orientation(Qt::Horizontal),
           toolButtonStyle(Qt::ToolButtonIconOnly),
-          handle(0), extension(0),
-          inResizeEvent(false)
+          layout(0), state(0)
     { }
 
     void init();
@@ -91,8 +76,7 @@ public:
     void _q_toggleView(bool b);
     void _q_updateIconSize(const QSize &sz);
     void _q_updateToolButtonStyle(Qt::ToolButtonStyle style);
-    QToolBarItem createItem(QAction *action);
-    int indexOf(QAction *action) const;
+    void _q_waitForPopup();
 
     bool explicitIconSize;
     bool explicitToolButtonStyle;
@@ -101,23 +85,34 @@ public:
     Qt::Orientation orientation;
     Qt::ToolButtonStyle toolButtonStyle;
     QSize iconSize;
-
-    QToolBarHandle *handle;
-    QToolBarExtension *extension;
-
-    QList<QToolBarItem> items;
+    bool floatable;
 
     QAction *toggleViewAction;
 
-    bool inResizeEvent;
+    QToolBarLayout *layout;
+
+    struct DragState {
+        QPoint pressPos;
+        bool dragging;
+        QLayoutItem *widgetItem;
+    };
+    DragState *state;
+
+    void mousePressEvent(QMouseEvent *e);
+    void mouseReleaseEvent(QMouseEvent *e);
+    void mouseMoveEvent(QMouseEvent *e);
+
+    void setWindowState(bool floating, bool unplug = false, const QRect &rect = QRect());
+    void initDrag(const QPoint &pos);
+    void startDrag();
+    void endDrag();
+
+    void unplug(const QRect &r);
+    void plug(const QRect &r);
+
+    QTimer *waitForPopupTimer;
 };
 
-static inline int pick(Qt::Orientation o, const QPoint &p)
-{ return o == Qt::Horizontal ? p.x() : p.y(); }
-
-static inline int pick(Qt::Orientation o, const QSize &s)
-{ return o == Qt::Horizontal ? s.width() : s.height(); }
-
 #endif // QT_NO_TOOLBAR
 
-#endif // QTOOLBAR_P_H
+#endif // QDYNAMICTOOLBAR_P_H
