@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,6 +53,9 @@ QT_MODULE(Network)
 class QTcpSocket;
 class QTimerEvent;
 class QIODevice;
+class QAuthenticator;
+class QNetworkProxy;
+class QSslError;
 
 class QHttpPrivate;
 
@@ -150,8 +168,14 @@ class Q_NETWORK_EXPORT QHttp : public QObject
     Q_OBJECT
 
 public:
+    enum ConnectionMode {
+        ConnectionModeHttp,
+        ConnectionModeHttps
+    };
+    
     explicit QHttp(QObject *parent = 0);
     QHttp(const QString &hostname, quint16 port = 80, QObject *parent = 0);
+    QHttp(const QString &hostname, ConnectionMode mode, quint16 port = 0, QObject *parent = 0);
     virtual ~QHttp();
 
     enum State {
@@ -171,16 +195,23 @@ public:
         UnexpectedClose,
         InvalidResponseHeader,
         WrongContentLength,
-        Aborted
+        Aborted, 
+        AuthenticationRequiredError,
+        ProxyAuthenticationRequiredError
     };
 
     int setHost(const QString &hostname, quint16 port = 80);
+    int setHost(const QString &hostname, ConnectionMode mode, quint16 port = 0);
+    
     int setSocket(QTcpSocket *socket);
     int setUser(const QString &username, const QString &password = QString());
 
+#ifndef QT_NO_NETWORKPROXY
     int setProxy(const QString &host, int port,
                  const QString &username = QString(),
                  const QString &password = QString());
+    int setProxy(const QNetworkProxy &proxy);
+#endif
 
     int get(const QString &path, QIODevice *to=0);
     int post(const QString &path, QIODevice *data, QIODevice *to=0 );
@@ -216,6 +247,10 @@ public:
 public Q_SLOTS:
     void abort();
 
+#ifndef QT_NO_OPENSSL
+    void ignoreSslErrors();
+#endif
+
 Q_SIGNALS:
     void stateChanged(int);
     void responseHeaderReceived(const QHttpResponseHeader &resp);
@@ -227,6 +262,15 @@ Q_SIGNALS:
     void requestFinished(int, bool);
     void done(bool);
 
+#ifndef QT_NO_NETWORKPROXY
+    void proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *);
+#endif
+    void authenticationRequired(const QString &hostname, quint16 port, QAuthenticator *);
+
+#ifndef QT_NO_OPENSSL
+    void sslErrors(const QList<QSslError> &errors);
+#endif
+
 private:
     Q_DISABLE_COPY(QHttp)
     Q_DECLARE_PRIVATE(QHttp)
