@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -339,7 +354,7 @@ QSocks5BindStore::QSocks5BindStore()
     , sweepTimerId(-1)
 {
     QCoreApplication *app = QCoreApplication::instance();
-    if (app->thread() != thread())
+    if (app && app->thread() != thread())
         moveToThread(app->thread());
 }
 
@@ -351,7 +366,7 @@ void QSocks5BindStore::add(int socketDescriptor, QSocks5BindData *bindData)
 {
     QMutexLocker lock(&mutex);
     if (store.contains(socketDescriptor)) {
-        qDebug() << "delete it";
+        // qDebug() << "delete it";
     }
     bindData->timeStamp = QDateTime::currentDateTime();
     store.insert(socketDescriptor, bindData);
@@ -695,7 +710,7 @@ void QSocks5SocketEnginePrivate::parseRequestMethodReply()
         return;
     }
     if (buf[pos++] != S5_VERSION_5) {
-        QSOCKS5_DEBUG << "totaly lost";
+        QSOCKS5_DEBUG << "totally lost";
     }
     if (buf[pos++] != S5_SUCCESS ) {
         socks5Error = Socks5Error(buf[pos-1]);
@@ -706,7 +721,7 @@ void QSocks5SocketEnginePrivate::parseRequestMethodReply()
         return;
     }
     if (buf[pos++] != 0x00) {
-        QSOCKS5_DEBUG << "totaly lost";
+        QSOCKS5_DEBUG << "totally lost";
     }
     if (!qt_socks5_get_host_address_and_port(inBuf, &localAddress, &localPort, &pos)) {
         QSOCKS5_DEBUG << "error getting address";
@@ -745,7 +760,7 @@ void QSocks5SocketEnginePrivate::parseNewConnection()
         return;
     }
     if (buf[pos++] != S5_VERSION_5) {
-        QSOCKS5_D_DEBUG << "totaly lost";
+        QSOCKS5_D_DEBUG << "totally lost";
     }
     if (buf[pos++] != S5_SUCCESS) {
         QSOCKS5_D_DEBUG <<  "Request error :" << s5RequestErrorToString(buf[pos-1]);
@@ -756,7 +771,7 @@ void QSocks5SocketEnginePrivate::parseNewConnection()
         return;
     }
     if (buf[pos++] != 0x00) {
-        QSOCKS5_D_DEBUG << "totaly lost";
+        QSOCKS5_D_DEBUG << "totally lost";
     }
     if (!qt_socks5_get_host_address_and_port(inBuf, &bindData->peerAddress, &bindData->peerPort, &pos)) {
         QSOCKS5_D_DEBUG << "error getting address";
@@ -853,22 +868,12 @@ QSocks5SocketEngine::~QSocks5SocketEngine()
 }
 
 static QBasicAtomic descriptorCounter = Q_ATOMIC_INIT(1);
-static int qt_socks5_new_socket_descriptor()
-{
-    register int descriptor;
-    for (;;) {
-        descriptor = descriptorCounter;
-        if (descriptorCounter.testAndSet(descriptor, descriptor + 1))
-            break;
-    }
-    return descriptor;
-}
 
 bool QSocks5SocketEngine::initialize(QAbstractSocket::SocketType type, QAbstractSocket::NetworkLayerProtocol protocol)
 {
     Q_D(QSocks5SocketEngine);
 
-    d->socketDescriptor = qt_socks5_new_socket_descriptor();
+    d->socketDescriptor = descriptorCounter.fetchAndAdd(1);
 
     d->socketType = type;
     d->socketProtocol = protocol;
@@ -1001,7 +1006,7 @@ bool QSocks5SocketEngine::connectToHost(const QHostAddress &address, quint16 por
         QSOCKS5_DEBUG << "not yet connected";
         return false;
     } else {
-        qDebug() << "unexpected call to contectToHost";
+        // qDebug() << "unexpected call to contectToHost";
     }
     return false;
 }
@@ -1046,8 +1051,9 @@ void QSocks5SocketEnginePrivate::_q_controlSocketReadNotification()
             break;
         case Connected: {
             QByteArray buf;
-            if (!data->authenticator->unSeal(data->controlSocket, &buf))
-                qDebug() << "unseal error maybe need to wait for more data";
+            if (!data->authenticator->unSeal(data->controlSocket, &buf)) {
+                // qDebug() << "unseal error maybe need to wait for more data";
+            }
             if (buf.size()) {
                 QSOCKS5_DEBUG << dump(buf);
                 connectData->readBuffer += buf;
@@ -1567,7 +1573,7 @@ bool QSocks5SocketEngine::waitForWrite(int msecs, bool *timedOut) const
         stopWatch.start();
 
         if (!d->data->controlSocket->waitForConnected(qt_timeout_value(msecs, stopWatch.elapsed()))) {
-            qDebug() << "failed to connect to proxy";
+            // qDebug() << "failed to connect to proxy";
             if (timedOut && d->data->controlSocket->error() == QAbstractSocket::SocketTimeoutError)
                 *timedOut = true;
             return false; // ???
