@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -109,7 +124,7 @@
 
 #include <QtCore/qatomic.h>
 #include <QtCore/qdatetime.h>
-#ifndef QT_NO_DEBUG
+#ifndef QT_NO_DEBUG_STREAM
 #include <QtCore/qdebug.h>
 #endif
 #include <QtCore/qdir.h>
@@ -131,6 +146,7 @@
 QSslCertificate::QSslCertificate(QIODevice *device, QSsl::EncodingFormat format)
     : d(new QSslCertificatePrivate)
 {
+    QSslSocketPrivate::ensureInitialized();
     if (device)
         d->init(device->readAll(), format);
 }
@@ -144,6 +160,7 @@ QSslCertificate::QSslCertificate(QIODevice *device, QSsl::EncodingFormat format)
 QSslCertificate::QSslCertificate(const QByteArray &data, QSsl::EncodingFormat format)
     : d(new QSslCertificatePrivate)
 {
+    QSslSocketPrivate::ensureInitialized();
     d->init(data, format);
 }
 
@@ -508,6 +525,10 @@ QSslCertificate::fromPath(const QString &path,
 QList<QSslCertificate>
 QSslCertificate::fromDevice(QIODevice *device, QSsl::EncodingFormat format)
 {
+    if (!device) {
+        qWarning("QSslCertificate::fromDevice: cannot read from a null device");
+        return QList<QSslCertificate>();
+    }
     return fromData(device->readAll(), format);
 }
 
@@ -639,6 +660,7 @@ QList<QSslCertificate>
 QSslCertificatePrivate::certificatesFromPem(const QByteArray &pem, int count)
 {
     QList<QSslCertificate> certificates;
+    QSslSocketPrivate::ensureInitialized();
 
     int offset = 0;
     while (count == -1 || certificates.size() < count) {
@@ -674,6 +696,8 @@ QList<QSslCertificate>
 QSslCertificatePrivate::certificatesFromDer(const QByteArray &der, int count)
 {
     QList<QSslCertificate> certificates;
+    QSslSocketPrivate::ensureInitialized();
+
 
 #if OPENSSL_VERSION_NUMBER >= 0x00908000L
         const unsigned char *data = (const unsigned char *)der.data();
