@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -50,13 +65,13 @@ QMultiInputContext::QMultiInputContext()
 {
     QStringList keys = QInputContextFactory::keys();
     for (int i = keys.size()-1; i >= 0; --i)
-        if (keys.at(i).contains("imsw"))
+        if (keys.at(i).contains(QLatin1String("imsw")))
             keys.removeAt(i);
 
-    QString def = getenv("QT_IM_MODULE");
+    QString def = QLatin1String(getenv("QT_IM_MODULE"));
     if (def.isEmpty()) {
 	QSettings settings;
-        def = settings.value("/qt/DefaultInputMethod", QLatin1String("xim")).toString();
+        def = settings.value(QLatin1String("/qt/DefaultInputMethod"), QLatin1String("xim")).toString();
     }
     current = keys.indexOf(def);
     if (current < 0)
@@ -75,7 +90,7 @@ QMultiInputContext::QMultiInputContext()
         if (i == current)
             a->setChecked(true);
     }
-    connect(group, SIGNAL(triggered(QAction *)), this, SLOT(changeSlave(QAction *)));
+    connect(group, SIGNAL(triggered(QAction*)), this, SLOT(changeSlave(QAction*)));
 
 
 }
@@ -87,12 +102,12 @@ QMultiInputContext::~QMultiInputContext()
 
 QString QMultiInputContext::identifierName()
 {
-    return (slave()) ? slave()->identifierName() : "";
+    return (slave()) ? slave()->identifierName() : QLatin1String("");
 }
 
 QString QMultiInputContext::language()
 {
-    return (slave()) ? slave()->language() : "";
+    return (slave()) ? slave()->language() : QLatin1String("");
 }
 
 
