@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -212,7 +227,6 @@ Q3ProgressBar *Q3ProgressDialog::bar() const
 
 
 /*!
-  \obsolete
   Constructs a progress dialog.
 
   Default settings:
@@ -234,16 +248,15 @@ Q3ProgressBar *Q3ProgressDialog::bar() const
 */
 
 Q3ProgressDialog::Q3ProgressDialog(QWidget *creator, const char *name,
-                                  bool modal, Qt::WFlags f)
+                                  bool modal, Qt::WindowFlags f)
     : QDialog(creator, f)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
     setModal(modal);
     init(creator, QString::fromLatin1(""), tr("Cancel"), 100);
 }
 
 /*!
-  \obsolete
   Constructs a progress dialog.
 
    The \a labelText is text used to remind the user what is progressing.
@@ -274,10 +287,10 @@ Q3ProgressDialog::Q3ProgressDialog(const QString &labelText,
                                   const QString &cancelButtonText,
                                   int totalSteps,
                                   QWidget *creator, const char *name,
-                                  bool modal, Qt::WFlags f)
+                                  bool modal, Qt::WindowFlags f)
     : QDialog(creator, f)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
     setModal(modal);
     init(creator, labelText, cancelButtonText, totalSteps);
 }
@@ -298,7 +311,7 @@ Q3ProgressDialog::Q3ProgressDialog(const QString &labelText,
   \sa setLabelText(), setLabel(), setCancelButtonText(), setCancelButton(),
   setTotalSteps()
 */
-Q3ProgressDialog::Q3ProgressDialog(QWidget *creator, Qt::WFlags f)
+Q3ProgressDialog::Q3ProgressDialog(QWidget *creator, Qt::WindowFlags f)
     : QDialog(creator, f)
 {
     init(creator, QString::fromLatin1(""), tr("Cancel"), 100);
@@ -328,7 +341,7 @@ Q3ProgressDialog::Q3ProgressDialog(QWidget *creator, Qt::WFlags f)
 */
 Q3ProgressDialog::Q3ProgressDialog(const QString &labelText,
                                   const QString &cancelButtonText,
-                                  int totalSteps, QWidget *creator, Qt::WFlags f)
+                                  int totalSteps, QWidget *creator, Qt::WindowFlags f)
     : QDialog(creator, f)
 {
     init(creator, labelText, cancelButtonText, totalSteps);
@@ -356,6 +369,7 @@ void Q3ProgressDialog::init(QWidget *creator,
     d->autoReset = true;
     d->forceHide = false;
     setCancelButtonText(canc);
+    connect(this, SIGNAL(canceled()), this, SIGNAL(cancelled()));
     connect(this, SIGNAL(canceled()), this, SLOT(cancel()));
     forceTimer = new QTimer(this);
     connect(forceTimer, SIGNAL(timeout()), this, SLOT(forceShow()));
@@ -371,6 +385,11 @@ void Q3ProgressDialog::init(QWidget *creator,
   \sa wasCanceled()
 */
 
+/*!
+    \fn void Q3ProgressDialog::cancelled()
+
+    Use canceled() instead.
+*/
 
 /*!
   Sets the label to \a label. The progress dialog resizes to fit. The
@@ -508,6 +527,21 @@ bool Q3ProgressDialog::wasCanceled() const
     return d->cancellation_flag;
 }
 
+/*!
+  \property Q3ProgressDialog::wasCancelled
+  \brief whether the dialog was canceled
+  \since 4.2
+
+  Use \l wasCanceled instead.
+*/
+
+/*!
+    Use wasCanceled() instead.
+*/
+bool Q3ProgressDialog::wasCancelled() const
+{
+    return d->cancellation_flag;
+}
 
 /*!
   \property Q3ProgressDialog::totalSteps
