@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -546,7 +561,7 @@ void Q3NetworkProtocol::registerNetworkProtocol( const QString &protocol,
 {
     if ( !q3networkProtocolRegister ) {
 	q3networkProtocolRegister = new Q3NetworkProtocolDict;
-	Q3NetworkProtocol::registerNetworkProtocol( "file", new Q3NetworkProtocolFactory< Q3LocalFs > );
+	Q3NetworkProtocol::registerNetworkProtocol( QLatin1String("file"), new Q3NetworkProtocolFactory< Q3LocalFs > );
     }
 
     q3networkProtocolRegister->insert( protocol, protocolFactory );
@@ -576,7 +591,7 @@ Q3NetworkProtocol *Q3NetworkProtocol::getNetworkProtocol( const QString &protoco
 {
     if ( !q3networkProtocolRegister ) {
 	q3networkProtocolRegister = new Q3NetworkProtocolDict;
-	Q3NetworkProtocol::registerNetworkProtocol( "file", new Q3NetworkProtocolFactory< Q3LocalFs > );
+	Q3NetworkProtocol::registerNetworkProtocol( QLatin1String("file"), new Q3NetworkProtocolFactory< Q3LocalFs > );
     }
 
     if ( protocol.isNull() )
@@ -602,7 +617,7 @@ bool Q3NetworkProtocol::hasOnlyLocalFileSystem()
 
     Q3DictIterator< Q3NetworkProtocolFactoryBase > it( *q3networkProtocolRegister );
     for ( ; it.current(); ++it )
-	if ( it.currentKey() != "file" )
+	if ( it.currentKey() != QLatin1String("file") )
 	    return false;
     return true;
 }
