@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -273,7 +288,7 @@ void Q3SocketPrivate::setSocketDevice( Q3Socket *q, Q3SocketDevice *device )
 Q3Socket::Q3Socket( QObject *parent, const char *name )
     : QIODevice( parent )
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
     d = new Q3SocketPrivate;
     setSocketDevice( 0 );
     resetStatus();
@@ -715,6 +730,7 @@ bool Q3Socket::consumeWriteBuf( Q_ULONG nbytes )
 
 /*!
     Implementation of the abstract virtual QIODevice::flush() function.
+    This function always returns true.
 */
 
 bool Q3Socket::flush()
@@ -1118,7 +1134,7 @@ bool Q3Socket::canReadLine() const
     if ( ((Q3Socket*)this)->d->rba.scanNewline( 0 ) )
 	return true;
     return ( bytesAvailable() > 0 &&
-	     ((Q3Socket*)this)->d->rba.scanNewline( 0 ) );
+	     (((Q3Socket*)this)->d->rba.scanNewline( 0 ) || QIODevice::canReadLine()) );
 }
 
 /*!
@@ -1180,12 +1196,13 @@ void Q3Socket::sn_read( bool force )
 	// event loop.
 	nread = d->socket->readBlock( buf, maxToRead ? QMIN((Q_LONG)sizeof(buf),maxToRead) : sizeof(buf) );
 	if ( nread == 0 ) {			// really closed
+            if ( !d->socket->isOpen() ) {
 #if defined(Q3SOCKET_DEBUG)
-	    qDebug( "Q3Socket (%s): sn_read: Connection closed", name() );
+                qDebug( "Q3Socket (%s): sn_read: Connection closed", name() );
 #endif
-	    // ### we should rather ask the socket device if it is closed
-	    d->connectionClosed();
-	    emit connectionClosed();
+                d->connectionClosed();
+                emit connectionClosed();
+            }
 	    Q3SocketPrivate::sn_read_alreadyCalled.removeRef( this );
 	    return;
 	} else {
