@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -456,7 +471,7 @@ QString Q3Process::readLineStdout()
 	return QString();
 
       if ( !buf->scanNewline( &a ) )
-	return QString( buf->readAll() );
+	return QString( QLatin1String(buf->readAll()) );
     }
 
     uint size = a.size();
@@ -465,11 +480,11 @@ QString Q3Process::readLineStdout()
     // get rid of terminating \n or \r\n
     if ( size>0 && a.at( size - 1 ) == '\n' ) {
       if ( size>1 && a.at( size - 2 ) == '\r' )
-	a[size - 2] = '\0';
+	a.chop(2);
       else
-	a[size - 1] = '\0';
+	a.chop(1);
     }
-    return QString( a );
+    return QString(QString::fromLatin1(a.constData()));
 }
 
 /*!
@@ -492,7 +507,7 @@ QString Q3Process::readLineStderr()
 	return QString();
 
       if ( !buf->scanNewline( &a ) )
-	return QString( buf->readAll() );
+	return QString( QString::fromLatin1( buf->readAll().constData() ) );
     }
 
     uint size = a.size();
@@ -501,11 +516,11 @@ QString Q3Process::readLineStderr()
     // get rid of terminating \n or \r\n
     if ( size>0 && a.at( size - 1 ) == '\n' ) {
       if ( size>1 && a.at( size - 2 ) == '\r' )
-	a[size - 2] = '\0';
+	a.chop(2);
       else
-	a[size - 1] = '\0';
+	a.chop(1);
     }
-    return QString( a );
+    return QString( QString::fromLatin1( a.constData() ) );
 }
 
 /*!
