@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -787,16 +802,17 @@ void Q3DataBrowser::del()
 }
 
 /*!
-    Moves the default cursor to the record specified by the index \a i
-    and refreshes the default form to display this record. If there is
-    no default form or no default cursor, nothing happens. If \a
-    relative is true (the default is false), the cursor is moved
-    relative to its current position. If the data browser successfully
-    navigated to the desired record, the default cursor is primed for
-    update and the primeUpdate() signal is emitted.
-
-    If the browser is already positioned on the desired record nothing
-    happens.
+  Moves the default cursor to the record specified by index \a i
+  and refreshes the default form to display that record. If there is
+  no default form or no default cursor, nothing happens. If
+  \a relative is true (the default is false), the cursor is moved
+  relative to its current position. If the data browser successfully
+  navigated to the desired record, the default cursor is primed for
+  update and the primeUpdate() signal is emitted.
+  
+  If the browser is already positioned on the desired record nothing
+  happens. Returns false if there is no cursor. Otherwise returns
+  true.
 */
 
 bool Q3DataBrowser::seek(int i, bool relative)
