@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -144,8 +159,8 @@ QWidget * Q3SqlEditorFactory::createEditor(QWidget * parent,
             break;
         case QVariant::Bool:
             w = new QComboBox(parent, "qt_editor_bool");
-            ((QComboBox *) w)->insertItem("False");
-            ((QComboBox *) w)->insertItem("True");
+            ((QComboBox *) w)->insertItem(QLatin1String("False"));
+            ((QComboBox *) w)->insertItem(QLatin1String("True"));
             break;
         case QVariant::UInt:
             w = new QSpinBox(0, 2147483647, 1, parent, "qt_editor_spinbox");
@@ -162,19 +177,19 @@ QWidget * Q3SqlEditorFactory::createEditor(QWidget * parent,
             break;
         case QVariant::Date: {
             QDateTimeEdit *edit = new QDateTimeEdit(parent);
-            edit->setDisplayFormat("yyyy/MM/dd");
-            edit->setObjectName("qt_editor_date");
+            edit->setDisplayFormat(QLatin1String("yyyy/MM/dd"));
+            edit->setObjectName(QLatin1String("qt_editor_date"));
             w = edit; }
             break;
         case QVariant::Time: {
             QDateTimeEdit *edit = new QDateTimeEdit(parent);
-            edit->setDisplayFormat("hh:mm");
-            edit->setObjectName("qt_editor_time");
+            edit->setDisplayFormat(QLatin1String("hh:mm"));
+            edit->setObjectName(QLatin1String("qt_editor_time"));
             w = edit; }
             break;
         case QVariant::DateTime:
             w = new QDateTimeEdit(parent);
-            w->setObjectName("qt_editor_datetime");
+            w->setObjectName(QLatin1String("qt_editor_datetime"));
             break;
 #ifndef QT_NO_LABEL
         case QVariant::Pixmap:
