@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -67,36 +82,38 @@
 */
 
 /*!
-    \fn bool Q3MultiLineEdit::autoUpdate() const
+  \fn bool Q3MultiLineEdit::autoUpdate() const
+
+  This function is a noop that always returns true.
 */
 
 /*!
-    \fn virtual void Q3MultiLineEdit::setAutoUpdate(bool b)
+  \fn virtual void Q3MultiLineEdit::setAutoUpdate(bool b)
 
-    \internal
+  \internal
 */
 
 /*!
-    \fn int Q3MultiLineEdit::totalWidth() const
+  \fn int Q3MultiLineEdit::totalWidth() const
 */
 
 /*!
-    \fn int Q3MultiLineEdit::totalHeight() const
+  \fn int Q3MultiLineEdit::totalHeight() const
 */
 
 /*!
-    \fn int Q3MultiLineEdit::maxLines() const
+  \fn int Q3MultiLineEdit::maxLines() const
 */
 
 /*!
-    \fn void Q3MultiLineEdit::setMaxLines(int max)
+  \fn void Q3MultiLineEdit::setMaxLines(int max)
 
-    Sets the maximum number of lines this Q3MultiLineEdit will hold to
-    \a max.
+  Sets the maximum number of lines this Q3MultiLineEdit will hold to
+  \a max.
 */
 
 /*!
-    \fn void Q3MultiLineEdit::deselect()
+  \fn void Q3MultiLineEdit::deselect()
 */
 
 
@@ -275,7 +292,7 @@ void Q3MultiLineEdit::insertAndMark(const QString& str, bool mark)
 
 void Q3MultiLineEdit::newLine()
 {
-    insert("\n");
+    insert(QString(QLatin1Char('\n')));
 }
 
 
