@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -765,7 +780,7 @@ QString Q3StyleSheetItem::contexts() const
 */
 void Q3StyleSheetItem::setContexts(const QString& c)
 {
-    d->contxt = QChar(' ') + c + QChar(' ');
+    d->contxt = QLatin1Char(' ') + c + QLatin1Char(' ');
 }
 
 /*!
@@ -778,7 +793,7 @@ bool Q3StyleSheetItem::allowedInContext(const Q3StyleSheetItem* s) const
 {
     if (d->contxt.isEmpty())
         return true;
-    return d->contxt.contains(QChar(' ')+s->name()+QChar(' '));
+    return d->contxt.contains(QLatin1Char(' ')+s->name()+QLatin1Char(' '));
 }
 
 
@@ -1065,7 +1080,7 @@ int Q3StyleSheetItem::lineSpacing() const
 Q3StyleSheet::Q3StyleSheet(QObject *parent, const char *name)
     : QObject(parent)
 {
-    setObjectName(name);
+    setObjectName(QLatin1String(name));
     init();
 }
 
@@ -1092,7 +1107,7 @@ void Q3StyleSheet::init()
 
     Q3StyleSheetItem *style;
 
-    style = new Q3StyleSheetItem(this, "qml"); // compatibility
+    style = new Q3StyleSheetItem(this, QLatin1String("qml")); // compatibility
     style->setDisplayMode(Q3StyleSheetItem::DisplayBlock);
 
     style = new Q3StyleSheetItem(this, QString::fromLatin1("qt"));
