@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -171,7 +186,7 @@ QString Q3TextBrowser::source() const
 void Q3TextBrowser::reload()
 {
     QString s = d->curmain;
-    d->curmain = "";
+    d->curmain = QLatin1String("");
     setSource(s);
 }
 
@@ -185,13 +200,13 @@ void Q3TextBrowser::setSource(const QString& name)
     d->textOrSourceChanged = true;
     QString source = name;
     QString mark;
-    int hash = name.indexOf('#');
+    int hash = name.indexOf(QLatin1Char('#'));
     if (hash != -1) {
         source = name.left(hash);
         mark = name.mid(hash+1);
     }
 
-    if (source.left(5) == "file:")
+    if (source.left(5) == QLatin1String("file:"))
         source = source.mid(6);
 
     QString url = mimeSourceFactory()->makeAbsolute(source, context());
@@ -210,8 +225,8 @@ void Q3TextBrowser::setSource(const QString& name)
             }
         }
         if (isVisible()) {
-            QString firstTag = txt.left(txt.indexOf('>') + 1);
-            if (firstTag.left(3) == "<qt" && firstTag.contains("type") && firstTag.contains("detail")) {
+            QString firstTag = txt.left(txt.indexOf(QLatin1Char('>')) + 1);
+            if (firstTag.left(3) == QLatin1String("<qt") && firstTag.contains(QLatin1String("type")) && firstTag.contains(QLatin1String("detail"))) {
                 popupDetail(txt, QCursor::pos());
 #ifndef QT_NO_CURSOR
                 qApp->restoreOverrideCursor();
@@ -227,7 +242,7 @@ void Q3TextBrowser::setSource(const QString& name)
     d->curmark = mark;
 
     if (!mark.isEmpty()) {
-        url += "#";
+        url += QLatin1Char('#');
         url += mark;
     }
     if (d->home.count() == 0)
@@ -443,7 +458,7 @@ void Q3TextBrowser::popupDetail(const QString& contents, const QPoint& pos)
     // how many window system problems it skirts around.
 
     QPainter p(popup);
-    p.setPen(QApplication::palette().color(QPalette::Active, QPalette::Foreground));
+    p.setPen(QApplication::palette().color(QPalette::Active, QPalette::WindowText));
     p.drawRect(0, 0, w, h);
     p.setPen(QApplication::palette().color(QPalette::Active, QPalette::Mid));
     p.setBrush(QColor(255, 255, 240));
@@ -485,8 +500,8 @@ void Q3TextBrowser::popupDetail(const QString& contents, const QPoint& pos)
 void Q3TextBrowser::setText(const QString &txt, const QString &context)
 {
     d->textOrSourceChanged = true;
-    d->curmark = "";
-    d->curmain = "";
+    d->curmark = QLatin1String("");
+    d->curmain = QLatin1String("");
     Q3TextEdit::setText(txt, context);
 }
 
