@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -1248,9 +1263,9 @@ QTextStream &operator<<(QTextStream &ts, const Q3DockArea &dockArea)
 
     for (int i = 0; i < l.size(); ++i) {
         Q3DockWindow *dw = l.at(i);
-        str += "[" + QString(dw->windowTitle()) + "," + QString::number((int)dw->offset()) +
-               "," + QString::number((int)dw->newLine()) + "," + QString::number(dw->fixedExtent().width()) +
-               "," + QString::number(dw->fixedExtent().height()) + "," + QString::number((int)!dw->isHidden()) + "]";
+        str += QLatin1Char('[') + QString(dw->windowTitle()) + QLatin1Char(',') + QString::number((int)dw->offset()) +
+               QLatin1Char(',') + QString::number((int)dw->newLine()) + QLatin1Char(',') + QString::number(dw->fixedExtent().width()) +
+               QLatin1Char(',') + QString::number(dw->fixedExtent().height()) + QLatin1Char(',') + QString::number((int)!dw->isHidden()) + QLatin1Char(']');
     }
     ts << str << endl;
 
@@ -1279,16 +1294,16 @@ QTextStream &operator>>(QTextStream &ts, Q3DockArea &dockArea)
 
     for (int i = 0; i < s.length(); ++i) {
         c = s[i];
-        if (state == Pre && c == '[') {
+        if (state == Pre && c == QLatin1Char('[')) {
             state++;
             continue;
         }
-        if (c == ',' &&
+        if (c == QLatin1Char(',') &&
              (state == Name || state == Offset || state == NewLine || state == Width || state == Height)) {
             state++;
             continue;
         }
-        if (state == Visible && c == ']') {
+        if (state == Visible && c == QLatin1Char(']')) {
             for (int j = 0; j < l.size(); ++j) {
                 Q3DockWindow *dw = l.at(j);
                 if (QString(dw->windowTitle()) == name) {
@@ -1304,7 +1319,7 @@ QTextStream &operator>>(QTextStream &ts, Q3DockArea &dockArea)
                 }
             }
 
-            name = offset = newLine = width = height = visible = "";
+            name = offset = newLine = width = height = visible = QLatin1String("");
 
             state = Pre;
             continue;
