@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,11 +50,11 @@
     Creates a new frame with the given \a parent, object \a name, and
     with widget flags \a f.
 */
-Q3Frame::Q3Frame(QWidget* parent, const char* name, Qt::WFlags f)
+Q3Frame::Q3Frame(QWidget* parent, const char* name, Qt::WindowFlags f)
     :QFrame(parent, f), marg(0)
 {
     if (name)
-        setObjectName(name);
+        setObjectName(QLatin1String(name));
     setAttribute(Qt::WA_LayoutOnEntireRect);
 }
 
@@ -59,12 +74,12 @@ void Q3Frame::paintEvent(QPaintEvent * event)
     QPainter paint(this);
     if (!contentsRect().contains(event->rect())) {
         paint.save();
-        paint.setClipRegion(event->region().intersect(frameRect()));
+        paint.setClipRegion(event->region().intersected(frameRect()));
         drawFrame(&paint);
         paint.restore();
     }
     if (event->rect().intersects(contentsRect())) {
-        paint.setClipRegion(event->region().intersect(contentsRect()));
+        paint.setClipRegion(event->region().intersected(contentsRect()));
         drawContents(&paint);
     }
 }
@@ -116,9 +131,10 @@ void Q3Frame::drawFrame(QPainter *p)
     This just calls frameChanged(); it does not make use of the \a
     event itself.
 */
-void Q3Frame::resizeEvent(QResizeEvent *)
+void Q3Frame::resizeEvent(QResizeEvent *e)
 {
-    frameChanged();
+    if (e->size() == e->oldSize())
+        frameChanged();
 }
 
 /*!
