@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -89,7 +104,7 @@ class QAbstractScrollAreaWidget : public QWidget
     Q_OBJECT
 
 public:
-    QAbstractScrollAreaWidget(Q3ScrollView* parent=0, const char* name=0, Qt::WFlags f = 0)
+    QAbstractScrollAreaWidget(Q3ScrollView* parent=0, const char* name=0, Qt::WindowFlags f = 0)
         : QWidget(parent, name, f)
     {
         setAutoFillBackground(true);
@@ -101,7 +116,7 @@ class QClipperWidget : public QWidget
     Q_OBJECT
 
 public:
-    QClipperWidget(QWidget * parent=0, const char * name=0, Qt::WFlags f=0)
+    QClipperWidget(QWidget * parent=0, const char * name=0, Qt::WindowFlags f=0)
         : QWidget (parent,name,f) {}
 };
 
@@ -198,7 +213,7 @@ public:
     uint signal_choke : 1;
 
     // This variables indicates in updateScrollBars() that we are
-    // in a resizeEvent() and thus don't want to flash scrollbars
+    // in a resizeEvent() and thus don't want to flash scroll bars
     uint inresize : 1;
     uint use_cached_size_hint : 1;
     QSize cachedSizeHint;
@@ -366,23 +381,27 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     viewport() of the scrollview and be added with addChild():
     \code
         Q3ScrollView* sv = new Q3ScrollView(...);
-        QVBoxWidget* big_box = new QVBoxWidget(sv->viewport());
-        sv->addChild(big_box);
+        QWidget *widget = new QWidget(sv->viewport());
+        QVBoxLayout *layout = new QVBoxLayout(widget);
+        addChild(widget);
     \endcode
     You can go on to add arbitrary child widgets to the single child
     in the scrollview as you would with any widget:
     \code
-        QLabel* child1 = new QLabel("CHILD", big_box);
-        QLabel* child2 = new QLabel("CHILD", big_box);
-        QLabel* child3 = new QLabel("CHILD", big_box);
+        QLabel* child1 = new QLabel("CHILD", widget);
+        QLabel* child2 = new QLabel("CHILD", widget);
+        QLabel* child3 = new QLabel("CHILD", widget);
+        layout->addWidget(child1);
+        layout->addWidget(child2);
+        layout->addWidget(child3);
         ...
     \endcode
 
     Here the Q3ScrollView has four children: the viewport(), the
     verticalScrollBar(), the horizontalScrollBar() and a small
-    cornerWidget(). The viewport() has one child: the big QVBoxWidget. The
-    QVBoxWidget has the three QLabel objects as child widgets. When the view
-    is scrolled, the QVBoxWidget is moved; its children move with it as
+    cornerWidget(). The viewport() has one child: the QWidget. The
+    QWidget has the three QLabel objects as child widgets. When the view
+    is scrolled, the QWidget is moved; its children move with it as
     child widgets normally do.
 
     \section1 Using a Very Big View with Some Widgets
@@ -497,8 +516,8 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     childX() and childY() to get the position of a child widget.
 
     A widget may be placed in the corner between the vertical and
-    horizontal scrollbars with setCornerWidget(). You can get access
-    to the scrollbars using horizontalScrollBar() and
+    horizontal scroll bars with setCornerWidget(). You can get access
+    to the scroll bars using horizontalScrollBar() and
     verticalScrollBar(), and to the viewport with viewport(). The
     scroll view can be scrolled using scrollBy(), ensureVisible(),
     setContentsPos() or center().
@@ -517,7 +536,7 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     \warning Q3ScrollView currently does not erase the background when
     resized, i.e. you must always clear the background manually in
     scrollview subclasses. This will change in a future version of Qt
-    and we recommend specifying the WNoAutoErase flag explicitly.
+    and we recommend specifying the \c WNoAutoErase flag explicitly.
 */
 
 
@@ -555,10 +574,10 @@ void Q3ScrollViewData::viewportResized(int w, int h)
     widget flags are propagated to the parent constructor as usual.
 */
 
-Q3ScrollView::Q3ScrollView(QWidget *parent, const char *name, Qt::WFlags f) :
+Q3ScrollView::Q3ScrollView(QWidget *parent, const char *name, Qt::WindowFlags f) :
     Q3Frame(parent, name, f & (~WStaticContents) & (~WNoAutoErase) & (~WResizeNoErase))
 {
-    WFlags flags = WResizeNoErase | (f&WPaintClever) | (f&WRepaintNoErase) | (f&WStaticContents);
+    WindowFlags flags = WResizeNoErase | (f&WPaintClever) | (f&WRepaintNoErase) | (f&WStaticContents);
     d = new Q3ScrollViewData(this, flags);
 
 #ifndef QT_NO_DRAGANDDROP
@@ -774,11 +793,11 @@ QSize Q3ScrollView::viewportSize(int x, int y) const
     int vsbExt = verticalScrollBar()->sizeHint().width();
 
     if (d->policy != AutoOne || d->anyVisibleChildren()) {
-        // Do we definitely need the scrollbar?
+        // Do we definitely need the scroll bar?
         needh = w-lmarg-rmarg < x;
         needv = h-tmarg-bmarg < y;
 
-        // Do we intend to show the scrollbar?
+        // Do we intend to show the scroll bar?
         if (d->hMode == AlwaysOn)
             showh = true;
         else if (d->hMode == AlwaysOff)
@@ -793,7 +812,7 @@ QSize Q3ScrollView::viewportSize(int x, int y) const
         else
             showv = needv;
 
-        // Given other scrollbar will be shown, NOW do we need one?
+        // Given other scroll bar will be shown, NOW do we need one?
         if (showh && h-vsbExt-tmarg-bmarg < y) {
             if (d->vMode == Auto)
                 showv=true;
@@ -803,7 +822,7 @@ QSize Q3ScrollView::viewportSize(int x, int y) const
                 showh=true;
         }
     } else {
-        // Scrollbars not needed, only show scrollbar that are always on.
+        // Scroll bars not needed, only show scroll bar that are always on.
         showh = d->hMode == AlwaysOn;
         showv = d->vMode == AlwaysOn;
     }
@@ -847,13 +866,13 @@ void Q3ScrollView::updateScrollBars()
     QSize oldVisibleSize(visibleWidth(), visibleHeight());
 
     if (d->policy != AutoOne || d->anyVisibleChildren()) {
-        // Do we definitely need the scrollbar?
+        // Do we definitely need the scroll bar?
         needh = w-lmarg-rmarg < d->contentsWidth();
         if (d->inresize)
             needh  = !horizontalScrollBar()->isHidden();
         needv = h-tmarg-bmarg < contentsHeight();
 
-        // Do we intend to show the scrollbar?
+        // Do we intend to show the scroll bar?
         if (d->hMode == AlwaysOn)
             showh = true;
         else if (d->hMode == AlwaysOff)
@@ -893,7 +912,7 @@ void Q3ScrollView::updateScrollBars()
         }
 #endif
 
-        // Given other scrollbar will be shown, NOW do we need one?
+        // Given other scroll bar will be shown, NOW do we need one?
         if (showh && h-vsbExt-tmarg-bmarg < contentsHeight()) {
             needv=true;
             if (d->vMode == Auto)
@@ -905,7 +924,7 @@ void Q3ScrollView::updateScrollBars()
                 showh=true;
         }
     } else {
-        // Scrollbars not needed, only show scrollbar that are always on.
+        // Scrollbars not needed, only show scroll bar that are always on.
         needh = needv = false;
         showh = d->hMode == AlwaysOn;
         showv = d->vMode == AlwaysOn;
@@ -914,7 +933,7 @@ void Q3ScrollView::updateScrollBars()
     bool sc = d->signal_choke;
     d->signal_choke=true;
 
-    // Hide unneeded scrollbar, calculate viewport size
+    // Hide unneeded scroll bar, calculate viewport size
     if (showh) {
         porth=h-hsbExt-tmarg-bmarg;
     } else {
@@ -932,7 +951,7 @@ void Q3ScrollView::updateScrollBars()
         portw=w-lmarg-rmarg;
     }
 
-    // Configure scrollbars that we will show
+    // Configure scroll bars that we will show
     if (needv) {
         d->vbar->setRange(0, contentsHeight()-porth);
         d->vbar->setSteps(Q3ScrollView::d->vbar->lineStep(), porth);
@@ -946,7 +965,7 @@ void Q3ScrollView::updateScrollBars()
         d->hbar->setRange(0, 0);
     }
 
-    // Position the scrollbars, viewport and corner widget.
+    // Position the scroll bars, viewport and corner widget.
     int bottom;
     bool reverse = QApplication::reverseLayout();
     int xoffset = (reverse && (showv || cornerWidget())) ? vsbExt : 0;
@@ -1123,7 +1142,7 @@ void Q3ScrollView::resizeEvent(QResizeEvent* event)
 /*!
     \reimp
 */
-void  Q3ScrollView::mousePressEvent(QMouseEvent * e) //#### remove for 4.0
+void  Q3ScrollView::mousePressEvent(QMouseEvent * e)
 {
     e->ignore();
 }
@@ -1131,7 +1150,7 @@ void  Q3ScrollView::mousePressEvent(QMouseEvent * e) //#### remove for 4.0
 /*!
     \reimp
 */
-void  Q3ScrollView::mouseReleaseEvent(QMouseEvent *e) //#### remove for 4.0
+void  Q3ScrollView::mouseReleaseEvent(QMouseEvent *e)
 {
     e->ignore();
 }
@@ -1140,7 +1159,7 @@ void  Q3ScrollView::mouseReleaseEvent(QMouseEvent *e) //#### remove for 4.0
 /*!
     \reimp
 */
-void  Q3ScrollView::mouseDoubleClickEvent(QMouseEvent *e) //#### remove for 4.0
+void  Q3ScrollView::mouseDoubleClickEvent(QMouseEvent *e)
 {
     e->ignore();
 }
@@ -1148,7 +1167,7 @@ void  Q3ScrollView::mouseDoubleClickEvent(QMouseEvent *e) //#### remove for 4.0
 /*!
     \reimp
 */
-void  Q3ScrollView::mouseMoveEvent(QMouseEvent *e) //#### remove for 4.0
+void  Q3ScrollView::mouseMoveEvent(QMouseEvent *e)
 {
     e->ignore();
 }
@@ -1164,9 +1183,9 @@ void Q3ScrollView::wheelEvent(QWheelEvent *e)
     viewportWheelEvent(&ce);
     if (!ce.isAccepted()) {
         if (e->orientation() == Horizontal && horizontalScrollBar())
-            QApplication::sendEvent(horizontalScrollBar(), e);
+            horizontalScrollBar()->event(e);
         else  if (e->orientation() == Vertical && verticalScrollBar())
-            QApplication::sendEvent(verticalScrollBar(), e);
+            verticalScrollBar()->event(e);
     } else {
         e->accept();
     }
@@ -1455,6 +1474,7 @@ int Q3ScrollView::childY(QWidget* child)
 
 bool Q3ScrollView::eventFilter(QObject *obj, QEvent *e)
 {
+    bool disabled = !(qobject_cast<QWidget*>(obj)->isEnabled());
     if (!d)
         return false; // we are destructing
     if (obj == d->viewport || obj == d->clipped_viewport) {
@@ -1468,30 +1488,42 @@ bool Q3ScrollView::eventFilter(QObject *obj, QEvent *e)
                 viewportResizeEvent((QResizeEvent *)e);
             break;
         case QEvent::MouseButtonPress:
+            if (disabled)
+                return false;
             viewportMousePressEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
         case QEvent::MouseButtonRelease:
+            if (disabled)
+                return false;
             viewportMouseReleaseEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
         case QEvent::MouseButtonDblClick:
+            if (disabled)
+                return false;
             viewportMouseDoubleClickEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
         case QEvent::MouseMove:
+            if (disabled)
+                return false;
             viewportMouseMoveEvent((QMouseEvent*)e);
             if (((QMouseEvent*)e)->isAccepted())
                 return true;
             break;
 #ifndef QT_NO_DRAGANDDROP
         case QEvent::DragEnter:
+            if (disabled)
+                return false;
             viewportDragEnterEvent((QDragEnterEvent*)e);
             break;
         case QEvent::DragMove: {
+            if (disabled)
+                return false;
             if (d->drag_autoscroll) {
                 QPoint vp = ((QDragMoveEvent*) e)->pos();
                 QRect inside_margin(autoscroll_margin, autoscroll_margin,
@@ -1506,15 +1538,27 @@ bool Q3ScrollView::eventFilter(QObject *obj, QEvent *e)
             viewportDragMoveEvent((QDragMoveEvent*)e);
         } break;
         case QEvent::DragLeave:
+            if (disabled)
+                return false;
             stopDragAutoScroll();
             viewportDragLeaveEvent((QDragLeaveEvent*)e);
             break;
         case QEvent::Drop:
+            if (disabled)
+                return false;
             stopDragAutoScroll();
             viewportDropEvent((QDropEvent*)e);
             break;
 #endif // QT_NO_DRAGANDDROP
+#ifndef QT_NO_WHEELEVENT
+        case QEvent::Wheel:
+            if (disabled)
+                return false;
+            break;
+#endif
         case QEvent::ContextMenu:
+            if (disabled)
+                return false;
             viewportContextMenuEvent((QContextMenuEvent*)e);
             if (((QContextMenuEvent*)e)->isAccepted())
                 return true;
@@ -2417,12 +2461,12 @@ void Q3ScrollView::changeFrameRect(const QRect& r)
     if (oldr != r) {
         QRect cr = contentsRect();
         QRegion fr(frameRect());
-        fr = fr.subtract(contentsRect());
+        fr = fr.subtracted(contentsRect());
         setFrameRect(r);
         if (isVisible()) {
-            cr = cr.intersect(contentsRect());
-            fr = fr.unite(frameRect());
-            fr = fr.subtract(cr);
+            cr = cr.intersected(contentsRect());
+            fr = fr.united(frameRect());
+            fr = fr.subtracted(cr);
             if (!fr.isEmpty())
                 update(fr);
         }
