@@ -2,19 +2,34 @@
 **
 ** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
-** This file is part of the $MODULE$ of the Qt Toolkit.
+** This file is part of the QtScript module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -1462,6 +1477,7 @@ void QScriptEnginePrivate::init()
     m_id_table.id_callee      = nameId(QLatin1String("callee"), true);
     m_id_table.id___proto__   = nameId(QLatin1String("__proto__"), true);
     m_id_table.id___fileName__ = nameId(QLatin1String("__fileName__"), true);
+    m_id_table.id___qt_sender__  = nameId(QLatin1String("__qt_sender__"), true);
 
     const int TEMP_STACK_SIZE = 10 * 1024;
     tempStackBegin = new QScriptValueImpl[TEMP_STACK_SIZE];
