@@ -2,19 +2,34 @@
 **
 ** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
-** This file is part of the $MODULE$ of the Qt Toolkit.
+** This file is part of the QtScript module of the Qt Toolkit.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -51,6 +66,15 @@ enum {
 
 static const bool GeneratePropertyFunctions = true;
 
+QByteArray QScriptMetaType::name() const
+{
+    if (!m_name.isEmpty())
+        return m_name;
+    else if (m_kind == Variant)
+        return "QVariant";
+    return QByteArray();
+}
+
 namespace QScript {
 
 class QtPropertyFunction: public QScriptFunction
@@ -695,8 +719,6 @@ QScript::ConnectionQObject::ConnectionQObject(const QMetaMethod &method,
       m_receiver(receiver)
 {
     m_slot = slot;
-    m_hasReceiver = (sender.isObject() && receiver.isObject()
-                     && sender.objectValue() != receiver.objectValue());
 
     QScriptEngine *eng = m_slot.engine();
     QScriptEnginePrivate *eng_p = QScriptEnginePrivate::get(eng);
@@ -785,8 +807,8 @@ void QScript::ConnectionQObject::execute(void **argv)
 
     int formalCount = fun->formals.count();
     int mx = qMax(formalCount, argc);
-    activation_data->m_members.resize(mx);
-    activation_data->m_objects.resize(mx);
+    activation_data->m_members.resize(mx + 1);
+    activation_data->m_objects.resize(mx + 1);
     for (int i = 0; i < mx; ++i) {
         QScriptNameIdImpl *nameId;
         if (i < formalCount)
@@ -809,12 +831,15 @@ void QScript::ConnectionQObject::execute(void **argv)
         senderObject = m_sender;
     else
         senderObject = eng->newQObject(sender());
+    activation_data->m_members[mx].object(eng->idTable()->id___qt_sender__, mx,
+                                          QScriptValue::SkipInEnumeration);
+    activation_data->m_objects[mx] = senderObject;
 
     QScriptValueImpl thisObject;
-    if (m_hasReceiver)
+    if (m_receiver.isObject())
         thisObject = m_receiver;
     else
-        thisObject = senderObject;
+        thisObject = eng->globalObject();
 
     QScriptContext *context = eng->pushContext();
     QScriptContextPrivate *context_data = QScriptContextPrivate::get(context);
@@ -824,17 +849,8 @@ void QScript::ConnectionQObject::execute(void **argv)
     context_data->argc = argc;
     context_data->args = const_cast<QScriptValueImpl*> (activation_data->m_objects.constData());
 
-    QScriptValueImpl tmp;
-    if (m_hasReceiver) {
-        tmp = m_receiver.property(QLatin1String("sender"));
-        m_receiver.setProperty(QLatin1String("sender"), senderObject);
-    }
-
     fun->execute(context_data);
 
-    if (m_hasReceiver)
-        m_receiver.setProperty(QLatin1String("sender"), tmp);
-
     if (context->state() == QScriptContext::ExceptionState) {
         qWarning() << "***" << context->returnValue().toString(); // ### fixme
     }
@@ -855,8 +871,13 @@ void QScript::ConnectionQObject::mark(int generation)
 bool QScript::ConnectionQObject::hasTarget(const QScriptValueImpl &receiver,
                                            const QScriptValueImpl &slot) const
 {
-    return ((receiver.objectValue() == m_receiver.objectValue())
-            && (slot.objectValue() == m_slot.objectValue()));
+    if (receiver.isObject() != m_receiver.isObject())
+        return false;
+    if ((receiver.isObject() && m_receiver.isObject())
+        && (receiver.objectValue() != m_receiver.objectValue())) {
+        return false;
+    }
+    return (slot.objectValue() == m_slot.objectValue());
 }
 
 void QScript::QtPropertyFunction::execute(QScriptContextPrivate *context)
@@ -1184,11 +1205,23 @@ void QScript::QtFunction::execute(QScriptContextPrivate *context)
                     }
                 } else if (actual.isVariant()) {
                     if (actual.variantValue().userType()
-                        || argType.name() == "QVariant") {
+                        || argType.isVariant()) {
                         // perfect
                     } else {
                         matchDistance += 10;
                     }
+                } else if (actual.isQObject()) {
+                    switch (tid) {
+                    case QMetaType::QObjectStar:
+                    case QMetaType::QWidgetStar:
+                        // perfect
+                        break;
+                    default:
+                        matchDistance += 10;
+                        break;
+                    }
+                } else {
+                    matchDistance += 10;
                 }
             }
 
@@ -1277,7 +1310,7 @@ void QScript::QtFunction::execute(QScriptContextPrivate *context)
                     result = eng_p->create(retType.typeId(), params[0]);
                     if (!result.isValid())
                         result = eng_p->newVariant(QVariant(retType.typeId(), params[0]));
-                } else if (retType.name() == "QVariant") {
+                } else if (retType.isVariant()) {
                     result = eng_p->newVariant(*(QVariant *)params[0]);
                 } else {
                     result = eng_p->undefinedValue();
