@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -44,21 +59,23 @@
 Q_DECLARE_METATYPE(sqlite3*)
 Q_DECLARE_METATYPE(sqlite3_stmt*)
 
-static QVariant::Type qSqliteType(int tp)
+static QVariant::Type qGetColumnType(const QString &tpName)
 {
-    switch (tp) {
-    case SQLITE_INTEGER:
+    const QString typeName = tpName.toLower();
+
+    if (typeName == QLatin1String("integer")
+        || typeName == QLatin1String("int"))
         return QVariant::Int;
-    case SQLITE_FLOAT:
+    if (typeName == QLatin1String("double")
+        || typeName == QLatin1String("float")
+        || typeName.startsWith(QLatin1String("numeric")))
         return QVariant::Double;
-    case SQLITE_BLOB:
+    if (typeName == QLatin1String("blob"))
         return QVariant::ByteArray;
-    case SQLITE_TEXT:
-    default:
-        return QVariant::String;
-    }
+    return QVariant::String;
 }
 
+
 static QSqlError qMakeError(sqlite3 *access, const QString &descr, QSqlError::ErrorType type,
                             int errorCode = -1)
 {
@@ -81,9 +98,8 @@ public:
     QSQLiteResultPrivate(QSQLiteResult *res);
     void cleanup();
     bool fetchNext(QSqlCachedResult::ValueCache &values, int idx, bool initialFetch);
-    bool isSelect();
     // initializes the recordInfo and the cache
-    void initColumns();
+    void initColumns(bool emptyResultset);
     void finalize();
 
     QSQLiteResult* q;
@@ -124,7 +140,7 @@ void QSQLiteResultPrivate::finalize()
     stmt = 0;
 }
 
-void QSQLiteResultPrivate::initColumns()
+void QSQLiteResultPrivate::initColumns(bool emptyResultset)
 {
     rInf.clear();
 
@@ -136,11 +152,18 @@ void QSQLiteResultPrivate::initColumns()
 
     for (int i = 0; i < nCols; ++i) {
         QString colName = QString::fromUtf16(
-                    static_cast<const ushort *>(sqlite3_column_name16(stmt, i)));
+                    static_cast<const ushort *>(sqlite3_column_name16(stmt, i))
+                    ).remove(QLatin1Char('"'));
+
+        // must use typeName for resolving the type to match QSqliteDriver::record
+        QString typeName = QString::fromUtf16(
+                    static_cast<const ushort *>(sqlite3_column_decltype16(stmt, i)));
 
         int dotIdx = colName.lastIndexOf(QLatin1Char('.'));
-        int stp = sqlite3_column_type(stmt, i);
-        QSqlField fld(colName.mid(dotIdx == -1 ? 0 : dotIdx + 1), qSqliteType(stp));
+        QSqlField fld(colName.mid(dotIdx == -1 ? 0 : dotIdx + 1), qGetColumnType(typeName));
+
+        // sqlite3_column_type is documented to have undefined behavior if the result set is empty
+        int stp = emptyResultset ? -1 : sqlite3_column_type(stmt, i);
         fld.setSqlType(stp);
         rInf.append(fld);
     }
@@ -159,9 +182,6 @@ bool QSQLiteResultPrivate::fetchNext(QSqlCachedResult::ValueCache &values, int i
     }
     skipRow = initialFetch;
 
-    if (!stmt)
-        return false;
-
     res = sqlite3_step(stmt);
 
     switch(res) {
@@ -169,7 +189,7 @@ bool QSQLiteResultPrivate::fetchNext(QSqlCachedResult::ValueCache &values, int i
         // check to see if should fill out columns
         if (rInf.isEmpty())
             // must be first call.
-            initColumns();
+            initColumns(false);
         if (idx < 0 && !initialFetch)
             return true;
         for (i = 0; i < rInf.count(); ++i) {
@@ -199,17 +219,25 @@ bool QSQLiteResultPrivate::fetchNext(QSqlCachedResult::ValueCache &values, int i
     case SQLITE_DONE:
         if (rInf.isEmpty())
             // must be first call.
-            initColumns();
+            initColumns(true);
         q->setAt(QSql::AfterLastRow);
+        sqlite3_reset(stmt);
         return false;
     case SQLITE_ERROR:
+        // SQLITE_ERROR is a generic error code and we must call sqlite3_reset()
+        // to get the specific error message.
+        res = sqlite3_reset(stmt);
+        q->setLastError(qMakeError(access, QCoreApplication::translate("QSQLiteResult",
+                        "Unable to fetch row"), QSqlError::ConnectionError, res));
+        q->setAt(QSql::AfterLastRow);
+        return false;
     case SQLITE_MISUSE:
     case SQLITE_BUSY:
     default:
         // something wrong, don't get col info, but still return false
         q->setLastError(qMakeError(access, QCoreApplication::translate("QSQLiteResult",
                         "Unable to fetch row"), QSqlError::ConnectionError, res));
-        finalize();
+        sqlite3_reset(stmt);
         q->setAt(QSql::AfterLastRow);
         return false;
     }
@@ -229,6 +257,16 @@ QSQLiteResult::~QSQLiteResult()
     delete d;
 }
 
+void QSQLiteResult::virtual_hook(int id, void *data)
+{
+    if (id == DetachFromResultSet) {
+        if (d->stmt)
+            sqlite3_reset(d->stmt);
+        return;
+    }
+    QSqlResult::virtual_hook(id, data);
+}
+
 bool QSQLiteResult::reset(const QString &query)
 {
     if (!prepare(query))
@@ -245,8 +283,13 @@ bool QSQLiteResult::prepare(const QString &query)
 
     setSelect(false);
 
+#if (SQLITE_VERSION_NUMBER >= 3003011)
+    int res = sqlite3_prepare16_v2(d->access, query.constData(), (query.size() + 1) * sizeof(QChar),
+                                   &d->stmt, 0);
+#else
     int res = sqlite3_prepare16(d->access, query.constData(), (query.size() + 1) * sizeof(QChar),
                                 &d->stmt, 0);
+#endif
 
     if (res != SQLITE_OK) {
         setLastError(qMakeError(d->access, QCoreApplication::translate("QSQLiteResult",
@@ -264,6 +307,7 @@ bool QSQLiteResult::exec()
     d->skippedStatus = false;
     d->skipRow = false;
     clearValues();
+    setLastError(QSqlError());
 
     int res = sqlite3_reset(d->stmt);
     if (res != SQLITE_OK) {
@@ -317,8 +361,17 @@ bool QSQLiteResult::exec()
                 return false;
             }
         }
+    } else {
+        setLastError(QSqlError(QCoreApplication::translate("QSQLiteResult",
+                        "Parameter count mismatch"), QString(), QSqlError::StatementError));
+        return false;
     }
     d->skippedStatus = d->fetchNext(cache(), 0, true);
+    if (lastError().isValid()) {
+        setSelect(false);
+        setActive(false);
+        return false;
+    }
     setSelect(!d->rInf.isEmpty());
     setActive(true);
     return true;
@@ -393,9 +446,12 @@ bool QSQLiteDriver::hasFeature(DriverFeature f) const
     case LastInsertId:
     case PreparedQueries:
     case PositionalPlaceholders:
+    case SimpleLocking:
         return true;
     case QuerySize:
     case NamedPlaceholders:
+    case BatchOperations:
+    case LowPrecisionNumbers:
         return false;
     }
     return false;
@@ -494,7 +550,7 @@ bool QSQLiteDriver::rollbackTransaction()
 
     QSqlQuery q(createResult());
     if (!q.exec(QLatin1String("ROLLBACK"))) {
-        setLastError(QSqlError(tr("Unable to roll back transaction"),
+        setLastError(QSqlError(tr("Unable to rollback transaction"),
                                q.lastError().databaseText(), QSqlError::TransactionError));
         return false;
     }
@@ -510,14 +566,19 @@ QStringList QSQLiteDriver::tables(QSql::TableType type) const
 
     QSqlQuery q(createResult());
     q.setForwardOnly(true);
-    if ((type & QSql::Tables) && (type & QSql::Views))
-        q.exec(QLatin1String("SELECT name FROM sqlite_master WHERE type='table' OR type='view'"));
-    else if (type & QSql::Tables)
-        q.exec(QLatin1String("SELECT name FROM sqlite_master WHERE type='table'"));
-    else if (type & QSql::Views)
-        q.exec(QLatin1String("SELECT name FROM sqlite_master WHERE type='view'"));
 
-    if (q.isActive()) {
+    QString sql = QLatin1String("SELECT name FROM sqlite_master WHERE %1 "
+                                "UNION ALL SELECT name FROM sqlite_temp_master WHERE %1");
+    if ((type & QSql::Tables) && (type & QSql::Views))
+        sql = sql.arg(QLatin1String("type='table' OR type='view'"));
+    else if (type & QSql::Tables)
+        sql = sql.arg(QLatin1String("type='table'"));
+    else if (type & QSql::Views)
+        sql = sql.arg(QLatin1String("type='view'"));
+    else
+        sql.clear();
+
+    if (!sql.isEmpty() && q.exec(sql)) {
         while(q.next())
             res.append(q.value(0).toString());
     }
@@ -530,24 +591,18 @@ QStringList QSQLiteDriver::tables(QSql::TableType type) const
     return res;
 }
 
-static QVariant::Type qGetColumnType(const QString &typeName)
-{
-    if (typeName == QLatin1String("integer")
-        || typeName == QLatin1String("int"))
-        return QVariant::Int;
-    if (typeName == QLatin1String("double")
-        || typeName == QLatin1String("float")
-        || typeName.startsWith(QLatin1String("numeric")))
-        return QVariant::Double;
-    if (typeName == QLatin1String("blob"))
-        return QVariant::ByteArray;
-    return QVariant::String;
-}
-
 static QSqlIndex qGetTableInfo(QSqlQuery &q, const QString &tableName, bool onlyPIndex = false)
 {
+    QString schema;
+    QString table(tableName);
+    int indexOfSeparator = tableName.indexOf(QLatin1String("."));
+    if (indexOfSeparator > -1) {
+        schema = tableName.left(indexOfSeparator).append(QLatin1String("."));
+        table = tableName.mid(indexOfSeparator + 1);
+    }
+    q.exec(QLatin1String("PRAGMA ") + schema + QLatin1String("table_info ('") + table + QLatin1String("')"));
+
     QSqlIndex ind;
-    q.exec(QLatin1String("PRAGMA table_info ('") + tableName + QLatin1String("')"));
     while (q.next()) {
         bool isPk = q.value(5).toInt();
         if (onlyPIndex && !isPk)
@@ -589,3 +644,12 @@ QVariant QSQLiteDriver::handle() const
     return qVariantFromValue(d->access);
 }
 
+QString QSQLiteDriver::escapeIdentifier(const QString &identifier, IdentifierType /*type*/) const
+{
+    QString res = identifier;
+    res.replace(QLatin1Char('"'), QLatin1String("\"\""));
+    res.prepend(QLatin1Char('"')).append(QLatin1Char('"'));
+    res.replace(QLatin1Char('.'), QLatin1String("\".\""));
+    return res;
+}
+
