@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -97,6 +112,7 @@ public:
 
     mutable QReadWriteLock lock;
 };
+Q_GLOBAL_STATIC(QConnectionDict, dbDict)
 
 class QSqlDatabasePrivate
 {
@@ -129,7 +145,6 @@ public:
     static void removeDatabase(const QString& name);
     static void invalidateDb(const QSqlDatabase &db, const QString &name);
     static DriverDict &driverDict();
-    Q_GLOBAL_STATIC(QConnectionDict, dbDict)
     static void cleanConnections();
 };
 
@@ -344,7 +359,7 @@ void QSqlDatabasePrivate::disable()
     Once the parameters have been set up you can call open() to open
     the connection.
 
-    The connection defined above is a nameless connection. If is the
+    The connection defined above is a nameless connection. It is the
     default connection and can be accessed using database() later on:
 
     \skipto QSqlDatabase db =
@@ -372,7 +387,8 @@ void QSqlDatabasePrivate::disable()
     If transactions are supported, you can use transaction() to start
     a transaction, and then commit() or rollback() to complete it.
     You can find out whether transactions are supported using
-    QSqlDriver::hasFeature().
+    QSqlDriver::hasFeature(). When using transactions you must start
+    the transaction before you create your query.
 
     If an error occurred, it is given by lastError().
 
@@ -381,7 +397,7 @@ void QSqlDatabasePrivate::disable()
     isDriverAvailable(). If you have created your own custom driver,
     you can register it with registerSqlDriver().
 
-    \sa QSqlDriver, QSqlQuery, {QtSql Module}
+    \sa QSqlDriver, QSqlQuery, {QtSql Module}, {Threads and the SQL Module}
 */
 
 /*!
@@ -413,7 +429,7 @@ void QSqlDatabasePrivate::disable()
     setConnectOptions(), and then you'll need to open() the
     connection.
 
-    \sa database() removeDatabase()
+    \sa database() removeDatabase() {Threads and the SQL Module}
 */
 QSqlDatabase QSqlDatabase::addDatabase(const QString &type, const QString &connectionName)
 {
@@ -433,7 +449,7 @@ QSqlDatabase QSqlDatabase::addDatabase(const QString &type, const QString &conne
     connectionName does not exist in the list of databases, an invalid
     connection is returned.
 
-    \sa isOpen()
+    \sa isOpen() {Threads and the SQL Module}
 */
 
 QSqlDatabase QSqlDatabase::database(const QString& connectionName, bool open)
@@ -474,7 +490,7 @@ QSqlDatabase QSqlDatabase::database(const QString& connectionName, bool open)
     QSqlDatabase::removeDatabase("sales"); // correct
     \endcode
 
-    \sa database()
+    \sa database() {Threads and the SQL Module}
 */
 
 void QSqlDatabase::removeDatabase(const QString& connectionName)
@@ -574,12 +590,12 @@ void QSqlDatabase::registerSqlDriver(const QString& name, QSqlDriverCreatorBase
     Returns true if the list of database connections contains \a
     connectionName; otherwise returns false.
 
-    \sa connectionNames(), database()
+    \sa connectionNames(), database(), {Threads and the SQL Module}
 */
 
 bool QSqlDatabase::contains(const QString& connectionName)
 {
-    return QSqlDatabasePrivate::dbDict()->contains_ts(connectionName);
+    return dbDict()->contains_ts(connectionName);
 }
 
 /*!
@@ -587,11 +603,11 @@ bool QSqlDatabase::contains(const QString& connectionName)
 
     Returns a list containing the names of all connections.
 
-    \sa contains(), database()
+    \sa contains(), database(), {Threads and the SQL Module}
 */
 QStringList QSqlDatabase::connectionNames()
 {
-    return QSqlDatabasePrivate::dbDict()->keys_ts();
+    return dbDict()->keys_ts();
 }
 
 /*!
@@ -814,7 +830,10 @@ bool QSqlDatabase::open(const QString& user, const QString& password)
 }
 
 /*!
-    Closes the database connection, freeing any resources acquired.
+    Closes the database connection, freeing any resources acquired, and
+    invalidating any existing QSqlQuery objects that are used with the
+    database.
+
     This will also affect copies of this QSqlDatabase object.
 
     \sa removeDatabase()
@@ -1200,7 +1219,7 @@ QSqlRecord QSqlDatabase::record(const QString& tablename) const
     \i
     \e none
 
-    \header \i SQLite
+    \header \i SQLite \i Interbase
     \row
 
     \i
@@ -1208,6 +1227,12 @@ QSqlRecord QSqlDatabase::record(const QString& tablename) const
     \i QSQLITE_BUSY_TIMEOUT
     \endlist
 
+    \i
+    \list
+    \i ISC_DPB_LC_CTYPE
+    \i ISC_DPB_SQL_ROLE_NAME
+    \endlist
+
     \endtable
 
     Examples:
