@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -41,13 +56,15 @@ public:
     ~QSqlQueryPrivate();
     QAtomic ref;
     QSqlResult* sqlResult;
+    QSql::NumericalPrecisionPolicy precisionPolicy;
 
-    Q_GLOBAL_STATIC_WITH_ARGS(QSqlQueryPrivate, nullQueryPrivate, (0))
-    Q_GLOBAL_STATIC(QSqlNullDriver, nullDriver)
-    Q_GLOBAL_STATIC_WITH_ARGS(QSqlNullResult, nullResult, (nullDriver()))
     static QSqlQueryPrivate* shared_null();
 };
 
+Q_GLOBAL_STATIC_WITH_ARGS(QSqlQueryPrivate, nullQueryPrivate, (0))
+Q_GLOBAL_STATIC(QSqlNullDriver, nullDriver)
+Q_GLOBAL_STATIC_WITH_ARGS(QSqlNullResult, nullResult, (nullDriver()))
+
 QSqlQueryPrivate* QSqlQueryPrivate::shared_null()
 {
     QSqlQueryPrivate *null = nullQueryPrivate();
@@ -58,7 +75,8 @@ QSqlQueryPrivate* QSqlQueryPrivate::shared_null()
 /*!
 \internal
 */
-QSqlQueryPrivate::QSqlQueryPrivate(QSqlResult* result): ref(1), sqlResult(result)
+QSqlQueryPrivate::QSqlQueryPrivate(QSqlResult* result)
+    : ref(1), sqlResult(result), precisionPolicy(QSql::HighPrecision)
 {
     if (!sqlResult)
         sqlResult = nullResult();
@@ -110,14 +128,14 @@ QSqlQueryPrivate::~QSqlQueryPrivate()
     \o seek()
     \endlist
 
-    These functions allow the programmer to move forward, backward or
-    arbitrarily through the records returned by the query. If you only
-    need to move forward through the results, e.g. using next() or
-    using seek() with a positive offset, you can use setForwardOnly()
-    and save a significant amount of memory overhead. Once an active
-    query is positioned on a valid record, data can be retrieved using
-    value(). All data is transferred from the SQL backend using
-    QVariants.
+    These functions allow the programmer to move forward, backward
+    or arbitrarily through the records returned by the query. If you
+    only need to move forward through the results (e.g., by using
+    next()), you can use setForwardOnly(), which will save a
+    significant amount of memory overhead and improve performance on
+    some databases. Once an active query is positioned on a valid
+    record, data can be retrieved using value(). All data is
+    transferred from the SQL backend using QVariants.
 
     For example:
 
@@ -202,6 +220,14 @@ QSqlQueryPrivate::~QSqlQueryPrivate()
 
     Note that unbound parameters will retain their values.
 
+    Stored procedures that uses the return statement to return values,
+    or return multiple result sets, are not fully supported. For specific
+    details see \l{SQL Database Drivers}.
+
+    \warning You must load the SQL driver and open the connection before a
+    QSqlQuery is created. Also, the connection must remain open while the
+    query exists; otherwise, the behavior of QSqlQuery is undefined.
+
     \sa QSqlDatabase, QSqlQueryModel, QSqlTableModel, QVariant
 */
 
@@ -333,12 +359,14 @@ bool QSqlQuery::exec(const QString& query)
     if (d->ref != 1) {
         bool fo = isForwardOnly();
         *this = QSqlQuery(driver()->createResult());
+        d->sqlResult->setNumericalPrecisionPolicy(d->precisionPolicy);
         setForwardOnly(fo);
     } else {
         d->sqlResult->clear();
         d->sqlResult->setActive(false);
         d->sqlResult->setLastError(QSqlError());
         d->sqlResult->setAt(QSql::BeforeFirstRow);
+        d->sqlResult->setNumericalPrecisionPolicy(d->precisionPolicy);
     }
     d->sqlResult->setQuery(query.trimmed());
     if (!driver()->isOpen() || driver()->isOpenError()) {
@@ -763,10 +791,16 @@ bool QSqlQuery::isForwardOnly() const
 }
 
 /*!
-    Sets forward only mode to \a forward. If \a forward is true, only
-    next() and seek() with positive values, are allowed for
-    navigating the results. Forward only mode needs far less memory
-    since results do not need to be cached.
+    Sets forward only mode to \a forward. If \a forward is true,
+    only next() and seek() with positive values, are allowed for
+    navigating the results.
+
+    Forward only mode can be (depending on the driver) more memory
+    efficient since results do not need to be cached. It will also
+    improve performance on some databases. For this to be true, you
+    must call \c setForwardMode() before the query is prepared or
+    executed. Note that the constructor that takes a query and a
+    database may execute the query.
 
     Forward only mode is off by default.
 
@@ -846,10 +880,12 @@ bool QSqlQuery::prepare(const QString& query)
         bool fo = isForwardOnly();
         *this = QSqlQuery(driver()->createResult());
         setForwardOnly(fo);
+        d->sqlResult->setNumericalPrecisionPolicy(d->precisionPolicy);
     } else {
         d->sqlResult->setActive(false);
         d->sqlResult->setLastError(QSqlError());
         d->sqlResult->setAt(QSql::BeforeFirstRow);
+        d->sqlResult->setNumericalPrecisionPolicy(d->precisionPolicy);
     }
     if (!driver()) {
         qWarning("QSqlQuery::prepare: no driver");
@@ -925,8 +961,9 @@ bool QSqlQuery::exec()
         4  NULL
     \endcode
 
-    To bind NULL values, a null QVariant has to be added to the bound QVariantList,
-    for example: \c {QVariant(QVariant::String)}
+    To bind NULL values, a null QVariant of the relevant type has to be added to
+    the bound QVariantList; for example, \c {QVariant(QVariant::String)} should be
+    used if you are using strings.
 
     Note that every bound QVariantList must contain the same amount of variants.
     Note that the type of the QVariants in a list must not change. For example,
@@ -955,6 +992,9 @@ bool QSqlQuery::execBatch(BatchExecutionMode mode)
     is QSql::Out or QSql::InOut, the placeholder will be
     overwritten with data from the database after the exec() call.
 
+    To bind a NULL value, use a null QVariant; for example, use
+    \c {QVariant(QVariant::String)} if you are binding a string.
+
     \sa addBindValue(), prepare(), exec(), boundValue() boundValues()
 */
 void QSqlQuery::bindValue(const QString& placeholder, const QVariant& val,
@@ -984,6 +1024,9 @@ void QSqlQuery::bindValue(int pos, const QVariant& val, QSql::ParamType paramTyp
     If \a paramType is QSql::Out or QSql::InOut, the placeholder will
     be overwritten with data from the database after the exec() call.
 
+    To bind a NULL value, use a null QVariant; for example, use
+    \c {QVariant(QVariant::String)} if you are binding a string.
+
     \sa bindValue(), prepare(), exec(), boundValue() boundValues()
 */
 void QSqlQuery::addBindValue(const QVariant& val, QSql::ParamType paramType)
@@ -1071,6 +1114,10 @@ QString QSqlQuery::executedQuery() const
     If more than one row was touched by the insert, the behavior is
     undefined.
 
+    Note that for Oracle databases the row's ROWID will be returned,
+    while for MySQL databases the row's auto-increment field will
+    be returned.
+
     \sa QSqlDriver::hasFeature()
 */
 QVariant QSqlQuery::lastInsertId() const
@@ -1078,3 +1125,35 @@ QVariant QSqlQuery::lastInsertId() const
     return d->sqlResult->lastInsertId();
 }
 
+/*!
+    Instruct the database driver to return numerical values with a precision specified by
+    \a precisionPolicy.
+
+    The Oracle driver, for example, retrieves numerical values as strings by default to
+    prevent the loss of precision. If the high precision doesn't matter, use this method
+    to increase execution speed by bypassing string conversions.
+
+    Note: Drivers that don't support fetching numerical values with low precision will
+    ignore the precision policy. You can use QSqlDriver::hasFeature() to find out whether a
+    driver supports this feature.
+
+    Note: Setting the precision policy doesn't affect the currently active query. Call
+    \l{exec()}{exec(QString)} or prepare() in order to activate the policy.
+
+    \sa QSql::NumericalPrecisionPolicy, numericalPrecisionPolicy()
+*/
+void QSqlQuery::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy precisionPolicy)
+{
+    d->precisionPolicy = precisionPolicy;
+}
+
+/*!
+    Returns the current precision policy.
+
+    \sa QSql::NumericalPrecisionPolicy, setNumericalPrecisionPolicy()
+*/
+QSql::NumericalPrecisionPolicy QSqlQuery::numericalPrecisionPolicy() const
+{
+    return d->precisionPolicy;
+}
+
