@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -70,6 +85,12 @@ QSqlQueryModelPrivate::~QSqlQueryModelPrivate()
 {
 }
 
+void QSqlQueryModelPrivate::initColOffsets(int size)
+{
+    colOffsets.resize(size);
+    memset(colOffsets.data(), 0, colOffsets.size() * sizeof(int));
+}
+
 /*!
     \class QSqlQueryModel
     \brief The QSqlQueryModel class provides a read-only data model for SQL
@@ -295,22 +316,26 @@ void QSqlQueryModel::setQuery(const QSqlQuery &query)
     Q_D(QSqlQueryModel);
     QSqlRecord newRec = query.record();
     bool columnsChanged = (newRec != d->rec);
-    bool hasQuerySize = d->query.driver()->hasFeature(QSqlDriver::QuerySize);
+    bool hasQuerySize = query.driver()->hasFeature(QSqlDriver::QuerySize);
 
-    if (d->colOffsets.size() != newRec.count() || columnsChanged) {
-        d->colOffsets.resize(newRec.count());
-        memset(d->colOffsets.data(), 0, d->colOffsets.size() * sizeof(int));
+    if (d->colOffsets.size() != newRec.count() || columnsChanged)
+        d->initColOffsets(newRec.count());
+
+    bool mustClearModel = d->bottom.isValid();
+    if (mustClearModel) {
+        d->atEnd = true;
+        beginRemoveRows(QModelIndex(), 0, qMax(d->bottom.row(), 0));
+        d->bottom = QModelIndex();
     }
 
-    beginRemoveRows(QModelIndex(), 0, qMax(d->bottom.row(), 0));
-
-    d->bottom = QModelIndex();
     d->error = QSqlError();
-    d->atEnd = false;
     d->query = query;
     d->rec = newRec;
-
-    endRemoveRows();
+   
+    if (mustClearModel)
+        endRemoveRows();
+    
+    d->atEnd = false;    
 
     if (columnsChanged)
         reset();
