@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -121,6 +136,7 @@ public:
     QSqlRecord baseRec; // the record without relations
     void clearChanges();
     void clearEditBuffer();
+    void clearCache();
     void revertCachedRow(int row);
 
     void translateFieldNames(int row, QSqlRecord &values) const;
@@ -175,6 +191,17 @@ void QSqlRelationalTableModelPrivate::clearEditBuffer()
     editBuffer = baseRec;
 }
 
+/*!
+    \reimp
+*/
+void QSqlRelationalTableModelPrivate::clearCache()
+{
+    for (int i = 0; i < relations.count(); ++i)
+        relations[i].displayValues.clear();
+
+    QSqlTableModelPrivate::clearCache();
+}
+
 /*!
     \class QSqlRelationalTableModel
     \brief The QSqlRelationalTableModel class provides an editable
@@ -235,6 +262,7 @@ void QSqlRelationalTableModelPrivate::clearEditBuffer()
     Notes:
 
     \list
+    \o The table must have a primary key declared.
     \o The table's primary key may not contain a relation to
        another table.
     \o If a relational table contains keys that refer to non-existent
@@ -383,10 +411,11 @@ QString QSqlRelationalTableModel::selectStatement() const
         if (relation.isValid()) {
             QString relTableAlias = QString::fromLatin1("relTblAl_%1").arg(i);
             fList.append(d->escapedRelationField(relTableAlias, relation.displayColumn()));
+            fList.append(QString::fromLatin1(" AS %1_%2").arg(relTableAlias).arg(relation.displayColumn()));
             fList.append(QLatin1Char(','));
             if (!tables.contains(relation.tableName()))
                 tables.append(d->db.driver()->escapeIdentifier(relation.tableName(),
-                       QSqlDriver::TableName).append(QLatin1String(" AS ")).append(
+                       QSqlDriver::TableName).append(QLatin1String(" ")).append(
                        d->db.driver()->escapeIdentifier(relTableAlias, QSqlDriver::TableName)));
             where.append(d->escapedRelationField(tableName(), rec.fieldName(i)));
             where.append(QLatin1Char('='));
