@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -22,9 +37,9 @@
 ****************************************************************************/
 
 #include "preprocessor.h"
-#include "scanner.h"
 #include "moc.h"
 #include "outputrevision.h"
+#include "../../corelib/global/qconfig.cpp"
 #include <QFile>
 #include <QFileInfo>
 #include <QDir>
@@ -49,8 +64,8 @@ static QByteArray combinePath(const char *infile, const char *outfile)
     QFileInfo outFileInfo(QDir::current(), QFile::decodeName(outfile));
     int numCommonComponents = 0;
 
-    QStringList inSplitted = inFileInfo.dir().canonicalPath().split('/');
-    QStringList outSplitted = outFileInfo.dir().canonicalPath().split('/');
+    QStringList inSplitted = inFileInfo.dir().canonicalPath().split(QLatin1Char('/'));
+    QStringList outSplitted = outFileInfo.dir().canonicalPath().split(QLatin1Char('/'));
 
     while (!inSplitted.isEmpty() && !outSplitted.isEmpty() &&
             inSplitted.first() == outSplitted.first()) {
@@ -71,10 +86,10 @@ static QByteArray combinePath(const char *infile, const char *outfile)
      */
     while (!outSplitted.isEmpty()) {
         outSplitted.removeFirst();
-        inSplitted.prepend("..");
+        inSplitted.prepend(QLatin1String(".."));
     }
     inSplitted.append(inFileInfo.fileName());
-    return QFile::encodeName(inSplitted.join("/"));
+    return QFile::encodeName(inSplitted.join(QLatin1String("/")));
 }
 
 
@@ -83,31 +98,135 @@ void error(const char *msg = "Invalid argument")
     if (msg)
         fprintf(stderr, "moc: %s\n", msg);
     fprintf(stderr, "Usage: moc [options] <header-file>\n"
-            "    -o<file>           Write output to file rather than stdout\n"
-            "    -I<dir>            Add dir to the include path for header files\n"
-            "    -E                 Preprocess only; do not generate meta object code\n"
-            "    -D<macro>[=<def>]  Define macro, with optional definition\n"
-            "    -U<macro>          Undefine macro\n"
-            "    -i                 Do not generate an #include statement\n"
-            "    -p<path>           Path prefix for included file\n"
-            "    -f[<file>]         Force #include, optional file name\n"
-            "    -nw                Do not display warnings\n"
-            "    -v                 Display version of moc\n");
+            "  -o<file>           write output to file rather than stdout\n"
+            "  -I<dir>            add dir to the include path for header files\n"
+            "  -E                 preprocess only; do not generate meta object code\n"
+            "  -D<macro>[=<def>]  define macro, with optional definition\n"
+            "  -U<macro>          undefine macro\n"
+            "  -i                 do not generate an #include statement\n"
+            "  -p<path>           path prefix for included file\n"
+            "  -f[<file>]         force #include, optional file name\n"
+            "  -nw                do not display warnings\n"
+            "  -v                 display version of moc\n");
     exit(1);
 }
 
-int main(int argc, char **argv)
+
+static inline bool hasNext(const Symbols &symbols, int i)
+{ return (i < symbols.size()); }
+
+static inline const Symbol &next(const Symbols &symbols, int &i)
+{ return symbols.at(i++); }
+
+
+QByteArray composePreprocessorOutput(const Symbols &symbols) {
+    QByteArray output;
+    int lineNum = 1;
+    Token last = PP_NOTOKEN;
+    Token secondlast = last;
+    int i = 0;
+    while (hasNext(symbols, i)) {
+        Symbol sym = next(symbols, i);
+        switch (sym.token) {
+        case PP_NEWLINE:
+        case PP_WHITESPACE:
+            if (last != PP_WHITESPACE) {
+                secondlast = last;
+                last = PP_WHITESPACE;
+                output += ' ';
+            }
+            continue;
+        case PP_STRING_LITERAL:
+            if (last == PP_STRING_LITERAL)
+                output.chop(1);
+            else if (secondlast == PP_STRING_LITERAL && last == PP_WHITESPACE)
+                output.chop(2);
+            else
+                break;
+            output += sym.lexem().mid(1);
+            secondlast = last;
+            last = PP_STRING_LITERAL;
+            continue;
+        case MOC_INCLUDE_BEGIN:
+            lineNum = 0;
+            continue;
+        case MOC_INCLUDE_END:
+            lineNum = sym.lineNum;
+            continue;
+        default:
+            break;
+        }
+        secondlast = last;
+        last = sym.token;
+
+        const int padding = sym.lineNum - lineNum;
+        if (padding > 0) {
+            output.resize(output.size() + padding);
+            qMemSet(output.data() + output.size() - padding, '\n', padding);
+            lineNum = sym.lineNum;
+        }
+
+        output += sym.lexem();
+    }
+
+    return output;
+}
+
+
+int main(int _argc, char **_argv)
 {
     bool autoInclude = true;
-    Preprocessor::macros["Q_MOC_RUN"] = "";
-    Preprocessor::macros["__cplusplus"] = "";
+    Preprocessor pp;
     Moc moc;
+    pp.macros["Q_MOC_RUN"];
+    pp.macros["__cplusplus"];
     QByteArray filename;
     QByteArray output;
     FILE *in = 0;
     FILE *out = 0;
     bool ignoreConflictingOptions = false;
-    for (int n = 1; n < argc; ++n) {
+
+    QVector<QByteArray> argv;
+    argv.resize(_argc - 1);
+    for (int n = 1; n < _argc; ++n)
+        argv[n - 1] = _argv[n];
+    int argc = argv.count();
+
+    for (int n = 0; n < argv.count(); ++n) {
+        if (argv.at(n).startsWith('@')) {
+            QByteArray optionsFile = argv.at(n);
+            optionsFile.remove(0, 1);
+            if (optionsFile.isEmpty())
+                error("The @ option requires an input file");
+            QFile f(QString::fromLatin1(optionsFile.constData()));
+            if (!f.open(QIODevice::ReadOnly | QIODevice::Text))
+                error("Cannot open options file specified with @");
+            argv.remove(n);
+            while (!f.atEnd()) {
+                QByteArray line = f.readLine().trimmed();
+                if (!line.isEmpty())
+                    argv.insert(n++, line);
+            }
+        }
+    }
+
+    // report Qt usage for commercial customers with a "metered license" (currently experimental)
+#if QT_EDITION != QT_EDITION_OPENSOURCE
+#ifdef QT_CONFIGURE_BINARIES_PATH
+    const char *binariesPath = QT_CONFIGURE_BINARIES_PATH;
+    QString reporterPath = QString::fromLocal8Bit(binariesPath) + QDir::separator()
+                           + QLatin1String("qtusagereporter");
+#if defined(Q_OS_WIN)
+    reporterPath += QLatin1String(".exe");
+#endif
+    if (QFile::exists(reporterPath))
+        system(qPrintable(reporterPath + QLatin1String(" moc")));
+#endif
+#endif
+
+    argc = argv.count();
+
+    for (int n = 0; n < argc; ++n) {
         QByteArray arg(argv[n]);
         if (arg[0] != '-') {
             if (filename.isEmpty()) {
@@ -128,7 +247,7 @@ int main(int argc, char **argv)
                 output = opt.mid(1);
             break;
         case 'E': // only preprocessor
-            Preprocessor::onlyPreprocess = true;
+            pp.preprocessOnly = true;
             break;
         case 'i': // no #include statement
             if (more)
@@ -159,27 +278,42 @@ int main(int argc, char **argv)
             if (!more) {
                 if (!(n < argc-1))
                     error("Missing path name for the -I option.");
-                Preprocessor::includes += argv[++n];
+                pp.includes += Preprocessor::IncludePath(argv[++n]);
             } else {
-                Preprocessor::includes += opt.mid(1);
+                pp.includes += Preprocessor::IncludePath(opt.mid(1));
+            }
+            break;
+        case 'F': // minimalistic framework support for the mac
+            if (!more) {
+                if (!(n < argc-1))
+                    error("Missing path name for the -F option.");
+                Preprocessor::IncludePath p(argv[++n]);
+                p.isFrameworkPath = true;
+                pp.includes += p;
+            } else {
+                Preprocessor::IncludePath p(opt.mid(1));
+                p.isFrameworkPath = true;
+                pp.includes += p;
             }
             break;
         case 'D': // define macro
             {
-                QByteArray macro, value;
+                QByteArray name, value;
                 if (!more) {
                     if (n < argc-1)
-                        macro = argv[++n];
+                        name = argv[++n];
                 } else
-                    macro = opt.mid(1);
-                int eq = macro.indexOf('=');
+                    name = opt.mid(1);
+                int eq = name.indexOf('=');
                 if (eq >= 0) {
-                    value = macro.mid(eq + 1);
-                    macro = macro.left(eq);
+                    value = name.mid(eq + 1);
+                    name = name.left(eq);
                 }
-                if (macro.isEmpty())
+                if (name.isEmpty())
                     error("Missing macro name");
-                Preprocessor::macros[macro] = value;
+                Macro macro;
+                macro.symbols += Symbol(0, PP_IDENTIFIER, value);
+                pp.macros.insert(name, macro);
 
             }
             break;
@@ -193,12 +327,12 @@ int main(int argc, char **argv)
                     macro = opt.mid(1);
                 if (macro.isEmpty())
                     error("Missing macro name");
-                Preprocessor::macros.remove(macro);
+                pp.macros.remove(macro);
 
             }
             break;
         case 'v':  // version number
-            if (more)
+            if (more && opt != "version")
                 error();
             fprintf(stderr, "Qt Meta Object Compiler version %d (Qt %s)\n",
                     mocOutputRevision, QT_VERSION_STR);
@@ -211,9 +345,12 @@ int main(int argc, char **argv)
             moc.displayWarnings = false;
             break;
         case 'h': // help
-            error(0); // 0 means usage only
+            if (more && opt != "help")
+                error();
+            else
+                error(0); // 0 means usage only
             break;
-        case '-': 
+        case '-':
             if (more && arg == "--ignore-option-clashes") {
                 // -- ignore all following moc specific options that conflict
                 // with for example gcc, like -pthread conflicting with moc's
@@ -230,7 +367,7 @@ int main(int argc, char **argv)
 
     if (autoInclude) {
         int ppos = filename.lastIndexOf('.');
-        moc.noInclude = (ppos >= 0 
+        moc.noInclude = (ppos >= 0
                          && tolower(filename[ppos + 1]) != 'h'
                          && tolower(filename[ppos + 1]) != QDir::separator().toLatin1()
                         );
@@ -253,9 +390,9 @@ int main(int argc, char **argv)
         in = stdin;
     } else {
 #if defined(_MSC_VER) && _MSC_VER >= 1400
-		if (fopen_s(&in, filename.data(), "r")) {
+		if (fopen_s(&in, filename.data(), "rb")) {
 #else
-        in = fopen(filename.data(), "r");
+        in = fopen(filename.data(), "rb");
 		if (!in) {
 #endif
             fprintf(stderr, "moc: %s: No such file\n", (const char*)filename);
@@ -264,29 +401,27 @@ int main(int argc, char **argv)
         moc.filename = filename;
     }
 
+    moc.currentFilenames.push(filename);
+
     // 1. preprocess
-    QByteArray input = Preprocessor::preprocessed(moc.filename, in);
+    moc.symbols = pp.preprocessed(moc.filename, in);
     fclose(in);
 
-    if (Preprocessor::onlyPreprocess)
-        goto step4;
+    if (!pp.preprocessOnly) {
+        // 2. parse
+        moc.parse();
+    }
 
-    // 2. tokenize
-    moc.symbols = Scanner::scan(input);
-
-    // 3. parse
-    moc.parse();
-
-    // 4. and output meta object code
- step4:
+    // 3. and output meta object code
 
     if (output.size()) { // output file specified
 #if defined(_MSC_VER) && _MSC_VER >= 1400
-		if (fopen_s(&out, output.data(), "w")) {
+        if (fopen_s(&out, output.data(), "w"))
 #else
         out = fopen(output.data(), "w"); // create output file
-        if (!out) {
+        if (!out)
 #endif
+        {
             fprintf(stderr, "moc: Cannot create %s\n", (const char*)output);
             return 1;
         }
@@ -294,8 +429,8 @@ int main(int argc, char **argv)
         out = stdout;
     }
 
-    if (Preprocessor::onlyPreprocess) {
-        fprintf(out, "%s%s\n", Preprocessor::protocol.constData(), input.constData());
+    if (pp.preprocessOnly) {
+        fprintf(out, "%s\n", composePreprocessorOutput(moc.symbols).constData());
     } else {
         if (moc.classList.isEmpty())
             moc.warning("No relevant classes found. No output generated.");
