@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,11 +44,12 @@
 #include <QMap>
 
 class Driver;
+class DomScript;
 
 class CustomWidgetsInfo : public TreeWalker
 {
 public:
-    CustomWidgetsInfo(Driver *driver);
+    CustomWidgetsInfo();
 
     void acceptUI(DomUI *node);
 
@@ -49,13 +65,16 @@ public:
     inline DomCustomWidget *customWidget(const QString &name) const
     { return m_customWidgets.value(name); }
 
+    DomScript *customWidgetScript(const QString &name) const;
+
     QString realClassName(const QString &className) const;
 
     bool extends(const QString &className, const QString &baseClassName) const;
 
 private:
-    Driver *driver;
-    QMap<QString, DomCustomWidget*> m_customWidgets;
+    typedef QMap<QString, DomCustomWidget*> NameCustomWidgetMap;
+    NameCustomWidgetMap m_customWidgets;
+    bool m_scriptsActivated;
 };
 
 #endif // CUSTOMWIDGETSINFO_H
