@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -42,8 +57,8 @@ bool Ui3Reader::isMainWindow = false;
 static QString lineColDebug(int line, int col)
 {
     if (line >= 0) {
-        const QString ret("Line: %1%2");
-        return ret.arg(line).arg(col >= 0 ? QString(" Column: %1").arg(col) : QString());
+        const QString ret = QString::fromLatin1("Line: %1%2");
+        return ret.arg(line).arg(col >= 0 ? QString::fromLatin1(" Column: %1").arg(col) : QString());
     }
     return QString();
 }
@@ -225,6 +240,7 @@ Ui3Reader::Ui3Reader(QTextStream &outStream)
    : out(outStream), trout(&languageChangeBody)
 {
     m_porting = new Porting();
+    m_extractImages = false;
 }
 
 Ui3Reader::~Ui3Reader()
@@ -234,7 +250,7 @@ Ui3Reader::~Ui3Reader()
 
 void Ui3Reader::generate(const QString &fn, const QString &outputFn,
           QDomDocument doc, bool decl, bool subcl, const QString &trm,
-          const QString& subClass, bool omitForwardDecls)
+          const QString& subClass, bool omitForwardDecls, const QString &convertedUiFile)
 {
     init();
 
@@ -251,7 +267,9 @@ void Ui3Reader::generate(const QString &fn, const QString &outputFn,
     bareNameOfClass = namespaces.last();
     namespaces.removeLast();
 
-    if (subcl) {
+    if (!convertedUiFile.isEmpty()) {
+        createWrapperDecl(e, convertedUiFile);
+    } else if (subcl) {
         if (decl)
             createSubDecl(e, subClass);
         else
@@ -603,3 +621,9 @@ bool Ui3Reader::isLayout(const QString& name) const
 {
     return layoutObjects.contains(name);
 }
+
+void Ui3Reader::setExtractImages(bool extract, const QString &qrcOutputFile)
+{
+    m_extractImages = extract;
+    m_qrcOutputFile = qrcOutputFile;
+}
