@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,8 +46,27 @@
 #include <QPixmap>
 #include <QMap>
 
+#include <QtGui/QFont>
+#include <QtGui/QFontDatabase>
+
 class Profile;
 
+struct FontSettings
+{
+    FontSettings() : useWindowFont(false), useBrowserFont(false),
+        windowWritingSystem(QFontDatabase::Latin), browserWritingSystem(QFontDatabase::Latin)
+        { }
+
+    QFont windowFont;
+    QFont browserFont;
+
+    bool useWindowFont;
+    bool useBrowserFont;
+
+    QFontDatabase::WritingSystem windowWritingSystem;
+    QFontDatabase::WritingSystem browserWritingSystem;
+};
+
 class Config
 {
 public:
@@ -62,30 +96,32 @@ public:
     QPixmap applicationIcon() const;
 
     // From QSettings, read / write
-    QString webBrowser() const { return webBrows; }
-    void setWebBrowser( const QString &cmd ) { webBrows = cmd; }
-
     QString homePage() const;
     void setHomePage( const QString &hom ) { home = hom; }
 
-    QString pdfReader() const { return pdfApp; }
-    void setPdfReader( const QString &cmd ) { pdfApp = cmd; }
-
     QStringList source() const;
     void setSource( const QStringList &s ) { src = s; }
 
     int sideBarPage() const { return sideBar; }
     void setSideBarPage( int sbp ) { sideBar = sbp; }
 
-    QRect geometry() const { return geom; }
-    void setGeometry( const QRect &geo ) { geom = geo; }
-
-    bool isMaximized() const { return maximized; }
-    void setMaximized( bool max ) { maximized = max; }
-
+    QByteArray windowGeometry() const { return winGeometry; }
+    void setWindowGeometry( const QByteArray &geometry ) { winGeometry = geometry; }
+    
     QByteArray mainWindowState() const { return mainWinState; }
     void setMainWindowState( const QByteArray &state ) { mainWinState = state; }
 
+    qreal fontPointSize() const { return pointFntSize; }
+    void setFontPointSize(qreal size)
+    { 
+        pointFntSize = size;
+        m_fontSettings.useBrowserFont = true;
+        m_fontSettings.browserFont.setPointSizeF(size); 
+    }
+
+    FontSettings fontSettings() { return m_fontSettings; }
+    void setFontSettings(const FontSettings &settings) { m_fontSettings = settings; }
+
     QString assistantDocPath() const;
 
     bool docRebuild() const { return rebuildDocs; }
@@ -93,6 +129,7 @@ public:
 
     void saveProfile( Profile *profile );
     void loadDefaultProfile();
+    bool defaultProfileExists();
 
     static Config *configuration();
     static Config *loadConfig(const QString &profileFileName);
@@ -102,21 +139,20 @@ private:
     Config& operator=( const Config &c );
 
     void saveSettings();
-
+    
 private:
     Profile *profil;
 
     QStringList profileNames;
-    QString webBrows;
     QString home;
-    QString pdfApp;
     QStringList src;
     QByteArray mainWinState;
-    QRect geom;
+    QByteArray winGeometry;
+    qreal pointFntSize;
     int sideBar;
-    bool maximized;
     bool hideSidebar;
     bool rebuildDocs;
+    FontSettings m_fontSettings;
 };
 
 #endif // CONFIG_H
