@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -39,12 +54,101 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#ifdef Q_WS_WIN
-#define INDEX_CHECK( text ) if( i+1 >= argc ) { QMessageBox::information( 0, "Qt Assistant", text ); return 1; }
-#else
-#define INDEX_CHECK( text ) if( i+1 >= argc ) { fprintf( stderr, text "\n" ); return 1; }
+#if defined(USE_STATIC_JPEG_PLUGIN)
+  #include <QtPlugin>
+  Q_IMPORT_PLUGIN(qjpeg)
 #endif
 
+#define INDEX_CHECK( text ) if( i+1 >= argc ) { fprintf(stderr, "%s\n", text); return 1; }
+
+
+#if !defined(QT_NO_DBUS) && defined(Q_OS_UNIX)
+#include <QtDBus/QDBusConnection>
+#include <QtDBus/QDBusAbstractAdaptor>
+#include <QtDBus/QDBusObjectPath>
+#include "tabbedbrowser.h"
+
+class HelpWindowAdaptor : public QDBusAbstractAdaptor
+{
+    Q_OBJECT
+    Q_CLASSINFO("D-Bus Interface", "com.trolltech.Assistant.HelpWindow")
+
+    Q_PROPERTY(QString source READ source WRITE setSource)
+
+public:
+    HelpWindowAdaptor(HelpWindow *w) : QDBusAbstractAdaptor(w), helpWindow(w)
+    {
+        setAutoRelaySignals(true);
+    }
+
+public Q_SLOTS:
+    inline QString source() const { return helpWindow->source().toString(); }
+    inline void setSource(const QString &src) { helpWindow->setSource(src); }
+
+    inline void clearHistory() { helpWindow->clearHistory(); }
+    inline void backward() { helpWindow->backward(); }
+    inline void forward() { helpWindow->forward(); }
+    inline void reload() { helpWindow->reload(); }
+    inline void home() { helpWindow->home(); }
+
+private:
+    HelpWindow *helpWindow;
+};
+
+class AssistantAdaptor : public QDBusAbstractAdaptor
+{
+    Q_OBJECT
+    Q_CLASSINFO("D-Bus Interface", "com.trolltech.Assistant.HelpViewer")
+
+public:
+    AssistantAdaptor(MainWindow *mw) : QDBusAbstractAdaptor(mw), mw(mw)
+    {
+        QDBusConnection connection = QDBusConnection::sessionBus();
+        connection.registerService(QLatin1String("com.trolltech.Assistant"));
+        connection.registerObject(QLatin1String("/Assistant"), mw);
+    }
+
+public slots:
+    void showLink(const QString &link) { mw->showLink(link); }
+    QDBusObjectPath createNewTab();
+    QDBusObjectPath currentTab();
+
+private:
+    QDBusObjectPath pathForBrowser(HelpWindow *window);
+    MainWindow *mw;
+};
+
+QDBusObjectPath AssistantAdaptor::createNewTab()
+{
+    HelpWindow *window = mw->browsers()->newBackgroundTab();
+    return pathForBrowser(window);
+}
+
+QDBusObjectPath AssistantAdaptor::currentTab()
+{
+    HelpWindow *window = mw->browsers()->currentBrowser();
+    return pathForBrowser(window);
+}
+
+QDBusObjectPath AssistantAdaptor::pathForBrowser(HelpWindow *window)
+{
+    int index = mw->browsers()->browsers().indexOf(window);
+    if (index == -1)
+        return QDBusObjectPath();
+
+    QString name(QLatin1String("/Assistant/Tabs/"));
+    name += QString::number(index);
+    QDBusObjectPath path(name);
+
+    if (!window->findChild<HelpWindowAdaptor *>()) {
+        (void)new HelpWindowAdaptor(window);
+        QDBusConnection::sessionBus().registerObject(name, window);
+    }
+
+    return path;
+}
+
+#endif // QT_NO_DBUS
 
 class AssistantSocket : public QTcpSocket
 {
@@ -80,7 +184,6 @@ private:
     quint16 p;
 };
 
-
 AssistantSocket::AssistantSocket( int sock, QObject *parent )
     : QTcpSocket( parent )
 {
@@ -93,7 +196,7 @@ void AssistantSocket::readClient()
 {
     QString link = QString();
     while ( canReadLine() )
-        link = readLine();
+        link = QLatin1String(readLine());
     if ( !link.isNull() ) {
         link = link.replace(QLatin1String("\n"), QLatin1String(""));
         link = link.replace(QLatin1String("\r"), QLatin1String(""));
@@ -138,20 +241,21 @@ int main( int argc, char ** argv )
     Q_INIT_RESOURCE(assistant);
 
     bool withGUI = true;
+#ifndef Q_WS_WIN
     if ( argc > 1 ) {
-        QString arg = QString::fromAscii(argv[1]);
+        QString arg = QString::fromLocal8Bit(argv[1]);
         arg = arg.toLower();
         if ( arg == QLatin1String("-addcontentfile")
             || arg == QLatin1String("-removecontentfile")
-#ifndef Q_WS_WIN
             || arg == QLatin1String("-help")
-#endif
+            || arg == QLatin1String("/?")
             )
             withGUI = false;
     }
+#endif
     QApplication a(argc, argv, withGUI);
-    a.setOrganizationName("Trolltech");
-    a.setApplicationName("Assistant");
+    a.setOrganizationName(QLatin1String("Trolltech"));
+    a.setApplicationName(QLatin1String("Assistant"));
 
     QString resourceDir;
     AssistantServer *as = 0;
@@ -159,9 +263,12 @@ int main( int argc, char ** argv )
     QString file, profileName, aDocPath;
     bool server = false;
     bool hideSidebar = false;
+    bool configLoaded = false;
     if ( argc == 2 ) {
-        if ( (argv[1])[0] != '-' ) {
-            file = QString::fromUtf8(argv[1]);
+        file = QString::fromLocal8Bit(argv[1]);
+        if (file.startsWith(QLatin1String("-")) || file == QLatin1String("/?")) {
+            file.clear();
+        } else {
             QFileInfo fi(file);
             file = fi.absoluteFilePath();
             file = MainWindow::urlifyFileName(file);
@@ -169,7 +276,7 @@ int main( int argc, char ** argv )
     }
     if ( file.isEmpty() ) {
         for ( int i = 1; i < argc; i++ ) {
-            QString opt = QString::fromAscii(argv[i]).toLower();
+            QString opt = QString::fromLocal8Bit(argv[i]).toLower();
             if ( opt == QLatin1String("-file") ) {
                 INDEX_CHECK( "Missing file argument!" );
                 i++;
@@ -181,21 +288,17 @@ int main( int argc, char ** argv )
                 profileName = QFile::decodeName(argv[++i]);
             } else if ( opt == QLatin1String("-addcontentfile") ) {
                 INDEX_CHECK( "Missing content file!" );
-                Config *c = Config::loadConfig( QString() );
+                Config *c = Config::loadConfig(QString());
                 QFileInfo file( QFile::decodeName(argv[i+1]) );
                 if( !file.exists() ) {
-                    fprintf( stderr, "Could not locate content file: '%s'\n",
-                             file.absoluteFilePath().toLatin1().constData() );
-                    fflush( stderr );
+                    fprintf(stderr, "Could not locate content file: %s\n", qPrintable(file.absoluteFilePath()));
                     return 1;
                 }
                 DocuParser *parser = DocuParser::createParser( file.absoluteFilePath() );
                 if( parser ) {
                     QFile f( QFile::decodeName(argv[i+1]) );
                     if( !parser->parse( &f ) ) {
-                        fprintf( stderr, "Failed to parse file: '%s'\n, ",
-                                 file.absoluteFilePath().toLatin1().constData() );
-                        fflush( stderr );
+                        fprintf(stderr, "Failed to parse file: %s\n", qPrintable(file.absoluteFilePath()));
                         return 1;
                     }
                     parser->addTo( c->profile() );
@@ -204,26 +307,28 @@ int main( int argc, char ** argv )
                 }
                 return 0;
             } else if ( opt == QLatin1String("-removecontentfile") ) {
-                INDEX_CHECK( "Missing content file!" );
-                Config *c = Config::loadConfig( QString() );
+                INDEX_CHECK("Missing content file!");
+                Config *c = Config::loadConfig(QString());
                 Profile *profile = c->profile();
-                QStringList entries = profile->docs.filter(QString::fromAscii(argv[i+1]));
+                QString contentFile = QString::fromLocal8Bit(argv[i+i]);
+                QStringList entries;
+#ifdef Q_WS_WIN
+                contentFile.replace(QLatin1Char('\\'), QLatin1Char('/'));
+                entries = profile->docs.filter(contentFile, Qt::CaseInsensitive);
+#else
+                entries = profile->docs.filter(contentFile);
+#endif
                 if (entries.count() == 0) {
-                    fprintf(stderr, "Could not locate content file: '%s'\n",
-                            argv[i+1]);
-                    fflush(stderr);
+                    fprintf(stderr, "Could not locate content file: %s\n", qPrintable(contentFile));
                     return 1;
                 } else if (entries.count() > 1) {
                     fprintf(stderr, "More than one entry matching file name found, "
                         "please specify full path to file");
-                    fflush(stderr);
                     return 1;
                 } else {
                     QFileInfo file(entries[0]);
                     if( !file.exists() ) {
-                        fprintf( stderr, "Could not locate content file: '%s'\n",
-                            file.absoluteFilePath().toLatin1().constData() );
-                        fflush( stderr );
+                        fprintf(stderr, "Could not locate content file: %s\n", qPrintable(file.absoluteFilePath()));
                         return 1;
                     }
                     profile->removeDocFileEntry( file.absoluteFilePath() );
@@ -231,23 +336,24 @@ int main( int argc, char ** argv )
                     c->save();
                 }
                 return 0;
-            } else if ( QString( argv[i] ).toLower() == "-docpath" ) {
+            } else if ( QString( QLatin1String(argv[i]) ).toLower() == QLatin1String("-docpath") ) {
                 INDEX_CHECK( "Missing path!" );
-                QDir dir( argv[i+1] );
+                QDir dir(QString::fromLocal8Bit(argv[i+1]));
                 if ( dir.exists() ) {
                     Config *c = Config::loadConfig(QString());
                     c->saveProfile(Profile::createDefaultProfile(dir.absolutePath()));
                     c->loadDefaultProfile();
                     c->setDocRebuild(true);
                     c->save();
+                    configLoaded = true;
+                    ++i;
                 } else {
-                    fprintf( stderr, "The specified path does not exist!\n");
-                    fflush( stderr );
+                    fprintf(stderr, "The specified path does not exist!\n");
                     return 1;
                 }
             } else if ( opt == QLatin1String("-hidesidebar") ) {
                 hideSidebar = true;
-            } else if ( opt == QLatin1String("-help") ) {
+            } else if ( opt == QLatin1String("-help") || opt == QLatin1String("/?") ) {
                 QString helpText = QLatin1String( "Usage: assistant [option]\n"
                                   "Options:\n"
                                   " -file Filename             assistant opens the specified file\n"
@@ -269,16 +375,15 @@ int main( int argc, char ** argv )
                 QMessageBox::information( 0, QLatin1String("Qt Assistant"),
                     QLatin1String("<pre>") + helpText + QLatin1String("</pre>") );
 #else
-                printf( "%s\n", helpText.toLatin1().constData() );
+                fprintf(stdout, "%s\n", qPrintable(helpText));
 #endif
                 exit( 0 );
             } else if ( opt == QLatin1String("-resourcedir") ) {
                 INDEX_CHECK( "Missing resource directory argument!" );
                 resourceDir = QFile::decodeName( argv[++i] );
             } else {
-                fprintf( stderr, "Unrecognized option '%s'. Try -help to get help.\n",
-                         argv[i] );
-                fflush( stderr );
+                fprintf(stderr, "Unrecognized option %s. Try -help to get help.\n", qPrintable(opt));
+                return 1;
             }
         }
     }
@@ -287,21 +392,24 @@ int main( int argc, char ** argv )
         resourceDir = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
 
     QTranslator translator( 0 );
-    translator.load( QLatin1String("assistant_") + QLocale::system().name().toLower(), resourceDir );
+    translator.load( QLatin1String("assistant_") + QLocale::system().name(), resourceDir );
     a.installTranslator( &translator );
 
     QTranslator qtTranslator( 0 );
-    qtTranslator.load( QLatin1String("qt_") + QLocale::system().name().toLower(), resourceDir );
+    qtTranslator.load( QLatin1String("qt_") + QLocale::system().name(), resourceDir );
     a.installTranslator( &qtTranslator );
 
-    Config *conf = Config::loadConfig( profileName );
-    if ( !conf ) {
+    Config *conf = 0;
+    if (configLoaded)
+        conf = Config::configuration();
+    else
+        conf = Config::loadConfig( profileName );
+    if (!conf) {
         fprintf( stderr, "Profile '%s' does not exist!\n", profileName.toLatin1().constData() );
         fflush( stderr );
         return -1;
     }
 
-    bool max = conf->isMaximized();
     QStringList links = conf->source();
     conf->hideSideBar( hideSidebar );
 
@@ -316,17 +424,23 @@ int main( int argc, char ** argv )
                      mw, SLOT(showLinkFromClient(QString)) );
     }
 
-    if ( max )
-        mw->setWindowState(mw->windowState() | Qt::WindowMaximized);
+#if !defined(QT_NO_DBUS) && defined(Q_OS_UNIX)
+    new AssistantAdaptor(mw);
+#endif // QT_NO_DBUS
+
+    FontSettings settings = conf->fontSettings();
+    if (mw->font() != settings.windowFont)
+        a.setFont(settings.windowFont, "QWidget");
+
     mw->show();
 
-    if ( !file.isEmpty() ) {
+    if (!file.isEmpty())
         mw->showLink( MainWindow::urlifyFileName(file) );
-    } else if ( file.isEmpty() )
+    else if (file.isEmpty())
         mw->showLinks( links );
 
     a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
-
+    
     int appExec = a.exec();
     delete (MainWindow*)mw;
     return appExec;
