@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,6 +40,8 @@
 #define MAINWINDOW_H
 
 #include "ui_mainwindow.h"
+#include "config.h"
+
 #include <QPointer>
 #include <QMap>
 
@@ -82,6 +99,9 @@ private slots:
     void forwardAvailable(bool);
 
     void browserTabChanged();
+    void copyAvailable(bool yes);
+    void updateTabActions(int index);
+    void showFontSettingsDialog();
 
 protected:
     void closeEvent(QCloseEvent *);
@@ -90,6 +110,7 @@ protected:
 private:
     void setupGoActions();
     bool insertActionSeparator();
+    void updateApplicationFontSettings(FontSettings &settings);
 
 private:
     Ui::MainWindow ui;
