@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -41,8 +56,8 @@ Profile *Profile::createDefaultProfile(const QString &docPath)
     QString path = QLibraryInfo::location(QLibraryInfo::DocumentationPath);
     if (!docPath.isEmpty())
         path = docPath;
-    path = QDir::cleanPath(path) + QLatin1String("/html/");  
-    
+    path = QDir::cleanPath(path) + QLatin1String("/html/");
+
     Profile *profile = new Profile;
     profile->valid = true;
     profile->type = DefaultProfile;
@@ -50,7 +65,6 @@ Profile *Profile::createDefaultProfile(const QString &docPath)
     profile->props[QLatin1String("applicationicon")] = QLatin1String("assistant.png");
     profile->props[QLatin1String("aboutmenutext")] = QLatin1String("About Qt");
     profile->props[QLatin1String("abouturl")] = QLatin1String("about_qt");
-    profile->props[QLatin1String("title")] = QLatin1String("Qt Assistant");
     profile->props[QLatin1String("basepath")] = path;
     profile->props[QLatin1String("startpage")] = path + QLatin1String("index.html");
 
@@ -80,21 +94,63 @@ Profile *Profile::createDefaultProfile(const QString &docPath)
     return profile;
 }
 
-
 Profile::Profile()
     : valid( true ), dparser( 0 )
 {
+    type = DefaultProfile;
 }
 
-
-void Profile::removeDocFileEntry( const QString &docfile )
+bool Profile::isValid() const
 {
-    docs.removeAll( docfile );
+    return valid;
+}
 
+void Profile::addDCFTitle(const QString &dcf, const QString &title)
+{
+    QString absdcf = QFileInfo(dcf).absoluteFilePath();
+    dcfTitles[title] = absdcf;
+    if (!docs.contains(absdcf))
+        docs << absdcf;
+}
+
+void Profile::addDCF(const QString &docfile)
+{
+    if( !docs.contains( docfile ) == 0 )
+        docs << docfile;
+}
+
+void Profile::addDCFIcon(const QString docfile, const QString &icon)
+{
+    icons[docfile] = icon;
+}
+
+void Profile::addDCFIndexPage(const QString title, const QString &indexPage)
+{
+    indexPages[title] = indexPage;
+}
+
+void Profile::addDCFImageDir(const QString docfile, const QString &imgDir)
+{
+    imageDirs[docfile] = imgDir;
+}
+
+void Profile::addProperty(const QString &name, const QString &value)
+{
+    props[name] = value;
+}
+
+bool Profile::hasDocFile(const QString &name)
+{
+    return docs.contains( name );
+}
+
+void Profile::removeDocFileEntry(const QString &docfile)
+{
+    docs.removeAll(docfile);
     QStringList titles;
 
     for( QMap<QString,QString>::Iterator it = dcfTitles.begin();
-         it != dcfTitles.end(); ++it ) {
+        it != dcfTitles.end(); ++it ) {
         if( (*it) == docfile ) {
             indexPages.remove( *it );
             icons.remove( *it );
@@ -103,16 +159,32 @@ void Profile::removeDocFileEntry( const QString &docfile )
         }
     }
 
-    for( QStringList::ConstIterator title = titles.begin();
-         title != titles.end(); ++title ) {
-
+    for( QStringList::ConstIterator title = titles.constBegin();
+        title != titles.constEnd(); ++title )
         dcfTitles.remove( *title );
-    }
 
 #ifdef ASSISTANT_DEBUG
-    qDebug( "docs:\n  - " + docs.join( "\n  - " ) );
-    qDebug( "titles:\n  - " + titles.join( "\n  - " ) );
-    qDebug( "keys:\n  - " + ( (QStringList*) &(dcfTitles.keys()) )->join( "\n  - " ) );
-    qDebug( "values:\n  - " + ( (QStringList*) &(dcfTitles.values()) )->join( "\n  - " ) );
+    qDebug() << "docs:\n  - " << docs.join("\n  - ");
+    qDebug() << "titles:\n  - " << titles.join("\n  - ");
+    qDebug() << "keys:\n  - " << ((QStringList*)&(dcfTitles.keys()))->join("\n  - ");
+    qDebug() << "values:\n  - " << ((QStringList*)&(dcfTitles.values()))->join("\n  - ");
 #endif
 }
+
+QString Profile::storableFilePath(const QString &fileName)
+{
+    QString path = QLibraryInfo::location(QLibraryInfo::DocumentationPath).replace(QLatin1String("\\"), QLatin1String("/"));
+    QString fName = fileName;
+    if (fName.startsWith(path))
+        fName.replace(0, path.length(), QLatin1String("$DOCPATH$"));
+    return fName;
+}
+
+QString Profile::loadableFilePath(const QString &fileName)
+{
+    QString path = QLibraryInfo::location(QLibraryInfo::DocumentationPath).replace(QLatin1String("\\"), QLatin1String("/"));
+    QString fName = fileName;
+    if (fName.startsWith(QLatin1String("$DOCPATH$")))
+        fName.replace(0, 9, path);
+    return fName;
+}
