@@ -9,20 +9,35 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
-#include <QtDesigner/QtDesigner>
 #include "formwindow_widgetstack.h"
+#include <QtDesigner/QDesignerFormWindowToolInterface>
 
 #include <QtGui/QWidget>
 #include <QtGui/qevent.h>
@@ -59,7 +74,7 @@ QDesignerFormWindowToolInterface *FormWindowWidgetStack::currentTool() const
 void FormWindowWidgetStack::setCurrentTool(int index)
 {
     if (index < 0 || index >= count()) {
-        qWarning("FormWindowWidgetStack::setCurrentTool(): invalid index: %d", index);
+        qDebug("FormWindowWidgetStack::setCurrentTool(): invalid index: %d", index);
         return;
     }
 
@@ -96,7 +111,7 @@ void FormWindowWidgetStack::setSenderAsCurrentTool()
     QDesignerFormWindowToolInterface *tool = 0;
     QAction *action = qobject_cast<QAction*>(sender());
     if (action == 0) {
-        qWarning("FormWindowWidgetStack::setSenderAsCurrentTool(): sender is not a QAction");
+        qDebug("FormWindowWidgetStack::setSenderAsCurrentTool(): sender is not a QAction");
         return;
     }
 
@@ -108,7 +123,7 @@ void FormWindowWidgetStack::setSenderAsCurrentTool()
     }
 
     if (tool == 0) {
-        qWarning("FormWindowWidgetStack::setSenderAsCurrentTool(): unknown tool");
+        qDebug("FormWindowWidgetStack::setSenderAsCurrentTool(): unknown tool");
         return;
     }
 
@@ -124,7 +139,7 @@ void FormWindowWidgetStack::setCurrentTool(QDesignerFormWindowToolInterface *too
 {
     int index = indexOf(tool);
     if (index == -1) {
-        qWarning("FormWindowWidgetStack::setCurrentTool(): unknown tool");
+        qDebug("FormWindowWidgetStack::setCurrentTool(): unknown tool");
         return;
     }
 
