@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,22 +40,22 @@
 #define FORMWINDOWMANAGER_H
 
 #include "formeditor_global.h"
-#include "formwindow.h"
 
-#include <QtDesigner/QtDesigner>
+#include <QtDesigner/QDesignerFormWindowManagerInterface>
 
 #include <QtCore/QObject>
 #include <QtCore/QList>
 #include <QtCore/QPointer>
+#include <QtCore/QMap>
 
 class QAction;
 class QActionGroup;
 class QUndoGroup;
-class MetaDataBase;
 class QDesignerFormEditorInterface;
 class QDesignerWidgetBoxInterface;
 
 namespace qdesigner_internal {
+class FormWindow;
 
 class QT_FORMEDITOR_EXPORT FormWindowManager: public QDesignerFormWindowManagerInterface
 {
@@ -143,16 +158,11 @@ private:
     QAction *m_actionUndo;
     QAction *m_actionRedo;
 
-    // DnD stuff
-    void beginDrag(const QList<QDesignerDnDItemInterface*> &item_list, const QPoint &globalPos);
-    void endDrag(const QPoint &pos);
-    void setItemsPos(const QPoint &pos);
-    bool isDecoration(QWidget *widget) const;
-    QList<QDesignerDnDItemInterface*> m_drag_item_list;
-    QWidget *m_last_widget_under_mouse;
-    FormWindow *m_last_form_under_mouse;
-    QDesignerWidgetBoxInterface *m_widget_box_under_mouse;
-    Qt::ContextMenuPolicy m_savedContextMenuPolicy;
+    QMap<QWidget *, bool> getUnsortedLayoutsToBeBroken(bool firstOnly) const;
+    bool hasLayoutsToBeBroken() const;
+    QList<QWidget *> layoutsToBeBroken(QWidget *w) const;
+    QList<QWidget *> layoutsToBeBroken() const;
+
     QUndoGroup *m_undoGroup;
 };
 
