@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -52,9 +67,9 @@ bool LinePropertySheet::isVisible(int index) const
 {
     QString name = propertyName(index);
 
-    return (name == QLatin1String("geometry")
-            || name == QLatin1String("orientation")
-            || name == QLatin1String("objectName"));
+    if (name == QLatin1String("frameShape"))
+        return false;
+    return QDesignerPropertySheet::isVisible(index);
 }
 
 void LinePropertySheet::setProperty(int index, const QVariant &value)
@@ -64,11 +79,7 @@ void LinePropertySheet::setProperty(int index, const QVariant &value)
 
 QString LinePropertySheet::propertyGroup(int index) const
 {
-    QString g = QDesignerPropertySheet::propertyGroup(index);
-    if (g == QLatin1String("QObject"))
-        return g;
-
-    return QLatin1String("Line");
+    return QDesignerPropertySheet::propertyGroup(index);
 }
 
 LinePropertySheetFactory::LinePropertySheetFactory(QExtensionManager *parent)
