@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -29,21 +44,26 @@ TRANSLATOR qdesigner_internal::WidgetEditorTool
 #include "formwindow.h"
 
 // sdk
-#include <QtDesigner/QtDesigner>
+#include <QtDesigner/QDesignerFormEditorInterface>
+#include <QtDesigner/QDesignerWidgetFactoryInterface>
+#include <QtDesigner/QDesignerWidgetBoxInterface>
+
 #include <layoutinfo_p.h>
+#include <qdesigner_dnditem_p.h>
 
 #include <QtGui/qevent.h>
 #include <QtGui/QAction>
 #include <QtGui/QMainWindow>
+#include <QtGui/QCursor>
 #include <QtCore/qdebug.h>
 
 using namespace qdesigner_internal;
 
 WidgetEditorTool::WidgetEditorTool(FormWindow *formWindow)
     : QDesignerFormWindowToolInterface(formWindow),
-      m_formWindow(formWindow)
+      m_formWindow(formWindow),
+      m_action(new QAction(tr("Edit Widgets"), this))
 {
-    m_action = new QAction(tr("Edit Widgets"), this);
 }
 
 QAction *WidgetEditorTool::action() const
@@ -101,7 +121,7 @@ bool WidgetEditorTool::mainWindowSeparatorEvent(QWidget *widget, QEvent *event)
 
 bool WidgetEditorTool::handleEvent(QWidget *widget, QWidget *managedWidget, QEvent *event)
 {
-    bool passive = core()->widgetFactory()->isPassiveInteractor(widget) != 0
+    const bool passive = core()->widgetFactory()->isPassiveInteractor(widget) != 0
                     || mainWindowSeparatorEvent(widget, event); // separators in QMainWindow
                                                                 // are no longer widgets
 
@@ -138,7 +158,16 @@ bool WidgetEditorTool::handleEvent(QWidget *widget, QWidget *managedWidget, QEve
     case QEvent::ContextMenu:
         return !passive && handleContextMenu(widget, managedWidget, static_cast<QContextMenuEvent*>(event));
 
-    default: break;
+    case QEvent::DragEnter:
+        return handleDragEnterMoveEvent(widget, managedWidget, static_cast<QDragEnterEvent *>(event), true);
+    case QEvent::DragMove:
+        return handleDragEnterMoveEvent(widget, managedWidget, static_cast<QDragEnterEvent *>(event), false);
+    case QEvent::DragLeave:
+        return handleDragLeaveEvent(widget, managedWidget, static_cast<QDragLeaveEvent *>(event));
+    case QEvent::Drop:
+        return handleDropEvent(widget, managedWidget, static_cast<QDropEvent *>(event));
+    default:
+        break;
 
     } // end switch
 
@@ -191,6 +220,77 @@ bool WidgetEditorTool::handlePaintEvent(QWidget *widget, QWidget *managedWidget,
     return false;
 }
 
+bool WidgetEditorTool::handleDragEnterMoveEvent(QWidget *, QWidget *, QDragMoveEvent *e, bool isEnter)
+{
+    const QDesignerMimeData *mimeData = qobject_cast<const QDesignerMimeData *>(e->mimeData());
+    if (!mimeData)
+        return false;
+
+    if (!m_formWindow->hasFeature(QDesignerFormWindowInterface::EditFeature)) {
+        e->ignore();
+        return true;
+    }
+
+    const QPoint formPos = e->pos();
+    const QPoint globalPos = m_formWindow->mapToGlobal(formPos);
+    const FormWindowBase::WidgetUnderMouseMode wum = mimeData->items().size() == 1 ? FormWindowBase::FindSingleSelectionDropTarget : FormWindowBase::FindMultiSelectionDropTarget;
+    QWidget *dropTarget = m_formWindow->widgetUnderMouse(formPos, wum);
+    if (m_lastDropTarget && dropTarget != m_lastDropTarget)
+        m_formWindow->highlightWidget(m_lastDropTarget, m_lastDropTarget->mapFromGlobal(globalPos), FormWindow::Restore);
+
+    m_lastDropTarget = dropTarget;
+
+    if (m_lastDropTarget)
+        m_formWindow->highlightWidget(m_lastDropTarget, m_lastDropTarget->mapFromGlobal(globalPos), FormWindow::Highlight);
+
+    if (isEnter || m_lastDropTarget)
+        mimeData->acceptEvent(e);
+    else
+        e->ignore();
+    return true;
+}
+
+bool WidgetEditorTool::handleDropEvent(QWidget *widget, QWidget *, QDropEvent *e)
+{
+    const QDesignerMimeData *mimeData = qobject_cast<const QDesignerMimeData *>(e->mimeData());
+    if (!mimeData)
+        return false;
+
+    if (!m_lastDropTarget ||
+        !m_formWindow->hasFeature(QDesignerFormWindowInterface::EditFeature)) {
+        e->ignore();
+        return true;
+    }
+    // FormWindow determines the position from the decoration.
+    const QPoint globalPos = widget->mapToGlobal(e->pos());
+    mimeData->moveDecoration(globalPos);
+    if (!m_formWindow->dropWidgets(mimeData->items(), m_lastDropTarget, globalPos)) {
+        e->ignore();
+        return true;
+    }
+    mimeData->acceptEvent(e);
+    return true;
+}
+
+bool WidgetEditorTool::restoreDropHighlighting()
+{
+    if (!m_lastDropTarget)
+        return false;
+
+    m_formWindow->highlightWidget(m_lastDropTarget, m_lastDropTarget->mapFromGlobal(QCursor::pos()), FormWindow::Restore);
+    m_lastDropTarget = 0;
+    return true;
+}
+
+bool WidgetEditorTool::handleDragLeaveEvent(QWidget *, QWidget *, QDragLeaveEvent *event)
+{
+    if (restoreDropHighlighting()) {
+        event->accept();
+        return true;
+    }
+    return false;
+}
+
 QWidget *WidgetEditorTool::editor() const
 {
     Q_ASSERT(formWindow() != 0);
