@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -77,25 +92,24 @@ protected:
 private:
     void trySetGeometry(QWidget *w, int x, int y, int width, int height);
     void tryResize(QWidget *w, int width, int height);
-    static int adjustPoint(int x, int dx);
 
 private:
-    QWidget *widget;
-    Type type;
-    QPoint oldPressPos;
-    FormWindow *formWindow;
-    WidgetSelection *sel;
-    QRect geom, origGeom;
-    bool active;
+    QWidget *m_widget;
+    const Type m_type;
+    QPoint m_origPressPos;
+    FormWindow *m_formWindow;
+    WidgetSelection *m_sel;
+    QRect m_geom, m_origGeom;
+    bool m_active;
 };
 
 class QT_FORMEDITOR_EXPORT WidgetSelection: public QObject
 {
     Q_OBJECT
 public:
-    WidgetSelection(FormWindow *parent, QHash<QWidget *, WidgetSelection *> *selDict);
+    WidgetSelection(FormWindow *parent);
 
-    void setWidget(QWidget *w, bool updateDict = true);
+    void setWidget(QWidget *w);
     bool isUsed() const;
 
     void updateGeometry();
@@ -106,19 +120,18 @@ public:
     QWidget *widget() const;
 
     QDesignerTaskMenuExtension *taskMenuExtension() const
-    { return taskMenu; }
+    { return m_taskMenu; }
 
     QDesignerFormEditorInterface *core() const;
 
     virtual bool eventFilter(QObject *object, QEvent *event);
 
 protected:
-    QHash<int, WidgetHandle*> handles;
+    QHash<int, WidgetHandle*> m_handles;
     InvisibleWidget *m_topWidget;
-    QPointer<QWidget> wid;
-    FormWindow *formWindow;
-    QHash<QWidget *, WidgetSelection *> *selectionDict;
-    QDesignerTaskMenuExtension *taskMenu;
+    QPointer<QWidget> m_wid;
+    FormWindow *m_formWindow;
+    QDesignerTaskMenuExtension *m_taskMenu;
 };
 
 }  // namespace qdesigner_internal
