@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,18 +9,37 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
+/*
+TRANSLATOR qdesigner_internal::PaletteEditorButton
+*/
+
 #include "paletteeditorbutton.h"
 #include "paletteeditor.h"
 
@@ -28,11 +47,12 @@
 
 using namespace qdesigner_internal;
 
-PaletteEditorButton::PaletteEditorButton(const QPalette &palette,
+PaletteEditorButton::PaletteEditorButton(QDesignerFormEditorInterface *core, const QPalette &palette,
                 QWidget *selectedWidget, QWidget *parent)
     : QToolButton(parent),
       m_palette(palette)
 {
+    m_core = core;
     m_selectedWidget = selectedWidget;
     setFocusPolicy(Qt::NoFocus);
     setText(tr("Change Palette"));
@@ -61,7 +81,7 @@ void PaletteEditorButton::showPaletteEditor()
                 p = m_selectedWidget->parentWidget()->palette();
         }
     }
-    QPalette pal = PaletteEditor::getPalette(0, m_palette, p, &result);
+    QPalette pal = PaletteEditor::getPalette(m_core, 0, m_palette, p, &result);
     if (result == QDialog::Accepted) {
         m_palette = pal;
         emit changed();
