@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -70,8 +85,8 @@ public:
 
     void enableCurrent(bool enable);
     void setColorSpinBoxes(const QColor &color);
-    QMap<double, QColor> stopsData(const QMap<double, QtGradientStop *> &stops) const;
-    QGradientStops makeGradientStops(const QMap<double, QColor> &data) const;
+    QMap<qreal, QColor> stopsData(const QMap<qreal, QtGradientStop *> &stops) const;
+    QGradientStops makeGradientStops(const QMap<qreal, QColor> &data) const;
     void updateZoom();
 
     QtGradientStopsModel *m_model;
@@ -104,10 +119,10 @@ void QtGradientStopsEditorPrivate::enableCurrent(bool enable)
     m_ui.alphaSpinBox->setEnabled(enable);
 }
 
-QMap<double, QColor> QtGradientStopsEditorPrivate::stopsData(const QMap<double, QtGradientStop *> &stops) const
+QMap<qreal, QColor> QtGradientStopsEditorPrivate::stopsData(const QMap<qreal, QtGradientStop *> &stops) const
 {
-    QMap<double, QColor> data;
-    QMap<double, QtGradientStop *>::ConstIterator itStop = stops.constBegin();
+    QMap<qreal, QColor> data;
+    QMap<qreal, QtGradientStop *>::ConstIterator itStop = stops.constBegin();
     while (itStop != stops.constEnd()) {
         QtGradientStop *stop = itStop.value();
         data[stop->position()] = stop->color();
@@ -117,10 +132,10 @@ QMap<double, QColor> QtGradientStopsEditorPrivate::stopsData(const QMap<double,
     return data;
 }
 
-QGradientStops QtGradientStopsEditorPrivate::makeGradientStops(const QMap<double, QColor> &data) const
+QGradientStops QtGradientStopsEditorPrivate::makeGradientStops(const QMap<qreal, QColor> &data) const
 {
     QGradientStops stops;
-    QMap<double, QColor>::ConstIterator itData = data.constBegin();
+    QMap<qreal, QColor>::ConstIterator itData = data.constBegin();
     while (itData != data.constEnd()) {
         stops << QPair<qreal, QColor>(itData.key(), itData.value());
 
@@ -222,7 +237,7 @@ void QtGradientStopsEditorPrivate::slotStopMoved(QtGradientStop *stop, qreal new
 {
     QTimer::singleShot(0, q_ptr, SLOT(slotUpdatePositionSpinBox()));
 
-    QMap<double, QColor> stops = stopsData(m_model->stops());
+    QMap<qreal, QColor> stops = stopsData(m_model->stops());
     stops.remove(stop->position());
     stops[newPos] = stop->color();
 
@@ -232,7 +247,7 @@ void QtGradientStopsEditorPrivate::slotStopMoved(QtGradientStop *stop, qreal new
 
 void QtGradientStopsEditorPrivate::slotStopAdded(QtGradientStop *stop)
 {
-    QMap<double, QColor> stops = stopsData(m_model->stops());
+    QMap<qreal, QColor> stops = stopsData(m_model->stops());
     stops[stop->position()] = stop->color();
 
     QGradientStops gradStops = makeGradientStops(stops);
@@ -241,7 +256,7 @@ void QtGradientStopsEditorPrivate::slotStopAdded(QtGradientStop *stop)
 
 void QtGradientStopsEditorPrivate::slotStopRemoved(QtGradientStop *stop)
 {
-    QMap<double, QColor> stops = stopsData(m_model->stops());
+    QMap<qreal, QColor> stops = stopsData(m_model->stops());
     stops.remove(stop->position());
 
     QGradientStops gradStops = makeGradientStops(stops);
@@ -259,7 +274,7 @@ void QtGradientStopsEditorPrivate::slotStopChanged(QtGradientStop *stop, const Q
         setColorSpinBoxes(newColor);
     }
 
-    QMap<double, QColor> stops = stopsData(m_model->stops());
+    QMap<qreal, QColor> stops = stopsData(m_model->stops());
     stops[stop->position()] = newColor;
 
     QGradientStops gradStops = makeGradientStops(stops);
@@ -583,8 +598,8 @@ QtGradientStopsEditor::QtGradientStopsEditor(QWidget *parent)
                 this, SLOT(slotRgbClicked()));
 
     d_ptr->enableCurrent(false);
-    d_ptr->m_ui.zoomInButton->setIcon(QIcon(":/qtgradienteditor/images/zoomin.png"));
-    d_ptr->m_ui.zoomOutButton->setIcon(QIcon(":/qtgradienteditor/images/zoomout.png"));
+    d_ptr->m_ui.zoomInButton->setIcon(QIcon(QLatin1String(":/qtgradienteditor/images/zoomin.png")));
+    d_ptr->m_ui.zoomOutButton->setIcon(QIcon(QLatin1String(":/qtgradienteditor/images/zoomout.png")));
     d_ptr->updateZoom();
 }
 
