@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -78,8 +93,8 @@ public:
 
     bool m_dragging;
     int m_dragOffset;
-    QMap<QtGradientStop *, double> m_dragStops;
-    QMap<double, QColor> m_dragOriginal;
+    QMap<QtGradientStop *, qreal> m_dragStops;
+    QMap<qreal, QColor> m_dragOriginal;
 };
 
 }
@@ -402,8 +417,8 @@ void QtGradientStopsWidget::mousePressEvent(QMouseEvent *e)
         } else if (e->modifiers() & Qt::ShiftModifier) {
             QtGradientStop *oldCurrent = d_ptr->m_model->currentStop();
             if (oldCurrent) {
-                QMap<double, QtGradientStop *> stops = d_ptr->m_model->stops();
-                QMap<double, QtGradientStop *>::ConstIterator itSt = stops.constFind(oldCurrent->position());
+                QMap<qreal, QtGradientStop *> stops = d_ptr->m_model->stops();
+                QMap<qreal, QtGradientStop *>::ConstIterator itSt = stops.constFind(oldCurrent->position());
                 if (itSt != stops.constEnd()) {
                     while (itSt != stops.constFind(stop->position())) {
                         d_ptr->m_model->selectStop(itSt.value(), true);
@@ -459,7 +474,7 @@ void QtGradientStopsWidget::mouseMoveEvent(QMouseEvent *e)
         double maxOffset = 0.0;
         double minOffset = 0.0;
         bool first = true;
-        QMap<QtGradientStop *, double>::ConstIterator itStop = d_ptr->m_dragStops.constBegin();
+        QMap<QtGradientStop *, qreal>::ConstIterator itStop = d_ptr->m_dragStops.constBegin();
         while (itStop != d_ptr->m_dragStops.constEnd()) {
             double offset = itStop.value();
 
@@ -479,7 +494,7 @@ void QtGradientStopsWidget::mouseMoveEvent(QMouseEvent *e)
         double viewportMin = d_ptr->toViewport(-minOffset);
         double viewportMax = d_ptr->toViewport(1.0 - maxOffset);
 
-        QMap<double, QtGradientStop *> newPositions;
+        QMap<qreal, QtGradientStop *> newPositions;
 
         int viewportX = e->pos().x() - d_ptr->m_dragOffset;
 
@@ -505,7 +520,7 @@ void QtGradientStopsWidget::mouseMoveEvent(QMouseEvent *e)
         }
 
         bool forward = true;
-        QMap<double, QtGradientStop *>::ConstIterator itNewPos = newPositions.constBegin();
+        QMap<qreal, QtGradientStop *>::ConstIterator itNewPos = newPositions.constBegin();
         if (itNewPos.value()->position() < itNewPos.key())
             forward = false;
 
@@ -529,7 +544,7 @@ void QtGradientStopsWidget::mouseMoveEvent(QMouseEvent *e)
                 itNewPos++;
         }
 
-        QMap<double, QColor>::ConstIterator itOld = d_ptr->m_dragOriginal.constBegin();
+        QMap<qreal, QColor>::ConstIterator itOld = d_ptr->m_dragOriginal.constBegin();
         while (itOld != d_ptr->m_dragOriginal.constEnd()) {
             double position = itOld.key();
             if (!d_ptr->m_model->at(position))
@@ -628,7 +643,7 @@ void QtGradientStopsWidget::keyPressEvent(QKeyEvent *e)
         d_ptr->m_model->deleteStops();
     } else if (e->key() == Qt::Key_Left || e->key() == Qt::Key_Right ||
                 e->key() == Qt::Key_Home || e->key() == Qt::Key_End) {
-        QMap<double, QtGradientStop *> stops = d_ptr->m_model->stops();
+        QMap<qreal, QtGradientStop *> stops = d_ptr->m_model->stops();
         if (stops.isEmpty())
             return;
         QtGradientStop *newCurrent = 0;
@@ -639,7 +654,7 @@ void QtGradientStopsWidget::keyPressEvent(QKeyEvent *e)
             else if (e->key() == Qt::Key_Right || e->key() == Qt::Key_End)
                 newCurrent = (--stops.constEnd()).value();
         } else {
-            QMap<double, QtGradientStop *>::ConstIterator itStop = stops.constBegin();
+            QMap<qreal, QtGradientStop *>::ConstIterator itStop = stops.constBegin();
             while (itStop.value() != current)
                 itStop++;
             if (e->key() == Qt::Key_Left && itStop != stops.constBegin())
