@@ -9,48 +9,63 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
-#include <QtGui/QUndoCommand>
 #include "signalsloteditor.h"
-#include "default_membersheet.h"
 #include "signalsloteditor_p.h"
-#include "ui4_p.h"
 
-#include <QtDesigner/QtDesigner>
+#include <qdesigner_membersheet_p.h>
+#include <ui4_p.h>
+
+#include <QtDesigner/QDesignerFormWindowInterface>
+#include <QtDesigner/QDesignerFormEditorInterface>
+#include <QtDesigner/QExtensionManager>
+#include <QtDesigner/QDesignerMetaDataBaseInterface>
+#include <QtDesigner/QDesignerContainerExtension>
+#include <QtDesigner/QDesignerLanguageExtension>
+#include <QtDesigner/QDesignerFormWindowCursorInterface>
+#include <QtDesigner/QDesignerWidgetDataBaseInterface>
 
 #include <QtGui/QAction>
 #include <QtGui/QDialog>
 #include <QtGui/QDialogButtonBox>
 #include <QtGui/QListWidget>
-#include <QtGui/QTreeWidget>
-#include <QtGui/QItemDelegate>
-#include <QtGui/QItemEditorFactory>
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QPushButton>
 #include <QtGui/QLabel>
 #include <QtGui/QCheckBox>
-#include <QtGui/QComboBox>
-#include <QtGui/QStandardItemModel>
-#include <QtGui/QHeaderView>
 #include <QtGui/QApplication>
+#include <QtGui/QUndoCommand>
 #include <QtGui/QMenu>
 
 #include <QtCore/qdebug.h>
 
-namespace qdesigner_internal {
-
 /*******************************************************************************
 ** Tools
 */
@@ -82,6 +97,20 @@ static void merge(QDesignerFormWindowInterface *form, QStringList *lst, const QL
     }
 }
 
+static bool signalMatchesSlot(QDesignerFormEditorInterface *core,
+                              const QString &signal,
+                              const QString &slot)
+{    
+    QExtensionManager *em = core->extensionManager();
+    const QDesignerLanguageExtension *lang = qt_extension<QDesignerLanguageExtension*> (em, core);
+    if (lang)
+        return lang->signalMatchesSlot(signal, slot);
+
+    return QDesignerMemberSheet::signalMatchesSlot(signal, slot);
+}
+
+namespace qdesigner_internal {
+
 QStringList objectNameList(QDesignerFormWindowInterface *form)
 {
     QStringList result;
@@ -96,7 +125,7 @@ QStringList objectNameList(QDesignerFormWindowInterface *form)
 
     QDesignerFormWindowCursorInterface *cursor = form->cursor();
     for (int i = 0; i < cursor->widgetCount(); ++i) {
-        QString name = cursor->widget(i)->objectName().trimmed();
+        const QString name = cursor->widget(i)->objectName().trimmed();
         if (!name.isEmpty())
             result.append(name);
     }
@@ -138,42 +167,6 @@ QStringList memberList(QDesignerFormWindowInterface *form, QObject *object, Memb
     return result;
 }
 
-bool signalMatchesSlot(const QString &signal, const QString &slot)
-{
-    bool result = true;
-
-    do {
-        int signal_idx = signal.indexOf(QLatin1Char('('));
-        int slot_idx = slot.indexOf(QLatin1Char('('));
-        if (signal_idx == -1 || slot_idx == -1)
-            break;
-
-        ++signal_idx; ++slot_idx;
-
-        if (slot.at(slot_idx) == QLatin1Char(')'))
-            break;
-
-        while (signal_idx < signal.size() && slot_idx < slot.size()) {
-            QChar signal_c = signal.at(signal_idx);
-            QChar slot_c = slot.at(slot_idx);
-
-            if (signal_c == QLatin1Char(',') && slot_c == QLatin1Char(')'))
-                break;
-
-            if (signal_c == QLatin1Char(')') && slot_c == QLatin1Char(')'))
-                break;
-
-            if (signal_c != slot_c) {
-                result = false;
-                break;
-            }
-
-            ++signal_idx; ++slot_idx;
-        }
-    } while (false);
-
-    return result;
-}
 
 ClassList classList(const QString &obj_name, MemberType member_type,
                             const QString &peer, QDesignerFormWindowInterface *form)
@@ -185,14 +178,13 @@ ClassList classList(const QString &obj_name, MemberType member_type,
     if (object == 0)
         return result;
 
-    QDesignerMemberSheetExtension *members
-        = qt_extension<QDesignerMemberSheetExtension*>
-                (form->core()->extensionManager(), object);
+    QDesignerFormEditorInterface *core = form->core();
+    QDesignerMemberSheetExtension *members = qt_extension<QDesignerMemberSheetExtension*>(core->extensionManager(), object);
     Q_ASSERT(members != 0);
 
     QString class_name;
     QStringList member_list;
-    for (int i = members->count(); i >= 0; --i) {
+    for (int i = members->count() -  1; i >= 0; --i) {
         if (!members->isVisible(i))
             continue;
 
@@ -202,12 +194,12 @@ ClassList classList(const QString &obj_name, MemberType member_type,
         if (member_type == SlotMember && !members->isSlot(i))
             continue;
 
-        QString signal = member_type == SignalMember ? members->signature(i) : peer;
-        QString slot = member_type == SignalMember ? peer : members->signature(i);
-        if (!signalMatchesSlot(signal, slot))
+        const QString signal = member_type == SignalMember ? members->signature(i) : peer;
+        const QString slot = member_type == SignalMember ? peer : members->signature(i);
+        if (!signalMatchesSlot(core, signal, slot))
             continue;
 
-        QString s = members->declaredInClass(i);
+       const QString s = members->declaredInClass(i);
         if (s != class_name) {
             if (!member_list.isEmpty())
                 result.append(ClassInfo(class_name, member_list));
@@ -221,7 +213,9 @@ ClassList classList(const QString &obj_name, MemberType member_type,
 
     return result;
 }
+}
 
+namespace {
 /*******************************************************************************
 ** OldSignalSlotDialog
 */
@@ -256,30 +250,29 @@ private:
 };
 
 // ### deprecated
-static QString realObjectName(QDesignerFormEditorInterface *core, QObject *object)
+QString realObjectName(QDesignerFormEditorInterface *core, QObject *object)
 {
     if (object == 0)
         return QString();
 
-    QString object_name = object->objectName();
-    QDesignerMetaDataBaseInterface *mdb = core->metaDataBase();
-    QDesignerMetaDataBaseItemInterface *item = mdb->item(object);
-    if (item != 0)
-        object_name = item->name();
-    return object_name;
+    const QDesignerMetaDataBaseInterface *mdb = core->metaDataBase();
+    if (const QDesignerMetaDataBaseItemInterface *item = mdb->item(object))
+        return item->name();
+
+    return object->objectName();
 }
 
-static QString realClassName(QDesignerFormEditorInterface *core, QWidget *widget)
+QString realClassName(QDesignerFormEditorInterface *core, QWidget *widget)
 {
     QString class_name = QLatin1String(widget->metaObject()->className());
-    QDesignerWidgetDataBaseInterface *wdb = core->widgetDataBase();
-    int idx = wdb->indexOfObject(widget);
+    const QDesignerWidgetDataBaseInterface *wdb = core->widgetDataBase();
+    const int idx = wdb->indexOfObject(widget);
     if (idx != -1)
         class_name = wdb->item(idx)->name();
     return class_name;
 }
 
-static QString widgetLabel(QDesignerFormEditorInterface *core, QWidget *widget)
+QString widgetLabel(QDesignerFormEditorInterface *core, QWidget *widget)
 {
     return QString::fromUtf8("%1 (%2)")
             .arg(realObjectName(core, widget))
@@ -294,7 +287,7 @@ void OldSignalSlotDialog::populateSlotList(const QString &signal)
     }
     m_slot_list->clear();
 
-    bool show_all = m_show_all_checkbox->isChecked();
+    const bool show_all = m_show_all_checkbox->isChecked();
 
     QStringList signatures;
 
@@ -309,7 +302,7 @@ void OldSignalSlotDialog::populateSlotList(const QString &signal)
                 continue;
 
             if (members->isSlot(i)) {
-                if (!signalMatchesSlot(signal, members->signature(i)))
+                if (!signalMatchesSlot(m_core, signal, members->signature(i)))
                     continue;
 
                 signatures.append(members->signature(i));
@@ -341,7 +334,7 @@ void OldSignalSlotDialog::populateSignalList()
     }
     m_signal_list->clear();
 
-    bool show_all = m_show_all_checkbox->isChecked();
+    const bool show_all = m_show_all_checkbox->isChecked();
 
     QStringList signatures;
 
@@ -384,34 +377,36 @@ void OldSignalSlotDialog::populateSignalList()
 
 // ### use designer
 OldSignalSlotDialog::OldSignalSlotDialog(QDesignerFormEditorInterface *core, QWidget *source, QWidget *destination,
-                                    QWidget *parent)
-    : QDialog(parent)
+                                         QWidget *parent) :
+    QDialog(parent),
+    m_signal_list(new QListWidget(this)),
+    m_slot_list(new QListWidget(this)),
+    m_buttonBox(new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel,Qt::Horizontal, this)),
+    m_ok_button(m_buttonBox->button(QDialogButtonBox::Ok)),
+    m_source(source),
+    m_destination(destination),
+    m_core(core),
+    m_show_all_checkbox(new QCheckBox(QObject::tr("Show all signals and slots")))
 {
-    m_source = source;
-    m_destination = destination;
-    m_core = core;
+    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
+    m_signal_list->setTextElideMode (Qt::ElideMiddle);
+    m_slot_list->setTextElideMode (Qt::ElideMiddle);
 
-    m_signal_list = new QListWidget(this);
     connect(m_signal_list,
                 SIGNAL(itemClicked(QListWidgetItem*)),
             this,
                 SLOT(selectSignal(QListWidgetItem*)));
-    m_slot_list = new QListWidget(this);
     connect(m_slot_list,
                 SIGNAL(itemClicked(QListWidgetItem*)),
             this,
                 SLOT(selectSlot(QListWidgetItem*)));
     m_slot_list->setEnabled(false);
 
-    m_buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel,
-                                       Qt::Horizontal, this);
-    m_ok_button = m_buttonBox->button(QDialogButtonBox::Ok);
     m_ok_button->setDefault(true);
     m_ok_button->setEnabled(false);
     connect(m_buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
     connect(m_buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
 
-    m_show_all_checkbox = new QCheckBox(tr("Show all signals and slots"));
     connect(m_show_all_checkbox, SIGNAL(toggled(bool)), this, SLOT(populateSignalList()));
 
     QLabel *source_label = new QLabel(this);
@@ -439,12 +434,12 @@ OldSignalSlotDialog::OldSignalSlotDialog(QDesignerFormEditorInterface *core, QWi
     l1->addWidget(m_show_all_checkbox);
     l1->addWidget(m_buttonBox);
 
-    setWindowTitle(tr("Configure Connection"));
+    setWindowTitle(QObject::tr("Configure Connection"));
 
     populateSignalList();
 }
 
-static QListWidgetItem *findItem(const QListWidget &list_widget, const QString &text)
+QListWidgetItem *findItem(const QListWidget &list_widget, const QString &text)
 {
     QListWidgetItem *result = 0;
     for (int i = 0; i < list_widget.count(); ++i) {
@@ -529,6 +524,9 @@ QString OldSignalSlotDialog::slot() const
     return item_list.at(0)->text();
 }
 
+}
+namespace qdesigner_internal {
+
 /*******************************************************************************
 ** SignalSlotConnection
 */
@@ -573,8 +571,6 @@ DomConnection *SignalSlotConnection::toUi() const
 
 void SignalSlotConnection::setSignal(const QString &signal)
 {
-
-
     m_signal = signal;
     setLabel(EndPoint::Source, m_signal);
 }
@@ -620,21 +616,21 @@ public:
     virtual void redo();
     virtual void undo();
 private:
+    const QString m_old_member;
+    const QString m_new_member;
+    const EndPoint::Type m_type;
     SignalSlotConnection *m_con;
-    QString m_old_member, m_new_member;
-    EndPoint::Type m_type;
     SignalSlotEditor *m_editor;
 };
 
 SetMemberCommand::SetMemberCommand(SignalSlotConnection *con, EndPoint::Type type,
-                                    const QString &member, SignalSlotEditor *editor)
+                                   const QString &member, SignalSlotEditor *editor) :
+    m_old_member(type == EndPoint::Source ? con->signal() : con->slot()),
+    m_new_member(member),
+    m_type(type),
+    m_con(con),
+    m_editor(editor)
 {
-    m_con = con;
-    m_editor = editor;
-    m_type = type;
-    m_old_member = type == EndPoint::Source ? con->signal() : con->slot();
-    m_new_member = member;
-
     if (type == EndPoint::Source)
         setText(QApplication::translate("Command", "Change signal"));
     else
@@ -667,15 +663,12 @@ void SetMemberCommand::undo()
 ** SignalSlotEditor
 */
 
-SignalSlotEditor::SignalSlotEditor(QDesignerFormWindowInterface *form_window, QWidget *parent)
-    : ConnectionEdit(parent, form_window), m_showAllSignalsSlots(false)
+SignalSlotEditor::SignalSlotEditor(QDesignerFormWindowInterface *form_window, QWidget *parent) :
+     ConnectionEdit(parent, form_window),
+     m_form_window(form_window),
+     m_model(new ConnectionModel(this, this)),
+     m_showAllSignalsSlots(false)
 {
-    m_form_window = form_window;
-    m_model = new ConnectionModel(this, this);
-
-    QDesignerFormEditorInterface *core = form_window->core();
-    QDesignerMemberSheetFactory *factory = new QDesignerMemberSheetFactory(core->extensionManager());
-    core->extensionManager()->registerExtensions(factory, Q_TYPEID(QDesignerMemberSheetExtension));
     connect(this, SIGNAL(widgetActivated(QWidget*)), form_window, SIGNAL(activated(QWidget*)));
 }
 
@@ -689,8 +682,10 @@ void SignalSlotEditor::modifyConnection(Connection *con)
     SignalSlotConnection *sigslot_con = static_cast<SignalSlotConnection*>(con);
 
     OldSignalSlotDialog dialog(m_form_window->core(),
-                                                    sigslot_con->widget(EndPoint::Source),
-                                                    sigslot_con->widget(EndPoint::Target));
+                               sigslot_con->widget(EndPoint::Source),
+                               sigslot_con->widget(EndPoint::Target),
+                               m_form_window->core()->topLevel());
+
     dialog.setSignalSlot(sigslot_con->signal(), sigslot_con->slot());
     dialog.setShowAllSignalsSlots(m_showAllSignalsSlots);
 
@@ -709,7 +704,11 @@ Connection *SignalSlotEditor::createConnection(QWidget *source, QWidget *destina
     Q_ASSERT(source != 0);
     Q_ASSERT(destination != 0);
 
-    OldSignalSlotDialog dialog(m_form_window->core(), source, destination);
+    OldSignalSlotDialog dialog(m_form_window->core(),
+                               source,
+                               destination,
+                               m_form_window->core()->topLevel());
+
     dialog.setShowAllSignalsSlots(m_showAllSignalsSlots);
 
     if (dialog.exec() == QDialog::Accepted) {
@@ -728,26 +727,26 @@ DomConnections *SignalSlotEditor::toUi() const
     DomConnections *result = new DomConnections;
     QList<DomConnection*> list;
     for (int i = 0; i < connectionCount(); ++i) {
-        SignalSlotConnection *con = static_cast<SignalSlotConnection*>(connection(i));
+        const SignalSlotConnection *con = static_cast<const SignalSlotConnection*>(connection(i));
         Q_ASSERT(con != 0);
 
         // If a widget's parent has been removed, and the parent was not a managed widget
         // (a page in a tab widget), we never get a widgetRemoved(). So we filter out
         // these child widgets here.
-        QObject *source = con->object(EndPoint::Source);
+        const QObject *source = con->object(EndPoint::Source);
         if (source == 0)
             continue;
 
-        QObject *target = con->object(EndPoint::Target);
+        const QObject *target = con->object(EndPoint::Target);
         if (target == 0)
             continue;
 
-        if (QWidget *sourceWidget = qobject_cast<QWidget*>(source)) {
+        if (const QWidget *sourceWidget = qobject_cast<const QWidget*>(source)) {
             if (!background()->isAncestorOf(sourceWidget))
                 continue;
         }
 
-        if (QWidget *targetWidget = qobject_cast<QWidget*>(target)) {
+        if (const QWidget *targetWidget = qobject_cast<const QWidget*>(target)) {
             if (!background()->isAncestorOf(targetWidget))
                 continue;
         }
@@ -775,30 +774,30 @@ QObject *SignalSlotEditor::objectByName(QWidget *topLevel, const QString &name)
     return 0;
 }
 
-void SignalSlotEditor::fromUi(DomConnections *connections, QWidget *parent)
+void SignalSlotEditor::fromUi(const DomConnections *connections, QWidget *parent)
 {
     if (connections == 0)
         return;
 
     setBackground(parent);
-
-    QList<DomConnection*> list = connections->elementConnection();
-    foreach (DomConnection *dom_con, list) {
+    clear();
+    const QList<DomConnection*> list = connections->elementConnection();
+    foreach (const DomConnection *dom_con, list) {
         QObject *source = objectByName(parent, dom_con->elementSender());
         if (source == 0) {
-            qWarning("SignalSlotEditor::fromUi(): no source widget called \"%s\"",
+            qDebug("SignalSlotEditor::fromUi(): no source widget called \"%s\"",
                         dom_con->elementSender().toUtf8().constData());
             continue;
         }
         QObject *destination = objectByName(parent, dom_con->elementReceiver());
         if (destination == 0) {
-            qWarning("SignalSlotEditor::fromUi(): no destination widget called \"%s\"",
+            qDebug("SignalSlotEditor::fromUi(): no destination widget called \"%s\"",
                         dom_con->elementReceiver().toUtf8().constData());
             continue;
         }
 
         QPoint sp = QPoint(20, 20), tp = QPoint(20, 20);
-        DomConnectionHints *dom_hints = dom_con->elementHints();
+        const DomConnectionHints *dom_hints = dom_con->elementHints();
         if (dom_hints != 0) {
             QList<DomConnectionHint*> list = dom_hints->elementHint();
             foreach (DomConnectionHint *hint, list) {
@@ -821,9 +820,9 @@ void SignalSlotEditor::fromUi(DomConnections *connections, QWidget *parent)
     }
 }
 
-static bool skipWidget(QWidget *w)
+static bool skipWidget(const QWidget *w)
 {
-    QString name = QLatin1String(w->metaObject()->className());
+    const QString name = QLatin1String(w->metaObject()->className());
     if (name == QLatin1String("QDesignerWidget"))
         return true;
     if (name == QLatin1String("QLayoutWidget"))
@@ -858,10 +857,10 @@ void SignalSlotEditor::setSignal(SignalSlotConnection *con, const QString &membe
 {
     if (member == con->signal())
         return;
-
+    
     m_form_window->beginCommand(QApplication::translate("Command", "Change signal"));
     undoStack()->push(new SetMemberCommand(con, EndPoint::Source, member, this));
-    if (!signalMatchesSlot(member, con->slot()))
+    if (!signalMatchesSlot(m_form_window->core(), member, con->slot()))
         undoStack()->push(new SetMemberCommand(con, EndPoint::Target, QString(), this));
     m_form_window->endCommand();
 }
@@ -873,7 +872,7 @@ void SignalSlotEditor::setSlot(SignalSlotConnection *con, const QString &member)
 
     m_form_window->beginCommand(QApplication::translate("Command", "Change slot"));
     undoStack()->push(new SetMemberCommand(con, EndPoint::Target, member, this));
-    if (!signalMatchesSlot(con->signal(), member))
+    if (!signalMatchesSlot(m_form_window->core(), con->signal(), member))
         undoStack()->push(new SetMemberCommand(con, EndPoint::Source, QString(), this));
     m_form_window->endCommand();
 }
@@ -889,7 +888,7 @@ void SignalSlotEditor::setSource(Connection *_con, const QString &obj_name)
     ConnectionEdit::setSource(con, obj_name);
 
     QObject *sourceObject = con->object(EndPoint::Source);
-    QStringList member_list = memberList(m_form_window, sourceObject, SignalMember);
+    const QStringList member_list = memberList(m_form_window, sourceObject, SignalMember);
 
     if (!member_list.contains(con->signal()))
         undoStack()->push(new SetMemberCommand(con, EndPoint::Source, QString(), this));
