@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,27 +9,47 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
+/*
+TRANSLATOR qdesigner_internal::ComboBoxTaskMenu
+*/
+
 #include "combobox_taskmenu.h"
 #include "inplace_editor.h"
 #include "listwidgeteditor.h"
 #include <qdesigner_command_p.h>
 
-#include <QtDesigner/QtDesigner>
-
+#include <QtDesigner/QDesignerFormWindowInterface>
+    
 #include <QtGui/QAction>
 #include <QtGui/QStyle>
+#include <QtGui/QFontComboBox>
 #include <QtGui/QStyleOption>
 
 #include <QtCore/QEvent>
@@ -83,7 +103,7 @@ void ComboBoxTaskMenu::editItems()
         }
         ChangeListContentsCommand *cmd = new ChangeListContentsCommand(m_formWindow);
         cmd->init(m_comboBox, items);
-        cmd->setDescription(tr("Change Combobox Contents"));
+        cmd->setText(tr("Change Combobox Contents"));
         m_formWindow->commandHistory()->push(cmd);
     }
 }
@@ -95,7 +115,8 @@ ComboBoxTaskMenuFactory::ComboBoxTaskMenuFactory(QExtensionManager *extensionMan
 
 QObject *ComboBoxTaskMenuFactory::createExtension(QObject *object, const QString &iid, QObject *parent) const
 {
-    if (QComboBox *button = qobject_cast<QComboBox*>(object)) {
+    QComboBox *button = qobject_cast<QComboBox*>(object);
+    if (button && !qobject_cast<QFontComboBox*>(object)) {
         if (iid == Q_TYPEID(QDesignerTaskMenuExtension)) {
             return new ComboBoxTaskMenu(button, parent);
         }
