@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,26 +9,48 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
+/*
+TRANSLATOR qdesigner_internal::TreeWidgetEditor
+*/
+
 #include "treewidgeteditor.h"
 #include <findicondialog_p.h>
 #include <iconloader_p.h>
 #include <qdesigner_command_p.h>
 
-#include <QtDesigner/QtDesigner>
+#include <QtDesigner/QDesignerFormWindowInterface>
+#include <QtDesigner/QDesignerFormEditorInterface>
+#include <QtDesigner/QDesignerIconCacheInterface>
 #include <QtCore/QDir>
 #include <QtCore/QQueue>
+#include <QHeaderView>
 
 using namespace qdesigner_internal;
 
@@ -36,6 +58,7 @@ TreeWidgetEditor::TreeWidgetEditor(QDesignerFormWindowInterface *form, QWidget *
     : QDialog(parent), m_updating(false)
 {
     ui.setupUi(this);
+    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     m_form = form;
     QIcon resetIcon = createIconSet(QString::fromUtf8("editdelete.png"));
     ui.deletePixmapItemButton->setIcon(resetIcon);
@@ -59,6 +82,8 @@ TreeWidgetEditor::TreeWidgetEditor(QDesignerFormWindowInterface *form, QWidget *
     ui.moveItemDownButton->setIcon(downIcon);
     ui.moveItemRightButton->setIcon(forwardIcon);
     ui.moveItemLeftButton->setIcon(backIcon);
+
+    ui.treeWidget->header()->setMovable(false);
 }
 
 TreeWidgetEditor::~TreeWidgetEditor()
@@ -159,9 +184,10 @@ void TreeWidgetEditor::on_newItemButton_clicked()
     newItem->setText(0, tr("New Item"));
     newItem->setFlags(newItem->flags() | Qt::ItemIsEditable);
 
-    ui.treeWidget->setCurrentItem(newItem);
+    ui.treeWidget->setCurrentItem(newItem, ui.listWidget->currentRow());
     m_updating = false;
     updateEditor();
+    ui.treeWidget->editItem(newItem, ui.listWidget->currentRow());
 }
 
 void TreeWidgetEditor::on_newSubItemButton_clicked()
@@ -176,10 +202,10 @@ void TreeWidgetEditor::on_newSubItemButton_clicked()
     newItem->setText(0, tr("New Sub Item"));
     newItem->setFlags(newItem->flags() | Qt::ItemIsEditable);
 
-    ui.treeWidget->setCurrentItem(newItem);
-
+    ui.treeWidget->setCurrentItem(newItem, ui.listWidget->currentRow());
     m_updating = false;
     updateEditor();
+    ui.treeWidget->editItem(newItem, ui.listWidget->currentRow());
 }
 
 void TreeWidgetEditor::on_deleteItemButton_clicked()
@@ -210,10 +236,11 @@ void TreeWidgetEditor::on_deleteItemButton_clicked()
         if (idx >= 0)
             nextCurrent = ui.treeWidget->topLevelItem(idx);
     }
+    closeEditors();
     delete curItem;
 
     if (nextCurrent)
-        ui.treeWidget->setCurrentItem(nextCurrent);
+        ui.treeWidget->setCurrentItem(nextCurrent, ui.listWidget->currentRow());
 
     m_updating = false;
     updateEditor();
@@ -244,7 +271,7 @@ void TreeWidgetEditor::on_moveItemUpButton_clicked()
         takenItem = ui.treeWidget->takeTopLevelItem(idx);
         ui.treeWidget->insertTopLevelItem(idx - 1, takenItem);
     }
-    ui.treeWidget->setCurrentItem(takenItem);
+    ui.treeWidget->setCurrentItem(takenItem, ui.listWidget->currentRow());
 
     m_updating = false;
     updateEditor();
@@ -278,7 +305,7 @@ void TreeWidgetEditor::on_moveItemDownButton_clicked()
         takenItem = ui.treeWidget->takeTopLevelItem(idx);
         ui.treeWidget->insertTopLevelItem(idx + 1, takenItem);
     }
-    ui.treeWidget->setCurrentItem(takenItem);
+    ui.treeWidget->setCurrentItem(takenItem, ui.listWidget->currentRow());
 
     m_updating = false;
     updateEditor();
@@ -304,7 +331,7 @@ void TreeWidgetEditor::on_moveItemLeftButton_clicked()
         int idx = ui.treeWidget->indexOfTopLevelItem(parentItem);
         ui.treeWidget->insertTopLevelItem(idx, takenItem);
     }
-    ui.treeWidget->setCurrentItem(takenItem);
+    ui.treeWidget->setCurrentItem(takenItem, ui.listWidget->currentRow());
 
     m_updating = false;
     updateEditor();
@@ -339,7 +366,7 @@ void TreeWidgetEditor::on_moveItemRightButton_clicked()
         takenItem = ui.treeWidget->takeTopLevelItem(idx);
         parentItem->insertChild(0, takenItem);
     }
-    ui.treeWidget->setCurrentItem(takenItem);
+    ui.treeWidget->setCurrentItem(takenItem, ui.listWidget->currentRow());
 
     m_updating = false;
     updateEditor();
@@ -355,8 +382,6 @@ void TreeWidgetEditor::on_treeWidget_currentItemChanged(QTreeWidgetItem *,
     ui.listWidget->setCurrentRow(idx.column());
     m_updating = false;
     updateEditor();
-    ui.itemTextLineEdit->selectAll();
-    ui.itemTextLineEdit->setFocus();
 }
 
 void TreeWidgetEditor::on_treeWidget_itemChanged(QTreeWidgetItem *)
@@ -370,20 +395,29 @@ void TreeWidgetEditor::on_listWidget_currentRowChanged(int)
 {
     if (m_updating)
         return;
+    m_updating = true;
+    QListWidgetItem *currentColumn = ui.listWidget->currentItem();
+    QTreeWidgetItem *currentItem = ui.treeWidget->currentItem();
+    if (currentItem && currentColumn) {
+        int col = ui.listWidget->currentRow();
+        ui.treeWidget->setCurrentItem(currentItem, col);
+    }
+    m_updating = false;
     updateEditor();
 }
 
-void TreeWidgetEditor::on_listWidget_itemChanged(QListWidgetItem *item)
+void TreeWidgetEditor::on_listWidget_itemChanged(QListWidgetItem *)
 {
     if (m_updating)
         return;
-    QString str = item->text();
-    int row = ui.listWidget->row(item);
-    ui.treeWidget->headerItem()->setText(row, str);
+    updateEditor();
 }
 
 void TreeWidgetEditor::updateEditor()
 {
+    for (int i = 0; i < ui.listWidget->count(); i++)
+        ui.treeWidget->headerItem()->setText(i, ui.listWidget->item(i)->text());
+
     QTreeWidgetItem *current = ui.treeWidget->currentItem();
     QListWidgetItem *currentColumn = ui.listWidget->currentItem();
 
@@ -515,9 +549,7 @@ void TreeWidgetEditor::on_previewPixmapItemButton_clicked()
     QString qrc_path;
 
     QIcon icon = curItem->icon(currentRow);
-    if (icon.isNull()) {
-        file_path = m_form->absoluteDir().absolutePath();
-    } else {
+    if (!icon.isNull()) {
         file_path = m_form->core()->iconCache()->iconToFilePath(icon);
         qrc_path = m_form->core()->iconCache()->iconToQrcPath(icon);
     }
@@ -575,7 +607,6 @@ void TreeWidgetEditor::moveColumnsRight(int fromColumn, int toColumn)
     if (fromColumn >= toColumn)
         return;
 
-
     QTreeWidgetItem *headerItem = ui.treeWidget->headerItem();
     QString text = headerItem->text(fromColumn);
     QIcon icon = headerItem->icon(fromColumn);
@@ -628,7 +659,7 @@ void TreeWidgetEditor::on_newColumnButton_clicked()
     ui.listWidget->insertItem(idx, item);
     ui.listWidget->setCurrentItem(item);
 
-    ui.treeWidget->setCurrentItem(ui.treeWidget->currentItem());
+    ui.treeWidget->setCurrentItem(ui.treeWidget->currentItem(), ui.listWidget->currentRow());
 
     m_updating = false;
     updateEditor();
@@ -659,6 +690,7 @@ void TreeWidgetEditor::on_deleteColumnButton_clicked()
     moveColumnsRight(idx, columnCount - 1);
     ui.treeWidget->setColumnCount(columnCount - 1);
 
+    closeEditors();
     delete currentColumn;
     if (idx == columnCount - 1)
         idx--;
@@ -687,6 +719,8 @@ void TreeWidgetEditor::on_moveColumnUpButton_clicked()
     ui.listWidget->insertItem(idx - 1, currentColumn);
     ui.listWidget->setCurrentItem(currentColumn);
 
+    ui.treeWidget->setCurrentItem(ui.treeWidget->currentItem(), ui.listWidget->currentRow());
+
     m_updating = false;
     updateEditor();
 }
@@ -710,6 +744,8 @@ void TreeWidgetEditor::on_moveColumnDownButton_clicked()
     ui.listWidget->insertItem(idx + 1, currentColumn);
     ui.listWidget->setCurrentItem(currentColumn);
 
+    ui.treeWidget->setCurrentItem(ui.treeWidget->currentItem(), ui.listWidget->currentRow());
+
     m_updating = false;
     updateEditor();
 }
@@ -762,4 +798,11 @@ void TreeWidgetEditor::on_deletePixmapColumnButton_clicked()
 }
 
 
-
+void TreeWidgetEditor::closeEditors()
+{
+    if (QTreeWidgetItem *cur = ui.treeWidget->currentItem() ) {
+        const int numCols = cur->columnCount ();
+        for (int i = 0; i < numCols; i++) 
+            ui.treeWidget->closePersistentEditor (cur, i);
+    }
+}
