@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,21 +39,11 @@
 #ifndef WIDGETBOX_H
 #define WIDGETBOX_H
 
-#include <QtXml/QDomDocument>
-
-#include <QtDesigner/QtDesigner>
-
 #include "widgetbox_global.h"
+#include <qdesigner_widgetbox_p.h>
 
 class QDesignerFormEditorInterface;
 class QDesignerFormWindowInterface;
-class QStackedLayout;
-class QDomElement;
-class QEvent;
-class QMenu;
-class QAction;
-class QActionGroup;
-class DomWidget;
 
 namespace qdesigner_internal {
 
@@ -46,7 +51,7 @@ class WidgetBoxTreeView;
 class WidgetCollectionModel;
 class Scratchpad;
 
-class QT_WIDGETBOX_EXPORT WidgetBox : public QDesignerWidgetBoxInterface
+class QT_WIDGETBOX_EXPORT WidgetBox : public QDesignerWidgetBox
 {
     Q_OBJECT
 public:
@@ -72,8 +77,15 @@ public:
     virtual bool load();
     virtual bool save();
 
+    virtual bool loadContents(const QString &contents);
+
+protected:
+    virtual void dragEnterEvent (QDragEnterEvent * event);
+    virtual void dragMoveEvent(QDragMoveEvent * event);
+    virtual void dropEvent (QDropEvent * event);
+
 private slots:
-    void handleMousePress(const QString &xml, const QPoint &global_mouse_pos);
+    void handleMousePress(const QString &name, const QString &xml, bool custom, const QPoint &global_mouse_pos);
 
 private:
     QDesignerFormEditorInterface *m_core;
