@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 2005-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,26 +9,41 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/
 
+#include "itemdialog.h"
+#include "item.h"
+
 #include <QtGui/QGridLayout>
 #include <QtGui/QLabel>
 #include <QtGui/QPixmap>
 #include <QtGui/QPushButton>
-#include <QtCore/QList>
-
-#include "itemdialog.h"
-#include "item.h"
+#include <QtGui/QDialogButtonBox>
 
 QList<ItemDialog *> ItemDialog::openDialogs;
 
@@ -36,6 +51,7 @@ ItemDialog::ItemDialog(QWidget *parent, const Item *item)
     : QDialog(parent)
 {
     setAttribute(Qt::WA_DeleteOnClose);
+    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     QGridLayout *layout = new QGridLayout(this);
     QLabel *lblPix = new QLabel();
     const BusinessCard *bcard = static_cast<const BusinessCard *>(item);
@@ -55,14 +71,14 @@ ItemDialog::ItemDialog(QWidget *parent, const Item *item)
     descFont.setItalic(true);
     descFont.setPointSize(descFont.pointSize() - 2);
     lblDesc->setFont(descFont);
-    QPushButton *btn = new QPushButton(tr("Close"));
-    btn->setDefault(true);
-    connect(btn, SIGNAL(clicked()), this, SLOT(close()));
+
+    QDialogButtonBox *btnBox = new QDialogButtonBox(QDialogButtonBox::Close);
+    connect(btnBox, SIGNAL(rejected()), this, SLOT(close()));
 
     layout->addWidget(lblPix, 0, 0, 2, 2);
     layout->addWidget(lblName, 0, 2, 1, 1);
     layout->addWidget(lblDesc, 1, 2, 1, 1);
-    layout->addWidget(btn, 2, 3, 1, 1);
+    layout->addWidget(btnBox, 2, 3, 1, 1);
     openDialogs.append(this);
 }
 
