@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,18 +39,20 @@
 #ifndef QDESIGNER_ACTIONS_H
 #define QDESIGNER_ACTIONS_H
 
-#include <QtCore/QPointer>
 #include <QtCore/QObject>
-#include <QtGui/QAction>
+#include <QtCore/QPointer>
 
-class QDesignerMainWindow;
 class QDesignerWorkbench;
 
+class QDir;
+class QTimer;
 class QAction;
 class QActionGroup;
 class QDesignerFormEditorInterface;
 class QDesignerFormWindowInterface;
 class QAssistantClient;
+class QRect;
+class QWidget;
 
 class QDesignerActions: public QObject
 {
@@ -59,74 +76,27 @@ public:
     QActionGroup *toolActions() const;
     QActionGroup *helpActions() const;
     QActionGroup *uiMode() const;
+    QAction *preferencesAction() const;
     QActionGroup *styleActions() const;
 
-//
-// file actions
-//
-    QAction *newFormAction() const;
-    QAction *openFormAction() const;
-    QAction *saveFormAction() const;
-    QAction *saveFormAsAction() const;
-    QAction *saveFormAsTemplateAction() const;
-    QAction *closeFormAction() const;
-    QAction *quitAction() const;
-
-//
-// edit actions
-//
-    QAction *undoAction() const;
-    QAction *redoAction() const;
-    QAction *cutAction() const;
-    QAction *copyAction() const;
-    QAction *pasteAction() const;
-    QAction *deleteAction() const;
-    QAction *selectAllAction() const;
-    QAction *sendToBackAction() const;
-    QAction *bringToFrontAction() const;
-
-//
-// edit mode actions
-//
-    QAction *editWidgets() const;
-    QAction *editConnections() const;
-    QAction *editTabOrders() const;
-    QAction *editBuddies() const;
-
-//
-// form actions
-//
-    QAction *layoutHorizontallyAction() const;
-    QAction *layoutVerticallyAction() const;
-    QAction *layoutHorizontallyInSplitterAction() const;
-    QAction *layoutVerticallyInSplitterAction() const;
-    QAction *layoutGridAction() const;
-    QAction *breakLayoutAction() const;
-    QAction *adjustSizeAction() const;
-    QAction *previewFormAction() const;
-    QAction *formSettings() const;
-
-//
-// window actions
-//
+    // window actions
     QAction *minimizeAction() const;
-    QAction *bringAllToFront() const;
+    // edit mode actions
+    QAction *editWidgets() const;
+    // form actions
+    QAction *previewFormAction() const;
 
-//
-// help actions
-//
-    QAction *mainHelpAction() const;
-    QAction *widgetHelpAction() const;
-    QAction *whatsNewAction() const;
-    QAction *aboutQtAction() const;
-    QAction *aboutDesignerAction() const;
+    void setBringAllToFrontVisible(bool visible);
+    void setWindowListSeparatorVisible(bool visible);
 
     virtual bool eventFilter(QObject *watched, QEvent *event);
 
+    bool openForm(QWidget *parent);
+
 public slots:
     void activeFormWindowChanged(QDesignerFormWindowInterface *formWindow);
     void createForm();
-    bool openForm();
+    void slotOpenForm();
 
 signals:
     void useBigIcons(bool);
@@ -134,6 +104,7 @@ signals:
 private slots:
     void saveForm();
     void saveFormAs();
+    void saveAllForms();
     void saveFormAsTemplate();
     void previewForm(QAction *action = 0);
     void previewFormLater(QAction *action = 0);
@@ -143,15 +114,15 @@ private slots:
     void openRecentForm();
     void clearRecentFiles();
     void closeForm();
-    void minimizeForm();
-    void bringAllToFront();
     void showDesignerHelp();
     void showWhatsNew();
     void aboutPlugins();
     void aboutDesigner();
-    void updateUIMode(QAction *act);
     void showWidgetSpecificHelp();
     void showFormSettings();
+    void backupForms();
+    void showNewFormDialog(const QString &fileName);
+    void showPreferencesDialog();
 
 private:
     bool saveFormAs(QDesignerFormWindowInterface *fw);
@@ -160,9 +131,13 @@ private:
     void addRecentFile(const QString &fileName);
     void showHelp(const QString &help);
     void updateCloseAction();
-    void closePreview();
+    bool closePreview();
+    QRect fixDialogRect(const QRect &rect) const;
+    QString fixResourceFileBackupPath(QDesignerFormWindowInterface *fwi, const QDir& backupDir);
+    void showStatusBarMessage(const QString &message) const;
+    QActionGroup *createHelpActions();
+    bool ensureBackupDirectories();
 
-private:
     enum { MaxRecentFiles = 10 };
     QDesignerWorkbench *m_workbench;
     QDesignerFormEditorInterface *m_core;
@@ -170,6 +145,12 @@ private:
     QString m_openDirectory;
     QString m_saveDirectory;
 
+
+    QString m_backupPath;
+    QString m_backupTmpPath;
+
+    QTimer* m_backupTimer;
+
     QActionGroup *m_fileActions;
     QActionGroup *m_recentFilesActions;
     QActionGroup *m_editActions;
@@ -177,7 +158,6 @@ private:
     QActionGroup *m_windowActions;
     QActionGroup *m_toolActions;
     QActionGroup *m_helpActions;
-    QActionGroup *m_uiMode;
     QActionGroup *m_styleActions;
 
     QAction *m_editWidgetsAction;
@@ -186,43 +166,22 @@ private:
     QAction *m_openFormAction;
     QAction *m_saveFormAction;
     QAction *m_saveFormAsAction;
+    QAction *m_saveAllFormsAction;
     QAction *m_saveFormAsTemplateAction;
     QAction *m_closeFormAction;
 
     QAction *m_quitAction;
-    QAction *m_undoAction;
-    QAction *m_redoAction;
-    QAction *m_cutAction;
-    QAction *m_copyAction;
-    QAction *m_pasteAction;
-    QAction *m_deleteAction;
-    QAction *m_sendToBackAction;
-    QAction *m_bringToFrontAction;
-    QAction *m_selectAllAction;
 
-
-    QAction *m_layoutHorizontallyAction;
-    QAction *m_layoutVerticallyAction;
-    QAction *m_layoutHorizontallyInSplitterAction;
-    QAction *m_layoutVerticallyInSplitterAction;
-    QAction *m_layoutGridAction;
-    QAction *m_breakLayoutAction;
-    QAction *m_adjustSizeAction;
     QAction *m_previewFormAction;
     QAction *m_formSettings;
 
     QAction *m_minimizeAction;
+    QAction *m_bringAllToFrontSeparator;
     QAction *m_bringAllToFrontAction;
+    QAction *m_windowListSeparatorAction;
+
+    QAction *m_preferencesAction;
 
-    QAction *m_mainHelpAction;
-    QAction *m_widgetHelp;
-    QAction *m_whatsNewAction;
-    QAction *m_aboutQtAction;
-    QAction *m_aboutPluginsAction;
-    QAction *m_aboutDesignerAction;
-    QAction *m_sdiAction;
-    QAction *m_dockedMdiAction;
-    
     QPointer<QWidget> m_previewWidget;
 };
 
