@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,30 +40,39 @@
 #define QDESIGNER_FORMWINDOW_H
 
 #include <QtCore/QPointer>
-#include <QtGui/QMainWindow>
+#include <QtGui/QWidget>
 
 class QDesignerWorkbench;
 class QDesignerFormWindowInterface;
 
-class QDesignerFormWindow: public QMainWindow
+class QDesignerFormWindow: public QWidget
 {
     Q_OBJECT
 public:
     QDesignerFormWindow(QDesignerFormWindowInterface *formWindow, QDesignerWorkbench *workbench,
                         QWidget *parent = 0, Qt::WindowFlags flags = 0);
 
+    void firstShow();
+
     virtual ~QDesignerFormWindow();
 
     QAction *action() const;
     QDesignerWorkbench *workbench() const;
     QDesignerFormWindowInterface *editor() const;
 
-    virtual QRect geometryHint() const;
+    QRect geometryHint() const;
 
 public slots:
-    void updateWindowTitle(const QString &fileName);
     void updateChanged();
+
+private slots:
+    void updateWindowTitle(const QString &fileName);
     void geometryChanged();
+    void widgetActivated(QWidget *widget);
+
+signals:
+    void minimizationStateChanged(QDesignerFormWindowInterface *formWindow, bool minimized);
+    void triggerAction();
 
 protected:
     virtual void changeEvent(QEvent *e);
@@ -56,10 +80,12 @@ protected:
     virtual void resizeEvent(QResizeEvent* rev);
 
 private:
-    QDesignerFormWindowInterface *m_editor;
+    int getNumberOfUntitledWindows() const;
+    QPointer<QDesignerFormWindowInterface> m_editor;
     QPointer<QDesignerWorkbench> m_workbench;
     QAction *m_action;
-    bool initialized;
+    bool m_initialized;
+    bool m_windowTitleInitialized;
 };
 
 #endif // QDESIGNER_FORMWINDOW_H
