@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -24,26 +39,29 @@
 #ifndef QDESIGNER_WORKBENCH_H
 #define QDESIGNER_WORKBENCH_H
 
+#include "designer_enums.h"
+
 #include <QtCore/QObject>
-#include <QtCore/QPointer>
 #include <QtCore/QHash>
+#include <QtCore/QSet>
 #include <QtCore/QList>
 #include <QtCore/QRect>
 
-class QDesigner;
 class QDesignerActions;
 class QDesignerToolWindow;
 class QDesignerFormWindow;
+struct Preferences;
 
 class QAction;
 class QActionGroup;
 class QDockWidget;
 class QMenu;
 class QMenuBar;
-class QVariant;
 class QToolBar;
-class QWorkspace;
+class QMdiArea;
+class QMdiSubWindow;
 class QCloseEvent;
+class QFont;
 
 class QDesignerFormEditorInterface;
 class QDesignerFormWindowInterface;
@@ -56,13 +74,6 @@ class QDesignerIntegration;
 class QDesignerWorkbench: public QObject
 {
     Q_OBJECT
-public:
-    enum UIMode
-    {
-        NeutralMode,
-        TopLevelMode,
-        DockedMode
-    };
 
 public:
     QDesignerWorkbench();
@@ -77,6 +88,11 @@ public:
 
     QDesignerFormWindow *createFormWindow();
 
+    QDesignerFormWindow *openForm(const QString &fileName, QString *errorMessage);
+    QDesignerFormWindow *openTemplate(const QString &templateFileName,
+                                      const QString &editorFileName,
+                                      QString *errorMessage);
+
     int toolWindowCount() const;
     QDesignerToolWindow *toolWindow(int index) const;
 
@@ -91,12 +107,15 @@ public:
     int marginHint() const;
 
     void saveSettings() const;
+    void applyPreferences(const Preferences&);
 
     bool readInForm(const QString &fileName) const;
     bool writeOutForm(QDesignerFormWindowInterface *formWindow, const QString &fileName) const;
     bool saveForm(QDesignerFormWindowInterface *fw);
     bool handleClose();
     void closeAllToolWindows();
+    bool readInBackup();
+    void updateBackup(QDesignerFormWindowInterface* fwi);
 
 signals:
     void modeChanged(UIMode mode);
@@ -108,6 +127,8 @@ public slots:
     void removeToolWindow(QDesignerToolWindow *toolWindow);
     void removeFormWindow(QDesignerFormWindow *formWindow);
     void setUIMode(UIMode mode);
+    void bringAllToFront();
+    void toggleFormMinimizationState();
 
 // ### private slots:
     void switchToNeutralMode();
@@ -118,11 +139,12 @@ public slots:
 
 private slots:
     void initialize();
-    void activateWorkspaceChildWindow(QWidget *widget);
+    void activateMdiAreaChildWindow(QMdiSubWindow*);
     void updateWindowMenu(QDesignerFormWindowInterface *fw);
     void formWindowActionTriggered(QAction *a);
     void showToolBars();
-    void adjustFormPositions();
+    void adjustMDIFormPositions();
+    void minimizationStateChanged(QDesignerFormWindowInterface *formWindow, bool minimized);
 
 private:
     QWidget *magicalParent() const;
@@ -130,9 +152,17 @@ private:
     QDockWidget *magicalDockWidget(QWidget *widget) const;
 
     QDesignerFormWindowManagerInterface *formWindowManager() const;
-    void changeBringToFrontVisiblity(bool visible);
+
+    bool eventFilter(QObject *object, QEvent *event);
 
 private:
+    QDesignerFormWindow *loadForm(const QString &fileName, bool *uic3Converted, QString *errorMessage);
+    void resizeForm(QDesignerFormWindow *fw,  const QWidget *mainContainer) const;
+    void saveGeometries();
+    bool isFormWindowMinimized(const QDesignerFormWindow *fw);
+    void setFormWindowMinimized(QDesignerFormWindow *fw, bool minimized);
+    void setDesignerUIFont(const QFont &);
+
     QDesignerFormEditorInterface *m_core;
     qdesigner_internal::QDesignerIntegration *m_integration;
 
@@ -151,22 +181,40 @@ private:
     QToolBar *m_toolToolBar;
     QToolBar *m_formToolBar;
     QToolBar *m_editToolBar;
+    QToolBar *m_fileToolBar;
 
     UIMode m_mode;
 
     QList<QDesignerToolWindow*> m_toolWindows;
     QList<QDesignerFormWindow*> m_formWindows;
 
-    QWorkspace *m_workspace;
-    QHash<QWidget*, bool> m_visibilities;
-    QHash<QWidget*, QRect> m_geometries;
+    QMdiArea *m_mdiArea;
 
-    class ToolWindowExtra {};
-    class FormWindowExtra {};
+    // Helper class to remember the position of a window while switching user interface modes.
+    class Position {
+    public:
+        Position(const QDockWidget *dockWidget);
+        Position(const QMdiSubWindow *mdiSubWindow, const QPoint &mdiAreaOffset);
+        Position(const QWidget *topLevelWindow, const QPoint &desktopTopLeft);
 
-    QHash<QDesignerToolWindow*, ToolWindowExtra> m_toolWindowExtras;
-    QHash<QDesignerFormWindow*, FormWindowExtra> m_formWindowExtras;
-    bool m_initializing;
+        void applyTo(QMdiSubWindow *mdiSubWindow, const QPoint &mdiAreaOffset) const;
+        void applyTo(QWidget *topLevelWindow, const QPoint &desktopTopLeft) const;
+        void applyTo(QDockWidget *dockWidget) const;
+
+        QPoint position() const { return m_position; }
+    private:
+        bool m_minimized;
+        // Position referring to top-left corner (desktop in top-level mode or main window in MDI mode)
+        QPoint m_position;
+    };
+    typedef  QHash<QWidget*, Position> PositionMap;
+    PositionMap m_Positions;
+
+    QSet<QDesignerToolWindow*> m_toolWindowExtras;
+    QSet<QDesignerFormWindow*> m_formWindowExtras;
+
+    enum State { StateInitializing, StateUp, StateClosing };
+    State m_state;
 };
 
 #endif // QDESIGNER_WORKBENCH_H
