@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -31,8 +46,18 @@ TRANSLATOR qdesigner_internal::ActionEditor
 #include "newactiondialog_p.h"
 #include "qdesigner_menu_p.h"
 #include "qdesigner_command_p.h"
+#include "qdesigner_propertycommand_p.h"
+#include "resourcemimedata_p.h"
+#include "qdesigner_objectinspector_p.h"
+#include "qdesigner_utils_p.h"
 
-#include <QtDesigner/QtDesigner>
+#include <QtDesigner/QDesignerFormEditorInterface>
+#include <QtDesigner/QDesignerPropertyEditorInterface>
+#include <QtDesigner/QDesignerPropertySheetExtension>
+#include <QtDesigner/QExtensionManager>
+#include <QtDesigner/QDesignerMetaDataBaseInterface>
+#include <QtDesigner/QDesignerFormWindowInterface>
+#include <QtDesigner/QDesignerIconCacheInterface>
 
 #include <QtGui/QMenu>
 #include <QtGui/QMessageBox>
@@ -55,13 +80,15 @@ Q_DECLARE_METATYPE(QAction*)
 Q_DECLARE_METATYPE(QListWidgetItem*)
 
 namespace qdesigner_internal {
-
+//-------- ActionFilterWidget
 class ActionFilterWidget: public QWidget
 {
     Q_OBJECT
 public:
     ActionFilterWidget(ActionEditor *actionEditor, QToolBar *parent)
         : QWidget(parent),
+          m_button(new QPushButton(this)),
+          m_editor(new QLineEdit(this)),
           m_actionEditor(actionEditor)
     {
         QHBoxLayout *l = new QHBoxLayout(this);
@@ -74,11 +101,10 @@ public:
         label->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
         l->addWidget(label);
 
-        m_editor = new QLineEdit(this);
         l->addWidget(m_editor);
 
         connect(m_editor, SIGNAL(textChanged(QString)), actionEditor, SLOT(setFilter(QString)));
-        m_button = new QPushButton(this);
+
         m_button->setIcon(createIconSet(QLatin1String("resetproperty.png")));
         m_button->setIconSize(QSize(16, 16));
         m_button->setFlat(true);
@@ -88,7 +114,7 @@ public:
     }
 
 private slots:
-    void checkButton(QString text)
+    void checkButton(const QString &text)
     {
         m_button->setEnabled(!text.isEmpty());
     }
@@ -99,6 +125,7 @@ private:
     ActionEditor *m_actionEditor;
 };
 
+//--------  ActionGroupDelegate
 class ActionGroupDelegate: public QItemDelegate
 {
 public:
@@ -116,6 +143,7 @@ public:
     virtual void drawFocus(QPainter * /*painter*/, const QStyleOptionViewItem &/*option*/, const QRect &/*rect*/) const {}
 };
 
+//--------  ActionEditor
 ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent, Qt::WindowFlags flags)
     : QDesignerActionEditorInterface(parent, flags),
       m_core(core)
@@ -127,18 +155,17 @@ ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent,
     l->setSpacing(0);
 
     QToolBar *toolbar = new QToolBar(this);
-//    toolbar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon); // ### style
     toolbar->setIconSize(QSize(24, 24));
     toolbar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
     l->addWidget(toolbar);
 
     m_actionNew = toolbar->addAction(tr("New..."));
-    m_actionNew->setIcon(createIconSet("filenew.png"));
+    m_actionNew->setIcon(createIconSet(QLatin1String("filenew.png")));
     m_actionNew->setEnabled(false);
     connect(m_actionNew, SIGNAL(triggered()), this, SLOT(slotNewAction()));
 
     m_actionDelete = toolbar->addAction(tr("Delete"));
-    m_actionDelete->setIcon(createIconSet("editdelete.png"));
+    m_actionDelete->setIcon(createIconSet(QLatin1String("editdelete.png")));
     m_actionDelete->setEnabled(false);
 
     m_filterWidget = new ActionFilterWidget(this, toolbar);
@@ -147,7 +174,7 @@ ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent,
 
     connect(m_actionDelete, SIGNAL(triggered()), this, SLOT(slotDeleteAction()));
 
-    splitter = new QSplitter(Qt::Horizontal, this);
+    QSplitter *splitter = new QSplitter(Qt::Horizontal, this);
     splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
     l->addWidget(splitter);
@@ -165,6 +192,8 @@ ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent,
 #endif
 
     m_actionRepository = new ActionRepository(splitter);
+    connect(m_actionRepository, SIGNAL(resourceImageDropped(ResourceMimeData,QAction*)),
+            this, SLOT(resourceImageDropped(ResourceMimeData,QAction*)));
     splitter->addWidget(m_actionRepository);
 
     connect(m_actionRepository, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
@@ -175,7 +204,7 @@ ActionEditor::ActionEditor(QDesignerFormEditorInterface *core, QWidget *parent,
     connect(m_actionRepository, SIGNAL(contextMenuRequested(QContextMenuEvent*, QListWidgetItem*)),
             this, SIGNAL(contextMenuRequested(QContextMenuEvent*, QListWidgetItem*)));
     connect(this, SIGNAL(itemActivated(QListWidgetItem*)),
-            this, SLOT(editAction(QListWidgetItem*)));    
+            this, SLOT(editAction(QListWidgetItem*)));
 }
 
 ActionEditor::~ActionEditor()
@@ -207,7 +236,7 @@ void ActionEditor::setFormWindow(QDesignerFormWindowInterface *formWindow)
         return;
 
     if (m_formWindow != 0) {
-        QList<QAction*> actionList = qFindChildren<QAction*>(m_formWindow->mainContainer());
+        const QList<QAction*> actionList = qFindChildren<QAction*>(m_formWindow->mainContainer());
         foreach (QAction *action, actionList)
             disconnect(action, SIGNAL(changed()), this, SLOT(slotActionChanged()));
     }
@@ -226,7 +255,7 @@ void ActionEditor::setFormWindow(QDesignerFormWindowInterface *formWindow)
     m_actionNew->setEnabled(true);
     m_filterWidget->setEnabled(true);
 
-    QList<QAction*> actionList = qFindChildren<QAction*>(formWindow->mainContainer());
+    const QList<QAction*> actionList = qFindChildren<QAction*>(formWindow->mainContainer());
     foreach (QAction *action, actionList) {
         if (!core()->metaDataBase()->item(action)
             || action->isSeparator()
@@ -242,17 +271,27 @@ void ActionEditor::setFormWindow(QDesignerFormWindowInterface *formWindow)
     setFilter(m_filter);
 }
 
-QString fixActionText(QString text)
+static QIcon fixActionIcon(QIcon icon)
 {
-    return text.replace(QLatin1String("&"), QString());
+    if (icon.isNull())
+        return emptyIcon();
+    return icon;
 }
 
-QIcon fixActionIcon(QIcon icon)
+// Set up list widget item, icon, tooltip
+static void setListWidgetItem(const QAction *action,  QListWidgetItem *item)
 {
-    static const QIcon empty_icon(":/trolltech/formeditor/images/emptyicon.png");
-    if (icon.isNull())
-        return empty_icon;
-    return icon;
+    item->setText(action->objectName());
+    item->setIcon(fixActionIcon(action->icon()));
+
+    QString tooltip = action->objectName();
+    const QString text = action->text();
+    if (!text.isEmpty()) {
+        tooltip += QLatin1Char('\n');
+        tooltip += text;
+    }
+    item->setToolTip(tooltip);
+    item->setWhatsThis(tooltip);
 }
 
 QListWidgetItem *ActionEditor::createListWidgetItem(QAction *action)
@@ -261,54 +300,52 @@ QListWidgetItem *ActionEditor::createListWidgetItem(QAction *action)
         return 0;
 
     QListWidgetItem *item = new QListWidgetItem(m_actionRepository);
-    QSize s = m_actionRepository->iconSize();
-    item->setSizeHint(QSize(s.width()*3, s.height()*2));
-    item->setText(fixActionText(action->objectName()));
-    item->setIcon(fixActionIcon(action->icon()));
+    const QSize s = m_actionRepository->iconSize();
+    item->setSizeHint(QSize(s.width() * 3, s.height() * 2));
+
+    setListWidgetItem(action, item);
 
     QVariant itemData;
     qVariantSetValue(itemData, action);
     item->setData(ActionRepository::ActionRole, itemData);
-
-/*    QVariant actionData;
-    qVariantSetValue(actionData, item);
-    action->setData(actionData); */
-
+    item->setFlags(item->flags() | Qt::ItemIsDropEnabled);
     return item;
 }
 
-void ActionEditor::updatePropertyEditor(QAction *action)
-{
-    if (!action || !core()->propertyEditor())
-        return;
-
-    QObject *sel = action;
-    if (action->menu())
-        sel = action->menu();
-
-    core()->propertyEditor()->setObject(sel);
-}
-
 void ActionEditor::slotItemChanged(QListWidgetItem *item)
 {
-    if (core()->propertyEditor() == 0 || formWindow() == 0)
+    QDesignerFormWindowInterface *fw = formWindow();
+    if (!fw)
         return;
 
-    m_actionDelete->setEnabled(item != 0);
+    QAction *action = 0;
+    if (item)
+        action = itemToAction(item);
 
-    if (!item) {
-        core()->propertyEditor()->setObject(formWindow()->mainContainer());
+    m_actionDelete->setEnabled(action != 0);
+
+    if (!action) {
+        fw->clearSelection();
         return;
     }
 
-    if (QAction *action = itemToAction(item)) {
-        updatePropertyEditor(action);
+    QDesignerObjectInspector *oi = qobject_cast<QDesignerObjectInspector *>(core()->objectInspector());
+
+    if (action->associatedWidgets().empty()) {
+        // Special case: action not in object tree. Unselect all and set in property editor
+        fw->clearSelection(false);
+        if (oi)
+            oi->clearSelection();
+        core()->propertyEditor()->setObject(action);
+    } else {
+        if (oi)
+            oi->selectObject(action);
     }
 }
 
 QListWidgetItem *ActionEditor::actionToItem(QAction *action) const
 {
-    int cnt = m_actionRepository->count();
+    const int cnt = m_actionRepository->count();
     for (int i = 0; i < cnt; ++i) {
         QListWidgetItem *item = m_actionRepository->item(i);
         if (itemToAction(item) == action)
@@ -335,8 +372,7 @@ void ActionEditor::slotActionChanged()
         delete item;
     } else {
         // action text or icon changed, update item
-        item->setText(fixActionText(action->objectName()));
-        item->setIcon(fixActionIcon(action->icon()));
+        setListWidgetItem(action, item);
     }
 }
 
@@ -356,23 +392,24 @@ void ActionEditor::setFilter(const QString &f)
     m_actionRepository->filter(m_filter);
 }
 
+// Set changed state of icon property,  reset when icon is cleared
+static void refreshIconPropertyChanged(const QAction *action, QDesignerPropertySheetExtension *sheet)
+{
+    sheet->setChanged(sheet->indexOf(QLatin1String("icon")), !action->icon().isNull());
+}
+
 void ActionEditor::manageAction(QAction *action)
 {
     action->setParent(formWindow()->mainContainer());
     core()->metaDataBase()->add(action);
 
-    if (action->isSeparator() || action->menu() != 0) {
-/*        QVariant actionData;
-        qVariantSetValue(actionData, (QListWidgetItem*)0);
-        action->setData(actionData); */
+    if (action->isSeparator() || action->menu() != 0)
         return;
-    }
 
-    QDesignerPropertySheetExtension *sheet = 0;
-    sheet = qt_extension<QDesignerPropertySheetExtension*>(core()->extensionManager(), action);
-    sheet->setChanged(sheet->indexOf("objectName"), true);
-    sheet->setChanged(sheet->indexOf("text"), true);
-    sheet->setChanged(sheet->indexOf("icon"), !action->icon().isNull());
+    QDesignerPropertySheetExtension *sheet = qt_extension<QDesignerPropertySheetExtension*>(core()->extensionManager(), action);
+    sheet->setChanged(sheet->indexOf(QLatin1String("objectName")), true);
+    sheet->setChanged(sheet->indexOf(QLatin1String("text")), true);
+    refreshIconPropertyChanged(action, sheet);
 
     QListWidgetItem *item = createListWidgetItem(action);
     m_actionRepository->setCurrentItem(item);
@@ -391,10 +428,6 @@ void ActionEditor::unmanageAction(QAction *action)
     if (item == 0)
         return;
 
-/*    QVariant actionData;
-    qVariantSetValue(actionData, (QListWidgetItem*)0);
-    action->setData(actionData); */
-
     delete item;
 }
 
@@ -406,6 +439,7 @@ void ActionEditor::slotNewAction()
     if (dlg.exec() == QDialog::Accepted) {
         QAction *action = new QAction(formWindow());
         action->setObjectName(dlg.actionName());
+        formWindow()->ensureUniqueObjectName(action);
         action->setText(dlg.actionText());
         action->setIcon(dlg.actionIcon());
 
@@ -415,6 +449,37 @@ void ActionEditor::slotNewAction()
     }
 }
 
+static inline bool isSameIcon(const QIcon &i1, const QIcon &i2)
+{
+    return i1.serialNumber() == i2.serialNumber();
+}
+
+// return a FormWindow command to apply an icon
+static QDesignerFormWindowCommand *setIconPropertyCommand(const QIcon &newIcon, QAction *action, QDesignerFormWindowInterface *fw)
+{
+    const QString iconProperty = QLatin1String("icon");
+    if (newIcon.isNull()) {
+        ResetPropertyCommand *cmd = new ResetPropertyCommand(fw);
+        cmd->init(action, iconProperty);
+        return cmd;
+    }
+    SetPropertyCommand *cmd = new SetPropertyCommand(fw);
+    cmd->init(action, iconProperty, newIcon);
+    return cmd;
+}
+
+static QDesignerFormWindowCommand *setTextPropertyCommand(const QString &propertyName, const QString &text, QAction *action, QDesignerFormWindowInterface *fw)
+{
+    if (text.isEmpty()) {
+        ResetPropertyCommand *cmd = new ResetPropertyCommand(fw);
+        cmd->init(action, propertyName);
+        return cmd;
+    }
+    SetPropertyCommand *cmd = new SetPropertyCommand(fw);
+    cmd->init(action, propertyName, text);
+    return cmd;
+}
+
 void ActionEditor::editAction(QListWidgetItem *item)
 {
     if (!item)
@@ -431,27 +496,40 @@ void ActionEditor::editAction(QListWidgetItem *item)
     if (!dlg.exec())
         return;
 
-    formWindow()->beginCommand(QLatin1String("Edit action"));
-    if (action->objectName() != dlg.actionName()) {
-        SetPropertyCommand *cmd = new SetPropertyCommand(formWindow());
-        cmd->init(action, "objectName", dlg.actionName());
-        formWindow()->commandHistory()->push(cmd);
-    }
-    if (action->text() != dlg.actionText()) {
-        SetPropertyCommand *cmd = new SetPropertyCommand(formWindow());
-        cmd->init(action, "text", dlg.actionText());
-        formWindow()->commandHistory()->push(cmd);
-    }
-    if (action->icon().serialNumber() != dlg.actionIcon().serialNumber()) {
-        SetPropertyCommand *cmd = new SetPropertyCommand(formWindow());
-        cmd->init(action, "icon", dlg.actionIcon());
-        formWindow()->commandHistory()->push(cmd);
-    }
-    formWindow()->endCommand();
+    // figure out changes and whether to start a macro
+    enum ChangedMask { NameChanged = 1, TextChanged = 2 , IconChanged = 4 };
+    const QString newName = dlg.actionName();
+    const QString newText = dlg.actionText();
+    const QIcon newIcon = dlg.actionIcon();
+
+    int changedMask = 0;
+
+    if (newName != action->objectName())
+        changedMask |= NameChanged;
+    if (newText != action->text())
+        changedMask |= TextChanged;
+    if (!isSameIcon(newIcon, action->icon()))
+        changedMask |= IconChanged;
+
+    if (!changedMask)
+        return;
+
+    const bool severalChanges = (changedMask != NameChanged) && (changedMask != TextChanged) && (changedMask != IconChanged);
+    if (severalChanges)
+        formWindow()->beginCommand(QLatin1String("Edit action"));
+
+    if (changedMask & NameChanged)
+        formWindow()->commandHistory()->push(setTextPropertyCommand(QLatin1String("objectName"), newName, action, formWindow()));
+
+    if (changedMask & TextChanged)
+        formWindow()->commandHistory()->push(setTextPropertyCommand(QLatin1String("text"), newText, action, formWindow()));
+
+    if (changedMask & IconChanged)
+        formWindow()->commandHistory()->push(setIconPropertyCommand(newIcon, action, formWindow()));
+
+    if (severalChanges)
+        formWindow()->endCommand();
 
-    QDesignerPropertySheetExtension *sheet = 0;
-    sheet = qt_extension<QDesignerPropertySheetExtension*>(core()->extensionManager(), action);
-    sheet->setChanged(sheet->indexOf("icon"), !action->icon().isNull());
 }
 
 void ActionEditor::slotDeleteAction()
@@ -474,23 +552,43 @@ void ActionEditor::slotNotImplemented()
     QMessageBox::information(this, tr("Designer"), tr("Feature not implemented!"));
 }
 
-QString ActionEditor::actionTextToName(const QString &text)
+QString ActionEditor::actionTextToName(const QString &text, const QString &prefix)
 {
     QString name = text;
     if (name.isEmpty())
         return QString();
 
     name[0] = name.at(0).toUpper();
-    name.prepend(QLatin1String("action"));
-    name.replace(QRegExp(QString("[^a-zA-Z_0-9]")), QString("_"));
-    name.replace(QRegExp("__*"), QString("_"));
-    if (name.endsWith("_"))
+    name.prepend(prefix);
+    const QString underscore = QString(QLatin1Char('_'));
+    name.replace(QRegExp(QString(QLatin1String("[^a-zA-Z_0-9]"))), underscore);
+    name.replace(QRegExp(QLatin1String("__*")), underscore);
+    if (name.endsWith(underscore.at(0)))
         name.truncate(name.size() - 1);
 
     return name;
 }
 
+void  ActionEditor::resourceImageDropped(const ResourceMimeData &data, QAction *action)
+{
+    QDesignerFormWindowInterface *fw =  formWindow();
+    if (!fw)
+        return;
 
+    const QIcon icon = resourceMimeDataToIcon(data, fw);
+
+    if (icon.isNull() || isSameIcon(icon, action->icon()))
+        return;
+
+    fw->commandHistory()->push(setIconPropertyCommand(icon , action, fw));
+}
+
+void ActionEditor::mainContainerChanged()
+{
+    // Invalidate references to objects kept in model
+    if (sender() == formWindow())
+        setFormWindow(0);
+}
 } // namespace qdesigner_internal
 
 #include "actioneditor.moc"
