@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -43,11 +58,11 @@
 class QDesignerPropertyEditorInterface;
 class QListWidget;
 class QListWidgetItem;
-class QSplitter;
 
 namespace qdesigner_internal {
 
 class ActionRepository;
+class ResourceMimeData;
 
 class QDESIGNER_SHARED_EXPORT ActionEditor: public QDesignerActionEditorInterface
 {
@@ -69,13 +84,14 @@ public:
     virtual void manageAction(QAction *action);
     virtual void unmanageAction(QAction *action);
 
-    static QString actionTextToName(const QString &text);
+    static QString actionTextToName(const QString &text, const QString &prefix = QLatin1String("action"));
 
     QAction *itemToAction(QListWidgetItem *item) const;
     QListWidgetItem *actionToItem(QAction *action) const;
 
 public slots:
     void setFilter(const QString &filter);
+    void mainContainerChanged();
 
 private slots:
     void slotItemChanged(QListWidgetItem *item);
@@ -84,17 +100,17 @@ private slots:
     void slotNewAction();
     void slotDeleteAction();
     void slotNotImplemented();
-
+    void resourceImageDropped(const ResourceMimeData &data, QAction *action);
+    
 signals:
     void itemActivated(QListWidgetItem *item);
+    void contextMenuRequested(QContextMenuEvent*, QListWidgetItem *item);
 
 private:
     QListWidgetItem *createListWidgetItem(QAction *action);
-    void updatePropertyEditor(QAction *action);
 
     QDesignerFormEditorInterface *m_core;
     QPointer<QDesignerFormWindowInterface> m_formWindow;
-    QSplitter *splitter;
     QListWidget *m_actionGroups;
     ActionRepository *m_actionRepository;
     QAction *m_actionNew;
