@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -36,6 +51,8 @@
 #ifndef CONNECTIONEDIT_H
 #define CONNECTIONEDIT_H
 
+#include "shared_global_p.h"
+
 #include <QtCore/QMultiMap>
 #include <QtCore/QList>
 #include <QtCore/QPointer>
@@ -44,11 +61,11 @@
 #include <QtGui/QPixmap>
 #include <QtGui/QPolygonF>
 
-#include "qtundo_p.h"
-#include "shared_global_p.h"
+#include <QtGui/QUndoCommand>
+#
 
 class QDesignerFormWindowInterface;
-class QtUndoStack;
+class QUndoStack;
 
 namespace qdesigner_internal {
 
@@ -146,8 +163,9 @@ class QDESIGNER_SHARED_EXPORT ConnectionEdit : public QWidget, public CETypes
     Q_OBJECT
 public:
     ConnectionEdit(QWidget *parent, QDesignerFormWindowInterface *form);
+    virtual ~ConnectionEdit();
 
-    inline QWidget *background() const { return m_bg_widget; }
+    inline const QPointer<QWidget> &background() const { return m_bg_widget; }
 
     void setSelected(Connection *con, bool sel);
     bool selected(const Connection *con) const;
@@ -157,15 +175,15 @@ public:
     Connection *connection(int i) const { return m_con_list.at(i); }
     int indexOfConnection(Connection *con) const { return m_con_list.indexOf(con); }
 
-    void deleteSelected();
+    virtual void deleteSelected();
 
     virtual void setSource(Connection *con, const QString &obj_name);
     virtual void setTarget(Connection *con, const QString &obj_name);
 
-    QtUndoStack *undoStack() const { return m_undo_stack; }
+    QUndoStack *undoStack() const { return m_undo_stack; }
 
     void clear();
-    
+
     void showEvent(QShowEvent * /*e*/)
     {
         updateBackground();
@@ -182,8 +200,8 @@ signals:
 
 public slots:
     virtual void setBackground(QWidget *background);
-    void updateBackground();
-    void widgetRemoved(QWidget *w);
+    virtual void updateBackground();
+    virtual void widgetRemoved(QWidget *w);
     void updateLines();
     void enableUpdateBackground(bool enable);
 
@@ -200,68 +218,71 @@ protected:
     virtual void modifyConnection(Connection *con);
 
     virtual QWidget *widgetAt(const QPoint &pos) const;
-    QRect widgetRect(QWidget *w) const;
     void addConnection(Connection *con);
-
+    QRect widgetRect(QWidget *w) const;
+    
     enum State { Editing, Connecting, Dragging };
     State state() const;
 
+    virtual void endConnection(QWidget *target, const QPoint &pos);
 private:
-    QWidget *m_bg_widget;
-    QtUndoStack *m_undo_stack;
+    void startConnection(QWidget *source, const QPoint &pos);
+    void continueConnection(QWidget *target, const QPoint &pos);
+    void abortConnection();
+
+    void findObjectsUnderMouse(const QPoint &pos);
+    void startDrag(const EndPoint &end_point, const QPoint &pos);
+    void continueDrag(const QPoint &pos);
+    void endDrag(const QPoint &pos);
+    void adjustHotSopt(const EndPoint &end_point, const QPoint &pos);
+    Connection *connectionAt(const QPoint &pos) const;
+    EndPoint endPointAt(const QPoint &pos) const;
+    void paintConnection(QPainter *p, Connection *con,
+			 WidgetSet *heavy_highlight_set,
+			 WidgetSet *light_highlight_set) const;
+    void paintLabel(QPainter *p, EndPoint::Type type, Connection *con);
+
+    
+    QPointer<QWidget> m_bg_widget;
+    QUndoStack *m_undo_stack;
     bool m_enable_update_background;
 
     Connection *m_tmp_con; // the connection we are currently editing
     ConnectionList m_con_list;
     bool m_start_connection_on_drag;
-    void startConnection(QWidget *source, const QPoint &pos);
-    void continueConnection(QWidget *target, const QPoint &pos);
-    void endConnection(QWidget *target, const QPoint &pos);
-    void abortConnection();
-
-    void findObjectsUnderMouse(const QPoint &pos);
     EndPoint m_end_point_under_mouse;
     QPointer<QWidget> m_widget_under_mouse;
 
     EndPoint m_drag_end_point;
     QPoint m_old_source_pos, m_old_target_pos;
-    void startDrag(const EndPoint &end_point, const QPoint &pos);
-    void continueDrag(const QPoint &pos);
-    void endDrag(const QPoint &pos);
-    void adjustHotSopt(const EndPoint &end_point, const QPoint &pos);
-
-    Connection *connectionAt(const QPoint &pos) const;
-    EndPoint endPointAt(const QPoint &pos) const;
     ConnectionSet m_sel_con_set;
-
-    void paintConnection(QPainter *p, Connection *con,
-                            WidgetSet *heavy_highlight_set,
-                            WidgetSet *light_highlight_set) const;
-    void paintLabel(QPainter *p, EndPoint::Type type, Connection *con);
-
-    QColor m_inactive_color, m_active_color;
+    const QColor m_inactive_color;
+    const QColor m_active_color;
 
 private:
+    friend class BuddyEditor;
     friend class Connection;
     friend class AddConnectionCommand;
     friend class DeleteConnectionsCommand;
     friend class SetEndPointCommand;
 };
 
-class QDESIGNER_SHARED_EXPORT CECommand : public QtCommand, public CETypes
+class QDESIGNER_SHARED_EXPORT CECommand : public QUndoCommand, public CETypes
 {
-    Q_OBJECT
 public:
     CECommand(ConnectionEdit *edit)
-        : m_edit(edit) { setCanMerge(false); }
+        : m_edit(edit) {}
+
+    virtual bool mergeWith(const QUndoCommand *) { return false; }
+
     ConnectionEdit *edit() const { return m_edit; }
+
 private:
     ConnectionEdit *m_edit;
 };
 
 class QDESIGNER_SHARED_EXPORT AddConnectionCommand : public CECommand
 {
-    Q_OBJECT
 public:
     AddConnectionCommand(ConnectionEdit *edit, Connection *con);
     virtual void redo();
