@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -44,13 +59,15 @@ class QDesignerFormWindowInterface;
 class QListWidgetItem;
 class QModelIndex;
 
+class QDesignerResourceBrowserInterface;
+
+namespace qdesigner_internal {
+
 namespace Ui
 {
     class FindIconDialog;
 } // namespace Ui
 
-namespace qdesigner_internal {
-
 class ResourceEditor;
 
 class QDESIGNER_SHARED_EXPORT FindIconDialog : public QDialog
@@ -68,25 +85,31 @@ public:
     virtual void accept();
 
 private slots:
+    void setActiveBox(int box);
     void setActiveBox();
     void updateButtons();
 
     void setFile(const QString &path);
     void setQrc(const QString &qrc, const QString &file);
+    void setLanguagePath(const QString &path);
     void cdUp();
 
-    void itemActivated(QListWidgetItem *item);
+    void itemActivated(QListWidgetItem *item); // File page
     void currentItemChanged(QListWidgetItem *item);
-    void itemActivated(const QString &qrc_path, const QString &file_name);
+    
+    void itemActivated(const QString &qrc_path, const QString &file_name); // QRC page
     void itemChanged(const QString &qrc_path, const QString &file_name);
+    
+    void itemActivated(const QString &file_name); // Language plugin page
+    void itemChanged(const QString &file_name);
 
 private:
-    enum InputBox { FileBox, ResourceBox };
+    enum InputBox { ResourceBox, FileBox, LanguageBox };
 
-    void setActiveBox(InputBox box);
     InputBox activeBox() const;
+    bool isIconValid(const QString &file) const;
 
-    Ui::FindIconDialog *ui;
+    qdesigner_internal::Ui::FindIconDialog *ui;
     QDesignerFormWindowInterface *m_form;
 
     void setViewDir(const QString &path);
@@ -94,11 +117,29 @@ private:
     struct FileData {
         QString file;
     } m_file_data;
+    struct LanguageData {
+        QString file;
+    } m_language_data;
     struct ResourceData {
         QString file;
         QString qrc;
     } m_resource_data;
     ResourceEditor *m_resource_editor;
+    QDesignerResourceBrowserInterface *m_language_editor;
+
+    static QString defaultQrcPath();
+    static QString defaultFilePath(QDesignerFormWindowInterface *form);
+    static QString defaultLanguagePath();
+    static void setDefaultQrcPath(const QString &path);
+    static void setDefaultFilePath(const QString &path);
+    static void setDefaultLanguagePath(const QString &path);
+    static InputBox previousInputBox();
+    static void setPreviousInputBox(InputBox box);
+
+#ifdef Q_OS_WIN
+    bool isRoot;
+    QString rootDir;
+#endif
 };
 
 } // namespace qdesigner_internal
