@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -23,7 +38,10 @@
 
 #include "layoutinfo_p.h"
 
-#include <QtDesigner/QtDesigner>
+#include <QtDesigner/QDesignerFormEditorInterface>
+#include <QtDesigner/QDesignerContainerExtension>
+#include <QtDesigner/QDesignerMetaDataBaseInterface>
+#include <QtDesigner/QExtensionManager>
 
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QSplitter>
@@ -112,7 +130,7 @@ void LayoutInfo::deleteLayout(QDesignerFormEditorInterface *core, QWidget *widge
         return;
     }
 
-    qWarning() << "trying to delete an unmanaged layout:" << "widget:" << widget << "layout:" << layout;
+    qDebug() << "trying to delete an unmanaged layout:" << "widget:" << widget << "layout:" << layout;
 }
 
 void LayoutInfo::cells(QLayout *layout, IntervalList *rows, IntervalList *columns)
@@ -160,19 +178,47 @@ bool LayoutInfo::isWidgetLaidout(QDesignerFormEditorInterface *core, QWidget *wi
     return false;
 }
 
+QLayout *LayoutInfo::internalLayout(QWidget *widget)
+{
+    QLayout *widgetLayout = widget->layout();
+    if (widgetLayout && widget->inherits("Q3GroupBox")) {
+        if (widgetLayout->count()) {
+            widgetLayout = widgetLayout->itemAt(0)->layout();
+        } else {
+            widgetLayout = 0;
+        }
+    }
+    return widgetLayout;
+}
+
+
 QLayout *LayoutInfo::managedLayout(QDesignerFormEditorInterface *core, QWidget *widget)
 {
     if (widget == 0)
         return 0;
-
+    
     QLayout *layout = widget->layout();
-    QDesignerMetaDataBaseItemInterface *item = core->metaDataBase()->item(layout);
-    if (layout != 0 && item == 0) {
-        layout = qFindChild<QLayout*>(layout);
-        item = core->metaDataBase()->item(layout);
-    }
+    if (!layout)
+        return 0;
 
-    return item != 0 ? layout : 0;
+    return managedLayout(core, layout);    
+}
+
+QLayout *LayoutInfo::managedLayout(QDesignerFormEditorInterface *core, QLayout *layout)
+{
+    QDesignerMetaDataBaseInterface *metaDataBase = core->metaDataBase();
+
+    if (!metaDataBase)
+        return layout;
+    
+    const QDesignerMetaDataBaseItemInterface *item = metaDataBase->item(layout);
+    if (item == 0) {
+        layout = qFindChild<QLayout*>(layout);
+        item = metaDataBase->item(layout);
+    }
+    if (!item)
+        return 0;
+    return layout;
 }
 
 
