@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -25,22 +40,28 @@
 #include "qdesigner_menubar_p.h"
 #include "qdesigner_toolbar_p.h"
 #include "qdesigner_command_p.h"
+#include "qdesigner_propertycommand_p.h"
 #include "actionrepository_p.h"
 #include "actionprovider_p.h"
 #include "actioneditor_p.h"
 #include "qdesigner_utils_p.h"
-
-#include <QtDesigner/QtDesigner>
+#include "qdesigner_objectinspector_p.h"
 
 #include <QtCore/QTimer>
 #include <QtCore/qdebug.h>
 
+#include <QtDesigner/QDesignerFormEditorInterface>
+#include <QtDesigner/QDesignerWidgetFactoryInterface>
+#include <QtDesigner/QDesignerMetaDataBaseInterface>
+#include <QtDesigner/QExtensionManager>
+
 #include <QtGui/QAction>
 #include <QtGui/QApplication>
 #include <QtGui/QLineEdit>
 #include <QtGui/QPainter>
 #include <QtGui/QRubberBand>
 #include <QtGui/QToolTip>
+#include <QtGui/QToolBar>
 #include <QtGui/qevent.h>
 
 Q_DECLARE_METATYPE(QAction*)
@@ -48,37 +69,34 @@ Q_DECLARE_METATYPE(QListWidgetItem*)
 
 using namespace qdesigner_internal;
 
-QDesignerMenu::QDesignerMenu(QWidget *parent)
-    : QMenu(parent)
+QDesignerMenu::QDesignerMenu(QWidget *parent) :
+    QMenu(parent),
+    m_currentIndex(0),
+    m_addItem(new SpecialMenuAction(this)),
+    m_addSeparator(new SpecialMenuAction(this)),
+    m_showSubMenuTimer(new QTimer(this)),
+    m_deactivateWindowTimer(new QTimer(this)),
+    m_adjustSizeTimer(new QTimer(this)),
+    m_editor(new QLineEdit(this)),
+    m_dragging(false),
+    m_lastSubMenuIndex(-1)
 {
-    m_interactive = true;
-    m_dragging = false;
-    m_currentIndex = 0;
-    m_lastSubMenuIndex = -1;
-
     setContextMenuPolicy(Qt::DefaultContextMenu);
     setAcceptDrops(true); // ### fake
     setSeparatorsCollapsible(false);
 
-    m_adjustSizeTimer = new QTimer(this);
     connect(m_adjustSizeTimer, SIGNAL(timeout()), this, SLOT(slotAdjustSizeNow()));
-
-    m_addItem = new SpecialMenuAction(this);
     m_addItem->setText(tr("Type Here"));
     addAction(m_addItem);
 
-    m_addSeparator = new SpecialMenuAction(this);
     m_addSeparator->setText(tr("Add Separator"));
     addAction(m_addSeparator);
 
-    m_showSubMenuTimer = new QTimer(this);
     connect(m_showSubMenuTimer, SIGNAL(timeout()), this, SLOT(slotShowSubMenuNow()));
 
-    m_deactivateWindowTimer = new QTimer(this);
     connect(m_deactivateWindowTimer, SIGNAL(timeout()), this, SLOT(slotDeactivateNow()));
 
-    m_editor = new QLineEdit(this);
-    m_editor->setObjectName("__qt__passive_editor");
+    m_editor->setObjectName(QLatin1String("__qt__passive_editor"));
     m_editor->hide();
 
     m_editor->installEventFilter(this);
@@ -126,34 +144,36 @@ bool QDesignerMenu::handleEvent(QWidget *widget, QEvent *event)
     return true;
 }
 
-void QDesignerMenu::startDrag(const QPoint &pos)
+void QDesignerMenu::startDrag(const QPoint &pos, Qt::KeyboardModifiers modifiers)
 {
-    int index = findAction(pos);
+    const int index = findAction(pos);
     if (index >= realActionCount())
         return;
 
     QAction *action = safeActionAt(index);
 
-    RemoveActionFromCommand *cmd = new RemoveActionFromCommand(formWindow());
-    cmd->init(this, action, actions().at(index + 1));
-    formWindow()->commandHistory()->push(cmd);
+    QDesignerFormWindowInterface *fw = formWindow();
+    const Qt::DropAction dropAction = (modifiers & Qt::ControlModifier) ? Qt::CopyAction : Qt::MoveAction;
+    if (dropAction == Qt::MoveAction) {
+        RemoveActionFromCommand *cmd = new RemoveActionFromCommand(fw);
+        cmd->init(this, action, actions().at(index + 1));
+        fw->commandHistory()->push(cmd);
+    }
 
     QDrag *drag = new QDrag(this);
-    drag->setPixmap(action->icon().pixmap(QSize(22, 22)));
+    drag->setPixmap(ActionRepositoryMimeData::actionDragPixmap(action));
+    drag->setMimeData(new ActionRepositoryMimeData(action, dropAction));
 
-    ActionRepositoryMimeData *data = new ActionRepositoryMimeData();
-    data->items.append(action);
-    drag->setMimeData(data);
-
-    int old_index = m_currentIndex;
+    const int old_index = m_currentIndex;
     m_currentIndex = -1;
 
-    if (drag->start() == Qt::IgnoreAction) {
-        QAction *previous = safeActionAt(index);
-
-        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(formWindow());
-        cmd->init(this, action, previous);
-        formWindow()->commandHistory()->push(cmd);
+    if (drag->start(dropAction) == Qt::IgnoreAction) {
+        if (dropAction == Qt::MoveAction) {
+            QAction *previous = safeActionAt(index);
+            InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
+            cmd->init(this, action, previous);
+            fw->commandHistory()->push(cmd);
+        }
 
         m_currentIndex = old_index;
     }
@@ -222,8 +242,9 @@ bool QDesignerMenu::handleKeyPressEvent(QWidget * /*widget*/, QKeyEvent *e)
             setFocus(); // FIXME: this is because some other widget get the focus when CTRL is pressed
             return true; // no update
 
-        default:
-            if (!currentAction() || currentAction()->isSeparator() || currentAction() == m_addSeparator) {
+        default: {
+            QAction *action = currentAction();
+            if (!action || action->isSeparator() || action == m_addSeparator) {
                 e->ignore();
                 return true;
             } else if (!e->text().isEmpty() && e->text().at(0).toLatin1() >= 32) {
@@ -233,6 +254,7 @@ bool QDesignerMenu::handleKeyPressEvent(QWidget * /*widget*/, QKeyEvent *e)
             } else {
                 e->ignore();
             }
+        }
             return true;
         }
     } else if (m_editor->hasFocus()) { // In edit mode
@@ -265,6 +287,12 @@ bool QDesignerMenu::handleKeyPressEvent(QWidget * /*widget*/, QKeyEvent *e)
     return true;
 }
 
+static void sendMouseEventTo(QWidget *target, const QPoint &targetPoint, const QMouseEvent *event)
+{
+    QMouseEvent e(event->type(), targetPoint, event->globalPos(), event->button(), event->buttons(), event->modifiers());
+    QApplication::sendEvent(target, &e);
+}
+
 bool QDesignerMenu::handleMouseDoubleClickEvent(QWidget *, QMouseEvent *event)
 {
     event->accept();
@@ -279,13 +307,10 @@ bool QDesignerMenu::handleMouseDoubleClickEvent(QWidget *, QMouseEvent *event)
         QMenuBar *mb = qobject_cast<QMenuBar*>(target);
         QDesignerMenu *menu = qobject_cast<QDesignerMenu*>(target);
         if (mb != 0 || menu != 0) {
-            QPoint pt = target->mapFromGlobal(event->globalPos());
-
+            const QPoint pt = target->mapFromGlobal(event->globalPos());
             QAction *action = mb == 0 ? menu->actionAt(pt) : mb->actionAt(pt);
-            if (action) {
-                QMouseEvent e(event->type(), pt, event->globalPos(), event->button(), event->buttons(), event->modifiers());
-                QApplication::sendEvent(target, &e);
-            }
+            if (action)
+                 sendMouseEventTo(target, pt, event);
         }
         return true;
     }
@@ -309,20 +334,27 @@ bool QDesignerMenu::handleMouseDoubleClickEvent(QWidget *, QMouseEvent *event)
 bool QDesignerMenu::handleMousePressEvent(QWidget * /*widget*/, QMouseEvent *event)
 {
     if (!rect().contains(event->pos())) {
-        if (QMenuBar *mb = qobject_cast<QMenuBar*>(QApplication::widgetAt(event->globalPos()))) {
-            QPoint pt = mb->mapFromGlobal(event->globalPos());
-            QAction *action = mb->actionAt(pt);
-
-            if (action && action->menu() == findRootMenu()) {
-                // propagate the mouse press event (but don't close the popup)
-                QMouseEvent e(event->type(), pt, event->globalPos(), event->button(), event->buttons(), event->modifiers());
-                QApplication::sendEvent(mb, &e);
-                return true;
+        QWidget *clickedWidget = QApplication::widgetAt(event->globalPos());
+        if (QMenuBar *mb = qobject_cast<QMenuBar*>(clickedWidget)) {
+            const QPoint pt = mb->mapFromGlobal(event->globalPos());
+            if (QAction *action = mb->actionAt(pt)) {
+                QMenu * menu = action->menu();
+                if (menu == findRootMenu()) {
+                    // propagate the mouse press event (but don't close the popup)
+                    sendMouseEventTo(mb, pt, event);
+                    return true;
+                }
             }
         }
 
-        // hide the popup Qt will replay the event
+        // hide the popup Qt will replay the event. Try to focus the clicked window.
         slotDeactivateNow();
+        if (clickedWidget) {
+            if (QWidget *focusProxy = clickedWidget->focusProxy())
+                clickedWidget = focusProxy;
+            if (clickedWidget->focusPolicy() != Qt::NoFocus)
+                clickedWidget->setFocus(Qt::OtherFocusReason);
+        }
         return true;
     }
 
@@ -335,15 +367,14 @@ bool QDesignerMenu::handleMousePressEvent(QWidget * /*widget*/, QMouseEvent *eve
 
     m_startPosition = mapFromGlobal(event->globalPos());
 
-    int index = findAction(m_startPosition);
+    const int index = findAction(m_startPosition);
 
     QAction *action = safeActionAt(index);
     QRect pm_rect = subMenuPixmapRect(action);
     pm_rect.setLeft(pm_rect.left() - 20); // give the user a little more space to click
 
-    int old_index = m_currentIndex;
+    const int old_index = m_currentIndex;
     m_currentIndex = index;
-
     if ((hasSubMenuPixmap(action) || action->menu() != 0)
         && pm_rect.contains(m_startPosition)) {
         if (m_currentIndex == m_lastSubMenuIndex) {
@@ -352,16 +383,16 @@ bool QDesignerMenu::handleMousePressEvent(QWidget * /*widget*/, QMouseEvent *eve
             slotShowSubMenuNow();
     } else {
         if (index == old_index) {
-            if (m_currentIndex == m_lastSubMenuIndex) {
+            if (m_currentIndex == m_lastSubMenuIndex)
                 hideSubMenu();
-            } else
-                slotShowSubMenuNow();
         } else {
             hideSubMenu();
         }
     }
 
-    updateCurrentAction();
+    update();
+    if (index != old_index)
+        selectCurrentAction();
 
     return true;
 }
@@ -382,13 +413,11 @@ bool QDesignerMenu::handleMouseMoveEvent(QWidget *, QMouseEvent *event)
     if (!rect().contains(event->pos())) {
 
         if (QMenuBar *mb = qobject_cast<QMenuBar*>(QApplication::widgetAt(event->globalPos()))) {
-            QPoint pt = mb->mapFromGlobal(event->globalPos());
+            const QPoint pt = mb->mapFromGlobal(event->globalPos());
             QAction *action = mb->actionAt(pt);
-
             if (action && action->menu() == findRootMenu()) {
                 // propagate the mouse press event (but don't close the popup)
-                QMouseEvent e(event->type(), pt, event->globalPos(), event->button(), event->buttons(), event->modifiers());
-                QApplication::sendEvent(mb, &e);
+                sendMouseEventTo(mb, pt, event);
                 return true;
             }
             // hide the popup Qt will replay the event
@@ -402,12 +431,12 @@ bool QDesignerMenu::handleMouseMoveEvent(QWidget *, QMouseEvent *event)
 
     event->accept();
 
-    QPoint pos = mapFromGlobal(event->globalPos());
+    const QPoint pos = mapFromGlobal(event->globalPos());
 
     if ((pos - m_startPosition).manhattanLength() < qApp->startDragDistance())
         return true;
 
-    startDrag(m_startPosition);
+    startDrag(m_startPosition, event->modifiers());
     m_startPosition = QPoint();
 
     return true;
@@ -417,44 +446,92 @@ bool QDesignerMenu::handleContextMenuEvent(QWidget *, QContextMenuEvent *event)
 {
     event->accept();
 
-    int index = findAction(mapFromGlobal(event->globalPos()));
+    const int index = findAction(mapFromGlobal(event->globalPos()));
     QAction *action = safeActionAt(index);
     if (qobject_cast<SpecialMenuAction*>(action))
         return true;
 
     QMenu menu(this);
-    QAction *a = menu.addAction(tr("Remove action '%1'").arg(action->objectName()));
     QVariant itemData;
     qVariantSetValue(itemData, action);
-    a->setData(itemData);
 
-    connect(&menu, SIGNAL(triggered(QAction*)), this, SLOT(slotRemoveSelectedAction(QAction*)));
+    QAction *addSeparatorAction = menu.addAction(tr("Insert separator"));
+    addSeparatorAction->setData(itemData);
+
+    QAction *removeAction = 0;
+    if (action->isSeparator())
+        removeAction = menu.addAction(tr("Remove separator"));
+    else
+        removeAction = menu.addAction(tr("Remove action '%1'").arg(action->objectName()));
+    removeAction->setData(itemData);
+
+    connect(addSeparatorAction, SIGNAL(triggered(bool)), this, SLOT(slotAddSeparator()));
+    connect(removeAction, SIGNAL(triggered(bool)), this, SLOT(slotRemoveSelectedAction()));
     menu.exec(event->globalPos());
 
     return true;
 }
 
-void QDesignerMenu::slotRemoveSelectedAction(QAction *action)
+void QDesignerMenu::slotAddSeparator()
 {
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (!action)
+        return;
+
     QAction *a = qvariant_cast<QAction*>(action->data());
     Q_ASSERT(a != 0);
 
-    int pos = actions().indexOf(a);
+    const int pos = actions().indexOf(a);
+    QAction *action_before = 0;
+    if (pos != -1)
+        action_before = safeActionAt(pos);
+
+    QDesignerFormWindowInterface *fw = formWindow();
+    fw->beginCommand(tr("Add separator"));
+    QAction *sep = createAction(QString(), true);
+
+    InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
+    cmd->init(this, sep, action_before);
+    fw->commandHistory()->push(cmd);
+
+    if (parentMenu()) {
+        QAction *parent_action = parentMenu()->currentAction();
+        if (parent_action->menu() == 0) {
+            CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
+            cmd->init(parentMenu(), parentMenu()->currentAction());
+            fw->commandHistory()->push(cmd);
+        }
+    }
+
+    fw->endCommand();
+}
+
+void QDesignerMenu::slotRemoveSelectedAction()
+{
+    QAction *action = qobject_cast<QAction *>(sender());
+    if (!action)
+        return;
+
+    QAction *a = qvariant_cast<QAction*>(action->data());
+    Q_ASSERT(a != 0);
+
+    const int pos = actions().indexOf(a);
     QAction *action_before = 0;
     if (pos != -1)
         action_before = safeActionAt(pos + 1);
 
-    RemoveActionFromCommand *cmd = new RemoveActionFromCommand(formWindow());
+    QDesignerFormWindowInterface *fw = formWindow();
+    RemoveActionFromCommand *cmd = new RemoveActionFromCommand(fw);
     cmd->init(this, a, action_before);
-    formWindow()->commandHistory()->push(cmd);
+    fw->commandHistory()->push(cmd);
 }
 
 QRect QDesignerMenu::subMenuPixmapRect(QAction *action) const
 {
-    static const QPixmap pm(":/trolltech/formeditor/images/submenu.png");
-    QRect g = actionGeometry(action);
-    int x = g.right() - pm.width() - 2;
-    int y = g.top() + (g.height() - pm.height())/2 + 1;
+    static const QPixmap pm(QLatin1String(":/trolltech/formeditor/images/submenu.png"));
+    const QRect g = actionGeometry(action);
+    const int x = g.right() - pm.width() - 2;
+    const int y = g.top() + (g.height() - pm.height())/2 + 1;
     return QRect(x, y, pm.width(), pm.height());
 }
 
@@ -467,6 +544,12 @@ bool QDesignerMenu::hasSubMenuPixmap(QAction *action) const
             && canCreateSubMenu(action);
 }
 
+void QDesignerMenu::showEvent ( QShowEvent * event )
+{
+    selectCurrentAction();
+    QMenu::showEvent (event);
+}
+
 void QDesignerMenu::paintEvent(QPaintEvent *event)
 {
     QMenu::paintEvent(event);
@@ -476,7 +559,7 @@ void QDesignerMenu::paintEvent(QPaintEvent *event)
     QAction *current = currentAction();
 
     foreach (QAction *a, actions()) {
-        QRect g = actionGeometry(a);
+        const QRect g = actionGeometry(a);
 
         if (qobject_cast<SpecialMenuAction*>(a)) {
             QLinearGradient lg(g.left(), g.top(), g.left(), g.bottom());
@@ -486,7 +569,7 @@ void QDesignerMenu::paintEvent(QPaintEvent *event)
 
             p.fillRect(g, lg);
         } else if (hasSubMenuPixmap(a)) {
-            static const QPixmap pm(":/trolltech/formeditor/images/submenu.png");
+            static const QPixmap pm(QLatin1String(":/trolltech/formeditor/images/submenu.png"));
             p.drawPixmap(subMenuPixmapRect(a).topLeft(), pm);
         }
     }
@@ -504,7 +587,7 @@ void QDesignerMenu::paintEvent(QPaintEvent *event)
             return;
     }
 
-    QRect g = actionGeometry(current);
+    const QRect g = actionGeometry(current);
     drawSelection(&p, g.adjusted(1, 1, -3, -3));
 }
 
@@ -576,12 +659,11 @@ bool QDesignerMenu::eventFilter(QObject *object, QEvent *event)
         case QEvent::Leave:
         case QEvent::FocusIn:
         case QEvent::FocusOut:
-        {
-            QWidget *widget = qobject_cast<QWidget*>(object);
-
-            if (dispatch && widget && (widget == this || isAncestorOf(widget)))
-                return handleEvent(widget, event);
-        } break;
+        if (dispatch)
+            if (QWidget *widget = qobject_cast<QWidget*>(object))
+                if (widget == this || isAncestorOf(widget))
+                    return handleEvent(widget, event);
+        break;
     }
 
     return false;
@@ -602,7 +684,7 @@ int QDesignerMenu::actionAtPosition(const QPoint &pos) const
 
 int QDesignerMenu::findAction(const QPoint &pos) const
 {
-    int index = actionAtPosition(pos);
+    const int index = actionAtPosition(pos);
     if (index == -1)
         return realActionCount();
 
@@ -616,42 +698,43 @@ void QDesignerMenu::adjustIndicator(const QPoint &pos)
     }
 }
 
-QAction *QDesignerMenu::actionMimeData(const QMimeData *mimeData) const
-{
-    if (const ActionRepositoryMimeData *d = qobject_cast<const ActionRepositoryMimeData*>(mimeData)) {
-        Q_ASSERT(!d->items.isEmpty());
-
-        return d->items.first();
-    }
-
-    return 0;
-}
-
-bool QDesignerMenu::checkAction(QAction *action) const
+QDesignerMenu::ActionDragCheck QDesignerMenu::checkAction(QAction *action) const
 {
     if (!action || (action->menu() && action->menu()->parentWidget() != const_cast<QDesignerMenu*>(this)))
-        return false; // menu action!! nothing to do
-
-    if (actions().contains(action))
-        return false; // we already have the action in the menu
+        return NoActionDrag; // menu action!! nothing to do
 
     if (!Utils::isObjectAncestorOf(formWindow()->mainContainer(), action))
-        return false; // the action belongs to another form window
+        return NoActionDrag; // the action belongs to another form window
 
-    return true;
+    if (actions().contains(action))
+        return ActionDragOnSubMenu; // we already have the action in the menu
+
+    return AcceptActionDrag;
 }
 
 void QDesignerMenu::dragEnterEvent(QDragEnterEvent *event)
 {
-    QAction *action = actionMimeData(event->mimeData());
-    if (!action)
+    const ActionRepositoryMimeData *d = qobject_cast<const ActionRepositoryMimeData*>(event->mimeData());
+    if (!d || d->actionList().empty()) {
+        event->ignore();
         return;
+    }
 
-    m_dragging = true;
-    event->acceptProposedAction();
+    QAction *action = d->actionList().first();
 
-    if (checkAction(action)) {
+    switch (checkAction(action)) {
+    case NoActionDrag:
+        event->ignore();
+        break;
+    case ActionDragOnSubMenu:
+        d->accept(event);
+        m_dragging = true;
+        break;
+    case AcceptActionDrag:
+        d->accept(event);
+        m_dragging = true;
         adjustIndicator(event->pos());
+        break;
     }
 }
 
@@ -663,18 +746,35 @@ void QDesignerMenu::dragMoveEvent(QDragMoveEvent *event)
         return;
     }
 
-    QAction *action = actionMimeData(event->mimeData());
-    if (action == 0 || !checkAction(action)) {
+    const ActionRepositoryMimeData *d = qobject_cast<const ActionRepositoryMimeData*>(event->mimeData());
+    if (!d || d->actionList().empty()) {
         event->ignore();
         return;
     }
 
-    event->acceptProposedAction();
-    adjustIndicator(event->pos());
-    m_currentIndex = findAction(event->pos());
-
-    if (m_lastSubMenuIndex != m_currentIndex)
-        m_showSubMenuTimer->start(300);
+    QAction *action = d->actionList().first();
+    const ActionDragCheck dc = checkAction(action);
+    switch (dc) {
+    case NoActionDrag:
+        event->ignore();
+        break;
+    case ActionDragOnSubMenu:
+    case AcceptActionDrag: { // Do not pop up submenu of action being dragged
+        const int newIndex = findAction(event->pos());
+        if (safeActionAt(newIndex) != action) {
+            m_currentIndex = newIndex;
+            if (m_lastSubMenuIndex != m_currentIndex)
+                m_showSubMenuTimer->start(300);
+        }
+        if (dc == AcceptActionDrag) {
+            adjustIndicator(event->pos());
+            d->accept(event);
+        } else {
+            event->ignore();
+        }
+    }
+        break;
+    }
 }
 
 void QDesignerMenu::dragLeaveEvent(QDragLeaveEvent *)
@@ -690,29 +790,35 @@ void QDesignerMenu::dropEvent(QDropEvent *event)
     hideSubMenu();
     m_dragging = false;
 
-    QAction *action = actionMimeData(event->mimeData());
-    if (action && checkAction(action)) {
+    QDesignerFormWindowInterface *fw = formWindow();
+    const ActionRepositoryMimeData *d = qobject_cast<const ActionRepositoryMimeData*>(event->mimeData());
+    if (!d || d->actionList().empty()) {
+        event->ignore();
+        return;
+    }
+    QAction *action = d->actionList().first();
+    if (action && checkAction(action) == AcceptActionDrag) {
         event->acceptProposedAction();
         int index = findAction(event->pos());
         index = qMin(index, actions().count() - 1);
 
-        formWindow()->beginCommand(tr("Insert action"));
-        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(formWindow());
+        fw->beginCommand(tr("Insert action"));
+        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
         cmd->init(this, action, safeActionAt(index));
-        formWindow()->commandHistory()->push(cmd);
+        fw->commandHistory()->push(cmd);
 
         m_currentIndex = index;
 
         if (parentMenu()) {
             QAction *parent_action = parentMenu()->currentAction();
             if (parent_action->menu() == 0) {
-                CreateSubmenuCommand *cmd = new CreateSubmenuCommand(formWindow());
-                cmd->init(parentMenu(), parentMenu()->currentAction());
-                formWindow()->commandHistory()->push(cmd);
+                CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
+                cmd->init(parentMenu(), parentMenu()->currentAction(), action);
+                fw->commandHistory()->push(cmd);
             }
         }
-        updateCurrentAction();
-        formWindow()->endCommand();
+        update();
+        fw->endCommand();
     } else {
         event->ignore();
     }
@@ -773,7 +879,7 @@ void QDesignerMenu::moveLeft()
                 mb->moveRight();
         }
     }
-    updateCurrentAction();
+    update();
 }
 
 void QDesignerMenu::moveRight()
@@ -805,7 +911,9 @@ void QDesignerMenu::moveUp(bool ctrl)
         (void) swap(m_currentIndex, m_currentIndex - 1);
 
     m_currentIndex = qMax(0, --m_currentIndex);
-    updateCurrentAction();
+    // Always re-select, swapping destroys order
+    update();
+    selectCurrentAction();
 }
 
 void QDesignerMenu::moveDown(bool ctrl)
@@ -818,7 +926,9 @@ void QDesignerMenu::moveDown(bool ctrl)
         (void) swap(m_currentIndex + 1, m_currentIndex);
 
     m_currentIndex = qMin(actions().count() - 1, ++m_currentIndex);
-    updateCurrentAction();
+    update();
+    if (!ctrl)
+        selectCurrentAction();
 }
 
 QAction *QDesignerMenu::currentAction() const
@@ -834,9 +944,24 @@ int QDesignerMenu::realActionCount() const
     return actions().count() - 2; // 2 fake actions
 }
 
-void QDesignerMenu::updateCurrentAction()
+void QDesignerMenu::selectCurrentAction()
 {
-    update();
+    QAction *action = currentAction();
+    if (!action || action == m_addSeparator || action == m_addItem)
+        return;
+
+    QDesignerObjectInspector *oi = 0;
+    if (QDesignerFormWindowInterface *fw = formWindow())
+        oi = qobject_cast<QDesignerObjectInspector *>(fw->core()->objectInspector());
+
+    if (!oi)
+        return;
+
+    oi->clearSelection();
+    if (QMenu *menu = action->menu())
+        oi->selectObject(menu);
+    else
+        oi->selectObject(action);
 }
 
 void QDesignerMenu::createRealMenuAction(QAction *action)
@@ -853,13 +978,11 @@ void QDesignerMenu::createRealMenuAction(QAction *action)
     action->setMenu(menu);
     menu->setTitle(action->text());
 
-    Q_ASSERT(formWindow() != 0);
+    Q_ASSERT(fw);
 
     core->widgetFactory()->initialize(menu);
 
-    QString niceObjectName = ActionEditor::actionTextToName(menu->title());
-    if (niceObjectName.startsWith("action"))
-        niceObjectName.replace(0, 6, QLatin1String("menu"));
+    const QString niceObjectName = ActionEditor::actionTextToName(menu->title(), QLatin1String("menu"));
     menu->setObjectName(niceObjectName);
 
     core->metaDataBase()->add(menu);
@@ -896,22 +1019,17 @@ QDesignerMenu *QDesignerMenu::findOrCreateSubMenu(QAction *action)
 
 bool QDesignerMenu::canCreateSubMenu(QAction *action) const // ### improve it's a bit too slow
 {
-    QWidget *topLevel = formWindow()->mainContainer();
-    QList<QMenu*> menus = qFindChildren<QMenu*>(topLevel);
-    QList<QToolBar*> toolBars = qFindChildren<QToolBar*>(topLevel);
-
-    foreach (const QMenu *m, menus) {
-        if (m != this && m->actions().contains(action)) {
-            return false; // sorry
+    foreach (const QWidget *aw, action->associatedWidgets())
+        if (aw != this) {
+            if (const QMenu *m = qobject_cast<const QMenu *>(aw)) {
+                if (m->actions().contains(action))
+                    return false; // sorry
+            } else {
+                if (const QToolBar *tb = qobject_cast<const QToolBar *>(aw))
+                    if (tb->actions().contains(action))
+                        return false; // sorry
+            }
         }
-    }
-
-    foreach (QToolBar *tb, toolBars) {
-        if (tb->actions().contains(action)) {
-            return false; // sorry
-        }
-    }
-
     return true;
 }
 
@@ -937,7 +1055,7 @@ void QDesignerMenu::slotShowSubMenuNow()
         if (!menu->isVisible()) {
             if ((menu->windowFlags() & Qt::Popup) != Qt::Popup)
                 menu->setWindowFlags(Qt::Popup);
-            QRect g = actionGeometry(action);
+            const QRect g = actionGeometry(action);
             menu->move(mapToGlobal(g.topRight()));
             menu->show();
             menu->setFocus();
@@ -998,7 +1116,7 @@ void QDesignerMenu::adjustSpecialActions()
 
 bool QDesignerMenu::interactive(bool i)
 {
-    bool old = m_interactive;
+    const bool old = m_interactive;
     m_interactive = i;
     return old;
 }
@@ -1009,23 +1127,24 @@ void QDesignerMenu::enterEditMode()
         showLineEdit();
     } else {
         hideSubMenu();
-        formWindow()->beginCommand(tr("Add separator"));
+        QDesignerFormWindowInterface *fw = formWindow();
+        fw->beginCommand(tr("Add separator"));
         QAction *sep = createAction(QString(), true);
 
-        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(formWindow());
+        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
         cmd->init(this, sep, safeActionAt(realActionCount()));
-        formWindow()->commandHistory()->push(cmd);
+        fw->commandHistory()->push(cmd);
 
         if (parentMenu()) {
             QAction *parent_action = parentMenu()->currentAction();
             if (parent_action->menu() == 0) {
-                CreateSubmenuCommand *cmd = new CreateSubmenuCommand(formWindow());
+                CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
                 cmd->init(parentMenu(), parentMenu()->currentAction());
-                formWindow()->commandHistory()->push(cmd);
+                fw->commandHistory()->push(cmd);
             }
         }
 
-        formWindow()->endCommand();
+        fw->endCommand();
 
         m_currentIndex = actions().indexOf(m_addItem);
         update();
@@ -1039,33 +1158,34 @@ void QDesignerMenu::leaveEditMode(LeaveEditMode mode)
 
     QAction *action = 0;
 
+    QDesignerFormWindowInterface *fw = formWindow();
     if (m_currentIndex < realActionCount()) {
         action = safeActionAt(m_currentIndex);
-        formWindow()->beginCommand(QLatin1String("Set action text"));
+        fw->beginCommand(QApplication::translate("Command", "Set action text"));
     } else {
-        Q_ASSERT(formWindow() != 0);
-        formWindow()->beginCommand(QLatin1String("Insert action"));
+        Q_ASSERT(fw != 0);
+        fw->beginCommand(QApplication::translate("Command", "Insert action"));
         action = createAction(ActionEditor::actionTextToName(m_editor->text()));
-        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(formWindow());
+        InsertActionIntoCommand *cmd = new InsertActionIntoCommand(fw);
         cmd->init(this, action, currentAction());
-        formWindow()->commandHistory()->push(cmd);
+        fw->commandHistory()->push(cmd);
     }
 
-    SetPropertyCommand *cmd = new SetPropertyCommand(formWindow());
+    SetPropertyCommand *cmd = new SetPropertyCommand(fw);
     cmd->init(action, QLatin1String("text"), m_editor->text());
-    formWindow()->commandHistory()->push(cmd);
+    fw->commandHistory()->push(cmd);
 
     if (parentMenu()) {
         QAction *parent_action = parentMenu()->currentAction();
         if (parent_action->menu() == 0) {
-            CreateSubmenuCommand *cmd = new CreateSubmenuCommand(formWindow());
-            cmd->init(parentMenu(), parentMenu()->currentAction());
-            formWindow()->commandHistory()->push(cmd);
+            CreateSubmenuCommand *cmd = new CreateSubmenuCommand(fw);
+            cmd->init(parentMenu(), parentMenu()->currentAction(), action);
+            fw->commandHistory()->push(cmd);
         }
     }
 
-    updateCurrentAction();
-    formWindow()->endCommand();
+    update();
+    fw->endCommand();
 }
 
 QAction *QDesignerMenu::safeMenuAction(QDesignerMenu *menu) const
@@ -1097,7 +1217,7 @@ void QDesignerMenu::showLineEdit()
     // open edit field for item name
     setFocus();
 
-    QString text = action != m_addItem ? action->text() : QString();
+    const QString text = action != m_addItem ? action->text() : QString();
     m_editor->setText(text);
     m_editor->selectAll();
     m_editor->setGeometry(actionGeometry(action).adjusted(1, 1, -2, -2));
@@ -1105,31 +1225,17 @@ void QDesignerMenu::showLineEdit()
     m_editor->setFocus();
 }
 
-// ### Share me with QDesignerToolBar (a.k.a. he's a copy of me)
 QAction *QDesignerMenu::createAction(const QString &objectName, bool separator)
 {
-    Q_ASSERT(formWindow() != 0);
     QDesignerFormWindowInterface *fw = formWindow();
-
-    QAction *action = new QAction(fw);
-    fw->core()->widgetFactory()->initialize(action);
-    if (separator)
-        action->setSeparator(true);
-
-    action->setObjectName(objectName);
-    fw->ensureUniqueObjectName(action);
-
-    AddActionCommand *cmd = new AddActionCommand(fw);
-    cmd->init(action);
-    fw->commandHistory()->push(cmd);
-
-    return action;
+    Q_ASSERT(fw);
+    return ToolBarEventFilter::createAction(fw, objectName, separator);
 }
 
 // ### share with QDesignerMenu::swap
 bool QDesignerMenu::swap(int a, int b)
 {
-    int left = qMin(a, b);
+    const int left = qMin(a, b);
     int right = qMax(a, b);
 
     QAction *action_a = safeActionAt(left);
@@ -1146,29 +1252,30 @@ bool QDesignerMenu::swap(int a, int b)
     if (right < 0)
         return false; // nothing to do
 
-    formWindow()->beginCommand(QLatin1String("Move action"));
+    QDesignerFormWindowInterface *fw = formWindow();
+    fw->beginCommand(QApplication::translate("Command", "Move action"));
 
     QAction *action_b_before = safeActionAt(right + 1);
 
-    RemoveActionFromCommand *cmd1 = new RemoveActionFromCommand(formWindow());
+    RemoveActionFromCommand *cmd1 = new RemoveActionFromCommand(fw);
     cmd1->init(this, action_b, action_b_before, false);
-    formWindow()->commandHistory()->push(cmd1);
+    fw->commandHistory()->push(cmd1);
 
     QAction *action_a_before = safeActionAt(left + 1);
 
-    InsertActionIntoCommand *cmd2 = new InsertActionIntoCommand(formWindow());
+    InsertActionIntoCommand *cmd2 = new InsertActionIntoCommand(fw);
     cmd2->init(this, action_b, action_a_before, false);
-    formWindow()->commandHistory()->push(cmd2);
+    fw->commandHistory()->push(cmd2);
 
-    RemoveActionFromCommand *cmd3 = new RemoveActionFromCommand(formWindow());
+    RemoveActionFromCommand *cmd3 = new RemoveActionFromCommand(fw);
     cmd3->init(this, action_a, action_b, false);
-    formWindow()->commandHistory()->push(cmd3);
+    fw->commandHistory()->push(cmd3);
 
-    InsertActionIntoCommand *cmd4 = new InsertActionIntoCommand(formWindow());
+    InsertActionIntoCommand *cmd4 = new InsertActionIntoCommand(fw);
     cmd4->init(this, action_a, action_b_before, true);
-    formWindow()->commandHistory()->push(cmd4);
+    fw->commandHistory()->push(cmd4);
 
-    formWindow()->endCommand();
+    fw->endCommand();
 
     return true;
 }
@@ -1192,14 +1299,15 @@ void QDesignerMenu::hideSubMenu()
 void QDesignerMenu::deleteAction()
 {
     QAction *action = currentAction();
-    int pos = actions().indexOf(action);
+    const int pos = actions().indexOf(action);
     QAction *action_before = 0;
     if (pos != -1)
         action_before = safeActionAt(pos + 1);
 
-    RemoveActionFromCommand *cmd = new RemoveActionFromCommand(formWindow());
+    QDesignerFormWindowInterface *fw = formWindow();
+    RemoveActionFromCommand *cmd = new RemoveActionFromCommand(fw);
     cmd->init(this, action, action_before);
-    formWindow()->commandHistory()->push(cmd);
+    fw->commandHistory()->push(cmd);
 
     update();
 }
