@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -41,6 +56,12 @@
 
 class QDesignerFormWindowInterface;
 
+namespace qdesigner_internal {
+    class PromotionTaskMenu;
+}
+
+class QMenu;
+
 class QDESIGNER_SHARED_EXPORT QDesignerTabWidget : public QTabWidget
 {
     Q_OBJECT
@@ -65,14 +86,8 @@ public:
     QIcon currentTabIcon() const;
     void setCurrentTabIcon(const QIcon &tabIcon);
 
-    inline QAction *actionDeletePage() const
-    { return m_actionDeletePage; }
-
-    inline QAction *actionInsertPage() const
-    { return m_actionInsertPage; }
-
-    inline QAction *actionInsertPageAfter() const
-    { return m_actionInsertPageAfter; }
+    // Add context menu and return page submenu or 0.
+    QMenu *addContextMenuActions(QMenu *popup);
 
     bool eventFilter(QObject *o, QEvent *e);
 
@@ -90,16 +105,21 @@ protected:
     virtual void tabRemoved(int index);
 
 private:
-    QPoint pressPoint;
-    QWidget *dropIndicator;
-    int dragIndex;
-    QWidget *dragPage;
-    QString dragLabel;
-    QIcon dragIcon;
-    bool mousePressed;
+
+private:
+    int pageFromPosition(const QPoint &pos, QRect &rect) const;
+
+    QPoint m_pressPoint;
+    QWidget *m_dropIndicator;
+    int m_dragIndex;
+    QWidget *m_dragPage;
+    QString m_dragLabel;
+    QIcon m_dragIcon;
+    bool m_mousePressed;
     QAction *m_actionDeletePage;
     QAction *m_actionInsertPage;
     QAction *m_actionInsertPageAfter;
+    qdesigner_internal::PromotionTaskMenu* m_pagePromotionTaskMenu;
 };
 
 #endif // QDESIGNER_TABWIDGET_H
