@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -35,18 +50,16 @@
 // We mean it.
 //
 
-#include <QtGui/QWidget>
-
 #include "shared_global_p.h"
 
+#include <QtGui/QWidget>
+
 class QDesignerFormEditorInterface;
 class QDesignerFormWindowInterface;
 class QPushButton;
 class QToolButton;
-class QLineEdit;
 class QComboBox;
 class QStackedWidget;
-class QString;
 class QTreeView;
 class QModelIndex;
 
@@ -59,11 +72,14 @@ class QDESIGNER_SHARED_EXPORT ResourceEditor : public QWidget
     Q_OBJECT
 
 public:
-    ResourceEditor(QDesignerFormEditorInterface *core, QWidget *parent = 0);
-
+    ResourceEditor(QDesignerFormEditorInterface *core,
+                   bool dragEnabled,
+                   QWidget *parent = 0);
+    
     QDesignerFormWindowInterface *form() const { return m_form; }
     int qrcCount() const;
     void setCurrentFile(const QString &qrc_path, const QString &file_path);
+    bool isIcon(const QString &qrc_path, const QString &file_path) const;
 
 signals:
     void fileActivated(const QString &qrc_path, const QString &file_path);
@@ -91,8 +107,6 @@ private slots:
     void itemChanged(const QModelIndex &index);
 
 private:
-    QDesignerFormWindowInterface *m_form;
-
     void getCurrentItem(QString &prefix, QString &file);
     QTreeView *currentView() const;
     ResourceModel *currentModel() const;
@@ -105,6 +119,7 @@ private:
     void insertEmptyComboItem();
     void removeEmptyComboItem();
 
+    QDesignerFormWindowInterface *m_form;
     QComboBox *m_qrc_combo;
     QStackedWidget *m_qrc_stack;
     QToolButton *m_add_button;
@@ -112,6 +127,7 @@ private:
     QPushButton *m_add_files_button;
     QToolButton *m_remove_qrc_button;
     bool m_ignore_update;
+    const bool m_dragEnabled;
 };
 
 } // namespace qdesigner_internal
