@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -9,12 +9,27 @@
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
 ** General Public Licensing requirements will be met:
-** http://www.trolltech.com/products/qt/opensource.html
+** http://trolltech.com/products/qt/licenses/licensing/opensource/
 **
 ** If you are unsure which license is appropriate for your use, please
 ** review the following information:
-** http://www.trolltech.com/products/qt/licensing.html or contact the
-** sales department at sales@trolltech.com.
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
+**
+** In addition, as a special exception, Trolltech gives you certain
+** additional rights. These rights are described in the Trolltech GPL
+** Exception version 1.0, which can be found at
+** http://www.trolltech.com/products/qt/gplexception/ and in the file
+** GPL_EXCEPTION.txt in this package.
+**
+** In addition, as a special exception, Trolltech, as the sole copyright
+** holder for Qt Designer, grants users of the Qt/Eclipse Integration
+** plug-in the right for the Qt/Eclipse Integration to link to
+** functionality provided by Qt Designer and its related libraries.
+**
+** Trolltech reserves all rights not expressly granted herein.
+** 
+** Trolltech ASA (c) 2007
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -50,11 +65,13 @@ class QWidget;
 class QLayout;
 class QDesignerFormEditorInterface;
 class QDesignerCustomWidgetInterface;
+class QDesignerFormWindowInterface;
 
 namespace qdesigner_internal {
 
 class QDESIGNER_SHARED_EXPORT WidgetFactory: public QDesignerWidgetFactoryInterface
 {
+    Q_OBJECT
 public:
     WidgetFactory(QDesignerFormEditorInterface *core, QObject *parent = 0);
     ~WidgetFactory();
@@ -62,6 +79,8 @@ public:
     virtual QWidget* containerOfWidget(QWidget *widget) const;
     virtual QWidget* widgetOfContainer(QWidget *widget) const;
 
+    QObject* createObject(const QString &className, QObject* parent) const;
+    
     virtual QWidget *createWidget(const QString &className, QWidget *parentWidget) const;
     virtual QLayout *createLayout(QWidget *widget, QLayout *layout, int type) const;
 
@@ -70,14 +89,21 @@ public:
 
     virtual QDesignerFormEditorInterface *core() const;
 
-    static const char* classNameOf(QObject* o);
+    static QString classNameOf(QDesignerFormEditorInterface *core, QObject* o);
 
+    QDesignerFormWindowInterface *currentFormWindow(QDesignerFormWindowInterface *fw);
+
+    static QLayout *createUnmanagedLayout(QWidget *parentWidget, int type);
 public slots:
     void loadPlugins();
 
 private:
+    QWidget* createCustomWidget(const QString &className, QWidget *parentWidget) const;
+    
     QDesignerFormEditorInterface *m_core;
-    QMap<QString, QDesignerCustomWidgetInterface*> m_customFactory;
+    typedef QMap<QString, QDesignerCustomWidgetInterface*> CustomWidgetFactoryMap;
+    CustomWidgetFactoryMap m_customFactory;
+    QDesignerFormWindowInterface *m_formWindow;
 
     static QPointer<QWidget> *m_lastPassiveInteractor;
     static bool m_lastWasAPassiveInteractor;
